#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>

#include <particle/papi.h>
#include <GL/glut.h>

#include "flow.h"
#include "partDrawOGL.h"
#include "partTex.h"

/*-- Function Prototypes -----------------------------------*/
void Reshape(int w, int h);
void Update();
void UpdateTimer(int);
void Draw();
void KeyPress(unsigned char, int, int);

/*-- Globals -----------------------------------------------*/
static	GLuint	texture;
int		frame		= 0;
int		lastTimeVal = 0;
int		timeVal		= 0;
int		aveTimeDiff	= 0;
int		sleepTime	= 0;
int		rateAccum	= 0;
float	camRotX		= -0.09;
float	camRotY		=  0.15;
float	camRotZ		=  0.0;
double	fps			= 30.0;
double	rateAdj		= 1.0;
float	timeAdj		= 1.0;
PartDrawOGL	glDraw;
MMatrix		rotMat;

Flow	*flow = 0;

int		doAnim		= 1;
int		doLookAt	= 1;

#define F_RAND ((float)rand()/(float)RAND_MAX)

/*----------------------------------------------------------*/
int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
	glutInitWindowSize(640, 480);
	glutCreateWindow("flow code");
	glutDisplayFunc(Draw);
	glutReshapeFunc(Reshape);
	glutKeyboardFunc(KeyPress);
	
	glDisable(GL_CULL_FACE);
	glDisable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glColor4f(1.0, 1.0, 1.0, 1.0);
	
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	glTexImage2D(GL_TEXTURE_2D, 0,   tex_0Comp, tex_0w, tex_0h, 0, (GLenum)tex_0Format, GL_UNSIGNED_BYTE, tex_0); 
	gluBuild2DMipmaps(GL_TEXTURE_2D, tex_0Comp, tex_0w, tex_0h,    (GLenum)tex_0Format, GL_UNSIGNED_BYTE, tex_0);
	
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glBindTexture(GL_TEXTURE_2D, texture);

	flow = new Flow();

	glDraw.setEyePoint(flow->camPos.x(), flow->camPos.y(), flow->camPos.z());
	
	glMatrixMode(GL_MODELVIEW);
	glutTimerFunc(100, UpdateTimer, true);
	glutMainLoop();
	return 0;
}

/*----------------------------------------------------------*/
void Reshape(int w, int h)
{
	fprintf(stderr, "new size = %d x %d\n", w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective( flow->fov, (double)w/(double)h, flow->nearPlane, flow->farPlane);
	glMatrixMode(GL_MODELVIEW);
	glViewport(0,0,w,h);
}

/*----------------------------------------------------------*/
void UpdateTimer(int status) 
{
	int			timeDiff = status; /* assignment is meaningless - just to get rid of compiler warning */
	struct		timeval	tv;
	
	if(doAnim)
		glutTimerFunc(30, UpdateTimer, doAnim);
	Update();
	
	gettimeofday(&tv, 0);
	timeVal = (tv.tv_sec * 1000000) + tv.tv_usec;
	
	if(lastTimeVal)
	{
		timeDiff = timeVal-lastTimeVal;
		fps      = 0.8*fps     + 0.2*(1000000.0/timeDiff);
		timeAdj	 = 0.8*timeAdj + 0.2*(30.0/fps);
		if( !(frame%64) )
		{
			flow->timeAdj = 0.5*timeAdj + 0.5*flow->timeAdj;
			fprintf(stdout, "fps = %f\n", fps);
		}
	}
	lastTimeVal = timeVal;
}

/*----------------------------------------------------------*/
void Update (void) 
{
	frame++;
	flow->frame = frame;
	Draw();
}

/*----------------------------------------------------------*/
void KeyPress (unsigned char key, int, int) 
{
	static int oldWidth, oldHeight;
	static bool fullScrn = false;
	switch(key)
	{
		case '	':
			fullScrn = !fullScrn;
			if(fullScrn)
			{
				oldWidth = glutGet(GLenum(GLUT_WINDOW_WIDTH));
				oldHeight = glutGet(GLenum(GLUT_WINDOW_HEIGHT));
				glutSetCursor(GLUT_CURSOR_NONE);
				glutFullScreen(); 
			}
			else
			{
				glutSetCursor(GLUT_CURSOR_LEFT_ARROW);
				glutReshapeWindow(oldWidth, oldHeight);
			}
			break;
		case 'q':
		case 'Q':
		case '': /*ESC*/
			fprintf(stdout, "exit. frame = %d\n", frame);
			fflush(stdout);
			exit(0);
			break;
		case ' ':
			doAnim = !doAnim;
			glutTimerFunc(30, UpdateTimer, doAnim);
			break;
		case '<':
			camRotY -= 0.05;
			break;
		case '>':
			camRotY += 0.05;
			break;
		case '[':
			camRotX -= 0.05;
			break;
		case ']':
			camRotX += 0.05;
			break;
	}
}

/*----------------------------------------------------------*/
void	Draw()
{
	static	float rotX = 0.0;
	static	float rotY = 0.0;
	static	float rotZ = 0.0;
	MVertex	curCamPos;
		
	rotX -= camRotX * flow->timeAdj; 
	rotY -= camRotY * flow->timeAdj; 
	rotZ -= camRotZ * flow->timeAdj; 
	
	if(doLookAt)
	{
		rotMat.setIdentity();
		rotMat.rotateX(-rotX);
		rotMat.rotateZ(-rotZ);
		rotMat.rotateY(-rotY);
		curCamPos = rotMat*flow->camPos;
		glDraw.setEyePoint(curCamPos.x(), curCamPos.y(),curCamPos.z());
	}
	
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	glMultMatrixf(flow->camMat.mat());

	glRotatef ( rotX, 1.0, 0.0, 0.0);
	glRotatef ( rotZ, 0.0, 0.0, 1.0);
	glRotatef ( rotY, 0.0, 1.0, 0.0);
					
/*==========================================================*/
/*-- FLOW_EXEC --------*/
	flow->execParticleGroup00();
	glDraw.pDrawGroupp(Line);
	flow->execParticleGroup01();
	glDraw.pDrawGroupp(Line);
	flow->execParticleGroup02();
	glDraw.pDrawGroupp(Disk);
/*==========================================================*/

	glutSwapBuffers();
}

/*----------------------------------------------------------*/
void	DrawPrimtives()
{
}

