#include "partDrawOGL.h"

#include <stdio.h>

//extern	int				g_frame;
//extern	CameraView		*g_globalCam;
extern _ParticleState	_ps;

////-- Default material colors
static	float	emis[4] = { 0.00, 0.00, 0.00, 1.00 };
static	float	ambt[4] = { 0.00, 0.00, 0.00, 1.00 };
static	float	diff[4] = { 1.00, 1.00, 1.00, 1.00 };
static	float	spec[4] = { 0.50, 0.50, 0.50, 1.00 };

const int partPointSkip = 1;

//==============================================================================
PNode::PNode()
{
	rank		= -1.0;
	particle	=  0;
	less		=  0;
	more		=  0;
}

PNode::~PNode()
{
}

bool			PartDrawOGL::sort = false;
MVertex			PartDrawOGL::eyePos;
int				PartDrawOGL::poolSize = 0;
PNode			*PartDrawOGL::partPool = 0;

PPrimitiveType	PartDrawOGL::lastTypeDrawn = Point;
int				PartDrawOGL::tmpCount = 0;

int				PartDrawOGL::pCount = 0;
MMatrix			PartDrawOGL::camRot;
MMatrix			PartDrawOGL::velRot;

//==============================================================================
PartDrawOGL::PartDrawOGL()
{
}

PartDrawOGL::~PartDrawOGL()
{
}

//--------------------------------------------------------------------
void PartDrawOGL::setSorting(bool status)
{
	sort = status;
}	

//--------------------------------------------------------------------
void PartDrawOGL::setEyePoint(float x, float y, float z)
{
	eyePos.setX(x);
	eyePos.setY(y);
	eyePos.setZ(z);
	velRot.pointAt(	eyePos );
}
void PartDrawOGL::setEyePoint(float *pos)
{
	eyePos.setX(pos[0]);
	eyePos.setY(pos[1]);
	eyePos.setZ(pos[2]);
	velRot.pointAt(	eyePos );
}

//--------------------------------------------------------------------
GLfloat* PartDrawOGL::scaleColor(float *in)
{
	static GLfloat	out[4];
	out[0] = (float)in[0];
	out[1] = (float)in[1];
	out[2] = (float)in[2];
	out[3] = (float)in[3];
	float max = 1.0;
	if (out[0] > max)
		max = out[0];
	if (out[1] > max)
		max = out[1];
	if (out[2] > max)
		max = out[2];
	if (max > 1.0)
	{
		float tmp = 1.0/max;
		out[0] *= tmp;
		out[1] *= tmp;
		out[2] *= tmp;
	}
	return out;
}

/**
 * Draw current particle group. 
 * @param primitive primitive type
 */
//--------------------------------------------------------------------
void PartDrawOGL::pDrawGroupp(PPrimitiveType primitive)
{	
	// Get a pointer to the particles in gp memory
	ParticleGroup *pg = _ps.pgrp;
	if(pg == NULL)
		return; // ERROR
		
	if(pg->p_count < 1)
		return;
		
	//-- Set material to a known state...
	glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,	emis);
	glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,	ambt);
	glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,	diff);
	glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,	spec);
	glMaterialf  (GL_FRONT_AND_BACK, GL_SHININESS,	100);
	
	int 		i;
	if(sort)
	{
		sortParticles(pg);
		pCount = 0;
		walkTree(&partPool[0], primitive);
	}
	else
	{
		switch(primitive)
		{
			case Point:
				for(i = 0; i < pg->p_count; i++)
					drawSphere(pg->list[i]);
				break;
			case Line:
				glBegin(GL_LINES);
				for(i = 0; i < pg->p_count; i++)
					drawLine(pg->list[i]);
				glEnd();
				break;
			case Disk:
				for(i = 0; i < pg->p_count; i++)
					drawDisk(pg->list[i]);
				break;
			default:
				for(i = 0; i < pg->p_count; i++)
					drawSphere(pg->list[i]);
				break;
		}
	}
}

/**
 * Sort particle group contents into an unbalanced binary tree.
 * A pool of particle nodes are allocated, and is dynamically resized 
 * to 1.5 * num particles. This memory is NOT autmatically freed for the 
 * sake of speed. Call setPoolSize() to reclaim memory.
 * 
 */
//--------------------------------------------------------------------
void	PartDrawOGL::sortParticles(ParticleGroup *pg)
{
	int numParts = pg->p_count;
	//-- resize the pool, if necessary
	PNode	*newPool = 0;
	if(poolSize <= numParts)
	{
		poolSize = numParts+numParts/2;
		if(partPool)
			delete [] partPool;
		newPool  = new PNode[poolSize];
		partPool = newPool;
	}
	
	float	rank;
	float	diff[3];
	int		poolIndex = 0;
	for(int i = 0; i < pg->p_count; i++)
	{
		Particle &p =  pg->list[i];
		diff[0] = p.pos.x - eyePos.x();
		diff[1] = p.pos.y - eyePos.y();
		diff[2] = p.pos.z - eyePos.z();
		//-- don't bother taking square root
		rank = diff[0]*diff[0] + diff[1]*diff[1] + diff[2]*diff[2];
		partPool[poolIndex].rank	 = rank;
		partPool[poolIndex].particle = &pg->list[i];
		partPool[poolIndex].less	 = 0;
		partPool[poolIndex].more	 = 0;
		if(poolIndex)
			insertParticle(&partPool[poolIndex], &partPool[0]);
		poolIndex++;
	}
	
}
		
//--------------------------------------------------------------------
void	PartDrawOGL::insertParticle(PNode* node, PNode *treeNode)
{
	if(node->rank > treeNode->rank)
	{
		if(treeNode->more)
			insertParticle(node, treeNode->more);
		else
			treeNode->more = node;
	}
	else if(node->rank < treeNode->rank)
	{
		if(treeNode->less)
			insertParticle(node, treeNode->less);
		else
			treeNode->less = node;
	}		
}

		
//--------------------------------------------------------------------
void	PartDrawOGL::walkTree(PNode *node, PPrimitiveType primitive)
{
	if(node->more)
		walkTree(node->more, primitive);
	Particle	*p = node->particle;
	if(pCount++ % partPointSkip == 0)
	{
		MVertex	vel(-p->vel.x, -p->vel.y, -p->vel.z);
		camRot.pointAt(	MVertex(eyePos.x()- p->pos.x, eyePos.y() - p->pos.y, eyePos.z() - p->pos.z), 
						vel);
	}
	switch(primitive)
	{
		case Point:
			drawSphere(*p);
			break;
		case Line:
			drawLine(*p);
			break;
		case Disk:
			drawDisk(*p);
			break;
	}
	if(node->less)
		walkTree(node->less, primitive);
}

//--------------------------------------------------------------------
void	PartDrawOGL::walkTreePrint(PNode *node, PPrimitiveType primitive)
{
	if(node->more)
		walkTreePrint(node->more, primitive);
		
	fprintf(stderr, "%04d  %f", tmpCount, node->rank);
	tmpCount++;
	
	if(node->less)
		walkTreePrint(node->less, primitive);
}

/**
 * Set size of particle sort pool. If num particles > pool size, 
 * it is automatically resized to num particles * 1.5.
 * This member is mostly used for reclaiming memory if a huge
 * particle system has been sorted, but the current system
 * is smaller.
 */
//--------------------------------------------------------------------
void	PartDrawOGL::setPoolSize(int size)
{
	if(partPool)
			delete [] partPool;
	poolSize = size;
	partPool = new PNode[poolSize];
}

////--------------------------------------
//void	PartDrawOGL::drawSphere	(Particle &m, bool fast)
//{
//	glColor4fv(scaleColor((float*)&m.color));
////	glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, scaleColor((float*)&m.color));
//	drawTexQuad(m);
//}
//
////--------------------------------------
//void	PartDrawOGL::drawDisk	(Particle &m, bool fast)
//{
//	glColor4fv(scaleColor((float*)&m.color));
////	glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, scaleColor((float*)&m.color));
//	MVertex	vel(-m.vel.x, -m.vel.y, -m.vel.z);
//	camRot.pointAt(	MVertex(eyePos.x()- m.pos.x, eyePos.y() - m.pos.y, eyePos.z() - m.pos.z), vel);
//	drawTexQuad(m);
//}
//
////--------------------------------------
//void	PartDrawOGL::drawLine	(Particle &m, bool fast)
//{
//	glColor4fv(scaleColor((float*)&m.color));
////	glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, scaleColor((float*)&m.color));
//	MVertex	vel(-m.vel.x, -m.vel.y, -m.vel.z);
//	camRot.pointAt(	MVertex(eyePos.x()- m.pos.x, eyePos.y() - m.pos.y, eyePos.z() - m.pos.z), vel);
//	drawTexQuad(m, true);
//}

//--------------------------------------
void	PartDrawOGL::drawTexQuad(Particle &m, bool stretch)
{
	/* stuff for tail */
	float	r = 1.0;
	float	l = 0.0;
	if(stretch)
	{
		float	camDist;

		MVertex	velVec(m.vel.x, m.vel.y, m.vel.z);
		MVertex camVec(m.pos.x-eyePos.x(), m.pos.y-eyePos.y(), m.pos.z-eyePos.z());
		camDist = camVec.length();
		velVec.normalize();
		camVec.normalize();
		float	angle = fabs(velVec.dot(camVec));
		float	angleShrink = 1.0 - angle*angle;

		l = m.size.y * m.vel.length();

		if(camDist < l)		/* get rid of funky artifacts when particles get too close to camera */
			angleShrink *= camDist/l;

		l *= angleShrink;	/* faked perspective correction for length */
	}
	r = m.size.x;

	glColor4fv(scaleColor((float*)&m.color));
	glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, scaleColor((float*)&m.color));

	glPushMatrix();
	glTranslatef((GLfloat)m.pos.x, (GLfloat)m.pos.y, (GLfloat)m.pos.z);
	glMultMatrixf(camRot.matx);

	glBegin(GL_QUADS);

	glTexCoord2f(0.0, 0.0); glVertex3f((GLfloat) r, (GLfloat) r+l, (GLfloat)0.0 );
	glTexCoord2f(1.0, 0.0); glVertex3f((GLfloat)-r, (GLfloat) r+l, (GLfloat)0.0 );
	glTexCoord2f(1.0, 1.0); glVertex3f((GLfloat)-r, (GLfloat)-r, (GLfloat)0.0 );
	glTexCoord2f(0.0, 1.0); glVertex3f((GLfloat) r, (GLfloat)-r, (GLfloat)0.0 );

	glEnd();
	glPopMatrix();
}

////--------------------------------------
//void	PartDrawOGL::drawSphere	(Particle &m)
//{
//	drawDisk(m);
//}
//
////--------------------------------------
//void	PartDrawOGL::drawLine (Particle &m)
//{
//	drawDisk(m);
//}
//
////--------------------------------------
//void	PartDrawOGL::drawDisk	(Particle &m)
//{
//	/* stuff for tail */
//	float	r = 1.0;
//	float	l = 0.0;
//	float	camDist;
//	MVertex	velVec(m.vel.x, m.vel.y, m.vel.z);
//	MVertex camVec(m.pos.x-eyePos.x(), m.pos.y-eyePos.y(), m.pos.z-eyePos.z());
//	camDist = camVec.length();
//	velVec.normalize();
//	camVec.normalize();
//	float	angle = 1.0 - fabs(velVec.dot(camVec));
//	
//	float	r = m.size.x;
//	float	l = (1.0+m.size.x) * m.size.y * m.vel.length();
//	
//	if(camDist < l)
//		angle *= camDist/l;
//	l *= angle;
//	
//	glColor4fv(scaleColor((float*)&m.color));
////	glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, scaleColor((float*)&m.color));
//	glPushMatrix();
//	glTranslatef((GLfloat)m.pos.x, (GLfloat)m.pos.y, (GLfloat)m.pos.z);
//	glMultMatrixf(camRot.matx);
//	glBegin(GL_QUADS);
//	glTexCoord2f(0.0, 0.0); glVertex3f((GLfloat) r, (GLfloat) r+l, (GLfloat)0.0 );
//	glTexCoord2f(1.0, 0.0); glVertex3f((GLfloat)-r, (GLfloat) r+l, (GLfloat)0.0 );
//	glTexCoord2f(1.0, 1.0); glVertex3f((GLfloat)-r, (GLfloat)-r, (GLfloat)0.0 );
//	glTexCoord2f(0.0, 1.0); glVertex3f((GLfloat) r, (GLfloat)-r, (GLfloat)0.0 );
//	glEnd();
//	glPopMatrix();
//}
//
