#ifndef partDrawOGL_h
#define partDrawOGL_h

#include <GL/gl.h>
#include <particle/general.h>

#include "mmath.h"

enum PPrimitiveType	{	
	Point,		/**< change this to 'Sphere' */
	Line, 		/**< line, from position to position-velocity */
	Disk		/**< camera-facing disk */
};

/**
 * PNode
 */
//==============================================================================
class PNode
{
public:
	PNode();
	~PNode();
	
	float			rank;
	PPrimitiveType	type;
	Particle		*particle;
	PNode			*less;
	PNode			*more;
};

/**
 * PartDrawOGL
 */
//==============================================================================
class PartDrawOGL
{
public:
	PartDrawOGL();
	~PartDrawOGL();

	static	void	pDrawGroupp(PPrimitiveType primitive);
	
	static	void	sortParticles(ParticleGroup *pg);
	static	void	setPoolSize(int size = 100);
	
	static	void	setSorting(bool);
	static	void	setEyePoint(float x, float y, float z);
	static	void	setEyePoint(float *pos);
	
	static	void	drawTexQuad (Particle &m, bool stretch = false);
	static inline void	drawSphere (Particle &m) {
		glColor4fv(scaleColor((float*)&m.color));
		drawTexQuad(m);
	}
	static inline void drawDisk	(Particle &m) {
		glColor4fv(scaleColor((float*)&m.color));
		MVertex	vel(-m.vel.x, -m.vel.y, -m.vel.z);
		camRot.pointAt(	MVertex(eyePos.x()- m.pos.x, eyePos.y() - m.pos.y, eyePos.z() - m.pos.z), vel);
		drawTexQuad(m);
	}
	static inline void drawLine (Particle &m) {
		glColor4fv(scaleColor((float*)&m.color));
		MVertex	vel(-m.vel.x, -m.vel.y, -m.vel.z);
		camRot.pointAt(	MVertex(eyePos.x()- m.pos.x, eyePos.y() - m.pos.y, eyePos.z() - m.pos.z), vel);
		drawTexQuad(m, true);
	}
	
protected:

private:
	static	GLfloat* scaleColor(float *in);	
	static	void	insertParticle(PNode* node, PNode *treeNode);
	static	void	walkTree(PNode* node, PPrimitiveType primitive);
	static	void	walkTreePrint(PNode* node, PPrimitiveType primitive);
	
	static	bool		sort;
//	static	float		eyePos[3];
	static	MVertex		eyePos;
	
	static	int			poolSize;
	static	PNode		*partPool;
	
	static	PPrimitiveType	lastTypeDrawn;
	
	static	int	tmpCount;
	
	static	int		pCount;
	static	MMatrix	camRot;	//-- this shouldn't be static. Don't keep this class as singleton
	static	MMatrix	velRot;	//-- this shouldn't be static. Don't keep this class as singleton
};

#endif // partDrawOGL_h




