/*
 * bkgndStripes.sl
 *
 * DESCRIPTION:
 *   Diagonal stripes, intended use as a background for flow's 
 *   shader editor.
 *
 * PARAMETERS:
 *  repeats, lightColor, darkColor, roughness
 *
 * ANTIALIASING: not really
 *
 * AUTHOR: Mark B. Allan
 *
 * HISTORY:
 *
 * The RenderMan (R) Interface Procedures and RIB Protocol are:
 * Copyright 1988, 1989, Pixar.  All rights reserved.
 * RenderMan (R) is a registered trademark of Pixar.
 */

surface
bkgndStripes (
	uniform float	repeats = 12.0;
	uniform color	lightColor = 0.8;
	uniform color	darkColor  = 0.6;
	uniform float	roughness = 0.1;
	)
{
    normal	Nf = faceforward (normalize(N),I);
	color	Ct;
	float	stripe;
	
	stripe = floor ((-s-t) * repeats );
	Ct = mix(lightColor, darkColor, smoothstep(0, 0.5, mod(stripe, 2)) );
	Ci = Os * Ct * diffuse(Nf) + 0.75*specular(Nf,-normalize(I),roughness) ;
	Oi = Os;
}
