/* glMatRay.sl
 *
 * DESCRIPTION:
 *   juiced-up plastic shader that has similar parameters to OpenGL
 *   materials, with ray-tracing
 *
 * PARAMETERS:
 *  Kd, Ka, Ks, Ke(emission), rough, specularcolor, ambientcolor, 
 *  emissivecolor
 *	Kr(reflection), blur, samples
 *
 * ANTIALIASING: 
 *
 * AUTHOR: Mark B. Allan
 *
 * HISTORY:
 *  The ray-tracing bits were lifted from one of Larry Gritz's shaders.
 *
 * The RenderMan (R) Interface Procedures and RIB Protocol are:
 * Copyright 1988, 1989, Pixar.  All rights reserved.
 * RenderMan (R) is a registered trademark of Pixar.
 */
 
surface
glMatRay (	float Kd			= 0.5;
			float Ka 			= 1;
			float Ks 			= 1;
        	float Ke 			= 1.0;
			float rough 		= 0.05;
			color specularcolor = 1;
	 		color ambientcolor	= 1.0;
			color emissivecolor	= 0;
			float Kr 			= 0.5;
			float blur 			= 0;
			float samples 		= 1)
{
    normal	Nf;					/* Forward facing normal vector */
    vector	IN;					/* normalized incident vector */
    vector	Rdir;				/* Smooth reflection direction */
    vector	uoffset, voffset;	/* Offsets for blur */
    color	ev;					/* Color of the reflections */
    vector	R;					/* Direction to cast the ray */
    uniform float i, j;
    
    /* Construct a forward facing surface normal */
    Nf = faceforward (normalize(N), I);
    IN = normalize (I);
    ev = 0;

    /* Calculate the reflection color */
    if (Kr > 0.001) 
	{
		/* Rdir gets the perfect reflection direction */
		Rdir = normalize (reflect (IN, Nf));
		if (blur > 0) 
		{
	    	/* Construct orthogonal components to Rdir */
	    	uoffset = blur * normalize (point (zcomp(Rdir) - ycomp(Rdir),
					       				xcomp(Rdir) - zcomp(Rdir),
					       				ycomp(Rdir) - xcomp(Rdir)));
	    	voffset = Rdir ^ uoffset;
	    	for (i = 0;  i < samples;  i += 1) 
			{
				for (j = 0;  j < samples;  j += 1) 
				{
		    		/* Add a random offset to the smooth reflection vector */
		    		R = Rdir +
						((i + float random())/samples - 0.5) * uoffset +
						((j + float random())/samples - 0.5) * voffset;
		    		ev += trace (P, normalize(R));
				}
	    	}
	    	ev *= Kr / (samples*samples);
		}
		else
		{
	    	/* No blur, just do a simple trace */
	    	ev = Kr * trace (P, Rdir);
		}
    }
    
    /* The rest is like the plastic shader */
    Oi = Os;
    Ci =	Os * ( Cs * (ambientcolor *Ka*ambient() + Kd*diffuse(Nf) ) +
			specularcolor * (ev + Ks*specular(Nf,-IN,rough) ) ) +
	      	Ke*emissivecolor ;
}
