#include "JoystickDevice.h"

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>

#include <linux/joystick.h>

//==============================================================================
JoystickDevice::JoystickDevice()
{
	joy_fd	= -1;
	axes	=  0;
	buttons	=  0;
	strcpy(name, "Unknown");
	strcpy(devName, JOYSTICK_DEVICE);
}

JoystickDevice::~JoystickDevice()
{
	if(joy_fd > 0)
		shutdown();
}

/**
 * open up file descriptor to joystick device and get info on 
 * joystick if successful. Device name defaults to "/dev/js0"
 * @return success
 */
//--------------------------------------------------------------------
bool	JoystickDevice::initialize(const char* deviceName)
{
	int	tmpInt;
	//XXX Do check here to make sure we don't already
	//XXX a valid file descriptor...
	axes	=  0;
	buttons	=  0;
	
	strncpy(devName, deviceName, NAME_LENGTH);
	if( (joy_fd = open(devName, O_RDONLY)) < 0)
	{
		perror("JoysickDevice");
		return false;
	}
	
	ioctl(joy_fd, JSIOCGAXES,		&axes);
	ioctl(joy_fd, JSIOCGBUTTONS,	&buttons);
	ioctl(joy_fd, JSIOCGNAME(NAME_LENGTH), name);
	ioctl(joy_fd, JSIOCGVERSION,	&tmpInt);
	sprintf(version, "%d.%d.%d", tmpInt >> 16, (tmpInt >> 8) & 0xff, tmpInt & 0xff);	
	
	return true;
}

/**
 * closes joystick device file descriptor. Sets joy_fd to -1 if
 * successful.
 * @return success
 */
//--------------------------------------------------------------------
bool	JoystickDevice::shutdown()
{
	bool retVal;
	retVal = !close(joy_fd);
	if(retVal)
	{
		axes	=  0;
		buttons	=  0;
		joy_fd	= -1;
		strcpy(name, "Unknown");
	}
	else
	{
		perror("JoysickDevice");
	}
	return retVal;
}



