#include "JoystickWidget.h"

#include <stdio.h>

#include <qkeycode.h>
#include <qpopupmenu.h>
#include <qapplication.h>
#include <qmessagebox.h>

#include <errno.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>

#include <linux/joystick.h>

#include "extern.h"

//==============================================================================
JoystickWidget::JoystickWidget(const char* jsDevice, QWidget *parent, const char* name)
	: QWidget(parent, name)
{
	joyButGroup	= 0;
	axisGroup	= 0;

	setFont(*g_font_MB);

	fileMenu = new QPopupMenu(this);
	fileMenu->insertItem("&test", this, SLOT(testSlot()) );
	fileMenu->insertSeparator();
	fileMenu->insertItem("&quit", qApp, SLOT(quit()), CTRL+Key_Q );

	menuBar = new QMenuBar(this);
	menuBar->insertItem("&file", fileMenu );
	
	devLabel = new QLabel(this);
	devLabel->setText( jsDevice );
	devLabel->setAlignment(AlignCenter);
	devLabel->setFixedHeight(20);
	
	QLabel		*axesTextLabel;
	QLabel		*buttonsTextLabel;
	QLabel		*nameTextLabel;
	QLabel		*versionTextLabel;
	axesTextLabel		= defaultTextLabel("axes:");
	buttonsTextLabel	= defaultTextLabel("buttons:");
	nameTextLabel		= defaultTextLabel("name:");
	versionTextLabel	= defaultTextLabel("version:");
	axesLabel			= defaultLabel();
	buttonsLabel		= defaultLabel();
	nameLabel			= defaultLabel();
	versionLabel		= defaultLabel();
	
	axisXLabel = new QLabel(this);
	axisYLabel = new QLabel(this);
	
	joyButGroup	= new QButtonGroup( "buttons", this);
	joyButGroup->setFixedHeight(45);
	axisGroup	= new QList<QSlider>;
	//-- Initialize the joystick...
	if(this->initializeJoystick(jsDevice))
	{
		joyNotifier = new QSocketNotifier(joystick->getJoyFD(), QSocketNotifier::Read, this);
		QObject::connect( joyNotifier, SIGNAL(activated(int)), this, SLOT(joyActivity(int)) );
		
		//-- Create button widgets
		char	num[8];
		QPushButton	*tmpButton;
		for(int i = 0; i < joystick->getButtons(); i++)
		{
			sprintf(num, "%d", i);
			tmpButton = new QPushButton(joyButGroup);
			tmpButton->setFixedSize(20, 20);
			tmpButton->setPalette(QPalette(QColor(255, 32, 32)));
			tmpButton->move(10 + i*25, 15);
			tmpButton->setToggleButton(true);
			tmpButton->setFont(*g_font_S);
			tmpButton->setText(num);
		}
		
		//-- Create axis sliders
		QSlider	*tmpSlider;
		for(int i = 0; i < joystick->getAxes(); i++)
		{
			tmpSlider = new QSlider(-127, 127, 1, 0, QSlider::Vertical, this);
			tmpSlider->setFixedWidth(15);
			axisGroup->append(tmpSlider);
		}
	}
	//-- 
	topVLayout	= new QVBoxLayout(this, 5, 5);
	labelLayout	= new QGridLayout(2, 4);
	sliderLayout= new QHBoxLayout(10);
	
	topVLayout->addSpacing(25);
	topVLayout->addWidget(devLabel);
	topVLayout->addLayout(labelLayout);
		labelLayout->addWidget( axesTextLabel,		0, 0);
		labelLayout->addWidget( axesLabel,			0, 1);
		labelLayout->addWidget( buttonsTextLabel,	1, 0);
		labelLayout->addWidget( buttonsLabel,		1, 1);
		labelLayout->addWidget( nameTextLabel, 		0, 2);
		labelLayout->addWidget( nameLabel, 			0, 3);
		labelLayout->addWidget( versionTextLabel,	1, 2);
		labelLayout->addWidget( versionLabel,		1, 3);
	topVLayout->addLayout(sliderLayout);
		for(int i = 0; i < (int)axisGroup->count(); i++)
			sliderLayout->addWidget(axisGroup->at(i));
		sliderLayout->addStretch(1);
	topVLayout->addWidget(joyButGroup);
	
	topVLayout->activate();
	labelLayout->setColStretch(0, 0);
	labelLayout->setColStretch(1, 1);
	labelLayout->setColStretch(2, 0);
	labelLayout->setColStretch(3, 4);
	
}

//--------------------------------------------------------------------
JoystickWidget::~JoystickWidget()
{
	if(joystick)
		delete joystick;
}

//--------------------------------------------------------------------
QLabel	*JoystickWidget::defaultLabel()
{
	QLabel	*retVal;
	retVal = new QLabel(this);
	retVal->setFixedHeight(20);
	retVal->setMinimumWidth(50);
	retVal->setFrameStyle( QFrame::Box | QFrame::Sunken );
	return retVal;
}

//--------------------------------------------------------------------
QLabel	*JoystickWidget::defaultTextLabel(const char* text)
{
	QLabel	*retVal;
	retVal = new QLabel(this);
	retVal->setText(text);
	retVal->setMinimumSize(retVal->sizeHint());
	retVal->setFixedHeight(20);
	retVal->setAlignment(AlignRight|AlignVCenter);
	return retVal;
}

//--------------------------------------------------------------------
void	JoystickWidget::testSlot()
{
}

//--------------------------------------------------------------------
bool	JoystickWidget::initializeJoystick(const char* devName)
{
	QString	tmpString;
	char	message[NAME_LENGTH];
	joystick = new JoystickDevice();
	if(joystick->initialize(devName))
	{
		tmpString.setNum(joystick->getAxes());
		axesLabel->setText(tmpString);
		tmpString.setNum(joystick->getButtons());
		buttonsLabel->setText(tmpString);
		tmpString = joystick->getVersion();
		versionLabel->setText(tmpString);
		tmpString = joystick->getName();
		nameLabel->setText(tmpString);
		return true;
	}
	else
	{
		sprintf(message, "Joystick device %s failed\nto initialize.", devName);
		QMessageBox::critical(this, "joystick", message);
//		qApp->quit();	// bail out.
		return false;
	}
}

//--------------------------------------------------------------------
void	JoystickWidget::joyActivity(int fd)
{
	struct	js_event js;
	int		readrc;
	QButton	*tmpButton;
	QSlider	*tmpSlider;
//	char	axisText[64];

	readrc=read(fd, &js, sizeof(struct js_event));
	if (readrc == sizeof(struct js_event))
	{
		switch ((js.type&3))
		{
			case JS_EVENT_BUTTON:
				tmpButton = joyButGroup->find(js.number);
				if(tmpButton)
					tmpButton->setDown(js.value);
				break;
			case JS_EVENT_AXIS:
				tmpSlider = axisGroup->at(js.number);
				if(tmpSlider)
					axisGroup->at(js.number)->setValue( (int)(js.value>>8) );
			break;
		}
	}
	else if (errno != EAGAIN)
	{
		perror("\nJoystickWidget::joyActivity: error reading");
		exit (1);
	}

}
