/**
 * camera.h
 * Copyright 1999 by Mark B. Allan
 * 
 * camera class for GL and RM projection
 */
#ifndef camera_h
#define camera_h

#include <stdio.h>

#include <qobject.h>

#include "glPrimitives.h"
#include "mmath.h"

#define	CAM_ORTHO	0
#define	CAM_PERSPECTIVE 1

/**
 * This is ancient code that really needs to be totally
 * rewritten. The problem is that it works just fine for the time being.
 *
 */
//--------------------------------------------------------------
class	CameraVals
{
public:
	CameraVals();
	CameraVals(CameraVals&);
	~CameraVals();
	CameraVals& operator =(CameraVals&);

	inline	int	Frame()				{	return frame;	}
	inline	int	SetFrame(int in)	{	frame = in;	return frame; }
	
	void	PrintValues(FILE *outfile);

	int			projType;
	MVertex		position;		// XYZ location of camera
	MVertex		target;			// XYZ location of target
	MVertex		direction;		// vector to center of view
	MVertex		directionXZ;	// 2D vector to center on XZ plane 
	MVertex		rotate;			// XYZ rotation values
	float		fov;
	float		aspect;
	bool		gridX;
	bool		gridY;
	bool		gridZ;
	float		unitSize;
	int			numUnits;
	float		intensity;
	bool		targetLock;
	
	int	frame;
};

/**
 * This is ancient code that really needs to be totally
 * rewritten. The problem is that it works just fine for the time being.
 */
//--------------------------------------------------------------
class	CameraView : public QObject
{
	Q_OBJECT;
	
public:
	CameraView(int pType = CAM_PERSPECTIVE,  int gridNumUnits = 10, float gridUnitSize = 1.0);
	CameraView	(CameraView &);
	~CameraView();

	inline	const char	*name()				{	return (const char*)viewName;	}
	inline	int			ProjType()			{	return vals->projType;	}
	inline	float		fov()				{	return vals->fov;		}
	inline	float		aspect()			{	return vals->aspect;	}
	inline	bool		targetLock()		{	return vals->targetLock;}
	inline	float		gridUnitSize()		{	return vals->unitSize;	}
	inline	float		gridNumUnits()		{	return vals->numUnits;	}
	
	inline	float		*pos()			{	return vals->position.xyz();}
	inline	float		posX()			{	return vals->position.x();	}
	inline	float		posY()			{	return vals->position.y();	}
	inline	float		posZ()			{	return vals->position.z();	}
	inline	float		rotX()			{	return vals->rotate.x();	}
	inline	float		rotY()			{	return vals->rotate.y();	}
	inline	float		rotZ()			{	return vals->rotate.z();	}
	
	inline	CameraVals&	GetVals()	{	return	*vals;	}
	inline	CameraVals&	SetVals(CameraVals& in)	{	*vals = in;	return *vals;	}

	void		drawGL();
	void		ProjectGL();
	void		PlaceCamGL();
	void		ProjectRIB();
	void		PlaceCamRIB();
	void		Print();
	
	void		SetProjection(int);
	MVertex&	GetTarget()	{	return vals->target;	}
	void		SetName		(const char *);
	void		SetTargetLock(bool status)		{	vals->targetLock = status;	}
	void		SetTarget	(float, float, float);
	inline void	SetTargetX	(float in)	{	vals->target.setX(in);	}
	inline void	SetTargetY	(float in)	{	vals->target.setY(in);	}
	inline void	SetTargetZ	(float in)	{	vals->target.setZ(in);	}
	void 		SetGridNumUnits(int);
	void		SetGridUnitSize(float);
	void		SetGridIntensity(float);
	void		SetGrid(bool, bool, bool);
	void		SetYZGrid(bool);
	void		SetXZGrid(bool);
	void		SetXYGrid(bool);
 
	inline void SetNear(float in)	{  nearPlane = in;	}
	inline void	SetFar(float in)	{  farPlane = in;	}
	inline float getNear()			{  return nearPlane;}
	inline float getFar()			{  return farPlane;	}

	void		DrawGrid ();
	void		Reset ();

	void		MoveInOut	(float);
	void		MoveRightLeft(float);
	void		MoveUpDown	(float);
	void		PanLeftRight(float amnt);
	void		PanUpDown	(float amnt);
	void		TrackInOut	(float amnt);
	
	CameraVals		*vals;
	
public slots:
	void		SetFov		(float);
	float		AddFov		(float);
	void		SetAspect	(float);
	void		SetTrans	(MVertex coords);
	void		SetTrans	(float, float, float);
	void		SetTransX	(float);
	void		SetTransY	(float);
	void		SetTransZ	(float);
	void		AddTrans	(MVertex coords);
	void		AddTrans	(float, float, float);
	void		AddTransX	(float);
	void		AddTransY	(float);
	void		AddTransZ	(float);
	void		AddOrbitX	(float);
//	void		AddOrbitY	(float);
	void		LookatTarget();
	void		LookInDirection();
	
	void		SetRot	(float x, float y, float z);
	

signals:
	void		UpdateRequired();
	
	
protected:
	QCString			viewName;

	MVertex			targetDir;		// vector to target
	MVertex			dirDiff;		// difference between direction and vector to target

	CameraVals		*init;
	
	float		nearPlane;
	float		farPlane;
	
	enum FogType { NoFog, Fog, DepthCue };
	bool	hasFog;
	FogType	fogType;
	MVertex	fogColor;
	float	fogDensity;
	float	fogDistance;
	float	fogNear;
	float	fogFar;
};






#endif // camera_h
