/**
 * colorButton.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * Color selection widget
 */
#include "colorButton.h"

#include <stdio.h>
#include <math.h>

#include <qlabel.h>
#include <qimage.h> 
#include <qpen.h>
#include <qpainter.h>
#include <qtooltip.h>
#include <qcolordialog.h>

#include "extern.h"

//==============================================================================
ColorButton::ColorButton ( QWidget* parent, const char* name )
	: QWidget( parent, name)
{
	active			= false;
	mValue			= 1.0;
	rValue			= 1.0;
	gValue			= 1.0;
	bValue			= 1.0;
	rInt			= 255;
	gInt			= 255;
	bInt			= 255;
	
//	setFont(*g_font_MB);
	
	colorBut = new QPushButton(this);	
	colorBut->setMinimumSize(10, 10);
	
	colorMultEdit = new QLineEdit(this);
	colorMultEdit->setMinimumSize(10, 10);
	colorMultEdit->setFixedWidth(g_numInputWidth/3);
	colorMultEdit->setFont(*g_font_M);
		
	//-- Connect
	QObject::connect(colorBut, SIGNAL(clicked()), SLOT(getNewColor()) );
	QObject::connect(colorMultEdit, SIGNAL(textChanged(const QString&)), SLOT(setColorMultEditString(const QString&)) );
	QObject::connect(colorMultEdit, SIGNAL(returnPressed()), SLOT(setMValue()) );

	//-- Layout
	hLayout = new QHBoxLayout(this, 0, 1);
	
	hLayout->addWidget(colorBut, 2);
	hLayout->addWidget(colorMultEdit, 1);
	
}


ColorButton::~ColorButton()
{
}

//-----------------------------------------------------------------------
void	ColorButton::setEnabled(bool status)
{
	active = status;
	if(status)
	{
		QPalette butPalette;
		butPalette = palette();
		butPalette.setColor(QColorGroup::Button, QColor(rInt, gInt, bInt));
		colorBut->setPalette( butPalette );
	}
	else
		colorBut->setPalette(palette());
	colorBut->setEnabled(status);
	colorMultEdit->setEnabled(status);
}

//-----------------------------------------------------------------------
void	ColorButton::getNewColor()
{
	QColor	newColor;
	newColor = QColorDialog::getColor(QColor(rInt, gInt, bInt), this);
	if(newColor.isValid())
	{
		rInt = newColor.red();
		gInt = newColor.green();
		bInt = newColor.blue();
		rValue = mValue * (double)rInt/255.0;
		gValue = mValue * (double)gInt/255.0;
		bValue = mValue * (double)bInt/255.0;
		QPalette butPalette;
		butPalette = palette();
		butPalette.setColor(QColorGroup::Button, QColor(rInt, gInt, bInt));
		colorBut->setPalette( butPalette );
		emit RGBChanged (rValue, gValue, bValue);
	}
}

//-----------------------------------------------------------------------
void	ColorButton::setColor (GColor &in)
{
	setColor( in.R(), in.G(), in.B() );
}

//-----------------------------------------------------------------------
void	ColorButton::setColor (float *rgb)
{
	setColor( rgb[0], rgb[1], rgb[2] );
}

//-----------------------------------------------------------------------
void	ColorButton::SetR(double in){  setColor(in, gValue, bValue);   }
void	ColorButton::SetG(double in){  setColor(rValue, in, bValue);   }
void	ColorButton::SetB(double in){  setColor(rValue, gValue, in);   }

//-----------------------------------------------------------------------
void	ColorButton::setColor (double r, double g, double b)
{
	double	largest = 1.0;
	if( fabs(r) > 1.0 || fabs(g) > 1.0 ||  fabs(b) > 1.0 )
	{
		largest = r;
		if( r < g )
			largest = g;
		if( g < b )
			largest = b;
	}
	mValue = largest;
	r = r/mValue;
	g = g/mValue;
	b = b/mValue;
	setColor( (int)((r*255.0)+0.5), (int)((g*255.0)+0.5), (int)((b*255.0)+0.5) );
	
	char tmp[64];
	sprintf(tmp, "%3.2f", largest);
	colorMultEdit->setText(tmp);
}

//-----------------------------------------------------------------------
void	ColorButton::setColor(int rTemp, int gTemp, int bTemp)
{
	if(	(rTemp == rInt) && (gTemp == gInt) && (bTemp == bInt))
		return;
	
	rInt = rTemp;
	gInt = gTemp;
	bInt = bTemp;
	rValue = mValue * (double)rInt/255.0;
	gValue = mValue * (double)gInt/255.0;
	bValue = mValue * (double)bInt/255.0;
	
	QPalette butPalette;
	butPalette = palette();
	butPalette.setColor(QColorGroup::Button, QColor(rInt, gInt, bInt));
	colorBut->setPalette( butPalette );
	
	emit RGBChanged (rValue, gValue, bValue);
}

//-----------------------------------------------------------------------
void	ColorButton::setColorMultEditString(const QString &val)
{
	colorMultEditString = val;
}

//-----------------------------------------------------------------------
void	ColorButton::setMValue()
{
	bool	ok;
	double	tmp;
	
	tmp = colorMultEditString.toDouble(&ok);
	if(ok)
	{
		colorMultEdit->clearFocus();
		
		mValue = tmp;
		rValue = mValue * (double)rInt/255.0;
		gValue = mValue * (double)gInt/255.0;
		bValue = mValue * (double)bInt/255.0;
		
		emit RGBChanged (rValue, gValue, bValue);
	}
	else
	{
		warning("XXX - ColorButton::setMValue( \"%s\" ) not a float value.", (const char*)colorMultEditString );
	}
}
