#ifndef ColorButton_h
#define ColorButton_h

/**
 * colorButton.h
 * Copyright 1999 by Mark B. Allan
 * 
 * Color selection widget
 */

#include <qlayout.h>
#include <qlineedit.h>
#include <qpushbutton.h>

#include "basetypes.h"

/**
 * Button to bring up QColorDialog to select color.
 */
//===========================================================================
class ColorButton : public QWidget
{
    Q_OBJECT
public:
    ColorButton( QWidget* parent = 0, const char* name = 0);
	~ColorButton();
	
	inline	float	R()		{	return rValue;	}
	inline	float	G()		{	return gValue;	}
	inline	float	B()		{	return bValue;	}
	
	void	setColor	(float *rgb);
	void	setColor	(GColor&);
	void	setColor	(double r, double g, double b);
	
signals:
	void	RGBChanged(float, float, float);

public slots:
	void	setEnabled	(bool);

protected slots:
	void	getNewColor();
	void	setColorMultEditString(const QString&);
	void	setMValue();
	
protected:
	void	setColor	(int r, int g, int b);
	void	SetR	(double in);
	void	SetG 	(double in);
	void	SetB 	(double in);

protected:
	QHBoxLayout	*hLayout;
	
	QPushButton	*colorBut;
	QString		colorMultEditString;
	QLineEdit	*colorMultEdit;
	
	bool		active;
	
	double		mValue;		/**< multiplier value				*/
	double		rValue; 	/**< red value, unlimited range 	*/
	double		gValue; 	/**< green value, unlimited range	*/
	double		bValue; 	/**< blue value, unlimited range	*/
	int			rInt;		/**< red value, 0-255				*/
	int			gInt;		/**< green value, 0-255 			*/
	int			bInt;		/**< blue value, 0-255  			*/
};

#endif // ColorButton_h
