/**
 * configFileIO.h
 * Copyright 1999 by Mark B. Allan
 * 
 * Generic config file helper classes
 */
#ifndef configFileIO_h
#define configFileIO_h

#include <stdio.h>

#include <qasciidict.h>
#include <qstring.h>
#include <qmainwindow.h>

/**
 * Class to simplify config file reading
 */
//==============================================================================
class ConfigFileReader
{
public:
	ConfigFileReader(const char* filename);
	~ConfigFileReader();

	inline	bool	IsValid()	{	return isValid;	}
	
	bool	findBool(const char* key, bool& value);
	bool	findInt(const char* key, int& value);
	bool	findFloat(const char* key, float& value);
	bool	findString(const char* key, char*& value);
	bool	findToolBarDock(const char* key, int& dock);
	
protected:
	QMainWindow::ToolBarDock ToolBarValue(QCString& strValue);
	
	
protected:
	QAsciiDict<char>	*configDict;
	bool isValid;
};

/**
 * Class to simplify config file writing
 */
//==============================================================================
class ConfigFileWriter
{
public:
	ConfigFileWriter(const char* filename);
	~ConfigFileWriter();

	inline	bool	IsValid()	{	return isValid;	}
	
	void	writeBool(const char* key, bool value);
	void	writeInt(const char* key, int value);
	void	writeFloat(const char* key, float value);
	void	writeString(const char* key, char* value);
	void	writeToolBarDock(const char* key, int dock);
	void	writeComment(const char* comment);
	
protected:
	const char*	ToolBarString(int tbDock);

protected:
	bool	isValid;
	FILE	*outfile;
	char	buffer[256];
};



#endif // configFileReader_h




