/*
 * configFlowIO.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * Read and write flow config files
 */
#include "configFlowIO.h"

#include "extern.h"
#include "configFileIO.h"
#include "mainFlow.h"
#include "particleControlWidget.h"
#include "mbaSurfaceTabDialog.h"
#include "mbaSceneWidget.h"
#include "lightGroupWidget.h"
#include "renderPanelTabDialog.h"
#include "glWidgetOrtho.h"

//--------------------------------------------------------------------
bool	ConfigFlowIO::ReadConfigFile()
{
	QCString	infileName;
//	QCString	strKey;
//	QCString	strValue;
	int		liteR = -1;
	int		liteG = -1;
	int		liteB = -1;
	int		darkR = -1;
	int		darkG = -1;
	int		darkB = -1;
	
	infileName.sprintf("%s/.ReptileLabour/Init/flow.init", g_homeDir);
	ConfigFileReader	configFile((const char*)infileName);
	if (configFile.IsValid())
	{
		//########################################
		//## GUI config
		//########################################
		configFile.findString("FONT_NAME",			g_fontName);
		configFile.findInt ("FONT_SIZE",			g_fontSize);
		configFile.findBool("PERSISTENT_GEOMETRY",	g_persistentGeom);
		configFile.findBool("TOOLTIPS_ENABLED",		g_toolTipsEnabled);
		configFile.findBool("SAVE_BASE_SCENE_ON_EXIT",g_saveBaseSceneOnExit);
		configFile.findBool("KILL_CHILDREN_ON_EXIT",g_killChildrenOnExit);
		//########################################
		//## Shader preview stuff
		//########################################
		configFile.findInt("NUM_PREVIEW_COLS",		g_numPreviewCols);
		configFile.findInt("PREVIEW_CELL_SIZE",		g_previewTableCellSize);
		configFile.findInt("PREVIEW_INTERLACE",		g_rendInterlace);
		configFile.findInt("PREVIEW_RENDER_SIZE",	g_previewRenderSize);
		configFile.findInt("PIXMAP_CACHE_SIZE",		g_pixmapCacheSize);
		configFile.findString("DEFAULT_SURFACE",		g_defaultSurfaceShader);
		configFile.findString("DEFAULT_DISPLACEMENT",	g_defaultDisplaceShader);
		configFile.findString("SHADER_PATH_ENV",		g_envShaderPath);
		//########################################
		//## external executables, etc.
		//########################################
		configFile.findString("SHADER_INFO_EXEC", g_shaderInfoExec);
		configFile.findString("TIFF_TO_JPEG_EXEC", g_tiffToJpegExec);
		configFile.findString("RENDERMAN_RENDER", g_rendermanExec);
//		configFile.findString("RENDERMAN_OPTS", g_rendermanOpts);
		configFile.findString("COMPILED_SHADER_EXT", g_compiledShaderExt);
		//########################################
		//## GL stuff
		//########################################
		configFile.findBool("OGL_FAST_PARTS", g_glFastParticles);
		configFile.findBool("OGL_ALLOW_THICK", g_glThick);
		//########################################
		//## GUI colors, 0-255 range
		//########################################
		configFile.findInt("LITE_COLOR_R", liteR);
		configFile.findInt("LITE_COLOR_G", liteG);
		configFile.findInt("LITE_COLOR_B", liteB);
		configFile.findInt("DARK_COLOR_R", darkR);
		configFile.findInt("DARK_COLOR_G", darkG);
		configFile.findInt("DARK_COLOR_B", darkB);
		//########################################
		//## Done.
		//########################################
		
//		//-- set palettes, if appropriate
//		if ( liteR >= 0 && liteG >= 0 && liteB >= 0 )
//		{
//			delete g_litePalette;
//			g_litePalette = new QPalette(QColor(liteR, liteG, liteB));
//		}
//		if ( darkR >= 0 && darkG >= 0 && darkB >= 0 )
//		{
//			delete g_darkPalette;
//			g_darkPalette = new QPalette(QColor(darkR, darkG, darkB));
//			delete g_darkPaletteBut;
//			g_darkPaletteBut = new QPalette(QColor(darkR, darkG, darkB));
//			g_darkPaletteBut->setColor(QPalette::Normal, QColorGroup::Button, g_litePalette->normal().button());
//			g_darkPaletteBut->setColor(QPalette::Active, QColorGroup::Button, g_litePalette->normal().button());
////			g_darkPaletteBut->setColor(QPalette::Disabled, QColorGroup::Button, g_litePalette->normal().button());
////			g_darkPaletteBut->setColor(QPalette::Inactive, QColorGroup::Button, g_litePalette->normal().button());
//		}
//		g_litePalette->setColor(QColorGroup::Highlight, g_darkPalette->normal().mid());
//		g_litePalette->setColor(QPalette::Disabled, QColorGroup::Highlight, g_darkPalette->normal().midlight());
//		g_darkPalette->setColor(QColorGroup::Highlight, g_darkPalette->normal().mid());
//		g_darkPalette->setColor(QPalette::Disabled, QColorGroup::Highlight, g_darkPalette->normal().midlight());
//		g_darkPaletteBut->setColor(QColorGroup::Highlight, g_darkPalette->normal().mid());
//		g_darkPaletteBut->setColor(QPalette::Disabled, QColorGroup::Highlight, g_darkPalette->normal().midlight());
		
		return true;
	}
	else
	{
		WriteConfigFile();
		return false;
	}
}

//--------------------------------------------------------------------
bool ConfigFlowIO::WriteConfigFile()
{
	QCString	outfileName;
	
	outfileName.sprintf("%s/.ReptileLabour/Init/flow.init", g_homeDir);
	
	ConfigFileWriter configFile((const char*)outfileName);
	if(configFile.IsValid())
	{
		configFile.writeComment	("\n########################################\n## GUI config\n########################################");
		configFile.writeString	("FONT_NAME", g_fontName);
		configFile.writeInt		("FONT_SIZE", g_fontSize);
		configFile.writeBool	("PERSISTENT_GEOMETRY", g_persistentGeom);
		configFile.writeBool	("TOOLTIPS_ENABLED", g_toolTipsEnabled);
		configFile.writeBool	("SAVE_BASE_SCENE_ON_EXIT", g_saveBaseSceneOnExit);
		configFile.writeBool	("KILL_CHILDREN_ON_EXIT", g_killChildrenOnExit);
		configFile.writeComment	("\n########################################\n## Shader preview stuff\n########################################");
		configFile.writeInt		("NUM_PREVIEW_COLS", g_numPreviewCols);
		configFile.writeInt		("PREVIEW_CELL_SIZE", g_previewTableCellSize);
		configFile.writeInt		("PREVIEW_INTERLACE", g_rendInterlace);
		configFile.writeInt		("PREVIEW_RENDER_SIZE", g_previewRenderSize);
		configFile.writeInt		("PIXMAP_CACHE_SIZE", g_pixmapCacheSize);
		configFile.writeString	("DEFAULT_SURFACE", g_defaultSurfaceShader);
		configFile.writeString	("DEFAULT_DISPLACEMENT", g_defaultDisplaceShader);
		configFile.writeComment	("\n########################################\n## environment variables\n########################################");
		configFile.writeString	("SHADER_PATH_ENV", g_envShaderPath);
		configFile.writeComment	("\n########################################\n## external executables, etc.\n########################################");
		configFile.writeString	("SHADER_INFO_EXEC", g_shaderInfoExec);
		configFile.writeString	("TIFF_TO_JPEG_EXEC", g_tiffToJpegExec);
		configFile.writeString	("RENDERMAN_RENDER", g_rendermanExec);
//		configFile.writeString	("RENDERMAN_OPTS", g_rendermanOpts);
		configFile.writeString	("COMPILED_SHADER_EXT", g_compiledShaderExt);
		
		configFile.writeComment	("\n########################################\n## GL stuff, etc.\n########################################");
		configFile.writeBool	("OGL_FAST_PARTS", g_glFastParticles);
		configFile.writeBool	("OGL_ALLOW_THICK", g_glThick);
		
//		//-- Get current color values
//		QColorGroup	darkBase = g_darkPalette->normal();
//		QColor		darkBack = darkBase.background();
//		QColorGroup	liteBase = g_litePalette->normal();
//		QColor		liteBack = liteBase.background();
//		configFile.writeComment	("\n########################################\n## GUI colors, 0-255 range\n########################################");
//		configFile.writeInt		("LITE_COLOR_R", liteBack.red());
//		configFile.writeInt		("LITE_COLOR_G", liteBack.green());
//		configFile.writeInt		("LITE_COLOR_B", liteBack.blue());
//		configFile.writeInt		("DARK_COLOR_R", darkBack.red());
//		configFile.writeInt		("DARK_COLOR_G", darkBack.green());
//		configFile.writeInt		("DARK_COLOR_B", darkBack.blue());
//		configFile.writeComment	("############ Normal");
//		configFile.writeComment	("LITE_COLOR_R        191");
//		configFile.writeComment	("LITE_COLOR_G        191");
//		configFile.writeComment	("LITE_COLOR_B        191");
//		configFile.writeComment	("DARK_COLOR_R        159");
//		configFile.writeComment	("DARK_COLOR_G        159");
//		configFile.writeComment	("DARK_COLOR_B        159");
//		configFile.writeComment	("############ Steel Blue");
//		configFile.writeComment	("LITE_COLOR_R        190");
//		configFile.writeComment	("LITE_COLOR_G        190");
//		configFile.writeComment	("LITE_COLOR_B        200");
//		configFile.writeComment	("DARK_COLOR_R        140");
//		configFile.writeComment	("DARK_COLOR_G        140");
//		configFile.writeComment	("DARK_COLOR_B        150");
//		configFile.writeComment	("############ Darker");
//		configFile.writeComment	("LITE_COLOR_R        190");
//		configFile.writeComment	("LITE_COLOR_G        190");
//		configFile.writeComment	("LITE_COLOR_B        190");
//		configFile.writeComment	("DARK_COLOR_R        130");
//		configFile.writeComment	("DARK_COLOR_G        130");
//		configFile.writeComment	("DARK_COLOR_B        130");
		return true;
	}
	else
	{
		warning("ERROR: Could not write %s", (const char*)outfileName);
		return false;
	}
}

//--------------------------------------------------------------------
// geometry reading is done in startup.cpp MainStartup::MoveAndResize()

//--------------------------------------------------------------------
bool	ConfigFlowIO::WriteGeomFile()
{
	QCString	outfileName;
	
	outfileName.sprintf("%s/.ReptileLabour/Init/flow.geom", g_homeDir);
	
	ConfigFileWriter configFile((const char*)outfileName);
	if(configFile.IsValid())
	{
		configFile.writeComment	("\n########################################\n## Geometry\n########################################");
		configFile.writeComment	(" ");
		configFile.writeInt	 ("WINMANADD_X",	g_winManXAdd);
		configFile.writeInt	 ("WINMANADD_Y",	g_winManYAdd);
		configFile.writeComment	(" ");
		configFile.writeInt	 ("MAINFLOW_X",		g_mainFlow->x());
		configFile.writeInt	 ("MAINFLOW_Y",		g_mainFlow->y());
		configFile.writeInt	 ("MAINFLOW_W",		g_mainFlow->width());
		configFile.writeInt	 ("MAINFLOW_H",		g_mainFlow->height());
		configFile.writeBool ("MAINFLOW_SHOW",	g_mainFlow->isVisible());
		configFile.writeComment (" ");
		configFile.writeInt	 ("PARTICLECONTROL_X", 	  g_particleControlFrame->x());
		configFile.writeInt	 ("PARTICLECONTROL_Y", 	  g_particleControlFrame->y());
		configFile.writeInt	 ("PARTICLECONTROL_W", 	  g_particleControlFrame->width());
		configFile.writeInt	 ("PARTICLECONTROL_H", 	  g_particleControlFrame->height());
		configFile.writeBool ("PARTICLECONTROL_SHOW", g_particleControlFrame->isVisible());
		configFile.writeComment (" ");
		configFile.writeInt	 ("LIGHTGROUP_X",	 g_lightGroupWidgetFrame->x());
		configFile.writeInt	 ("LIGHTGROUP_Y",	 g_lightGroupWidgetFrame->y());
		configFile.writeInt	 ("LIGHTGROUP_W",	 g_lightGroupWidgetFrame->width());
		configFile.writeInt	 ("LIGHTGROUP_H",	 g_lightGroupWidgetFrame->height());
		configFile.writeBool ("LIGHTGROUP_SHOW", g_lightGroupWidgetFrame->isVisible());
		configFile.writeComment (" ");
		configFile.writeInt	 ("SURFACE_X",	  g_surfaceEditWidget->x());
		configFile.writeInt	 ("SURFACE_Y",	  g_surfaceEditWidget->y());
		configFile.writeInt	 ("SURFACE_W",	  g_surfaceEditWidget->width());
		configFile.writeInt	 ("SURFACE_H",	  g_surfaceEditWidget->height());
		configFile.writeBool ("SURFACE_SHOW", g_surfaceEditWidget->isVisible());
		configFile.writeComment (" ");
		configFile.writeInt	 ("RENDER_X",	 g_renderOptions->x());
		configFile.writeInt	 ("RENDER_Y",	 g_renderOptions->y());
		configFile.writeInt	 ("RENDER_W",	 g_renderOptions->width());
		configFile.writeInt	 ("RENDER_H",	 g_renderOptions->height());
		configFile.writeBool ("RENDER_SHOW", g_renderOptions->isVisible());
		configFile.writeComment (" ");
		configFile.writeInt	 ("SCENETREE_X",    g_sceneWidget->x());
		configFile.writeInt	 ("SCENETREE_Y",    g_sceneWidget->y());
		configFile.writeInt	 ("SCENETREE_W",    g_sceneWidget->width());
		configFile.writeInt	 ("SCENETREE_H",    g_sceneWidget->height());
		configFile.writeBool ("SCENETREE_SHOW", g_sceneWidget->isVisible());
		configFile.writeComment	("\n########################################\n## ToolBar locations\n########################################");
		configFile.writeToolBarDock	("TB0_MAIN_FLOW", g_mainFlow->toolBarDock(0));
		configFile.writeToolBarDock	("TB1_MAIN_FLOW", g_mainFlow->toolBarDock(1));
		configFile.writeToolBarDock	("TB0_PARTICLE_CONTROL", g_particleControlFrame->toolBarDock(0));
		configFile.writeToolBarDock	("TB1_PARTICLE_CONTROL", g_particleControlFrame->toolBarDock(1));
		configFile.writeToolBarDock	("TB0_SURFACE_TAB", g_surfaceEditWidget->toolBarDock(0));
		configFile.writeToolBarDock	("TB0_RENDER_TAB", g_renderOptions->toolBarDock(0));
		configFile.writeToolBarDock	("TB0_SCENE_WIDGET", g_sceneWidget->toolBarDock(0));
		configFile.writeToolBarDock	("TB0_LIGHT_GROUP", g_lightGroupWidgetFrame->toolBarDock(0));
		configFile.writeToolBarDock	("TB1_LIGHT_GROUP", g_lightGroupWidgetFrame->toolBarDock(1));
		configFile.writeToolBarDock	("TB2_LIGHT_GROUP", g_lightGroupWidgetFrame->toolBarDock(2));
		configFile.writeComment	("\n########################################\n## OrthoViews\n########################################");
		for(int i = 0; i < NUM_ORTHO_PORTS; i++)
		{
			char	xPosText[128];
			char	yPosText[128];
			char	wPosText[128];
			char	hPosText[128];
			char	showText[128];
			GLWidgetOrtho	*tmpOrtho = g_mainFlow->getGlOrthoPort(i);
			if(tmpOrtho)
			{
				sprintf(xPosText, "ORTHO_%02d_X", 	 i);
				sprintf(yPosText, "ORTHO_%02d_Y", 	 i);
				sprintf(wPosText, "ORTHO_%02d_W", 	 i);
				sprintf(hPosText, "ORTHO_%02d_H",	 i);
				sprintf(showText, "ORTHO_%02d_SHOW", i);
				configFile.writeInt	( xPosText, tmpOrtho->x() );
				configFile.writeInt	( yPosText, tmpOrtho->y() );
				configFile.writeInt	( wPosText, tmpOrtho->width() );
				configFile.writeInt	( hPosText, tmpOrtho->height() );
				configFile.writeBool( showText, tmpOrtho->isVisible() );
				configFile.writeComment (" ");
			}
		}
		return true;
	}
	else
	{
		warning("ConfigFlowIO::WriteGeomFile() couldn't write to %s", (const char*)outfileName);
		return false;
	}
}

////--------------------------------------------------------------------
//bool	ConfigFlowIO::WriteGeomFile()
//{
//	QCString	outfileName;
//	
//	outfileName.sprintf("%s/.ReptileLabour/Init/flow.init", g_homeDir);
//	
//	ConfigFileWriter configFile((const char*)outfileName);
//	if(configFile.IsValid())
//	{
//		configFile.writeInt		("MAINFLOW_X", 301     );
//		configFile.writeInt		("MAINFLOW_Y", 22      );
//		configFile.writeInt		("MAINFLOW_W", 480     );
//		configFile.writeInt		("MAINFLOW_H", 455     );
//		configFile.writeInt		("MAINFLOW_SHOW", true );
//
//		configFile.writeInt		("PARTICLECONTROL_X", 793	  );
//		configFile.writeInt		("PARTICLECONTROL_Y", 22	  );
//		configFile.writeInt		("PARTICLECONTROL_W", 481	  );
//		configFile.writeInt		("PARTICLECONTROL_H", 343	  );
//		configFile.writeInt		("PARTICLECONTROL_SHOW", true );
//
//		configFile.writeInt		("LIGHTGROUP_X", 549	 );
//		configFile.writeInt		("LIGHTGROUP_Y", 790	 );
//		configFile.writeInt		("LIGHTGROUP_W", 10 	 );
//		configFile.writeInt		("LIGHTGROUP_H", 203	 );
//		configFile.writeInt		("LIGHTGROUP_SHOW", true );
//
//		configFile.writeInt		("SURFACE_X", 793	   );
//		configFile.writeInt		("SURFACE_Y", 393	   );
//		configFile.writeInt		("SURFACE_W", 10	   );
//		configFile.writeInt		("SURFACE_H", 10	   );
//		configFile.writeInt		("SURFACE_SHOW", true  );
//
//		configFile.writeInt		("RENDER_X", 941	 );
//		configFile.writeInt		("RENDER_Y", 790	 );
//		configFile.writeInt		("RENDER_W", 331	 );
//		configFile.writeInt		("RENDER_H", 10 	 );
//		configFile.writeInt		("RENDER_SHOW", true );
//
//		configFile.writeInt		("SCENETREE_X", 20		 );
//		configFile.writeInt		("SCENETREE_Y", 20		 );
//		configFile.writeInt		("SCENETREE_W", 200 	 );
//		configFile.writeInt		("SCENETREE_H", 300 	 );
//		configFile.writeInt		("SCENETREE_SHOW", false );
//		return true;
//	}
//	else
//		return false;
//}
//





