/**
 * forkVector.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * generate a vector to use w/ the execv() call
 */
#include "forkVector.h"

#include <stdio.h>
#include <string.h>

#include "extern.h"

//==============================================================================
ForkVector::ForkVector()
{
	vecLen = 0;
	vector = 0;
}
ForkVector::~ForkVector()
{
	for(int i = 0; i < vecLen; i++)
		delete [] vector[i];
	delete [] vector;
	vecLen = 0;
}

//--------------------------------------------------------------------
char** ForkVector::createRenderVector(const char* ribname, bool interlace)
{
	int i;
	//-- get number of render options
	int numOpts;
	QCString	rendOpts(g_rendermanOpts);
	rendOpts = rendOpts.simplifyWhiteSpace();
	if(rendOpts.length() > 1)
	{
		numOpts = rendOpts.contains(' ') + 1;
	}
	else
		numOpts = 0;
	
	//-- create argv[0]
	QCString	execName;
	execName = (const char*)g_rendermanExec;
	int tmp = execName.findRev('/');
	if (tmp >= 0)
		execName.remove(0, tmp+1);
	
	//-- Do cleanup if necessary...
	if(vecLen)
	{
		for(int i = 0; i < vecLen; i++)
			delete [] vector[i];
		delete [] vector;
		vecLen = 0;
	}
	
	//-- how long is the vector?
	vecLen = 3; // exec name, filename, null
	vecLen += numOpts;
	if(interlace && g_rendInterlace > 1)
		vecLen += 2;
	
	
	//-- exec name 
	vector = new char*[vecLen];
	vector[0] = new char[strlen((const char*)execName)+1];
	strcpy(vector[0], (const char*)execName);
	
	int	space;
	QCString thisOpt;
	for(i = 0; i < numOpts; i++)
	{
		space = rendOpts.find(' ');
		if(space > 0)
		{
			thisOpt = (const char*)rendOpts;
			thisOpt.truncate(space);
			rendOpts.remove(0, space+1);
			vector[1+i] = new char[strlen((const char*)thisOpt)+1];
			strcpy(vector[1+i], (const char*)thisOpt);
		}
		else
		{
			vector[1+i] = new char[strlen((const char*)rendOpts)+1];
			strcpy(vector[1+i], (const char*)rendOpts);
		}
	}
	
	if(interlace && g_rendInterlace > 1)
	{
		vector[i+1] = new char[3];
		strcpy(vector[i+1], "-d");
		vector[i+2] = new char[4];
		if(g_rendInterlace < 100)
			sprintf(vector[i+2], "%d", g_rendInterlace);
		else
			sprintf(vector[i+2], "%d", 2);
	}

	
	//-- filename to render
	vector[vecLen-2] = new char[strlen(ribname)+1];
	strcpy(vector[vecLen-2], ribname);
	
	//-- terminate w/ null
	vector[vecLen-1] = (char*)0;

#ifdef DEBUG1
	for(i = 0; i < vecLen-1; i++)
		warning("vector[%d] = \'%s\'", i, vector[i]);
	warning("================================");
#endif
	return vector;
}

//--------------------------------------------------------------------
char** ForkVector::createConvertVector(const char* tiffFilename, const char* jpegFilename)
{
	QCString	execName;
	execName = (const char*)g_tiffToJpegExec;
	int tmp = execName.findRev('/');
	if (tmp >= 0)
		execName.remove(0, tmp+1);
	
	//-- Do cleanup if necessary...
	if(vecLen)
	{
		for(int i = 0; i < vecLen; i++)
			delete [] vector[i];
		delete [] vector;
		vecLen = 0;
	}
	vecLen = 4; // exec name, filename, null
	
	vector = new char*[vecLen];
	vector[0] = new char[strlen((const char*)execName)+1];
	strcpy(vector[0], (const char*)execName);
	vector[1] = new char[strlen(tiffFilename)+1];
	strcpy(vector[1], tiffFilename);
	vector[2] = new char[strlen(jpegFilename)+1];
	strcpy(vector[2], jpegFilename);

	//-- terminate w/ null
	vector[vecLen-1] = (char*)0;

	return vector;
}
