/**
 * glDomain.h
 * Copyright 1999 by Mark B. Allan
 * 
 * gl representation of a domain
 */
#ifndef glDomain_h
#define glDomain_h

#include "compatibility.h"
#include <GL/gl.h>
#include <particle/papi.h>

#include "mbaGLSurface.h"
#include "pDomain.h"
#include "mmath.h"

/**
 * GL rendering class for domains
 */
//==============================================================================
class GLDomain
{
public:
	GLDomain();
	~GLDomain();
	
	enum	DrawStyle { Filled, Wire };
	
	inline void		CallList()	{	glCallList(displayList);	}

	void	CreateListFromDomain(DrawStyle dS, PDomain &, int detail = 0, float depth = 0.0);
	
	void	DrawPointGL		(DrawStyle dS, PDomain &);
	void	DrawLineGL		(DrawStyle dS, PDomain &);
	void	DrawTriangleGL	(DrawStyle dS, PDomain &, float detail = 0, float depth = 0.0);
	void	DrawPlaneGL		(DrawStyle dS, PDomain &, float detail = 5, float depth = 0.0);
	void	DrawBoxGL		(DrawStyle dS, PDomain &);
	void	DrawSphereGL	(DrawStyle dS, PDomain &, float detail = 12);
	void	DrawCylinderGL	(DrawStyle dS, PDomain &);
	void	DrawConeGL		(DrawStyle dS, PDomain &);
	void	DrawBlobGL		(DrawStyle dS, PDomain &);
	void	DrawDiskGL		(DrawStyle dS, PDomain &, float detail = 12, float depth = 0.0);

	void	CreateListRMSphere(MMatrix &trans, float radius, float zmin, float zmax, float thetamax, float detail, bool closeTheta = false, bool flatCap = false);
	void	CreateListRMCone(MMatrix &trans, float height, float radius, float thetamax, float detail, bool closeTheta = false, bool flatCap = false, bool roundCap = false);
	void	CreateListRMDisk(MMatrix &trans, float height, float radius, float thetamax, float detail);
	void	CreateListRMCylinder(MMatrix &trans, float radius, float zmin, float zmax, float thetamax, float detail, bool closeTheta = false, bool flatCap = false, bool roundCap = false);
	
protected:
	GLuint			displayList;
	
	MBA_GLSurface	*glSurf;	//Won't be used for now, but should be in 
								//the display list for the object
	
};

#endif
