/**
 * glPort.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * base GL widget that does all the work
 */
#include "glPortCam.h"

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <GL/gl.h>

#include <qapp.h>

#include "extern.h"
#include "define.h"
#include "glPrimitives.h"
#include "renderman.h"
#include "glWidget.h"
#include "mbaScene.h"
#include "pAction.h"
#include "pPolygonObject.h"

#include "GLExt.h"

static	float	emis[4] = { 0.00, 0.00, 0.10, 1.0 };
static	float	ambt[4] = { 0.00, 0.00, 0.00, 1.0 };
static	float	diff[4] = { 1.00, 1.00, 1.00, 1.0 };
static	float	spec[4] = { 1.00, 1.00, 1.00, 1.0 };

//--------init static members
int	 GlPortCam::activeTimers= 0;

//===============================================================
GlPortCam::GlPortCam (QWidget* parent, const char* name, const QGLWidget* listShare, GLWidget* parentWidget)
	: GlPort(parent, name, listShare)
{
	glPortConstructor(parent, name, listShare, parentWidget);
}

GlPortCam::GlPortCam (const QGLFormat &/*format*/, QWidget* parent, const char* name, const QGLWidget* listShare, GLWidget* parentWidget )
	: GlPort(parent, name, listShare)
{
	glPortConstructor(parent, name, listShare, parentWidget);
}

void GlPortCam::glPortConstructor (QWidget* parent, const char* name, const QGLWidget* listShare, GLWidget* parentWidget)
{
	//Get rid of warnings...
	parent			= parent;
	name			= name;
	listShare		= listShare;
	parentWidget	= parentWidget;
	gridXY			= false;
	gridXZ			= true;
	gridYZ			= false;
	
	mouseMode		= MouseCamera;
//	mouseMode		= MouseDomain;
	moved			= false;
//	writeRIB		= false;
	targetLock		= false;
	mouseDown		= 0;
	mouseState		= 0;
	first			= true;
	xMouseLabel		= 0;
	yMouseLabel		= 0;
	tempAni			= false;
	frameNumEdit	= 0;
	fpsDisplayLabel	= 0;
	xMouseMoveLCD	= 0;
	yMouseMoveLCD	= 0;
	fpsCounter		= 0;
	fpsAverage		= 30.0;		
	
	g_frame			= 0;
	
	viewList.setAutoDelete(true);
	
	timer = new QTimer(this);
	timerID = 0;
	
	//------------ Connect	
//	QObject::connect( timer, SIGNAL(timeout()), SLOT(TimerRefresh()) );
	
	currentCam = new CameraView();
//	currentCam->SetTrans (-1.0, 1.5, 4.0);
//	currentCam->SetTarget (0.0, 0.5, 0.0);
	currentCam->SetTrans (-4.0, 2.0, 5.0);
	currentCam->SetTarget (0.0, 0.5, 0.0);
	currentCam->SetNear(0.1);
	currentCam->SetFar(100.0);
	currentCam->LookatTarget ();
	currentCam->SetTargetLock (false);	//set this from QWidget
	currentCam->SetFov(40.0);
//	currentCam->SetFov(90.0);		
	

/*!!! DANGER, WILL ROBINSON!!!*/
	if(portIDSeed == 1)
		g_globalCam = currentCam;	
	
//	QGLFormat	tester = format();
//	warning ("alpha = %d", tester.alpha() );
	
}

//----------------------------------------------------------
GlPortCam::~GlPortCam()
{
}

//----------------------------------------------------------
void GlPortCam::RecreateGridDisplayList()
{
	if(glGridDL)
	{
		glNewList(glGridDL, GL_COMPILE);
		currentCam->SetGrid(gridYZ, gridXZ, gridXY);
		currentCam->DrawGrid();
		glEndList();
	}
	else
		warning("WARNING: GlPortCam::RecreateGridDisplayList() called before GlPortCam::initializeGL()");
}

//----------------------------------------------------------
void	GlPortCam::setCameraProjection(int projection)
{
	currentCam->SetProjection(projection);
	currentCam->ProjectGL();
}

//----------------------------------------------------------
void	GlPortCam::updateGLIfNecessary()
{
	if(!timerID)
		updateGL();
}

//----------------------------------------------------------
void	GlPortCam::timerEvent ( QTimerEvent *te  )
{
//	warning ("Window %d - timerID = %d, timerEvent ID = %d", portID, timerID, te->timerId() );
	if (te->timerId() == timerID)
	{
//		warning ("\tWindow %d - timer event accepted (ID = %d)", portID, te->timerId() );
		if (mouseDown > 0)
		{
			switch (mouseMode)
			{
				case MouseCamera:
					doMouse();
					break;
				case MouseDomain:
					doMouseDomain();
					break;
			}
		}
		if (!tempAni)
			g_frame++;
		updateGL();
	}
}

//----------------------------------------------------------
void GlPortCam::initializeGL()
{
  	makeCurrent();
	GL_ERROR_MSG("GlPortCam::initializeGL -- enter");

	glState.setCullFace(g_glCullFace);
	glState.setDepthTest(true);
	glState.setNormalize(true);
	glState.setAlphaTest(g_glAlphaTest);
	glState.setBlend(g_glBlend);
	glState.setTwoSideLight(g_glTwoSideLight);
	glState.setWireframe(false);
	glState.setLighting(true);
	glState.initialize();
	g_glInitialized = true;
	
	GLExt::initialize();
	
	glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,	emis);
	glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,	ambt);
	glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,	diff);
	glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,	spec);
	glMaterialf  (GL_FRONT_AND_BACK, GL_SHININESS,	100);

	glGridDL = glGenLists(1);
	RecreateGridDisplayList();
	
	warning("direct rendering = %d", context()->format().directRendering());
	GL_ERROR_MSG("GlPortCam::initializeGL -- leave");
}

//----------------------------------------------------------
void	GlPortCam::SetYZGrid(bool in)
{	gridYZ = in;	RecreateGridDisplayList( );   }
void	GlPortCam::SetXZGrid(bool in)
{	gridXZ = in;	RecreateGridDisplayList( );   }
void	GlPortCam::SetXYGrid(bool in)
{	gridXY = in;	RecreateGridDisplayList( );   }

//----------------------------------------------------------
void GlPortCam::paintGL()
{
	if(noGLDraw)
		return;
		
  	makeCurrent();
	static	char	numBuffer[8] = "0";
//	static	int	lastFrame = 0;
	
//	static	int	timesDrawn = 0;
//	warning("paintGL() %d", timesDrawn++);
	
	if (g_scene)
		if (g_scene->BackgroundColor())
		{	
			GColor	*bg;
			bg = g_scene->BackgroundColor();
			glClearColor( bg->R(), bg->G(), bg->B(), 1.0 );
		}

	if( frameNumEdit && g_frame > 0 )
	{
		sprintf(numBuffer, "%d", g_frame);
		frameNumEdit->setText(numBuffer);
	}

	static	int	firstCamFrame = 1;
	if (g_frame == firstCamFrame && first)
	{
		CameraVals	*camLoc = new CameraVals(currentCam->GetVals());
		camLoc->SetFrame(g_frame);
		viewList.append(camLoc);
		first = false;
//		warning ("recorded at %d", firstCamFrame);
	}
		
	if ( viewList.count() > 0 && viewList.at() >= 0 )
	{
		if ( (viewList.current())->Frame() == g_frame )
		{
			currentCam->SetVals(*(viewList.current()) );
			viewList.next();
		}
		else if( (viewList.current())->Frame() < g_frame )
		{
			while( (viewList.current()) )
			{
				if ( (viewList.current())->Frame() == g_frame)
				{
					currentCam->SetVals(*(viewList.current()) );
					break;
				}
				else if ( (viewList.current())->Frame() > g_frame)
					break;
				viewList.next();
			}
		}
		else if( (viewList.current())->Frame() > g_frame )
		{
			viewList.first();
			while( (viewList.current()) )
			{
				if ( (viewList.current())->Frame() == g_frame)
				{
					currentCam->SetVals(*(viewList.current()) );
					break;
				}
				else if ( (viewList.current())->Frame() > g_frame)
					break;
				viewList.next();
			}
		}
	}
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glLoadIdentity();
	
	currentCam->PlaceCamGL();
	
	//-- draw grid, target etc
	if (GL__lighting)
	{	GL__lighting = false; glDisable(GL_LIGHTING);	}

	glCallList(glGridDL);
	
	if (targetLock)
	{
		MVertex	target = currentCam->GetTarget();
		glPushMatrix();
		glTranslatef(target.X(), target.Y(), target.Z());
		glLineWidth(1.0);
		glBegin(GL_LINES);
			glColor3f(1.0, 0.0, 0.0);
			glVertex3f( 0.00,-0.10, 0.00);
			glVertex3f( 0.00, 0.10, 0.00);
			glVertex3f(-0.10, 0.00, 0.00);
			glVertex3f( 0.10, 0.00, 0.00);
			glVertex3f( 0.00, 0.00,-0.10);
			glVertex3f( 0.00, 0.00, 0.10);
		glEnd();
		glPopMatrix();
	}
	
	static	int	lastFrame = -1;
	if (g_scene)
	{
		this->makeCurrent();
		if (lastFrame != g_frame)
			g_scene->execute(g_frame);
		g_scene->drawGL(g_frame);
	}
	lastFrame = g_frame;
	
		
	//-- Framerate ---------------------
	if (timerID)
	{
		fpsCounter++;
		if (fpsCounter == 30)
		{
			float	msElapsed = (float)qTime.elapsed();
			char	fpsStr[16];
			float	fps = 1000.0/(msElapsed/30.0);
			sprintf(fpsStr, "%.1f", fps );
			if(fps > 5.0) //-- window resize or move causes bad fps
				FlowParticleAction::dtAdjust = 30.0/fps;
			if(fpsDisplayLabel)
			{
				fpsDisplayLabel->setText(fpsStr);
			}
			//warning("fps = %s", fpsStr );
			fpsCounter = 0;
			qTime.start();
		}
	}
}


//----------------------------------------------------------
CameraVals GlPortCam::getCameraVals(int frame)
{
	int			currItem = viewList.at();
	int			count;
	CameraVals	valsA;
	
	count = viewList.count();
	if(count > 0)
	{
		for(int i = 0; i < count; i++)
		{
			valsA = *(viewList.at(i));
			if(valsA.frame == frame)
				break;
		}
	}
	if(currItem >= 0)	// restore previous current item.
		viewList.at(currItem);
	return valsA;
}

//----------------------------------------------------------
void GlPortCam::resizeGL( int w, int h )
{
	width	= w;
	height	= h;
	aspect	= (double)w/(double)h;
	
    glViewport( 0, 0, (GLint)w, (GLint)h );
	
	currentCam->SetAspect(aspect);
	currentCam->ProjectGL();
}

//----------------------------------------------------------
void GlPortCam::SetTargetLock (bool status)
{
	targetLock = status;
	currentCam->SetTargetLock(status);
	target = currentCam->GetTarget();
	updateGL();
}

//----------------------------------------------------------
void GlPortCam::SetWriteRIB (bool )
{
	g_frame = 0;
//	writeRIB = status;
}

//----------------------------------------------------------
void GlPortCam::SetAnimation (bool status)
{
	if ( status )
	{
		fpsCounter = 0;
		qTime.start();

		if ( !timerID )
		{
			timerID = QObject::startTimer(1000/g_fps);
//			timerID = QObject::startTimer(1000/15);
			activeTimers++;
		}
	}
	else
	{
		if ( timerID )
		{
			QObject::killTimer(timerID);
			timerID = 0;
			activeTimers--;
		}
		if(fpsDisplayLabel)
			fpsDisplayLabel->setText("");
	}
}

//----------------------------------------------------------
void GlPortCam::StartTimer ()
{
}
	
//----------------------------------------------------------
void	GlPortCam::SetQLCDPointers(MLineEdit *frameEdit, QLabel* xM, QLabel* yM, QLabel *fpsL)
{
	frameNumEdit = frameEdit;
	xMouseLabel = xM;
	yMouseLabel = yM;
	fpsDisplayLabel = fpsL;
}

//----------------------------------------------------------
void GlPortCam::ResetCamList()
{
	if(viewList.count() > 0)
	{
		CameraVals*	camVal = viewList.at(0);
		if(camVal)
			currentCam->SetVals(*camVal);
	}
}

//----------------------------------------------------------
void GlPortCam::ClearCamList()
{
	viewList.clear();
	first = true;
}

