/**
 * glPortCam.h
 * Copyright 1999 by Mark B. Allan
 * 
 * camera port for perspective camera
 */
#ifndef glPortCam_h
#define glPortCam_h

#include "define.h"
#include "glPort.h"
#include "camera.h"

/**
 * 
 */
//==============================================================================
class GlPortCam	: public GlPort
{
	Q_OBJECT

public:
	GlPortCam (QWidget* parent, const char* name = 0, const QGLWidget* listShare=0, GLWidget* parentWidget = 0 );
	GlPortCam (const QGLFormat & format, QWidget* parent, const char* name = 0, const QGLWidget* listShare=0, GLWidget* parentWidget = 0 );
	~GlPortCam();
	
	enum	MouseMode	{	MouseCamera, MouseDomain	};

	inline void	SetMouseMode(int mode)	{	warning("mouseMode%d",mode); mouseMode = (MouseMode)mode;	}
	void	SetQLCDPointers(MLineEdit *frameEdit, QLabel *xM, QLabel *yM, QLabel *f);

	void		setTargetPos(float x, float y, float z);
	void		setCamPos(float x, float y, float z);
	void		setCameraProjection(int);
	
    void		resizeGL( int w, int h );
	
	void		RecreateGridDisplayList();
	
	CameraVals	getCameraVals(int frame);
	
protected:
	void		glPortConstructor(QWidget* parent, const char* name = 0, const QGLWidget* listShare=0, GLWidget* parentWidget = 0);
    void		initializeGL();
    void		paintGL();
	void 		mousePressEvent(QMouseEvent *msEv);
	void 		mouseReleaseEvent(QMouseEvent *msEv);
	void 		mouseMoveEvent(QMouseEvent *msEv);
	void 		doMouse();
	void 		doMouseDomain ();
//	void		RecreateGridDisplayList(bool gridYZ, bool gridXZ, bool gridXY);
	
	void 		StartTimer();

signals:
	void	Domain123(float val01, float val02, float val03);
	void	Domain456(float val04, float val05, float val06);
	void	Domain789(float val04, float val05, float val06);
	void	DomainDone();
	
public slots:
	void	SetTargetLock(bool);
	void	SetWriteRIB(bool);
	void	SetAnimation(bool);
	void	ResetCamList();
	void	ClearCamList();
	void	updateGLIfNecessary();
	
	void	SetYZGrid(bool);
	void	SetXZGrid(bool);
	void	SetXYGrid(bool);
	
protected slots:
//	void 		TimerRefresh( );
	
protected: 
	void	timerEvent( QTimerEvent * );
	
protected:	
	QTimer	*timer;
	int		timerID;
	
    static int	activeTimers;
	bool	tempAni;

	int			xpos;
	int			ypos;
	float		xnow;
	float		ynow;
	float		xlast;
	float		ylast;
	float		xorig;
	float		yorig;
	float		xinc;
	float		yinc;
	float		xabs;
	float		yabs;
	bool		moved;
	QPoint		mousePos;
	float		moveBlend;
	float		moveBlendInc;
	
	MouseMode	mouseMode;
	int			mouseDown;
	int			mouseState;
	
	float		camMoveFact;
	
	bool		targetLock;
	MVertex		target;

	bool		first;
	QList<CameraVals>	viewList;
	
	MLineEdit	*frameNumEdit;
	QLCDNumber	*xMouseMoveLCD;
	QLCDNumber	*yMouseMoveLCD;
	QLabel		*xMouseLabel;
	QLabel		*yMouseLabel;
	QLabel		*fpsDisplayLabel;
	
	
	QTime	qTime;
	int		fpsCounter;
	float	fpsAverage;
};

#endif // glPortCam_h
