/**
 * glPortCamMouse.h
 * Copyright 1999 by Mark B. Allan
 * 
 * mouse routines to move camera around GLPort
 */
#include "glPortCam.h"

#include <qcursor.h>

#include "extern.h"
#include "particleControlWidget.h"

//----------------------------------------------------------
void GlPortCam::mousePressEvent ( QMouseEvent *msEv )
{
	mouseState = msEv->state() | msEv->button();
  	makeCurrent();
	mouseDown++;
	

	camMoveFact = 0.0;
	
	if( mouseState == (LeftButton | RightButton) )
	{
		if(msEv->button() == LeftButton)
		{
			moveBlend = 1.0;
			moveBlendInc = -0.02;
		}
		else
		{
			moveBlend = 0.0;
			moveBlendInc = 0.02;
		}
	}
	if (mouseDown < 2)
	{
		mousePos = QCursor::pos();
		xorig = xlast = (float)(xpos = msEv->x());
		yorig = ylast = (float)(ypos = msEv->y());
		xinc = 0;
		yinc = 0;
		if (!timerID)
		{
			tempAni = true;
			SetAnimation(true);
		}
	}
}

//- - - - - - - - - - - - - - - - - - - -
void GlPortCam::mouseReleaseEvent ( QMouseEvent *msEv )
{
	mouseState = msEv->state();
	mouseDown--;
	

//	if (	xMouseMoveLCD && yMouseMoveLCD )
	if ( xMouseLabel && yMouseLabel && !mouseDown )
	{
//		xMouseMoveLCD->display(0.0);
//		yMouseMoveLCD->display(0.0);
		xMouseLabel->setText("");
		yMouseLabel->setText("");
		//QCursor::setPos(mousePos);
	}
	if ( tempAni && mouseDown <= 0 )
	{
		mouseDown = 0;
		tempAni = false;
		SetAnimation(false);
		//warning("stop ani");
	}
	if(mouseMode == MouseDomain)
		emit DomainDone();
	
	if(mouseDown < 0)
	{
//		warning("WARNING: Qt screwed up. mouseDown < 0. mouseDown reset to 0.");
		mouseDown = 0;
	}
}

//- - - - - - - - - - - - - - - - - - - -
void GlPortCam::mouseMoveEvent ( QMouseEvent *msEv )
{
	mouseState = msEv->state();
	
	xnow = (float) msEv->x();
	ynow = (float) msEv->y();
	
 	xinc = ( xnow - xpos)/375.0f;
 	yinc = ( ynow - ypos)/375.0f;
	
	xabs = xnow - xlast;
	yabs = ynow - ylast;
	xlast = xnow;
	ylast = ynow;
	
//	if (	xMouseMoveLCD && yMouseMoveLCD )
	if ( xMouseLabel && yMouseLabel  )
	{
//		xMouseMoveLCD->display(xnow - xorig);
//		yMouseMoveLCD->display(-(ynow - yorig));
		char tmpS[16];
		sprintf(tmpS, "%d", (int)(xnow - xorig));
		xMouseLabel->setText(tmpS);
		sprintf(tmpS, "%d", (int)-(ynow - yorig));
		yMouseLabel->setText(tmpS);
	}

	moved = true;
//	doMouse();
//	ypos = msEv->y();
//	xpos = msEv->x();
}

//- - - - - - - - - - - - - - - - - - - -
void GlPortCam::doMouseDomain()
{
	static	bool	first = true;

	// Trying to move with respect to cam direction... this is the
	// right idea, but only if locked on (0, Y, 0)
	CameraVals	camVals = g_globalCam->GetVals();
	MVertex		camVec = camVals.position;
	camVec.setY(0.0);
	camVec.normalize();
	
	float	xCam = camVec.X();
	float	zCam = camVec.Z();
	
//	float	mscale = 0.01;
	float	mscale = 0.1;
	float	rscale = 1.0;
	if (first == true)
	{
		first = false;
		QObject::connect(this, SIGNAL(Domain123(float, float, float)), g_particleControlWidget, SLOT(SetDomain123(float, float, float)) );
		QObject::connect(this, SIGNAL(Domain456(float, float, float)), g_particleControlWidget, SLOT(SetDomain456(float, float, float)) );
		QObject::connect(this, SIGNAL(Domain789(float, float, float)), g_particleControlWidget, SLOT(SetDomain789(float, float, float)) );
		QObject::connect(this, SIGNAL(DomainDone()), g_particleControlWidget, SLOT(SetDomainDone()) );
	}
	
	float	xaccel = xinc*fabs(1.0+(0.05*(float)(xnow-xorig)) );
	float	yaccel = yinc*fabs(1.0+(0.05*(float)(ynow-yorig)) );
	
//	if (moved)
	{
		switch( mouseState )
		{
				//-- translate
				case (LeftButton):
					emit Domain123(	(xCam*yaccel+zCam*xaccel)*mscale, 
									0.0, 
									(zCam*yaccel-xCam*xaccel)*mscale);
					break;
				case (RightButton):
					emit Domain123(0.0, -yaccel*mscale, 0.0);
					break;
				
				//-- Rotate
				case (AltButton | LeftButton ):
					emit Domain456(yaccel*rscale, 0.0, 0.0);
					break;
				case (AltButton | MidButton ):
					emit Domain456(0.0, xaccel*rscale, 0.0);
					break;
				case (AltButton | RightButton ):
					emit Domain456(0.0, 0.0, yaccel*rscale);
					break;
					
				//-- Scale
				case (ShiftButton | LeftButton ):
					emit Domain789(xabs*mscale, 0.0, yabs*mscale);
					break;
				case (ShiftButton | RightButton ):
					emit Domain789(0.0, -yabs*mscale, 0.0);
					break;
		}
//		moved = false;
	}	
}

//--------------------------------------------------------------------
void GlPortCam::doMouse()
{
	MVertex	&target = currentCam->GetTarget();
	float	moveFactor		= 0.50 * g_moveSpeed;
	float	panFactor		= 1.25 * g_moveSpeed ;
	float	panMoveFactor	= 0.10 * g_moveSpeed;
	float	targetMoveFactor= 0.10 * g_moveSpeed ;
	
	static	float	lastXCamMove = 0.0;
	static	float	lastYCamMove = 0.0;

	//-- Target move stuff
	CameraVals&	camVals = currentCam->GetVals();
	MVertex		camVec;
	camVec = camVals.position-target;
	camVec.setY(0.0);
	camVec.normalize();
	float	xCam = camVec.X();
	float	zCam = camVec.Z();
	
	
	float	camTargDist;	// distance from camera to target
	float	leashLength;	// max distance camera-targer
	float	cameraMovement;
	if (camMoveFact < 1.0)
		camMoveFact += 0.016;
	else
		camMoveFact = 1.0;
	
	float	altSpeed = 2.0;
	switch( mouseState )
	{
		case (LeftButton):
			if (currentCam->ProjType() == CAM_PERSPECTIVE)
			{
				currentCam->MoveRightLeft(lastXCamMove = xinc*moveFactor);
				currentCam->MoveInOut(lastYCamMove = yinc*moveFactor);
			}
			else if (currentCam->ProjType() == CAM_ORTHO)
			{
				currentCam->AddTransX (xinc);
				currentCam->AddTransY (-yinc);
			}
			break;
		case (MidButton):
			if (targetLock)
			{
				currentCam->AddFov(yinc*2.0);
				g_globalCam->SetFov(currentCam->fov());
				g_globalCam->ProjectGL();
			}
			else
			{
				currentCam->PanLeftRight( xinc*panFactor );
				currentCam->PanUpDown( yinc*panFactor );
			}
			break;
		case (RightButton):
			if (currentCam->ProjType() == CAM_PERSPECTIVE)
			{
				currentCam->MoveRightLeft(xinc*moveFactor);
				currentCam->MoveUpDown(-yinc*moveFactor);
				//currentCam->TrackInOut(yinc*trackFactor);
			}
			else  if (currentCam->ProjType() == CAM_ORTHO)
			{
				currentCam->AddFov (yinc*4.0f);
				currentCam->ProjectGL();
			}
			break;
		case (LeftButton | MidButton):
			currentCam->PanLeftRight( xinc*panFactor );
			currentCam->MoveInOut( yinc*panMoveFactor );
			break;
		case (LeftButton | RightButton):
			currentCam->MoveRightLeft( xinc*moveFactor );
			currentCam->MoveUpDown   (      moveBlend  * (-yinc*moveFactor));
			currentCam->MoveInOut    ( (1.0-moveBlend) * ( yinc*moveFactor));
			if(moveBlendInc)
			{
				moveBlend += moveBlendInc;
				if(moveBlend > 1.0)
				{
					moveBlend = 1.0;
					moveBlendInc = 0.0;
				}
				else if(moveBlend < 0.0)
				{
					moveBlend = 0.0;
					moveBlendInc = 0.0;
				}
			}
			//warning("%f += %f", moveBlend, moveBlendInc);
			break;
		case (MidButton | RightButton):
			currentCam->PanLeftRight( xinc*panFactor );
			currentCam->MoveInOut( yinc*panMoveFactor );
			break;
		case (LeftButton | MidButton | RightButton):
			break;

		//-- Target movement...
		case (ControlButton | LeftButton ):
			currentCam->SetTargetX(target.X()+ (xCam*yinc+zCam*xinc) * targetMoveFactor);
			currentCam->SetTargetZ(target.Z()+ (zCam*yinc-xCam*xinc) * targetMoveFactor);
			break;
		case (ControlButton | RightButton ):
			currentCam->SetTargetY(target.Y()-yinc*targetMoveFactor);
			break;
		case (ControlButton | LeftButton | RightButton ):
			currentCam->SetTargetX(target.X()+ ( zCam*xinc) * targetMoveFactor);
			currentCam->SetTargetZ(target.Z()+ (-xCam*xinc) * targetMoveFactor);
			currentCam->SetTargetY(target.Y()-yinc*targetMoveFactor);
			break;


		case (ControlButton | AltButton | LeftButton):
			//----------------------------------------
			leashLength = 2.5;
			camTargDist = sqrt(	(target.X()-camVals.position.X())*(target.X()-camVals.position.X()) + 
								(target.Y()-camVals.position.Y())*(target.Y()-camVals.position.Y()) + 
								(target.X()-camVals.position.Z())*(target.X()-camVals.position.Z()) );
			if ( camTargDist > leashLength )
//				cameraMovement = 1.0;
				cameraMovement = camMoveFact;
			else
			{
//				cameraMovement = 0.0;
				cameraMovement = (1.0 - (leashLength - camTargDist)/leashLength)*camMoveFact;
			}
			//----------------------------------------
			currentCam->SetTargetX(target.X()+ (xCam*yinc+zCam*xinc) * targetMoveFactor);
			currentCam->SetTargetZ(target.Z()+ (zCam*yinc-xCam*xinc) * targetMoveFactor);
			//currentCam->MoveRightLeft( xinc*moveFactor * cameraMovement);
			//currentCam->MoveInOut    ( yinc*moveFactor * cameraMovement);
			currentCam->MoveRightLeft(lastXCamMove);
			currentCam->MoveInOut(lastYCamMove);

			break;
		case (ControlButton | AltButton | RightButton ):
			currentCam->SetTargetY(target.Y()-yinc*targetMoveFactor);
			break;
		case (ControlButton | AltButton | LeftButton | RightButton ):
			currentCam->SetTargetX(target.X()+ ( zCam*xinc) * targetMoveFactor);
			currentCam->SetTargetZ(target.Z()+ (-xCam*xinc) * targetMoveFactor);
			currentCam->SetTargetY(target.Y()-yinc*targetMoveFactor);
			break;

		case (ControlButton | MidButton ):
			break;
		case (ControlButton | LeftButton | MidButton):
			break;
		case (ControlButton | RightButton | MidButton ):
			break;
		case (ControlButton | RightButton | MidButton | LeftButton ):
			break;
		case (ShiftButton | LeftButton ):
			break;
		case (ShiftButton | LeftButton | MidButton):
			break;
		case (ShiftButton | MidButton ):
			break;
		case (ShiftButton | LeftButton | RightButton ):
			break;
		case (ShiftButton | RightButton ):
			break;
		case (ShiftButton | RightButton | MidButton ):
			break;
		case (ShiftButton | RightButton | MidButton | LeftButton ):
			break;

		case ( AltButton | LeftButton):
			currentCam->MoveRightLeft(lastXCamMove = xinc*moveFactor);
			currentCam->MoveInOut(lastYCamMove = yinc*moveFactor);
			//--target
			currentCam->SetTargetX(target.X()+ (xCam*yinc+zCam*xinc) * targetMoveFactor*altSpeed);
			currentCam->SetTargetZ(target.Z()+ (zCam*yinc-xCam*xinc) * targetMoveFactor*altSpeed);
			break;
		case ( AltButton | MidButton):
			currentCam->PanLeftRight( xinc*panFactor );
			currentCam->PanUpDown( yinc*panFactor );
			break;
		case ( AltButton | RightButton):
			currentCam->MoveRightLeft(xinc*moveFactor);
			currentCam->MoveUpDown(-yinc*moveFactor);
			//currentCam->TrackInOut(yinc*trackFactor);
			//--target
			currentCam->SetTargetY(target.Y()-yinc*targetMoveFactor*altSpeed);
			break;
		case ( AltButton | LeftButton | MidButton):
			currentCam->PanLeftRight( xinc*panFactor );
			currentCam->MoveInOut( yinc*panMoveFactor );
			break;
		case ( AltButton | LeftButton | RightButton):
			currentCam->MoveRightLeft(xinc*moveFactor);
			currentCam->MoveUpDown(-yinc*moveFactor);
			//--target
			currentCam->SetTargetX(target.X()+ ( zCam*xinc) * targetMoveFactor*altSpeed);
			currentCam->SetTargetZ(target.Z()+ (-xCam*xinc) * targetMoveFactor*altSpeed);
			currentCam->SetTargetY(target.Y()-yinc*targetMoveFactor*altSpeed);
			break;
		case ( AltButton | MidButton | RightButton):
			currentCam->PanLeftRight( xinc*panFactor );
			currentCam->MoveInOut( yinc*panMoveFactor );
			break;
	}
	
	// Really bad camera recording mechanism...
	if (!tempAni)
	{
		CameraVals	*camLoc = new CameraVals(currentCam->GetVals());
		camLoc->SetFrame(g_frame);
		viewList.append(camLoc); 
	}
}
