/**
 * glPort.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * base GL widget that does all the work
 */
#include "glPortOrtho.h"

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <GL/gl.h>

#include <qapp.h>

#include "extern.h"
#include "define.h"
#include "glPrimitives.h"
#include "renderman.h"
#include "glWidget.h"
#include "mbaScene.h"
#include "hitList.h"
#include "particleControl.h"

//===============================================================
HitList	*GlPortOrtho::hitList = new HitList();

//===============================================================
GlPortOrtho::GlPortOrtho (QWidget* parent, const char* name, const QGLWidget* listShare, GLWidgetOrtho* parentWidget, OrthoProj view)
	: GlPort(parent, name, listShare)
{
	GL_ERROR_MSG("GlPortOrtho::GlPortOrtho -- enter");
	glPortConstructor(parent, name, listShare, parentWidget, view);
	GL_ERROR_MSG("GlPortOrtho::GlPortOrtho -- leave");
}

GlPortOrtho::GlPortOrtho (const QGLFormat &/*format*/, QWidget* parent, const char* name, const QGLWidget* listShare, GLWidgetOrtho* parentWidget, OrthoProj view )
	: GlPort(parent, name, listShare)
{
	GL_ERROR_MSG("GlPortOrtho::GlPortOrtho -- enter");
	glPortConstructor(parent, name, listShare, parentWidget, view);
	GL_ERROR_MSG("GlPortOrtho::GlPortOrtho -- leave");
}

void GlPortOrtho::glPortConstructor (QWidget* parent, const char* name, const QGLWidget* listShare, GLWidgetOrtho* parentWidget, OrthoProj view)
{
	//Get rid of warnings...
	parent			= parent;
	name			= name;
	listShare		= listShare;
	parentWidget	= parentWidget;
	gridXY			= false;
	gridXZ			= true;
	gridYZ			= false;
	coordMult[0]			= 0.0;
	coordMult[1]			= 0.0;
	coordMult[2]			= 0.0;
	mouseDown 		= 0;
	mouseNew		= 0;
	gridSnap		= false;
	gridMult		= 1.0;
//	hitList = new HitList();
	
//	writeRIB		= false;
	first			= true;
	
	rectSel		= false;
	topLeft[0]  = -1.0;
	topLeft[1]  =  1.0;
	botRight[0] =  1.0;
	botRight[1] = -1.0;
	
	//------------ Connect	
//	QObject::connect( timer, SIGNAL(timeout()), SLOT(TimerRefresh()) );
	
	currentCam = new CameraView(CAM_ORTHO);
	currentCam->SetGridNumUnits(g_gridNumUnits);
	currentCam->SetGridUnitSize(g_gridUnitSize);
	
	orthoProj = view;
//	QGLFormat	tester = format();
//	warning ("alpha = %d", tester.alpha() );
	
}

//----------------------------------------------------------
GlPortOrtho::~GlPortOrtho()
{
}

//----------------------------------------------------------
void GlPortOrtho::setGridSnap(bool status)
{
	gridSnap = status;
}

//----------------------------------------------------------
void	GlPortOrtho::setHilightHitZones(bool in)	
{	
	hitList->setHilightHitZones(in);
	updateGL();
}

//----------------------------------------------------------
void GlPortOrtho::generateHitList(int sel)
{
//	warning("GlPortOrtho::generateHitList()");
	switch(sel)
	{
		case 0:
			if( g_scene->ActiveLightGroup())
				g_scene->ActiveLightGroup()->generateHitList(*hitList);
			break;
		case 1:
			if( g_scene->ActiveParticleSystem())
				g_scene->ActiveParticleSystem()->generateHitList(*hitList);
			else 
				warning("g_scene->ActiveParticleSystem() == 0");
			break;
		default:
			hitList->reset();
			break;
	}
	hitList->setFov(currentCam->fov());
	updateGL();
}

//----------------------------------------------------------
void GlPortOrtho::RecreateGridDisplayList()
{
	if(glGridDL)
	{
		glNewList(glGridDL, GL_COMPILE);
		currentCam->SetGridIntensity(0.4);
		currentCam->SetGrid(gridYZ, gridXZ, gridXY);
		currentCam->DrawGrid();
		glEndList();
	}
	else
		warning("WARNING: GlPortOrtho::RecreateGridDisplayList() called before GlPortOrtho::initializeGL()");
//	warning("grid %d %d %d", gridYZ, gridXZ, gridXY);
}

//----------------------------------------------------------
void	GlPortOrtho::setCameraProjection(int projection)
{
	currentCam->SetProjection(projection);
	currentCam->ProjectGL();
}

//----------------------------------------------------------
void GlPortOrtho::initializeGL()
{
	GL_ERROR_MSG("GlPortOrtho::initializeGL -- enter");
  	makeCurrent();
	GL_ERROR_MSG("GlPortOrtho::initializeGL -- makeCurrent");
//	warning("GlPortOrtho::initializeGL()");
	glDepthFunc	(GL_LEQUAL);
	glEnable	(GL_DEPTH_TEST);
	glShadeModel(GL_SMOOTH); 
	glCullFace	(GL_BACK);
	glEnable	(GL_CULL_FACE );

	//FIXME -  get rid of this at some point...
	glEnable(GL_NORMALIZE);
	
	float	emis[4] = { 0.00, 0.00, 0.10, 1.0 };
	float	ambt[4] = { 0.00, 0.00, 0.00, 1.0 };
	float	diff[4] = { 1.00, 1.00, 1.00, 1.0 };
	float	spec[4] = { 1.00, 1.00, 1.00, 1.0 };
	glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,	emis);
	glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,		ambt);
	glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,		diff);
	glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,	spec);
	glMaterialf  (GL_FRONT_AND_BACK, GL_SHININESS,	100);

	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);

//	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
//	glEnable(GL_BLEND);
	glDisable(GL_BLEND);


	glPolygonMode(GL_FRONT_AND_BACK, (GLenum)(GL__polygonMode = GL_FILL) ); 
	
	glPointSize(1.0);
	glLineWidth(1.0);

	GL__lighting = true;  glEnable(GL_LIGHTING);

	glGridDL = glGenLists(1);
	currentCam->SetFov(12);
	setView(orthoProj);
	GL_ERROR_MSG("GlPortOrtho::initializeGL -- leave");
}

//----------------------------------------------------------
void	GlPortOrtho::SetYZGrid(bool in)
{	gridYZ = in;	RecreateGridDisplayList( );   }
void	GlPortOrtho::SetXZGrid(bool in)
{	gridXZ = in;	RecreateGridDisplayList( );   }
void	GlPortOrtho::SetXYGrid(bool in)
{	gridXY = in;	RecreateGridDisplayList( );   }

//----------------------------------------------------------
void GlPortOrtho::paintGL()
{
	if(noGLDraw)
		return;

  	makeCurrent();
//	glClearColor( 0.5, 0.5, 0.5, 1.0 );
	glClearColor( 0.5, 0.5, 0.5, 1.0 );
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	glLoadIdentity();
	
	currentCam->PlaceCamGL();
	
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	
	//-- draw grid, target etc
	if (GL__lighting)
	{	GL__lighting = false; glDisable(GL_LIGHTING);	}
	
	glCallList(glGridDL);

	glColor3f(1.0, 0.0, 0.0);
	
	static	int	lastFrame = -1;
	if (g_scene)
	{
//		if (lastFrame != frame)
//			g_scene->execute(frame);
		g_scene->drawGL(g_frame);
	}
	lastFrame = g_frame;
	
	glColor3f(1.0, 0.93, 0.8);
	g_globalCam->drawGL();
	
	//-->GL state
	glDisable (GL_DEPTH_TEST);
	
	if(g_glThick) glLineWidth(2.0);
	hitList->drawGL();
	
//	if(rectSel)
//	{
//		glBegin(GL_LINE_LOOP);
//		glVertex3f( topLeft[0],  topLeft[1], 0.0);
//		glVertex3f(botRight[0],  topLeft[1], 0.0);
//		glVertex3f(botRight[0], botRight[1], 0.0);
//		glVertex3f( topLeft[0], botRight[1], 0.0);
//		glEnd();
//	}
	
	//-->GL state
	glEnable (GL_DEPTH_TEST);
	
}

//----------------------------------------------------------
void GlPortOrtho::resizeGL( int w, int h )
{
	width	= w;
	height	= h;
	aspect	= (double)w/(double)h;
	
    glViewport( 0, 0, (GLint)w, (GLint)h );
	
	currentCam->SetAspect(aspect);
	currentCam->ProjectGL();
}

//----------------------------------------------------------
void GlPortOrtho::setView(OrthoProj view, bool onlySet)
{
	switch(view)
	{
		case Front: 
			setViewFront(onlySet);
			break;
		case Back: 
			setViewBack(onlySet);
			break;
		case Left: 
			setViewLeft(onlySet);
			break;
		case Right: 
			setViewRight(onlySet);
			break;
		case Top: 
			setViewTop(onlySet);
			break;
		case Bottom:
			setViewBot(onlySet);
			break;
	}
}
//----------------------------------------------------------
void GlPortOrtho::setViewFront(bool onlySet)
{
	float	x, y;
	
	getXYOrtho(x, y);
	orthoProj = Front;
 	currentCam->SetRot (0.0, 0.0, 0.0);
// 	currentCam->SetTrans (0.0, 0.0, 250.0);
 	currentCam->SetTrans (x, y, 250.0);
	xCoord = 0;
	yCoord = 1;
	coordMult[0] = -1.0;
	coordMult[1] =  1.0;
	coordMult[2] =  0.0;
	gridXY = true;
	gridXZ = false;
	gridYZ = false;
	if(!onlySet)
	{
		makeCurrent();
		RecreateGridDisplayList();
		emit camUpdated(currentCam);
	}
}

//----------------------------------------------------------
void GlPortOrtho::setViewBack(bool onlySet)
{
	float	x, y;
	
	getXYOrtho(x, y);
	orthoProj = Back;
 	currentCam->SetRot (0.0, -180.0, 0.0);
// 	currentCam->SetTrans (0.0, 0.0, -250.0);
 	currentCam->SetTrans (x, y, -250.0);
	xCoord = 0;
	yCoord = 1;
	coordMult[0] =  1.0;
	coordMult[1] =  1.0;
	coordMult[2] =  0.0;
	gridXY = true;
	gridXZ = false;
	gridYZ = false;
	if(!onlySet)
	{
		makeCurrent();
		RecreateGridDisplayList();
		emit camUpdated(currentCam);
	}
}

//----------------------------------------------------------
void GlPortOrtho::setViewLeft(bool onlySet)
{
	float	x, y;
	
	getXYOrtho(x, y);
 	orthoProj = Left;
	currentCam->SetRot ( 0.0, 90.0, 0.0);
// 	currentCam->SetTrans ( -250.0,  0.0, 0.0);
 	currentCam->SetTrans ( -250.0,  y, -x);
	xCoord = 2;
	yCoord = 1;
	coordMult[0] =  0.0;
	coordMult[1] =  1.0;
	coordMult[2] =  1.0;
	gridXY = false;
	gridXZ = false;
	gridYZ = true;
	if(!onlySet)
	{
		makeCurrent();
		RecreateGridDisplayList();
		emit camUpdated(currentCam);
	}
}

//----------------------------------------------------------
void GlPortOrtho::setViewRight(bool onlySet)
{
	float	x, y;
	
	getXYOrtho(x, y);
	orthoProj = Right;
 	currentCam->SetRot ( 0.0, -90.0, 0.0);
// 	currentCam->SetTrans ( 250.0,  0.0, 0.0);
 	currentCam->SetTrans ( 250.0,  y, -x);
	xCoord = 2;
	yCoord = 1;
	coordMult[0] =  0.0;
	coordMult[1] =  1.0;
	coordMult[2] =  1.0;
	gridXY = false;
	gridXZ = false;
	gridYZ = true;
	if(!onlySet)
	{
		makeCurrent();
		RecreateGridDisplayList();
		emit camUpdated(currentCam);
	}
}

//----------------------------------------------------------
void GlPortOrtho::setViewTop(bool onlySet)
{
	float	x, y;
	
	getXYOrtho(x, y);
	orthoProj = Top;
 	currentCam->SetRot ( 90.0, 0.0, 0.0);
// 	currentCam->SetTrans ( 0.0,  250.0, 0.0);
 	currentCam->SetTrans ( x,  250.0, y);
	xCoord = 0;
	yCoord = 2;
	coordMult[0] =  0.0;
	coordMult[1] =  1.0;
	coordMult[2] =  1.0;
	gridXY = false;
	gridXZ = true;
	gridYZ = false;
	if(!onlySet)
	{
		makeCurrent();
		RecreateGridDisplayList();
		emit camUpdated(currentCam);
	}
}

//----------------------------------------------------------
void GlPortOrtho::setViewBot(bool onlySet)
{
	float	x, y;
	
	getXYOrtho(x, y);
	orthoProj = Bottom;
 	currentCam->SetRot ( -90.0, 0.0, 0.0);
// 	currentCam->SetTrans ( 0.0,  -250.0, 0.0);
 	currentCam->SetTrans ( x,  -250.0, y);
	xCoord = 0;
	yCoord = 2;
	coordMult[0] =  0.0;
	coordMult[1] =  1.0;
	coordMult[2] =  1.0;
	gridXY = false;
	gridXZ = true;
	gridYZ = false;
	if(!onlySet)
	{
		makeCurrent();
		RecreateGridDisplayList();
		emit camUpdated(currentCam);
	}
}

//----------------------------------------------------------
void GlPortOrtho::getXYOrtho(float &x, float &y)
{
	switch(orthoProj)
	{
		case Front:
			x =  currentCam->posX();
			y =  currentCam->posY();
			break;
		case Back:
			x =  currentCam->posX();
			y =  currentCam->posY();
			break;
		case Left:
			x = -currentCam->posZ();
			y =  currentCam->posY();
			break;
		case Right:
			x = -currentCam->posZ();
			y =  currentCam->posY();
			break;
		case Top:
			x =  currentCam->posX();
			y =  currentCam->posZ();
			break; 
		case Bottom:
			x =  currentCam->posX();
			y =  currentCam->posZ();
			break;
	}
}

