/**
 * glPortOrtho.h
 * Copyright 1999 by Mark B. Allan
 * 
 * camera port for perspective camera
 */
#ifndef glPortOrtho_h
#define glPortOrtho_h

#include "glPort.h"
#include "glWidgetOrtho.h"

class HitList;

/**
 * GlPortOrtho
 */
//==============================================================================
class GlPortOrtho	: public GlPort
{
	Q_OBJECT

public:
	enum		OrthoProj { Front = 0, Back, Left, Right, Top, Bottom };
	
	GlPortOrtho (QWidget* parent, const char* name = 0, const QGLWidget* listShare=0, GLWidgetOrtho* parentWidget = 0, OrthoProj view = Front );
	GlPortOrtho (const QGLFormat & format, QWidget* parent, const char* name = 0, const QGLWidget* listShare=0, GLWidgetOrtho* parentWidget = 0, OrthoProj view = Front );
	~GlPortOrtho();
	
	void		setTargetPos(float x, float y, float z);
	void		setCamPos(float x, float y, float z);
	void		setCameraProjection(int);
	
    void		resizeGL( int w, int h );
	
	void		RecreateGridDisplayList();
	
	void		setView(OrthoProj, bool onlySet = false);
	void		setViewFront(bool onlySet = false);
	void		setViewBack (bool onlySet = false);
	void		setViewLeft (bool onlySet = false);
	void		setViewRight(bool onlySet = false);
	void		setViewTop  (bool onlySet = false);
	void		setViewBot  (bool onlySet = false);

	void		generateHitList(int sel);
	
	inline HitList	*getHitList() 				{	return hitList;	}
	inline void		setGridSnapMult(float in)	{	gridMult = in;	}
	
	void		getXYOrtho(float &x, float &y);

signals:
	void		camUpdated(CameraView *);
	
protected:
	void		glPortConstructor(QWidget* parent, const char* name = 0, const QGLWidget* listShare=0, GLWidgetOrtho* parentWidget = 0, OrthoProj view = Front);
    void		initializeGL();
    void		paintGL();
	void 		mousePressEvent(QMouseEvent *msEv);
	void 		mouseReleaseEvent(QMouseEvent *msEv);
	void 		mouseMoveEvent(QMouseEvent *msEv);
	void 		doMouse();
	void 		doMouseDomain ();
	void		refreshInputWidgets();
	int			testForHit();
	
public slots:
	void	SetYZGrid(bool);
	void	SetXZGrid(bool);
	void	SetXYGrid(bool);
	void	setGridSnap(bool);
	void	setHilightHitZones(bool in);
	
protected:	
	OrthoProj	orthoProj;
	int			xpos;
	int			ypos;
	float		xnow;
	float		ynow;
	float		xlast;
	float		ylast;
	float		xorig;
	float		yorig;
	QPoint		mousePos;
	float	xMove;
	float	yMove;
	float	zMove;
	
	double	xnorm;
	double	ynorm;
	double	xnormLast;
	double	ynormLast;
	
	float		moveAmount;
	
	bool	rectSel;
	float	topLeft[2];
	float	botRight[2];
	
	int		mouseState;
	int		mouseDown;
	int		mouseNew;	
	
	bool		first;
	int		wasHit;
	
	float	coordMult[3];
	int		xCoord;
	int		yCoord;
	
	static	HitList	*hitList;
	bool	gridSnap;
	float	gridMult;
};

#endif // glPortOrtho_h
