/**
 * glPortOrthoMouse.h
 * Copyright 1999 by Mark B. Allan
 * 
 * mouse routines to move camera around GLPort
 */
#include "glPortOrtho.h"

#include <qcursor.h>
#include <qapplication.h>

#include "extern.h"
#include "camera.h"
#include "particleControlWidget.h"
#include "lightGroupWidget.h"
#include "hitList.h"

//----------------------------------------------------------
void GlPortOrtho::mousePressEvent ( QMouseEvent *msEv )
{
	mouseState = msEv->state() | msEv->button();
  	makeCurrent();
	mouseDown++;
	
	mousePos = QCursor::pos();
	xorig = xnow = xlast = (float)(xpos = msEv->x());
	yorig = ynow = ylast = (float)(ypos = msEv->y());
	
	xnormLast = xnorm = 2.0*(((double)msEv->x()/(double)width )-0.5);
	ynormLast = ynorm = 2.0*(((double)msEv->y()/(double)height)-0.5);
	topLeft[0] = -xnorm*aspect * currentCam->fov() + currentCam->posX();
	topLeft[1] = -ynorm        * currentCam->fov() + currentCam->posY();
	
	if( (msEv->button() == LeftButton || msEv->button() == RightButton) 
		&& msEv->state() == 0)
	{
		wasHit = testForHit();
		updateGL();
	}
//	doMouse();
//	updateGL();
}
 
//- - - - - - - - - - - - - - - - - - - -
void GlPortOrtho::mouseReleaseEvent ( QMouseEvent *msEv )
{
	mouseState = msEv->state();
	mouseDown--;
	if ( mouseDown <= 0 )
	{
		g_particleControlWidget->SetDomainDone();
		g_lightGroupWidget->updateCurrentLight();
		mouseNew = 0;
	}
	rectSel = false;
	emit camUpdated(currentCam);
	g_particleControlWidget->activateDeferredAction();
	updateGL();
	//--
}

//- - - - - - - - - - - - - - - - - - - -
void GlPortOrtho::mouseMoveEvent ( QMouseEvent *msEv )
{
//	moveAmount = 0.05;
	if(mouseNew++ < 4)
		return;

  	makeCurrent();
	moveAmount = 2.0*currentCam->fov()/(float)height;
	mouseState = msEv->state();
	
	xnow = (float) msEv->x();
	ynow = (float) msEv->y();
	
	xMove = moveAmount*(xnow - xlast);
	yMove = moveAmount*(ynow - ylast);

	xlast = xnow;
	ylast = ynow;
	
	xnormLast = xnorm;
	ynormLast = ynorm;	
	xnorm = 2.0*(((double)msEv->x()/(double)width )-0.5);
	ynorm = 2.0*(((double)msEv->y()/(double)height)-0.5);
//	if(xnorm < -1.0)	xnorm = -1.0;
//	if(xnorm >  1.0)	xnorm =  1.0;
//	if(ynorm < -1.0)	ynorm = -1.0;
//	if(ynorm >  1.0)	ynorm =  1.0;
	
	if(hitList->hasActive() && msEv->state() == LeftButton)
	{
		float x =  (xnorm-xnormLast)*aspect * currentCam->fov();
		float y =  (ynorm-ynormLast)        * currentCam->fov();
		switch(orthoProj)
		{
			case Front:
				hitList->moveActive( x, -y, 0.0, gridSnap, gridMult);
				break;
			case Back:
				hitList->moveActive(-x, -y, 0.0, gridSnap, gridMult);
				break;
			case Left:
				hitList->moveActive(0.0, -y,  x, gridSnap, gridMult);
				break;
			case Right:
				hitList->moveActive(0.0, -y, -x, gridSnap, gridMult);
				break;
			case Top:
				hitList->moveActive( x, 0.0,  y, gridSnap, gridMult);
				break; 
			case Bottom:
				hitList->moveActive( x, 0.0, -y, gridSnap, gridMult);
				break;
		}
	}
	else if(hitList->hasActive() && msEv->state() == RightButton)
	{
		float y =  (ynorm-ynormLast)        * currentCam->fov();
		switch(orthoProj)
		{
			case Front:
				hitList->moveActive( 0.0, 0.0,  y, gridSnap, gridMult);
				break;
			case Back:
				hitList->moveActive( 0.0, 0.0, -y, gridSnap, gridMult);
				break;
			case Left:
				hitList->moveActive( -y, 0.0, 0.0, gridSnap, gridMult);
				break;
			case Right:
				hitList->moveActive(  y, 0.0, 0.0, gridSnap, gridMult);
				break;
			case Top:
				hitList->moveActive( 0.0, -y, 0.0, gridSnap, gridMult);
				break; 
			case Bottom:
				hitList->moveActive( 0.0, -y, 0.0, gridSnap, gridMult);
				break;
		}
	}
	else
	{
		doMouse();
		emit camUpdated(currentCam);
	}
	updateGL();
//	qApp->processEvents(66);
}

//--------------------------------------------------------------------
void GlPortOrtho::doMouse()
{
	float tmp;
	switch( mouseState )
	{
		case (LeftButton):
			switch(orthoProj)
			{
				case Front:
					currentCam->AddTrans(-xMove,  yMove, 0.0);
					break;
				case Back:
					currentCam->AddTrans( xMove,  yMove, 0.0);
					break;
				case Left:
					currentCam->AddTrans(0.0,  yMove, -xMove);
					break;
				case Right:
					currentCam->AddTrans(0.0,  yMove,  xMove);
					break;
				case Top:
					currentCam->AddTrans(-xMove, 0.0, -yMove);
					break; 
				case Bottom:
					currentCam->AddTrans(-xMove, 0.0,  yMove);
					break;
			}
			break;
		case (MidButton):
			break;
		case (RightButton):
//			rectSel = true;
//			warning("xnorm = %f ynorm = %f", xnorm, ynorm);
////			topLeft[0]  = -0.5*(xnorm*width       );
////			topLeft[1]  = -0.5*(ynorm*height      );
//			botRight[0] = -xnorm*aspect * currentCam->fov() + currentCam->posX();
//			botRight[1] = -ynorm        * currentCam->fov() + currentCam->posY();
		break;
		case (LeftButton | MidButton):
			switch(orthoProj)
			{
				case Front:
					currentCam->AddTrans(-xMove,  yMove, 0.0);
					break;
				case Back:
					currentCam->AddTrans( xMove,  yMove, 0.0);
					break;
				case Left:
					currentCam->AddTrans(0.0,  yMove, -xMove);
					break;
				case Right:
					currentCam->AddTrans(0.0,  yMove,  xMove);
					break;
				case Top:
					currentCam->AddTrans(-xMove, 0.0, -yMove);
					break; 
				case Bottom:
					currentCam->AddTrans(-xMove, 0.0,  yMove);
					break;
			}
			break;
		case (LeftButton | RightButton):
			tmp = currentCam->AddFov( yMove );
			hitList->setFov(tmp);
			currentCam->ProjectGL();
			break;
		case (MidButton | RightButton):
			break;
		case (LeftButton | MidButton | RightButton):
			tmp = currentCam->AddFov( yMove );
			hitList->setFov(tmp);
			currentCam->ProjectGL();
			break;

		case (ControlButton | LeftButton ):
			break;
		case (ControlButton | RightButton ):
			break;
		case (ControlButton | LeftButton | RightButton ):
			break;


		case (ControlButton | AltButton | LeftButton):
			break;
		case (ControlButton | AltButton | RightButton ):
			break;
		case (ControlButton | AltButton | LeftButton | RightButton ):
			break;

		case (ControlButton | MidButton ):
			break;
		case (ControlButton | LeftButton | MidButton):
			break;
		case (ControlButton | RightButton | MidButton ):
			break;
		case (ControlButton | RightButton | MidButton | LeftButton ):
			break;
		case (ShiftButton | LeftButton ):
			break;
		case (ShiftButton | LeftButton | MidButton):
			break;
		case (ShiftButton | MidButton ):
			break;
		case (ShiftButton | LeftButton | RightButton ):
			break;
		case (ShiftButton | RightButton ):
			break;
		case (ShiftButton | RightButton | MidButton ):
			break;
		case (ShiftButton | RightButton | MidButton | LeftButton ):
			break;

		case ( AltButton | LeftButton):
			break;
		case ( AltButton | MidButton):
			break;
		case ( AltButton | RightButton):
			break;
		case ( AltButton | LeftButton | MidButton):
			break;
		case ( AltButton | LeftButton | RightButton):
			break;
		case ( AltButton | MidButton | RightButton):
			break;
	}
	
}

//----------------------------------------------------------
int GlPortOrtho::testForHit()
{
	float	x, y;
	int		wasHit = 0;
	switch(orthoProj)
	{
		case Front:
			x =  xnorm*aspect * currentCam->fov() + currentCam->posX();
			y = -ynorm        * currentCam->fov() + currentCam->posY();
			wasHit = hitList->checkForHit(x, y, 0.0);
			break;
		case Back:
			x = -xnorm*aspect * currentCam->fov() + currentCam->posX();
			y = -ynorm        * currentCam->fov() + currentCam->posY();
			wasHit = hitList->checkForHit(x, y, 0.0);
			break;
		case Left:
			x =  xnorm*aspect * currentCam->fov() + currentCam->posZ();
			y = -ynorm        * currentCam->fov() + currentCam->posY();
			wasHit = hitList->checkForHit(0.0, y, x);
			break;
		case Right:
			x = -xnorm*aspect * currentCam->fov() + currentCam->posZ();
			y = -ynorm        * currentCam->fov() + currentCam->posY();
			wasHit = hitList->checkForHit(0.0, y, x);
			break;
		case Top:
			x =  xnorm*aspect * currentCam->fov() + currentCam->posX();
			y =  ynorm        * currentCam->fov() + currentCam->posZ();
			wasHit = hitList->checkForHit(x, 0.0, y);
			break;
		case Bottom:
			x =  xnorm*aspect * currentCam->fov() + currentCam->posX();
			y = -ynorm        * currentCam->fov() + currentCam->posZ();
			wasHit = hitList->checkForHit(x, 0.0, y);
			break;
	}
	return wasHit;
}











