#include "glState.h"

#include <GL/gl.h>
#include <stdio.h>

//=====================================================================
GLState::GLState()
{
	initialized  = false;
	
	alphaTest	 = false;
	blend		 = false;
	cullFace	 = true;
	depthTest	 = true;
	depthMask	 = true;
	lighting	 = false;
	lineSmooth	 = false;
	normalize	 = false;
	pointSmooth  = false;
	twoSideLight = false;
	wireframe    = false;
}

GLState::~GLState()
{
}

GLState& GLState::operator = (GLState &in)
{
	alphaTest	  = in.alphaTest;
	blend		  = in.blend;
	cullFace	  = in.cullFace;
	depthTest	  = in.depthTest;
	depthMask	  = in.depthMask;
	lighting	  = in.lighting;
	lineSmooth    = in.lineSmooth;
	normalize	  = in.normalize;
	pointSmooth	  = in.pointSmooth;
	twoSideLight  = in.twoSideLight;
	wireframe	  = in.wireframe;
	execute();
	return *this;
}

void GLState::initialize()
{
	initialized = true;
	execute();
}

void GLState::execute()
{
	if(initialized)
	{
		setAlphaTest	( alphaTest		);
		setBlend		( blend			);
		setCullFace		( cullFace		);
		setDepthTest	( depthTest		);
		setDepthMask	( depthMask		);
		setLighting		( lighting		);
		setLineSmooth	( lineSmooth	);
		setNormalize	( normalize		);
		setPointSmooth	( pointSmooth	);
		setTwoSideLight	( twoSideLight	);
		setWireframe	( wireframe		);

		glCullFace	(GL_BACK);
		glShadeModel(GL_SMOOTH); 
		glDepthFunc	(GL_LEQUAL);
		glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glClearColor( 0.0, 0.0, 0.0, 1.0 );
		glPointSize(1.0);
		glLineWidth(1.0);
		
		// GL_NICEST GL_DONT_CARE GL_FASTEST
		glHint(GL_LINE_SMOOTH_HINT,		GL_NICEST);
		glHint(GL_POINT_SMOOTH_HINT,	GL_NICEST);
	}
}
	
void GLState::setAlphaTest(bool status)
{
	alphaTest = status;
	if(initialized)
	{
		if(status)	glEnable(GL_ALPHA_TEST);
		else	   glDisable(GL_ALPHA_TEST);
	}
}

void GLState::setBlend(bool status)
{
	blend = status;
	if(initialized)
	{
		if(status)	glEnable(GL_BLEND);
		else	   glDisable(GL_BLEND);
//		fprintf(stderr, "blend %d\n", status);
	}
}

void GLState::setCullFace(bool status)
{
	cullFace = status;
	if(initialized)
	{
		if(status)	glEnable(GL_CULL_FACE);
		else	   glDisable(GL_CULL_FACE);
	}
}

void GLState::setDepthTest(bool status)
{
	depthTest = status;
	if(initialized)
	{
		if(status)	glEnable(GL_DEPTH_TEST);
		else	   glDisable(GL_DEPTH_TEST);
	}
}

void GLState::setDepthMask(bool status)
{
	depthMask = status;
	if(initialized)
	{
		if(status)	glDepthMask(GL_TRUE);
		else	    glDepthMask(GL_FALSE);
	}
}

void GLState::setLighting(bool status)
{
	lighting = status;
	if(initialized)
	{
		if(status)	glEnable(GL_LIGHTING);
		else	   glDisable(GL_LIGHTING);
	}
}

void GLState::setLineSmooth(bool status)
{
	lineSmooth = status;
	if(initialized)
	{
		if(status)	glEnable(GL_LINE_SMOOTH);
		else	   glDisable(GL_LINE_SMOOTH);
	}
}

void GLState::setNormalize(bool status)
{
	normalize = status;
	if(initialized)
	{
		if(status)	glEnable(GL_NORMALIZE);
		else	   glDisable(GL_NORMALIZE);
	}
}

void GLState::setPointSmooth(bool status)
{
	pointSmooth = status;
	if(initialized)
	{
		if(status)	glEnable(GL_POINT_SMOOTH);
		else	   glDisable(GL_POINT_SMOOTH);
	}
}

void GLState::setTwoSideLight(bool status)
{
	twoSideLight = status;
	if(initialized)
	{
		if(status)	glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE );
		else	    glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE );
	}
}

void GLState::setWireframe(bool status)
{
	wireframe = status;
	if(initialized)
	{
		if(status) glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); 
		else	   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL); 
	}
}
	






