/**
 * glWidgetOrtho.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * frame around glPort
 */
#include "glWidgetOrtho.h"
#include <qkeycode.h>
#include <qtooltip.h>

#include "extern.h"
#include "define.h"
#include "glPortOrtho.h"
#include "particleControlWidget.h"
#include "camera.h"
#include "hitList.h"

int	GLWidgetOrtho::count	= 0;

//----------------------------------------------------------
GLWidgetOrtho::GLWidgetOrtho (QWidget * parent, const char* name, const QGLWidget* listShare, int view )
	: QWidget (parent, name)
{
	topVLayout		= 0;
	glPortFrame		= 0;
	glPortLayout	= 0;
	glPortOrtho		= 0;
	
//	setFont(*g_font_MB);
	
	setFocusPolicy(QWidget::StrongFocus);
	setCaption("flow::Ortho View");
	setIcon(QPixmap((const char**)flowIcon_xpm));
	
	selectionCombo = new QComboBox(false, this);
	selectionCombo->setFixedSize(80, g_comboHeight);
	selectionCombo->insertItem("lights");
	selectionCombo->insertItem("actions");
	selectionCombo->insertItem("polygon");
	selectionCombo->insertItem("geom");
	
	selectionModeLabel = new QLabel(this);
	selectionModeLabel->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	selectionModeLabel->setPalette(QColor(224, 224, 127));
	selectionModeLabel->setAlignment( AlignCenter );
	
	projectionCombo = new QComboBox(false, this);
	projectionCombo->setFixedSize(80, g_comboHeight);
	projectionCombo->insertItem("front");
	projectionCombo->insertItem("back");
	projectionCombo->insertItem("left");
	projectionCombo->insertItem("right");
	projectionCombo->insertItem("top");
	projectionCombo->insertItem("bottom");
	
	gridDivCombo = new QComboBox(this);
	gridDivCombo->setFixedSize(80, g_comboHeight);
	gridDivCombo->insertItem(" 1  ");
	gridDivCombo->insertItem(" 1/2  ");
	gridDivCombo->insertItem(" 1/4  ");
	gridDivCombo->insertItem(" 1/5  ");
	gridDivCombo->insertItem(" 1/10 ");
	
	hilightHitZonesBut = new MToolButton(this);
	hilightHitZonesBut->setFixedSize(g_height_M, g_height_M);
	hilightHitZonesBut->setToggleButton(true);
	hilightHitZonesBut->setText("h");

	hitZoneSlider = new QSlider(10, 100, 10, 50, QSlider::Horizontal, this);
	hitZoneSlider->setFixedHeight(15);

	gridSnapBut = new MToolButton(this);
	gridSnapBut->setFixedSize(80, g_height_M);
	gridSnapBut->setToggleButton(true);
	gridSnapBut->setText("grid snap");
	
	selectionCombo->setFocusPolicy(QWidget::NoFocus);
	projectionCombo->setFocusPolicy(QWidget::NoFocus);
	gridDivCombo->setFocusPolicy(QWidget::NoFocus);
	
//	camXPosInput = new NumInput(NumInput::Double, this);
//	camYPosInput = new NumInput(NumInput::Double, this);
//	camZPosInput = new NumInput(NumInput::Double, this);
//	camXRotInput = new NumInput(NumInput::Double, this);
//	camYRotInput = new NumInput(NumInput::Double, this);
//	camZRotInput = new NumInput(NumInput::Double, this);
//	camZoomInput = new NumInput(NumInput::Double, this);

	//----------------------------------glPort
	//frame for GL context
	glPortFrame = new QFrame(this);
    glPortFrame->setFrameStyle( QFrame::Sunken | QFrame::Panel );
    glPortFrame->setLineWidth( 1 );

	//layout to auto-resize
	glPortLayout = new QHBoxLayout(glPortFrame, 1, 1);

	//gl context
	glPortOrtho = new GlPortOrtho (glPortFrame, "glPortOrtho", listShare, this, (GlPortOrtho::OrthoProj)view );
	projectionCombo->setCurrentItem(view);
	setProjection(view, true);	
	
	glPortLayout->addWidget(glPortOrtho,1);
	glPortLayout->activate();
	
	if(g_toolTipsEnabled)
	{
		QToolTip::add(projectionCombo,		"projectionCombo");
		QToolTip::add(selectionCombo,		"selectionCombo");
		QToolTip::add(hilightHitZonesBut,	"hilightHitZonesBut");
		QToolTip::add(gridSnapBut,			"gridSnapBut");
		QToolTip::add(gridDivCombo,			"gridDivCombo");
	}
	
	//-- connect
	QObject::connect(projectionCombo,	SIGNAL(activated(int)), SLOT(setProjection(int)) );
	QObject::connect(selectionCombo,	SIGNAL(activated(int)), SLOT(selectionModeChanged(int)) );
	QObject::connect(gridSnapBut,	SIGNAL(toggled(bool)), glPortOrtho, SLOT(setGridSnap(bool)) );
	QObject::connect(gridDivCombo,	SIGNAL(activated(int)), SLOT(setGridSnapMult(int)) );
	QObject::connect(glPortOrtho->getHitList(),	SIGNAL(activePrimName(const char*)), SLOT(setCurrentName(const char*)) );
//	QObject::connect(glPortOrtho, SIGNAL(activePrimName(const char*)), SLOT(setCurrentName(const char*)) );
//	QObject::connect(glPortOrtho, SIGNAL(camUpdated(CameraView *)), SLOT(refreshInputWidgets(CameraView *)) );
	QObject::connect(hilightHitZonesBut,	SIGNAL(toggled(bool)), glPortOrtho, SLOT(setHilightHitZones(bool)) );

	QObject::connect(hitZoneSlider, 	SIGNAL(valueChanged(int)), SLOT(setHitZoneSize(int)));
	
	//-- layout
	topVLayout		= new QVBoxLayout (this, 5, 5);
//	topHLayout		= new QHBoxLayout (5);
	selectLayout	= new QHBoxLayout (5);
	toolLayout		= new QHBoxLayout (0); 
	
	topVLayout->addLayout(selectLayout, 0);
		selectLayout->addWidget(selectionCombo, 0);
		selectLayout->addWidget(selectionModeLabel, 1);
		selectLayout->addWidget(projectionCombo, 0);
//	topVLayout->addLayout(topHLayout, 1);
	topVLayout->addWidget(glPortFrame, 1);
	topVLayout->addLayout(toolLayout, 0);
		toolLayout->addWidget(hilightHitZonesBut, 0);
		toolLayout->addWidget(hitZoneSlider, 1);
		toolLayout->addStretch(1);
		toolLayout->addWidget(gridSnapBut, 0);
		toolLayout->addWidget(gridDivCombo, 0);
		
	topVLayout->activate();
GL_ERROR_MSG("GLWidgetOrtho::GLWidgetOrtho -- leave");
}


//----------------------------------------------------------
GLWidgetOrtho::~GLWidgetOrtho ()
{	
}

////----------------------------------------------------------
//void GLWidgetOrtho::ToggleMoveMode(bool on)
//{
//	if (on)
//		glPort->SetMouseMode( 1 );
//	else
//		glPort->SetMouseMode( 0 );
//}
//

//----------------------------------------------------------
void GLWidgetOrtho::focusInEvent ( QFocusEvent *fE )
{
GL_ERROR_MSG("GLWidgetOrtho::focusInEvent -- enter");
	glPortOrtho->generateHitList(selectionCombo->currentItem());
	QWidget::focusInEvent(fE);
GL_ERROR_MSG("GLWidgetOrtho::focusInEvent -- leave");
}

//----------------------------------------------------------
void GLWidgetOrtho::keyPressEvent ( QKeyEvent *kE )
{
	kE->ignore();
	QWidget::keyPressEvent(kE);
}

//----------------------------------------------------------
void GLWidgetOrtho::setProjection( int proj )
{
	this->setProjection(proj, false);
}

//----------------------------------------------------------
void GLWidgetOrtho::setProjection( int proj, bool onlySet )
{
GL_ERROR_MSG("GLWidgetOrtho::setProjection -- enter");

	switch(proj)
	{
		case 0:		
			glPortOrtho->setViewFront(onlySet);
			setCaption ("flow::Ortho View Front");
			setIconText("flow::Ortho View Front");
			break;
		case 1:		
			glPortOrtho->setViewBack(onlySet);
			setCaption ("flow::Ortho View Back");
			setIconText("flow::Ortho View Back");
			break;
		case 2:		
			glPortOrtho->setViewLeft(onlySet);
			setCaption ("flow::Ortho View Left");
			setIconText("flow::Ortho View Left");
			break;
		case 3:		
			glPortOrtho->setViewRight(onlySet);
			setCaption ("flow::Ortho View Right");
			setIconText("flow::Ortho View Right");
			break; 
		case 4:		
			glPortOrtho->setViewTop(onlySet);
			setCaption ("flow::Ortho View Top");
			setIconText("flow::Ortho View Top");
			break;
		case 5:		
			glPortOrtho->setViewBot(onlySet);
			setCaption ("flow::Ortho View Bottom");
			setIconText("flow::Ortho View Bottom");
			break;
	}
	if(!onlySet)
		glPortOrtho->updateGL();
		
GL_ERROR_MSG("GLWidgetOrtho::setProjection -- leave");
}

//----------------------------------------------------------
void GLWidgetOrtho::setCurrentName ( const char* name )
{
	selectionModeLabel->setText(name);
}

//----------------------------------------------------------
void GLWidgetOrtho::selectionModeChanged ( int sel )
{
	switch(sel)
	{
		case 0:	//lights
		selectionModeLabel->setPalette(QColor(224, 224, 127));
		break;
		case 1:	//actions
		selectionModeLabel->setPalette(QColor(224, 127, 127));
		break;
		case 2:	//polygons
		selectionModeLabel->setPalette(QColor(224, 127, 224));
		break;
		case 3:	//geom
		selectionModeLabel->setPalette(QColor(127, 127, 224));
		break;
	}
	setCurrentName(" ");		
}

//----------------------------------------------------------
void	GLWidgetOrtho::setGridSnapMult(int in)
{
	switch(in)
	{
		case 0:	glPortOrtho->setGridSnapMult(1.0     );	break;
		case 1:	glPortOrtho->setGridSnapMult(1.0/2.0 );	break;
		case 2:	glPortOrtho->setGridSnapMult(1.0/4.0 );	break;
		case 3:	glPortOrtho->setGridSnapMult(1.0/5.0 );	break;
		case 4:	glPortOrtho->setGridSnapMult(1.0/10.0);	break;
	}
}

//----------------------------------------------------------
void	GLWidgetOrtho::setHitZoneSize(int in)
{	
	glPortOrtho->getHitList()->setHitBox((float)in*0.001);
	glPortOrtho->updateGL();
GL_ERROR_MSG("GLWidgetOrtho::setHitZoneSize");
//	warning("%f", (float)in*0.0001);
}

//----------------------------------------------------------
void GLWidgetOrtho::refreshInputWidgets(CameraView * /*currentCam*/)
{
//	camXPosInput->SetValueSolid(currentCam->posX());
//	camYPosInput->SetValueSolid(currentCam->posY());
//	camZPosInput->SetValueSolid(currentCam->posZ());
//	camXRotInput->SetValueSolid(currentCam->rotX());
//	camYRotInput->SetValueSolid(currentCam->rotY());
//	camZRotInput->SetValueSolid(currentCam->rotZ());
//	camZoomInput->SetValueSolid(currentCam->fov());
}
	
