#ifndef hitList_h
#define hitList_h

#include <qobject.h>
#include <qlist.h>

class MBALight;
class FlowParticleAction;
class CameraView;

/**
 * 
 */
//==============================================================================
class HitPrimitive
{
public:
	enum	PrimType { Undefined = 0, Light, Action, Camera };
	
	HitPrimitive(PrimType pType, void* pTarget, const float *p, const float *h1 = 0, const float *h2 = 0);
//	HitPrimitive(float, float, float);

	
	PrimType	primType;
	float		pos[3];
	int			wasHit;
	bool		hasHit1;
	bool		hasHit2;
	float		hit1[3];
	float		hit2[3];
	void		*primTarget;
};

/**
 *
 */
//==============================================================================
class HitList : public QObject
{
	Q_OBJECT
public:	
	HitList();
	~HitList();
 
	void	reset();
	void	checkForOldActiveTarget();
	int		checkForHit(float x, float y, float z);
	bool	checkForHitPoint(float *p, float x, float y, float z);
	void	moveActive(float x, float y, float z, bool gridSnap, float gridSnapMult);
	
	void	moveLightFrom (float *moved, bool gridSnap, float gridSnapMult);
	void	moveLightTo   (float *moved, bool gridSnap, float gridSnapMult);
	void	moveActionDomainPos (float *moved, bool gridSnap, float gridSnapMult);
	void	moveActionDomainHit1(float *moved, bool gridSnap, float gridSnapMult);
	void	moveActionDomainHit2(float *moved, bool gridSnap, float gridSnapMult);

	void	addCameraPrimitive(CameraView *cv);	
	void	addLightPrimitive(MBALight *l);	
	void	addActionPrimitive(FlowParticleAction *pa);	
	void	setActionDomainHits(FlowParticleAction *pa, HitPrimitive *hp);
	
	inline bool	hasActive()			{	return (bool)activePrimitive; }
	inline void setFov(float in)	{	fov = in;	}
	inline void setHitBox(float in)	{	hitBox = in;	}
	inline void setHilightHitZones(bool in)	{	hilightHitZones = in;	}
	
	void	drawGL();
	void	drawGLBounds(float *primPos);

	void	makeSnapCoords(float *moved, float gridSnapMult);
	
signals:
	void activePrimName(const char*);
	
protected:
	float	hitBox;
	float	fov;
	QList<HitPrimitive>	primitiveList;
	HitPrimitive		*activePrimitive;
	void				*oldActivePrimitiveTarget;
	bool	hilightHitZones;	/**< highlight GL drawing of hit zones */
	bool	autoUpdate;
	float	snap[3];
};


#endif //hitList_h
