#include "hitList.h"

#include <math.h>

#include "pAction.h"
#include "extern.h"

//--------------------------------------------------------------------
void	HitList::setActionDomainHits(FlowParticleAction *pa, HitPrimitive	*hp)
{
	if(	pa->NumDomains() > 0 )
	{
		switch(pa->Domain(0)->Type())
		{
			case PDPoint:
				break;
			case PDLine:
				hp->hasHit1 = true;
				hp->hit1[0] = pa->Domain(0)->A3();
				hp->hit1[1] = pa->Domain(0)->A4();
				hp->hit1[2] = pa->Domain(0)->A5();
				hp->hasHit2 = false;
				break;
			case PDTriangle:
				hp->hasHit1 = true;
				hp->hit1[0] = pa->Domain(0)->A3();
				hp->hit1[1] = pa->Domain(0)->A4();
				hp->hit1[2] = pa->Domain(0)->A5();
				hp->hasHit2 = true;
				hp->hit2[0] = pa->Domain(0)->A6();
				hp->hit2[1] = pa->Domain(0)->A7();
				hp->hit2[2] = pa->Domain(0)->A8();
				break;
			case PDPlane:
				hp->hasHit1 = true;
				hp->hit1[0] = pa->Domain(0)->A3()+pa->Domain(0)->A0();
				hp->hit1[1] = pa->Domain(0)->A4()+pa->Domain(0)->A1();
				hp->hit1[2] = pa->Domain(0)->A5()+pa->Domain(0)->A2();
				hp->hasHit2 = true;
				hp->hit2[0] = pa->Domain(0)->A6()+pa->Domain(0)->A0();
				hp->hit2[1] = pa->Domain(0)->A7()+pa->Domain(0)->A1();
				hp->hit2[2] = pa->Domain(0)->A8()+pa->Domain(0)->A2();
				break;
			case PDBox:
				hp->hasHit1 = true;
				hp->hit1[0] = pa->Domain(0)->A3();
				hp->hit1[1] = pa->Domain(0)->A4();
				hp->hit1[2] = pa->Domain(0)->A5();
				hp->hasHit2 = false;
				break;
			case PDSphere:
				hp->hasHit1 = true;
				hp->hit1[0] =  pa->Domain(0)->A0() + pa->Domain(0)->A3();
				hp->hit1[1] =  pa->Domain(0)->A1();
				hp->hit1[2] =  pa->Domain(0)->A2();
				hp->hasHit2 = true;
				hp->hit2[0] = pa->Domain(0)->A0() - pa->Domain(0)->A4();
				hp->hit2[1] = pa->Domain(0)->A1();
				hp->hit2[2] = pa->Domain(0)->A2();
				break;
			case PDCylinder:
				hp->hasHit1 = true;
				hp->hit1[0] =  pa->Domain(0)->A3();
				hp->hit1[1] =  pa->Domain(0)->A4();
				hp->hit1[2] =  pa->Domain(0)->A5();
				hp->hasHit2 = true;
				hp->hit2[0] = -pa->Domain(0)->A6()+pa->Domain(0)->A0();
				hp->hit2[1] = -pa->Domain(0)->A6()+pa->Domain(0)->A1();
				hp->hit2[2] = -pa->Domain(0)->A6()+pa->Domain(0)->A2();
				break;
			case PDCone:
				hp->hasHit1 = true;
				hp->hit1[0] =  pa->Domain(0)->A3();
				hp->hit1[1] =  pa->Domain(0)->A4();
				hp->hit1[2] =  pa->Domain(0)->A5();
				hp->hasHit2 = true;
				hp->hit2[0] = -pa->Domain(0)->A6()+pa->Domain(0)->A0();
				hp->hit2[1] = -pa->Domain(0)->A6()+pa->Domain(0)->A1();
				hp->hit2[2] = -pa->Domain(0)->A6()+pa->Domain(0)->A2();
				break;
			case PDBlob:
				hp->hasHit1 = true;
				hp->hit1[0] =  pa->Domain(0)->A3()+pa->Domain(0)->A0();
				hp->hit1[1] =  pa->Domain(0)->A3()+pa->Domain(0)->A1();
				hp->hit1[2] =  pa->Domain(0)->A3()+pa->Domain(0)->A2();
				hp->hasHit2 = true;
				hp->hit2[0] = -pa->Domain(0)->A4()+pa->Domain(0)->A0();
				hp->hit2[1] = -pa->Domain(0)->A4()+pa->Domain(0)->A1();
				hp->hit2[2] = -pa->Domain(0)->A4()+pa->Domain(0)->A2();
				break;
			case PDDisc:
				hp->hasHit1 = true;
				hp->hit1[0] =  pa->Domain(0)->A0() + pa->Domain(0)->A3();
				hp->hit1[1] =  pa->Domain(0)->A1() + pa->Domain(0)->A4();
				hp->hit1[2] =  pa->Domain(0)->A2() + pa->Domain(0)->A5();
				hp->hasHit2 = false;
				break;
		}
	}
}

//--------------------------------------------------------------------
void	HitList::moveActionDomainPos(float *moved, bool gridSnap, float gridSnapMult)
{
	float	*tmp = moved;
	if(gridSnap)
	{
		makeSnapCoords(moved, gridSnapMult);
		tmp = snap;
	}
	
	FlowParticleAction	*pa;
	pa = (FlowParticleAction*)(activePrimitive->primTarget);

	pa->setDomainA0(0, tmp[0]);
	pa->setDomainA1(0, tmp[1]);
	pa->setDomainA2(0, tmp[2]);
	if(pa->Domain(0)->Type() == PDPlane)
	{
		activePrimitive->hit1[0] = pa->Domain(0)->A3() + pa->Domain(0)->A0();
		activePrimitive->hit1[1] = pa->Domain(0)->A4() + pa->Domain(0)->A1();
		activePrimitive->hit1[2] = pa->Domain(0)->A5() + pa->Domain(0)->A2();
		activePrimitive->hit2[0] = pa->Domain(0)->A6() + pa->Domain(0)->A0();
		activePrimitive->hit2[1] = pa->Domain(0)->A7() + pa->Domain(0)->A1();
		activePrimitive->hit2[2] = pa->Domain(0)->A8() + pa->Domain(0)->A2();
	}
	else if(pa->Domain(0)->Type() == PDSphere)
	{
		activePrimitive->hit1[0] =  pa->Domain(0)->A0()+pa->Domain(0)->A3();
		activePrimitive->hit1[1] =  pa->Domain(0)->A1();
		activePrimitive->hit1[2] =  pa->Domain(0)->A2();
		activePrimitive->hit2[0] =  pa->Domain(0)->A0()-pa->Domain(0)->A4();
		activePrimitive->hit2[1] =  pa->Domain(0)->A1();
		activePrimitive->hit2[2] =  pa->Domain(0)->A2();
	}
	else if(pa->Domain(0)->Type() == PDDisc)
	{
		activePrimitive->hit1[0] =  pa->Domain(0)->A0() + pa->Domain(0)->A3();
		activePrimitive->hit1[1] =  pa->Domain(0)->A1() + pa->Domain(0)->A4();
		activePrimitive->hit1[2] =  pa->Domain(0)->A2() + pa->Domain(0)->A5();
	}
}

//--------------------------------------------------------------------
void	HitList::moveActionDomainHit1(float *moved, bool gridSnap, float gridSnapMult)
{
	float	*tmp = moved;
	if(gridSnap)
	{
		makeSnapCoords(moved, gridSnapMult);
		tmp = snap;
	}
	
	FlowParticleAction	*pa;
	pa = (FlowParticleAction*)(activePrimitive->primTarget);
	switch(pa->Domain(0)->Type())
	{
		case PDPoint:
			break;
		case PDLine:
			pa->setDomainA3(0, tmp[0]);
			pa->setDomainA4(0, tmp[1]);
			pa->setDomainA5(0, tmp[2]);
			break;
		case PDTriangle:
			pa->setDomainA3(0, tmp[0]);
			pa->setDomainA4(0, tmp[1]);
			pa->setDomainA5(0, tmp[2]);
			break;
		case PDPlane:
			pa->setDomainA3(0, tmp[0] - pa->Domain(0)->A0());
			pa->setDomainA4(0, tmp[1] - pa->Domain(0)->A1());
			pa->setDomainA5(0, tmp[2] - pa->Domain(0)->A2());
			break;
		case PDBox:
			pa->setDomainA3(0, tmp[0]);
			pa->setDomainA4(0, tmp[1]);
			pa->setDomainA5(0, tmp[2]);
			break;
		case PDSphere:
			activePrimitive->hit1[1] = pa->Domain(0)->A1();
			activePrimitive->hit1[2] = pa->Domain(0)->A2();
			pa->setDomainA3(0, tmp[0] - pa->Domain(0)->A0());
			break;
		case PDCylinder:
			pa->setDomainA3(0, tmp[0]);
			pa->setDomainA4(0, tmp[1]);
			pa->setDomainA5(0, tmp[2]);
			break;
		case PDCone:
			pa->setDomainA3(0, tmp[0]);
			pa->setDomainA4(0, tmp[1]);
			pa->setDomainA5(0, tmp[2]);
			break;
		case PDBlob:
			activePrimitive->hit1[1] = 0.0;
			activePrimitive->hit1[2] = 0.0;
			pa->setDomainA3(0, tmp[0] - pa->Domain(0)->A0());
			break;
		case PDDisc:
			pa->setDomainA3(0, tmp[0] - pa->Domain(0)->A0());
			pa->setDomainA4(0, tmp[1] - pa->Domain(0)->A1());
			pa->setDomainA5(0, tmp[2] - pa->Domain(0)->A2());
			break;
	}
}

//--------------------------------------------------------------------
void	HitList::moveActionDomainHit2(float *moved, bool gridSnap, float gridSnapMult)
{
	float	*tmp = moved;
	if(gridSnap)
	{
		makeSnapCoords(moved, gridSnapMult);
		tmp = snap;
	}
	
	FlowParticleAction	*pa;
	pa = (FlowParticleAction*)(activePrimitive->primTarget);
	switch(pa->Domain(0)->Type())
	{
		case PDPoint:
			break;
		case PDLine:
			break;
		case PDTriangle:
			pa->setDomainA6(0, tmp[0]);
			pa->setDomainA7(0, tmp[1]);
			pa->setDomainA8(0, tmp[2]);
			break;
		case PDPlane:
			pa->setDomainA6(0, tmp[0] - pa->Domain(0)->A0());
			pa->setDomainA7(0, tmp[1] - pa->Domain(0)->A1());
			pa->setDomainA8(0, tmp[2] - pa->Domain(0)->A2());
			break;
		case PDBox:
			break;
		case PDSphere:
			pa->setDomainA4(0, -tmp[0] - pa->Domain(0)->A0());
			activePrimitive->hit2[1] = pa->Domain(0)->A1();
			activePrimitive->hit2[2] = pa->Domain(0)->A2();
			break;
		case PDCylinder:
			break;
		case PDCone:
			break;
		case PDBlob:
			break;
		case PDDisc:
			break;
	}
}

















