#include "imageFileDialog.h"

#include <qimage.h>
#include <qwmatrix.h>

#ifndef WIN32
	#include <tiffio.h>
#endif

#include "extern.h"

#define PREVIEW_HEIGHT	256

//==============================================================================
ImageFileDialog::ImageFileDialog(QWidget *parent, const char *name, bool modal)
	: QFileDialog(parent, name, modal)
{
	setFont(*g_font_M);

	picSizeLabel = new QLabel(this);
	picSizeLabel->setFixedWidth(55);
	picSizeLabel->setAlignment( AlignCenter );
	picSizeLabel->setFont(*g_font_MB);
	picSizeLabel->setAutoResize(false);
	
	picLabel = new QLabel(this);
	picLabel->setFixedHeight(PREVIEW_HEIGHT);
	picLabel->setFrameStyle( QFrame::Sunken | QFrame::Box );
	picLabel->setAlignment( AlignCenter );

	QFileDialog::addWidgets(picSizeLabel, picLabel, 0);
	
	QObject::connect(this, SIGNAL(fileHighlighted(const QString&)), SLOT(showImageFile(const QString&)) );
}

//--------------------------------------------------------------------
 void ImageFileDialog::showImageFile(const QString &filename)
{
#ifndef WIN32
	QPixmap	qPixmap;
	int		imageWidth	= 0;
	int		imageHeight	= 0;
	
	if(QFile::exists(filename))
	{
		QCString		sizeString;
		QWMatrix	mat;
		QImage		image;
		uint32		w, h;
		size_t		npixels;
		uint32*		raster;
		float		scaleSize;
//		uint32		r;
//		uint32		g;
//		uint32		b;

		TIFF* tif = TIFFOpen((const char*)filename, "r");
		if (tif) 
		{
			TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &w);
			TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &h);
			npixels = w * h;
			raster = (uint32*) _TIFFmalloc(npixels * sizeof (uint32));
			if (raster != NULL) 
			{
				if (TIFFReadRGBAImage(tif, w, h, raster, 0)) 
				{
					imageWidth	= w;
					imageHeight	= h;
					if (image.create(imageWidth, imageHeight, 32) )
					{
						for (uint32 y = 0; y < h; y++)
							for (uint32 x = 0; x < w; x++)
								image.setPixel(x, (h-1)-y, raster[(y*w)+x]);
						image = image.swapRGB();
					}
					else
						return;
				}
				_TIFFfree(raster);
			}
			TIFFClose(tif);
			qPixmap.convertFromImage(image);
			scaleSize = (float)PREVIEW_HEIGHT/(float)h;
			if( scaleSize < 1.0 )
			{
				mat.scale(scaleSize, scaleSize);
				qPixmap = qPixmap.xForm(mat);
			}
			picLabel->setPixmap(qPixmap);
			sizeString.sprintf("size:\n\n%u\nx\n%u", w, h);
			picSizeLabel->setText(sizeString);
		}
	}
	else
	{
		picLabel->setText("");
	}
#endif
}
