/**
 * lightGroup.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * Class that groups lights
 */
#include "compatibility.h"
#include <GL/gl.h>
#include <ri.h>

#include "lightGroup.h"
#include "mbaLight.h"
#include "hitList.h"
#include "extern.h"

//==============================================================================
LightGroup::LightGroup()
{
//	warning("LightGroup::LightGroup()");
	name = "default light group";
	lightList.setAutoDelete(true);
	particleLighting = false;
	
	MBALight	tmpLight1;
	tmpLight1.SetName("r light");
	tmpLight1.setLightType(MBALight::Local);
	tmpLight1.setFrom(-5.0, 5.5, 5.0);
	tmpLight1.setColor(1.00, 0.90, 0.90);
	tmpLight1.setIntensity(3.5);
	tmpLight1.setEnabled(true, true);
	
	MBALight	tmpLight2;
	tmpLight2.SetName("g light");
	tmpLight2.setLightType(MBALight::Local);
	tmpLight2.setFrom(6.0, 8.0, 6.0);
	tmpLight2.setColor(0.90, 1.0, 0.90);
	tmpLight2.setIntensity(2.5);
	tmpLight2.setEnabled(true, true);
	
	MBALight	tmpLight3;
	tmpLight3.SetName("b light");
	tmpLight3.setLightType(MBALight::Infinite);
	tmpLight3.setFrom( 0.75, 7.00, -5.00 );
	tmpLight3.setTo  ( 0.00, 5.25, -3.75 );
	tmpLight3.setColor(0.90, 0.90, 1.00);
	tmpLight3.setIntensity(0.9);
	tmpLight3.setEnabled(true, true);
	
	AddLight(tmpLight1);
	AddLight(tmpLight2);
	AddLight(tmpLight3);
	dirty = true;
}

LightGroup::~LightGroup()
{
}

//--------------------------------------------------------------------
int LightGroup::findLight(MBALight *cur)
{
	return lightList.find(cur);
}

//--------------------------------------------------------------------
void LightGroup::setEnabled(bool status)
{
	warning("---LightGroup::setEnabled");
	for ( int i = 0; i < NumLights(); i++)
	{
		if (status)
		{
			MakeClean();
		}
		else
		{
			glDisable(GL_LIGHT0);
			glDisable(GL_LIGHT1);
			glDisable(GL_LIGHT2);
			glDisable(GL_LIGHT3);
			glDisable(GL_LIGHT4);
			glDisable(GL_LIGHT5);
			glDisable(GL_LIGHT6);
			glDisable(GL_LIGHT7);
		}
	}
}

//--------------------------------------------------------------------
void LightGroup::drawGL(int frame)
{
	for ( int i = 0; i < (int)lightList.count(); i++)
		lightList.at(i)->drawGL(frame);
}

//--------------------------------------------------------------------
void LightGroup::illuminateGL(int frame)
{
	if(dirty)
		MakeClean();
	for ( int i = 0; i < (int)lightList.count(); i++)
	{
		if(lightList.at(i)->updated())
			dirty = true;
		lightList.at(i)->illuminateGL(frame);
	}
}

//--------------------------------------------------------------------
void LightGroup::drawRIB(int frame)
{
	frame = frame;
}

//--------------------------------------------------------------------
void LightGroup::illuminateRIB(int frame)
{
	char	idString[128];
	sprintf(idString, "====================LIGHT=GROUP==========> \"%s\"", (const char*)name );
	RiArchiveRecord ("comment", "", RI_NULL );		
	RiArchiveRecord ("comment", idString, RI_NULL );		
	for ( int i = 0; i < (int)lightList.count(); i++)
		lightList.at(i)->illuminateRIB(frame);
}

//--------------------------------------------------------------------
const char *LightGroup::LightName (int index)
{
	if (index >= 0 && index < (int)lightList.count() )
		return (const char*)(lightList.at(index))->Name();
	else
		return 0;
}

//--------------------------------------------------------------------
void LightGroup::AddLight (MBALight &light)
{
	lightList.append(new MBALight(light));
	dirty = true;
}

//--------------------------------------------------------------------
void LightGroup::DelLight (int index)
{
	if (index >= 0 && index < (int)lightList.count())
	{
		lightList.remove(index);
	}
	dirty = true;
//	int i;
//	warning("%d lights in group", NumLights());
//	for (i = 0; i < NumLights(); i++)
//	{
//		warning("\t%s", LightName(i));
//	}
}

//--------------------------------------------------------------------
MBALight *LightGroup::LightAt (int index)
{	
	if (index < (int)lightList.count())
		return lightList.at(index);	
	else 
		return 0;
}

////--------------------------------------------------------------------
//void LightGroup::MakeClean()
//{
//	glDisable(GL_LIGHT0);
//	glDisable(GL_LIGHT1);
//	glDisable(GL_LIGHT2);
//	glDisable(GL_LIGHT3);
//	glDisable(GL_LIGHT4);
//	glDisable(GL_LIGHT5);
//	glDisable(GL_LIGHT6);
//	glDisable(GL_LIGHT7);
//	int i;
//	for(i = 0; (i < (int)lightList.count() && i < 8); i++)
//	{
//		lightList.at(i)->setGlLightNum(i);
//		lightList.at(i)->UpdateVariables();
//		if(lightList.at(i)->isEnabled())
//			glEnable(lightList.at(i)->getGlLightNum());
//	}
//}

//--------------------------------------------------------------------
void LightGroup::MakeClean()
{
//	warning("LightGroup::MakeClean()");
	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
	glDisable(GL_LIGHT2);
	glDisable(GL_LIGHT3);
	glDisable(GL_LIGHT4);
	glDisable(GL_LIGHT5);
	glDisable(GL_LIGHT6);
	glDisable(GL_LIGHT7);
	//warning("%d %d %d %d", GL_LIGHT0, GL_LIGHT1, GL_LIGHT2, GL_LIGHT3);
	int i = 0;
	int j = 0;
	for(i = 0; (i < (int)lightList.count() && j < 8); i++)
	{
		if(lightList.at(i)->isEnabled())
		{
			lightList.at(i)->setGlLightNum(j);
			lightList.at(i)->UpdateVariables();
			glEnable(lightList.at(i)->getGlLightNum());
			//warning("j = %d, %d", j, lightList.at(i)->getGlLightNum());
			j++;
		}
		else
		{
			lightList.at(i)->setGlLightNum(-1);
		}
	}
	dirty = false;
}

//--------------------------------------------------------------------
void LightGroup::generateHitList(HitList &hL)
{
	int i;
	hL.reset();
	hL.addCameraPrimitive(g_globalCam);
	for(i = 0; i < NumLights(); i++)
	{
		if(lightList.at(i)->isEnabled())
			hL.addLightPrimitive(lightList.at(i));
	}
	hL.checkForOldActiveTarget();
}

