/**
 * lightGroup.h
 * Copyright 1999 by Mark B. Allan
 * 
 * Class that groups lights
 */
#ifndef lightGroup_h
#define lightGroup_h

#include <stdio.h>
#include <qobject.h>
#include <qlist.h>

#include "mbaLight.h"
class HitList;
//class MBALight;

/**
 * Light collection
 */
//==============================================================================
class	LightGroup	:	public QObject
{
	Q_OBJECT
public:
	LightGroup();
	~LightGroup();
	
	void		setEnabled(bool);
	inline bool	isEnabled()	{	return enabled;	}
	
	bool	SaveLightGroup(FILE *infile, int tabs = 0);
	bool	LoadLightGroup(FILE *infile, const char* nm);
	
	inline 	const char*	Name()	{	return (const char*)name;	}
	
	void	drawGL(int frame);
	void	illuminateGL(int frame);
	void	drawRIB(int frame);
	void	illuminateRIB(int frame);
	
	const char*	LightName(int index);
	
	inline int	NumLights()	{	return (int)lightList.count();	}
	
	inline bool	ParticleLighting()					{	return particleLighting;	}
	inline void	SetParticleLighting(bool status)	{	particleLighting = status;	}

	inline void	SetDirty() { dirty = true; }
	void		MakeClean();
	
	void	generateHitList(HitList &hL);
	
	MBALight *LightAt(int index);
	int		findLight(MBALight *);
	
public slots:
	void	AddLight(MBALight &light);
	void	DelLight(int index);

signals:
	void	Updated();
	
protected:	
	bool		dirty;
	bool		enabled;
	bool		particleLighting;
	QCString	name;
	
	int			maxGLNum;
	QList<MBALight>		lightList;

};

#endif
