/**
 * lightGroupWidget.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * light group widget
 */
#include "lightGroupWidget.h"

#include <qmessagebox.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qtooltip.h>

#include "compatibility.h"
#include <GL/gl.h>
#include "mbaLight.h"
#include "glPortCam.h"
#include "extern.h"
#include "define.h"
#include "colorButton.h"

//==============================================================================
LightGroupWidget::LightGroupWidget(LightGroupWidgetFrame *p, QToolBar *colorToolBar, QToolBar *typeToolBar, const char*name, LightGroup *lg)
	:	QWidget(p, name)
{
	currentLight = 0;
	parent = p;	// THIS IS DANGEROUS! I just spent 45 minutes tracking down a seg fault because of this...
//	setFont(*g_font_MB);	
	
	if (lg)
		lightGroup = lg;
	else
		lightGroup = 0;
		
	currentLight = 0;

	//-- 
	lightListBox = new QListBox(this);
	lightListBox->setMinimumSize (g_numInputWidth-20, 20);
	lightListBox->setFont(*g_font_M);
	int i;
	if(lightGroup)
		for (i = 0; i < lightGroup->NumLights(); i++)
			lightListBox->insertItem(lightGroup->LightName(i));
	else
		lightListBox->setEnabled(false);
	
	lightNameEdit = new QLineEdit(this);
	lightNameEdit->setFont( *g_font_LB );

	QWidget		*tmpWidget;
	QVBoxLayout	*tmpVLayout;
	//-- colorToolBar
	tmpWidget = new QWidget(colorToolBar);
	colorButton = new ColorButton (tmpWidget);
	colorButton->setFixedWidth(g_numInputWidth);
	colorButton->setEnabled(true);
	colorButton->setColor(1.0, 1.0, 1.0);
	
	intensityInput = new NumInput( NumInput::Double, tmpWidget );
	intensityInput->SetLabel("intensity");
	intensityInput->SetLabelWidth(g_numLabelWidth);
	intensityInput->SetTotalWidth(g_numInputWidth);
	intensityInput->SetValueSolid(1.0);
	
	drawLightToggle = new QCheckBox(tmpWidget);
	drawLightToggle->setText("draw light");
	
	castsShadowsToggle = new QCheckBox(tmpWidget);
	castsShadowsToggle->setText("casts shadows");
	
//	glLightToggle = new QCheckBox(tmpWidget);
//	glLightToggle->setText("casts shadows");
	
	tmpVLayout = new QVBoxLayout(tmpWidget, 2, 0);
	tmpVLayout->addWidget(colorButton);
	tmpVLayout->addWidget(intensityInput);
	tmpVLayout->addSpacing(5);
	tmpVLayout->addWidget(drawLightToggle);
	tmpVLayout->addWidget(castsShadowsToggle);
	
	//-- typeToolBar
	tmpWidget = new QWidget(typeToolBar);
	lightTypeCombo = new QComboBox(tmpWidget);
	lightTypeCombo->setFixedWidth(g_numInputWidth);
	lightTypeCombo->setFixedHeight(g_comboHeight);
	lightTypeCombo->insertItem("Ambient          ");
	lightTypeCombo->insertItem("Infinite         ");
	lightTypeCombo->insertItem("Local            ");
	lightTypeCombo->insertItem("Spot             ");
	lightTypeCombo->setFont( *g_font_MB );

	lightParam = 0;
	lightParamCombo = new QComboBox(tmpWidget);
	lightParamCombo->setFixedWidth(g_numInputWidth);
	lightParamCombo->setFixedHeight(g_comboHeight);
	lightParamCombo->insertItem("  From  ");
	lightParamCombo->insertItem("  To  ");
	lightParamCombo->insertItem("  Spot  ");
	
	p0Input = new NumInput (NumInput::Double, tmpWidget);
	p0Input->SetLabelWidth(g_numLabelWidth);
	p0Input->SetTotalWidth(g_numInputWidth);
	p1Input = new NumInput (NumInput::Double, tmpWidget);
	p1Input->SetLabelWidth(g_numLabelWidth);
	p1Input->SetTotalWidth(g_numInputWidth);
	p2Input = new NumInput (NumInput::Double, tmpWidget);
	p2Input->SetLabelWidth(g_numLabelWidth);
	p2Input->SetTotalWidth(g_numInputWidth);
	
	tmpVLayout = new QVBoxLayout(tmpWidget, 2, 0);
	tmpVLayout->addWidget(lightTypeCombo);
	tmpVLayout->addSpacing(5);
	tmpVLayout->addWidget(lightParamCombo);
	tmpVLayout->addWidget(p0Input);
	tmpVLayout->addWidget(p1Input);
	tmpVLayout->addWidget(p2Input);
	tmpVLayout->activate();
		
	//----------------------------------
	if (lightGroup)
		QObject::connect(lightGroup, SIGNAL(Updated()), SLOT(Update()) );
	
	QObject::connect(colorButton, SIGNAL(RGBChanged(float, float, float)), SLOT(ChangeLightColor(float, float, float)) );
	QObject::connect(p0Input, SIGNAL(ValueChanged(float)), SLOT(SetP0(float)) );
	QObject::connect(p1Input, SIGNAL(ValueChanged(float)), SLOT(SetP1(float)) );
	QObject::connect(p2Input, SIGNAL(ValueChanged(float)), SLOT(SetP2(float)) );
	QObject::connect(lightTypeCombo, SIGNAL(activated(int)), SLOT(SelectLightType(int)) );
	QObject::connect(lightParamCombo, SIGNAL(activated(int)), SLOT(SelectParamType(int)) );

	QObject::connect(castsShadowsToggle, SIGNAL(toggled(bool)), SLOT(setCastsShadows(bool)) );
	QObject::connect(drawLightToggle, SIGNAL(toggled(bool)), SLOT(ToggleDrawLight(bool)) );
	
	QObject::connect(intensityInput, SIGNAL(ValueChanged(float)), SLOT(setIntensity(float)) );
	QObject::connect(lightListBox, SIGNAL(highlighted(int)), SLOT(SwitchLight(int)) );
	QObject::connect(lightNameEdit, SIGNAL(returnPressed()), SLOT(SetLightName()) );
	
	QObject::connect(intensityInput, SIGNAL(Updated()), SIGNAL(Updated()) );
	QObject::connect(p0Input, SIGNAL(Updated()), SIGNAL(Updated()) );
	QObject::connect(p1Input, SIGNAL(Updated()), SIGNAL(Updated()) );
	QObject::connect(p2Input, SIGNAL(Updated()), SIGNAL(Updated()) );
	
	//----------------------------------
	topVLayout		= new QVBoxLayout (this, 5, 5);
	topHLayout		= new QHBoxLayout ( 10 );
	v1Layout		= new QVBoxLayout ( 5 );
	v2Layout		= new QVBoxLayout ( 5 );
	v3Layout		= new QVBoxLayout ( 0 );
	
	topVLayout->addLayout(topHLayout, 1);
		topHLayout->addLayout(v1Layout, 1);
			v1Layout->addWidget(lightNameEdit, 0);
			v1Layout->addWidget(lightListBox, 1);
	
	topVLayout->activate();
	
	SelectLightType(0);
	if (lightGroup)
	{
		if (lightGroup->NumLights() > 0)
		{
			lightListBox->setCurrentItem(0);
			currentLight = lightGroup->LightAt(0);
		}
		widgetsEnabled = true;
		if (!currentLight)
			SetWidgetsEnabled(false);
		else
			SwitchLight(0);
	}
	else 
		SetWidgetsEnabled(false);
}

LightGroupWidget::~LightGroupWidget()
{
}

//--------------------------------------------------------------------
void	LightGroupWidget::setActiveLight(MBALight* cur)
{
	int tmp = lightGroup->findLight(cur);
	if(tmp >= 0)
	{
//QT1.44	lightListBox->setSelected(tmp, true);
		lightListBox->setCurrentItem(tmp);
	}
}

//--------------------------------------------------------------------
void	LightGroupWidget::SetLightGroupPointer(LightGroup *newLightGroup)
{
	int	i;
	
	// Do any disconnects necessary...
	if(lightGroup)
		QObject::disconnect(lightGroup, SIGNAL(Updated()), this, SLOT(Update()) );
	currentLight = 0;
	lightListBox->clear();
	
	lightGroup = newLightGroup;
	if (lightGroup)
	{
		QObject::connect(lightGroup, SIGNAL(Updated()), this, SLOT(Update()) );
	
		lightListBox->blockSignals(true);
		for (i = 0; i < lightGroup->NumLights(); i++)
			lightListBox->insertItem(lightGroup->LightName(i));
		if (lightGroup->NumLights() > 0)
		{
			lightListBox->setCurrentItem(0);
			currentLight = lightGroup->LightAt(0);
			lightNameEdit->setText(lightGroup->LightName(0));
		}
		if (!currentLight)
			SetWidgetsEnabled(false);
		else
			SetWidgetsEnabled(true);
		lightListBox->blockSignals(false);
	}
	else
		SetWidgetsEnabled(false);
		
	emit Updated();
}
	
//----------------------------------------------------------
void LightGroupWidget::Update()
{
	int i;
	
	currentLight = 0;
	lightListBox->clear();
	for (i = 0; i < lightGroup->NumLights(); i++)
		lightListBox->insertItem(lightGroup->LightName(i));
	if (lightGroup->NumLights() > 0)
	{
		lightListBox->setCurrentItem(0);
		currentLight = lightGroup->LightAt(0);
		lightNameEdit->setText(lightGroup->LightName(0));
	}
	if (!currentLight)
		SetWidgetsEnabled(false);
	else
		SetWidgetsEnabled(true);
		
	
//	warning ("LightGroupWidget::Update()");
}

//----------------------------------------------------------
void LightGroupWidget::updateCurrentLight()
{
	if(currentLight && currentLight->updated())
	{
		currentLight->setUpdated(false);
		lightNameEdit->setText((const char*)currentLight->Name());
		SelectLightType(currentLight->LightType());
		p0Input->SetValueSolid(currentLight->getFromX());
		p1Input->SetValueSolid(currentLight->getFromY());
		p2Input->SetValueSolid(currentLight->getFromZ());
		parent->SetDisable(!currentLight->isEnabled());
		colorButton->setColor(currentLight->getColorR(),
							currentLight->getColorG(),
							currentLight->getColorB() );
		intensityInput->SetValueSolid(currentLight->getIntensity());
		castsShadowsToggle->setChecked(currentLight->getCastsShadows());
		drawLightToggle->setChecked(currentLight->DrawLight());
	}
}

//--------------------------------------------------------------------
void	LightGroupWidget::DisableCurrentLight(bool status)
{
	if(lightGroup && currentLight)
	{
		currentLight->setEnabled(!status);
		lightGroup->SetDirty();
	}
	emit Updated();
}
//--------------------------------------------------------------------
void	LightGroupWidget::CopyCurrentLight()
{
	if(lightGroup)
	{
		int curLight = lightListBox->currentItem();
		if ( curLight >= 0 && curLight < lightGroup->NumLights() )
		{
			MBALight tmpLight(*(lightGroup->LightAt(curLight)));
			lightGroup->AddLight(tmpLight);

			lightListBox->insertItem( lightGroup->LightAt(lightGroup->NumLights()-1)->Name() );
			lightListBox->setCurrentItem( lightGroup->NumLights()-1 );
		}
	}
}
	
//--------------------------------------------------------------------
void	LightGroupWidget::AddNewLight()
{
	if(lightGroup)
	{
		MBALight	tmpLight;

		if (!widgetsEnabled)
			SetWidgetsEnabled(true);

		lightGroup->AddLight(tmpLight);

		lightListBox->insertItem( lightGroup->LightAt(lightGroup->NumLights()-1)->Name() );
		lightListBox->setCurrentItem( lightGroup->NumLights()-1 );
	}
}
//--------------------------------------------------------------------
void	LightGroupWidget::DeleteCurrentLight()
{
	if(lightGroup)
	{
		int curLight = lightListBox->currentItem();
		if ( curLight >= 0 && curLight < lightGroup->NumLights() )
		{
			lightGroup->DelLight(curLight);
			lightListBox->removeItem(curLight);
		}
		if (lightGroup->NumLights() < 1)
		{
			SetWidgetsEnabled(false);
		}
	}
	emit Updated();
}

//--------------------------------------------------------------------
void	LightGroupWidget::SetAttenConstant(float in)
{
	if (currentLight)
		currentLight->setAttenuation( in, -1.0, -1.0);
}
void	LightGroupWidget::SetAttenLinear(float in)
{
	if (currentLight)
		currentLight->setAttenuation( -1.0, in, -1.0);
}
void	LightGroupWidget::SetAttenQuadratic(float in)
{
	if (currentLight && lightGroup)
		currentLight->setAttenuation( -1.0, -1.0, in);
}
	
//--------------------------------------------------------------------
void	LightGroupWidget::setCastsShadows(bool status)
{
	if (currentLight)
		currentLight->setCastsShadows(status);
}

//--------------------------------------------------------------------
void	LightGroupWidget::setIntensity(float in)
{
	if (currentLight)
		currentLight->setIntensity(in);
}
	
//--------------------------------------------------------------------
void	LightGroupWidget::ChangeLightColor(float r, float g, float b)
{
	if (currentLight)
		currentLight->setColor(r, g, b);
	emit Updated();
}

//--------------------------------------------------------------------
void	LightGroupWidget::SetWidgetsEnabled(bool status)
{
	if (status != widgetsEnabled)
	{
		widgetsEnabled = status;
		if (widgetsEnabled)
		{
			lightListBox->setEnabled(true);
			lightNameEdit->setEnabled(true);
			intensityInput->setEnabled(true);
			colorButton->setEnabled(true);
			lightTypeCombo->setEnabled(true);
			lightParamCombo->setEnabled(true);
			drawLightToggle->setEnabled(true);
			castsShadowsToggle->setEnabled(true);
			//Light parameters (from, to, spot) should be enabled by lightTypeCombo
		}
		else
		{
			lightListBox->setEnabled(false);
			lightNameEdit->setText(" ");
			lightNameEdit->setEnabled(false);
			intensityInput->setEnabled(false);
			colorButton->setEnabled(false);
			lightTypeCombo->setEnabled(false);
			lightParamCombo->setEnabled(false);
			drawLightToggle->setEnabled(false);
			p0Input->SetLabel("");
			p0Input->SetLabel("");
			p0Input->SetLabel("");
			p0Input->setEnabled(false);
			p1Input->setEnabled(false);
			p2Input->setEnabled(false);
			castsShadowsToggle->setEnabled(false);
		}
	}
}

//--------------------------------------------------------------------
void	LightGroupWidget::ActivateColorButton(bool status)
{
	colorButton->setEnabled(status);
}

//--------------------------------------------------------------------
void	LightGroupWidget::ToggleDrawLight(bool status)
{
	if(currentLight)
		currentLight->SetDrawLight(status);
	emit Updated();
}

//--------------------------------------------------------------------
void	LightGroupWidget::SwitchLight(int index)
{
	currentLight = lightGroup->LightAt(index);
	if (currentLight)
	{
		lightNameEdit->setText((const char*)currentLight->Name());
		SelectLightType(currentLight->LightType());
		p0Input->SetValueSolid(currentLight->getFromX());
		p1Input->SetValueSolid(currentLight->getFromY());
		p2Input->SetValueSolid(currentLight->getFromZ());
		parent->SetDisable(!currentLight->isEnabled());
		colorButton->setColor(currentLight->getColorR(),
							currentLight->getColorG(),
							currentLight->getColorB() );
		intensityInput->SetValueSolid(currentLight->getIntensity());
		castsShadowsToggle->setChecked(currentLight->getCastsShadows());
		drawLightToggle->setChecked(currentLight->DrawLight());
	}
	else
	{
		//FIXME -  Disable widgets that will try to update currentLight
	}
}


//--------------------------------------------------------------------
void	LightGroupWidget::SetLightName()
{
	if (currentLight)
	{
		currentLight->SetName(lightNameEdit->text());
		lightListBox->changeItem(currentLight->Name(), lightListBox->currentItem());
	}
}

//--------------------------------------------------------------------
void	LightGroupWidget::SelectLightType(int inType)
{
	if (currentLight)
	{
		switch(inType)
		{
			case MBALight::Ambient :
				currentLight->setLightType(MBALight::Ambient);
				lightParamCombo->clear();
				lightParamCombo->setEnabled(false);
				p0Input->setEnabled(false);
				p1Input->setEnabled(false);
				p2Input->setEnabled(false);
				lightTypeCombo->setCurrentItem( (int)MBALight::Ambient );
				break;
			case MBALight::Infinite :
				currentLight->setLightType(MBALight::Infinite);
				lightParamCombo->clear();
				lightParamCombo->setEnabled(true);
				lightParamCombo->insertItem("  From  ");
				lightParamCombo->insertItem("  To  ");
				p0Input->setEnabled(true);
				p1Input->setEnabled(true);
				p2Input->setEnabled(true);
				lightTypeCombo->setCurrentItem( (int)MBALight::Infinite );
				SelectParamType(0);
				break;
			case MBALight::Local :
				currentLight->setLightType(MBALight::Local);
				lightParamCombo->clear();
				lightParamCombo->setEnabled(true);
				lightParamCombo->insertItem("  From  ");
				p0Input->setEnabled(true);
				p1Input->setEnabled(true);
				p2Input->setEnabled(true);
				lightTypeCombo->setCurrentItem( (int)MBALight::Local );
				SelectParamType(0);
				break;
			case MBALight::Spot :
				currentLight->setLightType(MBALight::Spot);
				lightParamCombo->clear();
				lightParamCombo->setEnabled(true);
				lightParamCombo->insertItem("  From  ");
				lightParamCombo->insertItem("  To  ");
				lightParamCombo->insertItem("  Spot  ");
				p0Input->setEnabled(true);
				p1Input->setEnabled(true);
				p2Input->setEnabled(true);
				lightTypeCombo->setCurrentItem( (int)MBALight::Spot );
				SelectParamType(0);
				break;
		}
		p0Input->SetValueSolid(currentLight->getFromX());
		p1Input->SetValueSolid(currentLight->getFromY());
		p2Input->SetValueSolid(currentLight->getFromZ());
	}
	emit Updated();
}


//--------------------------------------------------------------------
void	LightGroupWidget::SelectParamType(int in)
{
	lightParam = in;
	switch(lightParam)
	{
		case 0:
			p0Input->SetLabel("x");
			p1Input->SetLabel("y");
			p2Input->SetLabel("z");
			p0Input->SetValueSolid(currentLight->getFromX());
			p1Input->SetValueSolid(currentLight->getFromY());
			p2Input->SetValueSolid(currentLight->getFromZ());
			break;
		case 1:
			p0Input->SetLabel("to x");
			p1Input->SetLabel("to y");
			p2Input->SetLabel("to z");
			p0Input->SetValueSolid(currentLight->getToX());
			p1Input->SetValueSolid(currentLight->getToY());
			p2Input->SetValueSolid(currentLight->getToZ());
			break;
		case 2:
			p0Input->SetLabel("angle");
			p1Input->SetLabel("delta");
			p2Input->SetLabel("distrib");
			p0Input->SetValueSolid(currentLight->getAngle());
			p1Input->SetValueSolid(currentLight->getDelta());
			p2Input->SetValueSolid(currentLight->getDistrib());
			break;
	}
			
}

//--------------------------------------------------------------------
void	LightGroupWidget::SetP0(float in)
{
	switch(lightParam)
	{
		case 0:
			SetLightFromX(in);
			break;
		case 1:
			SetLightToX(in);
			break;
		case 2:
			SetConeAngle(in);
			break;
	}
}

//--------------------------------------------------------------------
void	LightGroupWidget::SetP1(float in)
{
	switch(lightParam)
	{
		case 0:
			SetLightFromY(in);
			break;
		case 1:
			SetLightToY(in);
			break;
		case 2:
			SetConeDelta(in);
			break;
	}
}

//--------------------------------------------------------------------
void	LightGroupWidget::SetP2(float in)
{
	switch(lightParam)
	{
		case 0:
			SetLightFromZ(in);
			break;
		case 1:
			SetLightToZ(in);
			break;
		case 2:
			SetConeDistrib(in);
			break;
	}
}

//--------------------------------------------------------------------
void	LightGroupWidget::SetLightFromX(float in)
{
	if (currentLight)
		currentLight->setFromX(in);
}
//--------------------------------------------------------------------
void	LightGroupWidget::SetLightFromY(float in)
{
	if (currentLight)
		currentLight->setFromY(in);
}
//--------------------------------------------------------------------
void	LightGroupWidget::SetLightFromZ(float in)
{
	if (currentLight)
		currentLight->setFromZ(in);
}


//--------------------------------------------------------------------
void	LightGroupWidget::SetLightToX(float in)
{
	if (currentLight)
		currentLight->setToX(in);
}
//--------------------------------------------------------------------
void	LightGroupWidget::SetLightToY(float in)
{
	if (currentLight)
		currentLight->setToY(in);
}
//--------------------------------------------------------------------
void	LightGroupWidget::SetLightToZ(float in)
{
	if (currentLight)
		currentLight->setToZ(in);
}

//--------------------------------------------------------------------
void	LightGroupWidget::SetConeAngle(float in)
{
	if (currentLight)
		currentLight->setAngle(in);
}
//--------------------------------------------------------------------
void	LightGroupWidget::SetConeDelta(float in)
{
	if (currentLight)
		currentLight->setDelta(in);
}
//--------------------------------------------------------------------
void	LightGroupWidget::SetConeDistrib(float in)
{
	if (currentLight)
		currentLight->setDistrib(in);
}

//==============================================================================
LightGroupWidgetFrame::LightGroupWidgetFrame(QWidget *nullParent, const char* name, 
							ToolBarDock buttonBarDock,
							ToolBarDock colorBarDock,
							ToolBarDock typeBarDock)
	:	MbaMainWindow (nullParent, name)
{
	setCaption("flow::Light Group");
	setIcon(QPixmap((const char**)flowIcon_xpm));
	setFont(*g_font_MB);
	
	manipToolBar = new QToolBar(0, this, buttonBarDock, true);
	dockList.append(new QMainWindow::ToolBarDock(buttonBarDock));
	
	colorToolBar = new QToolBar(0, this, colorBarDock, true);
	dockList.append(new QMainWindow::ToolBarDock(colorBarDock));
	
	typeToolBar = new QToolBar(0, this, typeBarDock, true);
	dockList.append(new QMainWindow::ToolBarDock(typeBarDock));
	
	if(g_winStyleToolBars)
	{
		manipToolBar->setStyle(winStyle);
		colorToolBar->setStyle(winStyle);
		typeToolBar->setStyle(winStyle);
	}
	
	numToolBars = 3;
	
	MLabel *stretch;
	
	closeBut = new MToolButton(manipToolBar);
	closeBut->setAccel(ALT+'Q');
	closeBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	closeBut->setUsesBigPixmap(true);
	closeBut->setIconSet(*icoSet_close);

	stretch = new MLabel(manipToolBar);
	stretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	
	disableBut = new MToolButton(manipToolBar);
	disableBut->setAccel(ALT+'E');
	disableBut->setToggleButton(true);
	disableBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	disableBut->setUsesBigPixmap(true);
	disableBut->setIconSet(*icoSet_dis);
	disableBut->setIconSet(*icoSet_dis_on, true);

	stretch = new MLabel(manipToolBar);
	stretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	
	copyBut = new MToolButton(manipToolBar);
	copyBut->setAccel(CTRL+'C');
	copyBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	copyBut->setUsesBigPixmap(true);
	copyBut->setIconSet(*icoSet_clone);

	addBut = new MToolButton(manipToolBar);
	addBut->setAccel(ALT+'A');
	addBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	addBut->setUsesBigPixmap(true);
	addBut->setIconSet(*icoSet_add);

	delBut = new MToolButton(manipToolBar);
	delBut->setAccel(ALT+'D');
	delBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	delBut->setUsesBigPixmap(true);
	delBut->setIconSet(*icoSet_del);

	stretch = new MLabel(manipToolBar);
	manipToolBar->setStretchableWidget(stretch);
	
	if(g_toolTipsEnabled)
	{
		QToolTip::add(closeBut,		"close (alt-Q)");
		QToolTip::add(disableBut,	"enable/disable current light (alt-E)");
		QToolTip::add(copyBut,		"clone current light (ctrl-C)");
		QToolTip::add(addBut,		"add light (alt-A)");
		QToolTip::add(delBut,		"delete current light (alt-D)");
	}
	
	lightGroupWidget = new LightGroupWidget(this, colorToolBar, typeToolBar);
	setCentralWidget(lightGroupWidget);
	setRightJustification(true);
	
	//set this after creating lightGroupWidget
	colorToolBar->setStretchableWidget(new QLabel(colorToolBar));
	typeToolBar->setStretchableWidget(new QLabel(typeToolBar));

	//----------------------------------
	QObject::connect (closeBut, SIGNAL(clicked()), SLOT(hide()) );
	QObject::connect (disableBut, SIGNAL(toggled(bool)), lightGroupWidget, SLOT(DisableCurrentLight(bool)) );
	QObject::connect (copyBut, SIGNAL(clicked()), lightGroupWidget, SLOT(CopyCurrentLight()) );
	QObject::connect (addBut, SIGNAL(clicked()), lightGroupWidget, SLOT( AddNewLight()) );
	QObject::connect (delBut, SIGNAL(clicked()), this, SLOT( ConfirmDelete()) );
	
	
	//----------------------------------
	setDockEnabled( QMainWindow::TornOff, true);
	setDockEnabled( QMainWindow::Top	, true);
	setDockEnabled( QMainWindow::Bottom	, true); 
	setDockEnabled( QMainWindow::Right	, true);
	setDockEnabled( QMainWindow::Left	, true);
	
}

LightGroupWidgetFrame::~LightGroupWidgetFrame()
{
}

////----------------------------------------------------------
//void LightGroupWidgetFrame::show()
//{
//	//Work around bug in Qt...don't want the toolbar, 
//	//but it shows up if you do a hide(), then a show()
//	QMainWindow::show();
//	menuBar()->hide();
//}
	
//----------------------------------------------------------
bool LightGroupWidgetFrame::setToolBar(int num, QMainWindow::ToolBarDock dock, bool newline)
{
	switch(num)
	{
		case 0:
			removeToolBar(manipToolBar);
			addToolBar(manipToolBar, 0, dock, newline);
			*dockList.at(0) = dock;
			return true;
		case 1:
			removeToolBar(colorToolBar);
			addToolBar(colorToolBar, 0, dock, newline);
			*dockList.at(1) = dock;
			return true;
		case 2:
			removeToolBar(typeToolBar);
			addToolBar(typeToolBar, 0, dock, newline);
			*dockList.at(2) = dock;
			return true;
		default:
			return false;
	}
}

//----------------------------------------------------------
void LightGroupWidgetFrame::ConfirmDelete()
{
    switch( QMessageBox::warning( this, "Confirm Delete", "Delete current light?", "Ok", "Cancel" )) 
	{
		case 0:
			lightGroupWidget->DeleteCurrentLight();
			break;
		case 1:
			break;
	}	
}

//--------------------------------------------------------------------
QToolBar*	LightGroupWidgetFrame::GetToolBar(int toolBarNum)
{
	switch(toolBarNum)
	{
		case 0:
			return manipToolBar;
		default:
			return 0;
	}
}

