/**
 * lightGroupWidget.h
 * Copyright 1999 by Mark B. Allan
 * 
 * light group widget
 */
#ifndef lightGroupWidget_h
#define lightGroupWidget_h

#include "lightGroup.h"

#include <qtoolbar.h> 
#include <qwidget.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qcombobox.h>

#include "mbaMainWindow.h"
#include "mwidgets.h"
#include "lightGroup.h"
#include "numInput.h"

class LightGroupWidgetFrame;
class ColorButton;

/**
 * LightGroupWidget
 */
//==============================================================================
class	LightGroupWidget : public QWidget
{
	Q_OBJECT
public:
	LightGroupWidget(LightGroupWidgetFrame *p, QToolBar *colorToolBar, QToolBar *typeToolBar, const char* name = 0, LightGroup *lg = 0);
	~LightGroupWidget();
	
	void	SetLightFromX(float in);
	void	SetLightFromY(float in);
	void	SetLightFromZ(float in);
	void	SetLightToX(float in);
	void	SetLightToY(float in);
	void	SetLightToZ(float in);
	void	SetConeAngle(float);
	void	SetConeDelta(float);
	void	SetConeDistrib(float);
	
	void	SetAttenConstant(float);
	void	SetAttenLinear(float);
	void	SetAttenQuadratic(float);
	
	void	setActiveLight(MBALight*);

signals:
	void	Updated();
	
public slots:
	void	ActivateColorButton(bool);
	void	ToggleDrawLight(bool);
	void	ChangeLightColor(float r, float g, float b);
	void	SelectLightType(int);
	
	void	SetP0(float);
	void	SetP1(float);
	void	SetP2(float);
	
	void	setCastsShadows(bool status);
	
	void	setIntensity(float);
	void	SetLightGroupPointer(LightGroup *newLightGroup);
	
	void	DisableCurrentLight(bool status);
	void	CopyCurrentLight();
	void	AddNewLight();
	void	DeleteCurrentLight();
	
	void	Update();
	
	void	updateCurrentLight();
	
protected slots:
	void	SwitchLight(int index);
	void	SetLightName();
	void	SelectParamType(int);
	
protected:
	LightGroup		*lightGroup;
	MBALight		*currentLight;
	LightGroupWidgetFrame	*parent;
	
	QVBoxLayout	*topVLayout;
	QHBoxLayout	*topHLayout;
	QVBoxLayout	*v1Layout;
	QVBoxLayout	*v2Layout;
	QVBoxLayout	*v3Layout;
	
	ColorButton	*colorButton;
	QListBox	*lightListBox;
	
	QLineEdit	*lightNameEdit;
	
	NumInput	*intensityInput;
	NumInput	*p0Input;
	NumInput	*p1Input;
	NumInput	*p2Input;
//	NumInput	*toXInput;
//	NumInput	*toYInput;
//	NumInput	*toZInput;
//	NumInput	*coneAngleInput;
//	NumInput	*coneDeltaInput;
//	NumInput	*coneDistribInput;
	
	QCheckBox	*glLightToggle;
	QCheckBox	*castsShadowsToggle;
	QCheckBox	*drawLightToggle;
	QComboBox	*lightTypeCombo;
	
	QComboBox	*lightParamCombo;
	int			lightParam;
	
	bool		widgetsEnabled;
	void		SetWidgetsEnabled(bool status);
};


/**
 * LightGroupWidgetFrame
 */
//==============================================================================
class LightGroupWidgetFrame : public MbaMainWindow
{
	Q_OBJECT
	
public:
	LightGroupWidgetFrame(QWidget *nullParent = 0, const char* name = 0, 
							ToolBarDock buttonBarDock = Top,
							ToolBarDock colorBarDock = Right,
							ToolBarDock typeBarDock = Right);
	~LightGroupWidgetFrame();
	
	inline LightGroupWidget	*GetLightGroupWidget() {	return lightGroupWidget;	}

	inline void	SetDisable(bool status)	{	disableBut->setOn(status);	}

	bool	setToolBar(int num, QMainWindow::ToolBarDock dock, bool newline = true);
	
//public slots:
//	void	show();
	QToolBar	*GetToolBar(int toolBarNum);

protected slots:
	void	ConfirmDelete();
	
protected:
	LightGroupWidget	*lightGroupWidget;
	QToolBar			*manipToolBar;
	QToolBar			*colorToolBar;
	QToolBar			*typeToolBar;
	
	MToolButton	*closeBut;
	MToolButton	*disableBut;
	MToolButton	*copyBut;
	MToolButton	*addBut;
	MToolButton	*delBut;
};

#endif
