/**
 * main.h
 * Copyright 1999 by Mark B. Allan
 */
#include <qapp.h>
#include <qdict.h> 
#include <qfont.h>
#include <qstyle.h>
#include <qpixmapcache.h> 
#include "mbaMainWindow.h" 

#include "mainFlow.h"
#include "camera.h"
#include "particleControlWidget.h"
#include "mbaLight.h"
#include "glWidget.h"

#include "mbaSceneWidget.h"
#include "mbaSceneTreeWidget.h"
#include "mbaSurfaceTabDialog.h"
#include "mbaCurveCollection.h"

class RenderRIB;

#include "lightGroup.h"
#include "lightGroupWidget.h"
#include "pPolygonTweak.h"

#include "Pixmaps/flowIcon.xpm"

//--------------------------------------------------------------------
bool	g_disableOrtho	= false;

bool	g_persistentGeom		= false;
bool	g_toolTipsEnabled		= true;
bool	g_saveBaseSceneOnExit	= true;
bool	g_killChildrenOnExit	= true;

int		g_randSeed = 0;

int		g_partSkip		= 0;
float	g_moveSpeed		= 1.0;
float	g_gridUnitSize	= 1.0;
int		g_gridNumUnits	= 10;
bool	g_glThick		= true;		/**< allow point/line size > 1 */
bool	g_glFastParticles = true;	/**< draw particles as points/lines or full primitives */
bool	g_glSortParticles = true;	/**< depth sort particles (only when BLEND enabled) */

int			g_numLabelWidth	= 60;
int			g_numInputWidth	= 120;
int			g_height_S		= 20;
int			g_height_M		= 25;
int			g_comboHeight	= 25;

char		*g_fontName	= 0;
int			g_fontSize	= 0;

QStyle	*winStyle = 0;
bool	g_winStyleToolBars = false;

QFont		*g_font_S	= 0;
QFont		*g_font_M	= 0;
QFont		*g_font_L	= 0;
QFont		*g_font_SB	= 0;
QFont		*g_font_MB	= 0;
QFont		*g_font_LB	= 0;
QFont		*g_font_SBI	= 0;
QFont		*g_font_MBI	= 0;
QFont		*g_font_LBI	= 0;

QPalette	basePalette;
QPalette	flowPalette;

QIconSet	*icoSet_add		= 0;
QIconSet	*icoSet_del		= 0;
QIconSet	*icoSet_dis		= 0;
QIconSet	*icoSet_dis_on	= 0;
QIconSet	*icoSet_dwn		= 0;
QIconSet	*icoSet_up		= 0;
QIconSet	*icoSet_rm		= 0;
QIconSet	*icoSet_rew		= 0;
QIconSet	*icoSet_lock	= 0;
QIconSet	*icoSet_lock_on	= 0;
QIconSet	*icoSet_play	= 0;
QIconSet	*icoSet_copy	= 0;
QIconSet	*icoSet_shade	= 0;
QIconSet	*icoSet_close	= 0;
QIconSet	*icoSet_clone	= 0;
QIconSet	*icoSet_paste	= 0;
QIconSet	*icoSet_clear	= 0;
QIconSet	*icoSet_dis_s	= 0;
QIconSet	*icoSet_shade_s	= 0;
QPixmap		*pix_file		= 0;
QPixmap		*pix_save		= 0;

//LightGroup				*_lightGroup			= 0;	// create in mainFlow.cpp
LightGroupWidgetFrame	*g_lightGroupWidgetFrame	= 0;	// create in mainFlow.cpp
LightGroupWidget		*g_lightGroupWidget			= 0;	// create in mainFlow.cpp
MBAScene				*g_scene					= 0;
MBASceneWidget			*g_sceneWidget				= 0;
PolygonTweak			*g_polyTweakWidget			= 0;
GLWidget				*g_glWidget					= 0;
GlPortCam				*g_camGLPort				= 0;
CameraView				*g_globalCam				= 0;
ParticleControlWidget	*g_particleControlWidget	= 0;
ParticleControlFrame	*g_particleControlFrame		= 0;
RenderPanelTabDialog	*g_renderOptions			= 0;
MBASurfaceTabDialog		*g_surfaceEditWidget		= 0;
RenderRIB				*g_renderRIB				= 0;

MBACurveCollection		*g_curveControl	= 0;

//-- These will be set in an init file
int		g_numPreviewCols			= 2;
int		g_previewTableCellSize	= 60;
int		g_previewRenderSize		= 128;
int		g_pixmapCacheSize		= 5120;

char	*g_homeDir				= 0;
char	*g_flowHome				= 0;
char	*g_shadersPath			= 0;
char	*g_shaderSurfDir		= 0;
char	*g_shaderDispDir		= 0;
char	*g_rendName				= 0;

char	*g_rendermanExec		= 0;
char	*g_shaderInfoExec		= 0;
char	*g_tiffToJpegExec		= 0;
char	*g_envShaderPath		= 0;
char	*g_rendermanOpts		= 0;
char	*g_defaultSurfaceShader	= 0;
char	*g_defaultDisplaceShader= 0;
char	*g_compiledShaderExt	= 0;
bool	g_hasRendermanExec		= false;
bool	g_hasShaderInfoExec		= false;
bool	g_hasTiffToJpegExec		= false;

QStrList	g_surfaceShadersList;
QStrList	g_displacementShadersList;
QStrList	g_lightShadersList;
QStrList	g_imagerShadersList;
QStrList	g_volumeShadersList;

MainFlow	*g_mainFlow			= 0;
QWidget		*g_mainWidget		= 0;
int			g_winManXAdd		= 0; //5
int			g_winManYAdd		= 0; //21
int			g_dskTopW	= 640;
int			g_dskTopH	= 480;

int		g_frame = 0;
//bool	writeRIB = false;
int		g_rendW				= 160;
int		g_rendH				= 120;
float	g_rendA				= 1.0f;
float	g_rendRatio			= 4.0/3.0;
int		g_rendInterlace		= 1;
float	g_rendSamples[2]	= { 1, 1 };
bool	g_allowTrueDisplace	= false;
bool	g_allowShadows		= true;
bool	g_motionBlur		= false;
int		g_numBlurSteps		= 5;
float	g_rendCropTop		= 0.0;
float	g_rendCropBot		= 1.0;
float	g_rendCropLeft		= 0.0;
float	g_rendCropRight		= 1.0;

//---GL-State-variables-----------
bool	GL__lighting;
int		GL__polygonMode;
//--------------------------------
bool	g_glInitialized	= false;
bool	g_glCullFace	= false;
bool	g_glBlend		= false;
bool	g_glAlphaTest	= true;
bool	g_glTwoSideLight= false;

int		g_fps	= 30;

float	g_gridXOffset =  0.00;
float	g_gridYOffset = -0.05;
float	g_gridZOffset =  0.00;

//-------------------------------------------------------
void	SetGlobals();

