/**
 * mainFlow.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * main widget w/ menubar, etc.
 */
#include "mainFlow.h"

#include <limits.h>

#include <qkeycode.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qiconset.h>
#include <qapplication.h>
#include <qtooltip.h>

#include "extern.h"
#include "define.h"
#include "glWidget.h"
#include "glWidgetOrtho.h"
#include "mbaLight.h"
#include "mbaSurfaceTabDialog.h"
#include "mbaSceneWidget.h"
#include "particleControlWidget.h"
#include "pPolygonTweak.h"
#include "configFlowIO.h"
#include "aboutDialog.h"
#include "lightGroupWidget.h"
#include "renderPanelTabDialog.h"
#include "prefDialog.h"

#include "JoystickWidget.h"

//==============================================================================
MainFlow::MainFlow( QWidget* parent, const char* name, 
				ToolBarDock buttonBarDock, ToolBarDock infoBarDock)
    : MbaMainWindow( parent, name )
{
	g_mainWidget 	= this;
	numViews		= 0;
	fileMenu		= 0;
	editMenu		= 0;
	viewMenu		= 0;
	rendMenu		= 0;
	orthoWidgets[0] = 0;
	orthoWidgets[1] = 0;
	orthoWidgets[2] = 0;
	orthoWidgets[3] = 0;
	

//	setFont( *g_font_MB );
		
	setCaption("f l o w");
	setIcon(QPixmap((const char**)flowIcon_xpm));
	
	prefDialog = new PrefDialog(this);
	prefDialog->hide();
	
//	JoystickWidget *joystickWidget = new JoystickWidget();
//	joystickWidget->show();
//	joystickWidget->move(25, 25);
//	
	//----------------------------------widgets
	// Create main context before anything else
	g_glWidget = new GLWidget(this);
//	glWidget = new GLWidget(0);
	g_camGLPort = g_glWidget->GetGLPort();
	
	//-- Initialize file dialog variables...
	currentSceneFile = "";
	QFileInfo	testDirScene("./Scene");
	if (testDirScene.isDir())
		currentSceneDir = "./Scene";
	else
		currentSceneDir = ".";
	
	//-- validators --------------------
	QIntValidator		*intValidator;
	QDoubleValidator	*doubleValidator;
	
	//-- toolbars ----------------------
	mainToolBar = new QToolBar(0, this,  buttonBarDock, true);
	dockList.append(new QMainWindow::ToolBarDock(buttonBarDock));
	lcdToolBar = new QToolBar(0, this,  infoBarDock, true);
	dockList.append(new QMainWindow::ToolBarDock(infoBarDock));
	numToolBars = 2;

	if(g_winStyleToolBars)
	{
		mainToolBar->setStyle(winStyle);
		lcdToolBar->setStyle(winStyle);
	}
	QLabel	*tmpStretch;
	
	writeRIBBut  = new MToolButton (mainToolBar);
	writeRIBBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	writeRIBBut->setUsesBigPixmap(true);
	//writeRIBBut->setPixmap(QPixmap((const char**)but_rib_xpm ));
	writeRIBBut->setIconSet(*icoSet_rm);
	writeRIBBut->setAccel('R');
	
	tmpStretch = new MLabel(mainToolBar);
	tmpStretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	tmpStretch = new MLabel(mainToolBar);
	tmpStretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	
	clearCamBut  = new MToolButton (mainToolBar);
//	clearCamBut->setText ("Clear");
	clearCamBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	clearCamBut->setUsesBigPixmap(true);
	clearCamBut->setIconSet(*icoSet_clear);

	tmpStretch = new MLabel(mainToolBar);
	tmpStretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	
	resetFrameBut	 = new MToolButton (mainToolBar);
	resetFrameBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	resetFrameBut->setUsesBigPixmap(true);
	//resetFrameBut->setPixmap(QPixmap((const char**)but_rew_xpm ));
	resetFrameBut->setIconSet(*icoSet_rew);

	animationBut	= new MToolButton (mainToolBar);
	animationBut->setToggleButton(true);
	animationBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	animationBut->setUsesBigPixmap(true);
	//animationBut->setPixmap(QPixmap((const char**)but_play_xpm ));
	animationBut->setIconSet(*icoSet_play);
	
	tmpStretch = new MLabel(mainToolBar);
	tmpStretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	mainToolBar->setStretchableWidget(tmpStretch);
	
//	QIconSet	iconSet(QPixmap((const char**)but_lock_xpm), QIconSet::Large );
//	iconSet.setPixmap(QPixmap((const char**)but_play_xpm), QIconSet::Large, QIconSet::Active);
	
	extraButton	= new MToolButton (mainToolBar);
//	extraButton->setText ("Move");
	extraButton->setText ("");
	extraButton->setToggleButton(true);
	extraButton->setFixedSize(TOOL_SIZE, TOOL_SIZE);
//	extraButton->setUsesBigPixmap(true);
	extraButton->setAccel('M');
	extraButton->setEnabled(true);
		
	tmpStretch = new MLabel(mainToolBar);
	tmpStretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	
	targetLockBut	= new MToolButton (mainToolBar);
	targetLockBut->setIconSet(*icoSet_lock);
//	targetLockBut->setIconSet(*icoSet_lock_on, true);
	targetLockBut->setToggleButton(true);
	targetLockBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
//	targetLockBut->setText("T");
	targetLockBut->setUsesBigPixmap(true);
		
	//--
	frameNumberEdit	= new MLineEdit (lcdToolBar);
	frameNumberEdit->setFixedWidth(3*frameNumberEdit->sizeHint().height());
	intValidator = new QIntValidator(0, INT_MAX-1, frameNumberEdit);
	frameNumberEdit->setValidator(intValidator);
	
	tmpStretch = new MLabel(lcdToolBar);
	lcdToolBar->setStretchableWidget(tmpStretch);

	xMouseLabel = new QLabel(lcdToolBar);
	xMouseLabel->setMinimumSize(30, 20);
	xMouseLabel->setAlignment( AlignLeft | AlignBottom );
	xMouseLabel->setFont(*g_font_S);
	xMouseLabel->setText("");
	//xMouseLabel->setFrameStyle( QFrame::Panel | QFrame::Sunken);
	
	yMouseLabel = new QLabel(lcdToolBar);
	yMouseLabel->setMinimumSize(30, 20);
	yMouseLabel->setAlignment( AlignLeft | AlignBottom );
	yMouseLabel->setFont(*g_font_S);
	yMouseLabel->setText("");
	//yMouseLabel->setFrameStyle( QFrame::Panel | QFrame::Sunken);
	
	QCString tmpString;
	
	moveSpeedLabel = new QLabel(lcdToolBar);
	moveSpeedLabel->setText("   target fps ");
	moveSpeedLabel->setFont(*g_font_S);
//	moveSpeedLabel->setMinimumSize(40, 20);
	moveSpeedLabel->setAlignment( AlignRight | AlignVCenter );
	targetFPSEdit	= new MLineEdit (lcdToolBar);
	targetFPSEdit->setMaxLength(5);
	targetFPSEdit->setFixedWidth(2*targetFPSEdit->sizeHint().height());
	intValidator = new QIntValidator(1, 60, targetFPSEdit);
	targetFPSEdit->setValidator(intValidator);
	targetFPSEdit->setText(tmpString.setNum(g_fps));
	
	moveSpeedLabel = new QLabel(lcdToolBar);
	moveSpeedLabel->setText("   move speed ");
	moveSpeedLabel->setFont(*g_font_S);
//	moveSpeedLabel->setMinimumSize(40, 20);
	moveSpeedLabel->setAlignment( AlignRight | AlignVCenter );
	moveSpeedEdit	= new MLineEdit (lcdToolBar);
	moveSpeedEdit->setMaxLength(5);
	moveSpeedEdit->setFixedWidth(2*moveSpeedEdit->sizeHint().height());
	doubleValidator = new QDoubleValidator(-100.0, 100.0, 5, moveSpeedEdit);
	moveSpeedEdit->setValidator(doubleValidator);
	moveSpeedEdit->setText(tmpString.setNum(g_moveSpeed));
	
	tmpStretch = new MLabel(lcdToolBar);
	tmpStretch->setMinimumSize(10, 20);
	
	fpsLabel = new QLabel(lcdToolBar);
//	fpsLabel->setMinimumSize(37, 32);
	fpsLabel->setAlignment( AlignCenter );
	fpsLabel->setText("30.00");
	fpsLabel->setFrameStyle( QFrame::Box | QFrame::Sunken);
	fpsLabel->setFixedWidth(fpsLabel->sizeHint().width());
	fpsLabel->setText("");
	
	g_camGLPort->SetQLCDPointers(frameNumberEdit, xMouseLabel, yMouseLabel, fpsLabel);
	
	if(g_toolTipsEnabled)
	{
		QToolTip::add(writeRIBBut,		"preview RIB (R)");
		QToolTip::add(clearCamBut,		"clear camera and current particles");
		QToolTip::add(resetFrameBut,	"rewind to frame 0");
		QToolTip::add(animationBut,		"run simulation");
		QToolTip::add(extraButton,		"extra button for testing new stuff");
		QToolTip::add(targetLockBut,	"enable/disable camera target lock");
	}
	//-- connect -----------------------
	QObject::connect(writeRIBBut,	SIGNAL(clicked()),  SLOT(RenderFrame()) );
	QObject::connect(clearCamBut,	SIGNAL(clicked()),  SLOT(Clear()) );
	QObject::connect(resetFrameBut,	SIGNAL(clicked()),  SLOT(ResetFrameCount()) );
	QObject::connect(targetLockBut,	SIGNAL(toggled(bool)), g_camGLPort, SLOT(SetTargetLock(bool)) );
	QObject::connect(animationBut,	SIGNAL(toggled(bool)), g_camGLPort, SLOT(SetAnimation(bool)) );
	QObject::connect(extraButton,	SIGNAL(toggled(bool)), SLOT(ExtraFunction(bool)) );
	
	QObject::connect(frameNumberEdit,	SIGNAL(returnPressed()), SLOT(setFrameCount()) );
	QObject::connect(targetFPSEdit, 	SIGNAL(returnPressed()), SLOT(setTargetFPS()) );
	QObject::connect(moveSpeedEdit,		SIGNAL(returnPressed()), SLOT(setMoveSpeed()) );

	//-- layout ------------------------
//	glWidget->show();
//	glWidget->resize(320, 240);
	setCentralWidget(g_glWidget);
	g_glWidget->setFocus();
	setRightJustification(true);
	resize(400, 400);
	
	//-- other stuff -------------------
	targetLockBut->setOn(true);
	//mainToolBar->hide();
	
}

//--------------------------------------------------------------------
MainFlow::~MainFlow()
{
}

//--------------------------------------------------------------------
QToolBar*	MainFlow::GetToolBar(int toolBarNum)
{
	switch(toolBarNum)
	{
		case 0:
			return mainToolBar;
		case 1:
			return lcdToolBar;
		default:
			return 0;
	}
}

//----------------------------------------------------------
bool MainFlow::setToolBar(int num, QMainWindow::ToolBarDock dock, bool newline)
{
	switch(num)
	{
		case 0:
			removeToolBar(mainToolBar);
			addToolBar(mainToolBar, 0, dock, newline);
			*dockList.at(0) = dock;
			return true;
		case 1:
			removeToolBar(lcdToolBar);
			addToolBar(lcdToolBar, 0, dock, newline);
			*dockList.at(1) = dock;
			return true;
		default:
			return false;
	}
}

//--------------------------------------------------------------------
void	MainFlow::closeEvent(QCloseEvent *)
{
	// if this window closes, kill the app...
	qApp->quit();
}

void	MainFlow::showAndRaiseParticleControlFrame()
{	g_particleControlFrame->show();	g_particleControlFrame->raise();	}
void	MainFlow::showAndRaiseLightGroupWidgetFrame()
{	g_lightGroupWidgetFrame->show();	g_lightGroupWidgetFrame->raise();	}
void	MainFlow::showAndRaiseRenderOptions()
{	g_renderOptions->show();	g_renderOptions->raise();	}
void	MainFlow::showAndRaiseSurfaceEditWidget()
{	g_surfaceEditWidget->show();	g_surfaceEditWidget->raise();	}
void	MainFlow::showAndRaisePolyTweakWidget()
{	g_polyTweakWidget->show();	g_polyTweakWidget->raise();	}
void	MainFlow::showAndRaiseSceneWidget()
{	g_sceneWidget->show();	g_sceneWidget->raise();	}


//--------------------------------------------------------------------
void MainFlow::Init()
{
	g_particleControlWidget	= g_particleControlFrame->GetParticleControlWidget();

	
	g_sceneWidget->SetScenePointer(g_scene);
	lightGroupWidget = g_lightGroupWidgetFrame->GetLightGroupWidget();
	lightGroupWidget->SetLightGroupPointer(g_scene->ActiveLightGroup());
	
	//----------------------------------menuBar
	fileMenu = new QPopupMenu(this);
	fileMenu->insertItem ("&new",			this, SLOT(NewScene()), CTRL+Key_N);
	fileMenu->insertItem ("&open scene",	this, SLOT(LoadSceneDialog()), CTRL+Key_O);
    fileMenu->insertSeparator();
	fileMenu->insertItem ("&save scene",	this, SLOT(SaveThisScene()), CTRL+Key_S);
	fileMenu->insertItem ("save scene &as",	this, SLOT(SaveSceneDialog()), CTRL+Key_A);
    fileMenu->insertSeparator();
//	fileMenu->insertItem ("&write GLUT code",this, SLOT(writeGLUTCode()));
//    fileMenu->insertSeparator();
    fileMenu->insertItem( "e&xit",  		this, SLOT(Quit()), CTRL+Key_Q );
	
	editMenu = new QPopupMenu(this);
	editMenu->insertItem ("&particle control",		this, SLOT(showAndRaiseParticleControlFrame()), CTRL+Key_P);
	editMenu->insertItem ("&lights",				this, SLOT(showAndRaiseLightGroupWidgetFrame()), CTRL+Key_L );
	editMenu->insertItem ("&render options",		this, SLOT(showAndRaiseRenderOptions()), CTRL+Key_R );
	editMenu->insertItem ("&surface properties",	this, SLOT(showAndRaiseSurfaceEditWidget()), CTRL+Key_U );
	editMenu->insertItem ("&polygon tweaker",		this, SLOT(showAndRaisePolyTweakWidget()) );
    editMenu->insertSeparator();
	editMenu->insertItem ("scene &tree",			this, SLOT(showAndRaiseSceneWidget()), CTRL+Key_T );
    editMenu->insertSeparator();
	editMenu->insertItem ("&preferences",			this, SLOT(ShowPreferences()), ALT+Key_P);
	
	rendMenu = new QPopupMenu(this);
	rendMenu->setCheckable(true);
	rendID080	= rendMenu->insertItem( "width  80",	this, SLOT(setRenderSize080()) );
	rendID160	= rendMenu->insertItem( "width 160",	this, SLOT(setRenderSize160()) );
	rendID320	= rendMenu->insertItem( "width 320",	this, SLOT(setRenderSize320()) );
	rendID640	= rendMenu->insertItem( "width 640",	this, SLOT(setRenderSize640()) );
	rendID800	= rendMenu->insertItem( "width 800",	this, SLOT(setRenderSize800()) );
	rendID1024	= rendMenu->insertItem( "width 1024",	this, SLOT(setRenderSize1024()) );
	rendMenu->insertSeparator();
	rendID4x3	= rendMenu->insertItem( "ratio 4:3", 	this, SLOT(setRenderRatio4x3()) );
	rendID16x9	= rendMenu->insertItem( "ratio 16:9", 	this, SLOT(setRenderRatio16x9()) );
	rendMenu->insertSeparator();
//	rendIDfullGL= rendMenu->insertItem( "same as GL", 	this, SLOT(setRenderSizeFullGL()) );
//	rendIDhalfGL= rendMenu->insertItem( "half of GL", 	this, SLOT(setRenderSizeHalfGL()) );
//	rendMenu->insertSeparator();
	rendIDS11	= rendMenu->insertItem( "sample 1x1",	this, SLOT(setSamples11()) );
	rendIDS22	= rendMenu->insertItem( "sample 2x2",	this, SLOT(setSamples22()) );
	rendIDS33	= rendMenu->insertItem( "sample 3x3",	this, SLOT(setSamples33()) );
	rendIDS44	= rendMenu->insertItem( "sample 4x4",	this, SLOT(setSamples44()) );
	setRenderSize160();
	setRenderRatio4x3();
	setSamples11();
	
	viewMenu = new QPopupMenu(this);
	viewMenu->setCheckable(true);
	viewMenu->insertItem ("ortho view", 	  this,   SLOT(showGlOrtho1()), CTRL+Key_1 );
	viewMenu->insertItem ("ortho view", 	  this,   SLOT(showGlOrtho2()), CTRL+Key_2 );
	viewMenu->insertItem ("ortho view", 	  this,   SLOT(showGlOrtho3()), CTRL+Key_3 );
	viewMenu->insertItem ("ortho view", 	  this,   SLOT(showGlOrtho4()), CTRL+Key_4 );
//	viewMenu->insertSeparator();
//	viewMenu->insertItem ("free size", this, SLOT(DUMMYslot())  );
//	viewMenu->insertItem ("lock to 0.5x render size", this, SLOT(DUMMYslot()) );
//	viewMenu->insertItem ("lock to 1.0x render size", this, SLOT(DUMMYslot())  );
//	viewMenu->insertItem ("lock to 2.0x render size", this, SLOT(DUMMYslot())  );
	viewMenu->insertSeparator();
	viewMenu->insertItem ("&iconify all", this, SLOT(iconifyAll())  );
	
	helpMenu = new QPopupMenu(this);
	helpMenu->insertItem ("about", this, SLOT(showAboutDialog()) );
	
	menuBar()->insertItem ("&file", fileMenu);
	menuBar()->insertItem ("&edit", editMenu);
	menuBar()->insertItem ("&render", rendMenu);
	menuBar()->insertItem ("&view", viewMenu);
	menuBar()->insertSeparator();
	menuBar()->insertItem ("&help", helpMenu);
	
	if (!LoadScene("Scene/base.scene"))
	{
		NewScene();
	}
}

//--------------------------------------------------------------------
void	MainFlow::DUMMYslot()
{
}

//--------------------------------------------------------------------
void	MainFlow::ShowPreferences()
{
	prefDialog->show();
}

//--------------------------------------------------------------------
void	MainFlow::RenderFrame()
{
	g_renderOptions->RenderFrame();
}

//--------------------------------------------------------------------
void	MainFlow::iconifyAll()
{
	g_mainFlow->iconify();
	g_sceneWidget->iconify();
	g_lightGroupWidgetFrame->iconify();
	g_polyTweakWidget->iconify();
	g_particleControlFrame->iconify();
	g_surfaceEditWidget->iconify();
	g_renderOptions->iconify();
}
//--------------------------------------------------------------------
void	MainFlow::Quit()
{
	QCString	saveCurrent("./Scene");
    switch( QMessageBox::warning( this, "confirm exit", "c o n f i r m   e x i t . . . ", "&ok", "&cancel" )) 
	{
		case 0:
			if(g_persistentGeom)
				ConfigFlowIO::WriteGeomFile();
			if(g_saveBaseSceneOnExit)
			{
				saveCurrent += "/base.scene";
				SaveScene((const char*)saveCurrent);
			}
			qApp->quit();
			break;
		case 1:
			break;
	}	
}

//--------------------------------------------------------------------
void	MainFlow::showGlOrtho(int num)
{
	if(g_disableOrtho)
	{
		QMessageBox::warning(this, "MainFlow::showGlOrtho", "Ortho widgets are disabled because some XFree4.0 direct rendering GLX servers will crash\nwhen display lists are shared across contexts (driver bug).\nTo enable ortho views, change \'g_disableOrtho\' in main.h to \'false\' and recompile.");
	}
	else
	{
		GlPort::disableGLDraw(true);
		GL_ERROR_MSG("MainFlow::showGlOrtho -- enter");
		if(num >= 0 && num < NUM_ORTHO_PORTS)
		{
			if(orthoWidgets[num])
			{
				orthoWidgets[num]->show();
				orthoWidgets[num]->raise();
			}
			else
			{
				orthoWidgets[num] = new GLWidgetOrtho(0, 0, g_camGLPort, (4+num*2)%6);
				GL_ERROR_MSG("MainFlow::showGlOrtho -- created new ortho port");
				orthoWidgets[num]->resize(481,450);
				orthoWidgets[num]->show();
			}
		}
		GL_ERROR_MSG("MainFlow::showGlOrtho -- leave");
		GlPort::disableGLDraw(false);
	}
}

//--------------------------------------------------------------------
void	MainFlow::showGlOrtho1()
{
	showGlOrtho(0);
}

//--------------------------------------------------------------------
void	MainFlow::showGlOrtho2()
{
	showGlOrtho(1);
}

//--------------------------------------------------------------------
void	MainFlow::showGlOrtho3()
{
	showGlOrtho(2);
}

//--------------------------------------------------------------------
void	MainFlow::showGlOrtho4()
{
	showGlOrtho(3);
}

//--------------------------------------------------------------------
void	MainFlow::showAboutDialog()
{
	AboutDialog	aboutDialog;
	aboutDialog.exec();
}

//--------------------------------------------------------------------
void	MainFlow::setRenderSize080()
{
	setRenderItemsUnchecked();
	g_renderOptions->setSizeX(80);
	g_renderOptions->setSizeY((int)(0.1+80.0/g_rendRatio));
	rendMenu->setItemChecked(rendID080, true);
}
void	MainFlow::setRenderSize160()
{
	setRenderItemsUnchecked();
	g_renderOptions->setSizeX(160);
	g_renderOptions->setSizeY((int)(0.1+160.0/g_rendRatio));
	rendMenu->setItemChecked(rendID160, true);
}
void	MainFlow::setRenderSize320()
{
	setRenderItemsUnchecked();
	g_renderOptions->setSizeX(320);
	g_renderOptions->setSizeY((int)(0.1+320.0/g_rendRatio));
	rendMenu->setItemChecked(rendID320, true);
}
void	MainFlow::setRenderSize640()
{
	setRenderItemsUnchecked();
	g_renderOptions->setSizeX(640);
	g_renderOptions->setSizeY((int)(0.1+640.0/g_rendRatio));
	rendMenu->setItemChecked(rendID640, true);
}
void	MainFlow::setRenderSize800()
{
	setRenderItemsUnchecked();
	g_renderOptions->setSizeX(800);
	g_renderOptions->setSizeY((int)(0.1+800.0/g_rendRatio));
	rendMenu->setItemChecked(rendID800, true);
}
void	MainFlow::setRenderSize1024()
{
	setRenderItemsUnchecked();
	g_renderOptions->setSizeX(1024);
	g_renderOptions->setSizeY((int)(0.1+1024.0/g_rendRatio));
	rendMenu->setItemChecked(rendID1024, true);
}
void	MainFlow::setRenderItemsUnchecked()
{
	rendMenu->setItemChecked(rendID080, false);
	rendMenu->setItemChecked(rendID160, false);
	rendMenu->setItemChecked(rendID320, false);
	rendMenu->setItemChecked(rendID640, false);
	rendMenu->setItemChecked(rendID800, false);
	rendMenu->setItemChecked(rendID1024, false);
//	rendMenu->setItemChecked(rendIDfullGL, false);
//	rendMenu->setItemChecked(rendIDhalfGL, false);
}

//--------------------------------------------------------------------
void	MainFlow::setRenderRatio4x3()
{
	rendMenu->setItemChecked(rendID4x3,  true);
	rendMenu->setItemChecked(rendID16x9, false);
	g_rendRatio = 4.0/3.0;
	g_renderOptions->setSizeY((int)(0.1+g_rendW/g_rendRatio));
}

//--------------------------------------------------------------------
void	MainFlow::setRenderRatio16x9()
{
	rendMenu->setItemChecked(rendID4x3,  false);
	rendMenu->setItemChecked(rendID16x9, true);
	g_rendRatio = 16.0/9.0;
	g_renderOptions->setSizeY((int)(0.1+g_rendW/g_rendRatio));
}

////--------------------------------------------------------------------
//void	MainFlow::setRenderSizeFullGL()
//{
//	setRenderItemsUnchecked();
//	rendMenu->setItemChecked(rendID4x3,  false);
//	rendMenu->setItemChecked(rendID16x9, false);
//
//	rendMenu->setItemChecked(rendIDfullGL, true);
//	rendMenu->setItemChecked(rendIDhalfGL, false);
//}
//
////--------------------------------------------------------------------
//void	MainFlow::setRenderSizeHalfGL()
//{
//	setRenderItemsUnchecked();
//	rendMenu->setItemChecked(rendID4x3,  false);
//	rendMenu->setItemChecked(rendID16x9, false);
//
//	rendMenu->setItemChecked(rendIDfullGL, false);
//	rendMenu->setItemChecked(rendIDhalfGL, true);
//}
//
//--------------------------------------------------------------------
void	MainFlow::setSamples11()
{	setSamples(1);	}
void	MainFlow::setSamples22()
{	setSamples(2);	}
void	MainFlow::setSamples33()
{	setSamples(3);	}
void	MainFlow::setSamples44()
{	setSamples(4);	}

void	MainFlow::setSamples( int a )
{
	rendMenu->setItemChecked(rendIDS11, false);
	rendMenu->setItemChecked(rendIDS22, false);
	rendMenu->setItemChecked(rendIDS33, false);
	rendMenu->setItemChecked(rendIDS44, false);
	g_renderOptions->setSamplesX((float)a);
	g_renderOptions->setSamplesY((float)a);
	switch(a)
	{
		case 1:
			rendMenu->setItemChecked(rendIDS11, true);
			break;
		case 2:
			rendMenu->setItemChecked(rendIDS22, true);
			break;
		case 3:
			rendMenu->setItemChecked(rendIDS33, true);
			break;
		case 4:
			rendMenu->setItemChecked(rendIDS44, true);
			break;
	}
}

//--------------------------------------------------------------------
void	MainFlow::LoadSceneDialog()
{
	QCString s;
	QCString tmp;

	QFileDialog	fileDialog(currentSceneDir, 0, this, 0, true);
	fileDialog.setMode(QFileDialog::ExistingFile);
	fileDialog.resize(fileDialog.width(), (3*g_dskTopH)/4);
	if (!currentSceneFile.isEmpty())
		fileDialog.setSelection(currentSceneFile);
		
	if ( fileDialog.exec() == QDialog::Accepted)
		s = fileDialog.selectedFile();
	
	QFileInfo	selectedFile (s);
	if (selectedFile.exists())
	{
		if ( ! (selectedFile.isFile() && selectedFile.isReadable()) )
		{
			tmp.sprintf("\"%s\"\nis not a readable file.", (const char*)s);
			QMessageBox::critical ( this, "Read Particle System ERROR", tmp);
		}
		else
		{
			currentSceneFile = (const char*)s;
			LoadScene((const char*)s);
			currentSceneDir = fileDialog.dir()->path();
		}
	}
}

//--------------------------------------------------------------------
void	MainFlow::SaveSceneDialog()
{
	int		i;
	int		result;
	QCString s;
	QCString tmp;

	QFileDialog	fileDialog(currentSceneDir, 0, this, 0, true);
	fileDialog.resize(fileDialog.width(), (3*g_dskTopH)/4);
	if (!currentSceneFile.isEmpty())
		fileDialog.setSelection(currentSceneFile);
	
	if ( (result = fileDialog.exec()) == QDialog::Accepted)
	{
		currentSceneDir = fileDialog.dir()->path();
		s = fileDialog.selectedFile();
	}
	
	QFileInfo	selectedFile (s);
	if (selectedFile.exists())
	{
		tmp.sprintf("The file\n\"%s\"\nexists.\n\nOverwrite?", (const char*)s);
		i = QMessageBox::warning ( this, "WARNING - File Exists", tmp, "Ok", "Cancel");
		switch (i)
		{
			case 0:
				currentSceneFile = (const char*)s;
				SaveScene((const char*)s);
				break;
			case 1:
				// Don't overwrite
				break;
		}
	}
	else if (result == QDialog::Accepted)
	{
		currentSceneFile = (const char*)s;
		SaveScene((const char*)s);
	}
}

//--------------------------------------------------------------------
void	MainFlow::SaveThisScene()
{
	if(strlen(currentSceneFile) > 0)
	{
		SaveScene(currentSceneFile);
	}
	else
		SaveSceneDialog();
}

//--------------------------------------------------------------------
void	MainFlow::NewScene()
{
	GL_ERROR_MSG("MainFlow::NewScene -- enter");
	
	currentSceneFile = "";
	warning("//------------------------------------------------------------");
	g_particleControlWidget->SetParticleControl((ParticleControl*)0);
	
	MBAScene	*oldScene = g_scene;;
	MBAScene	*newScene = new MBAScene;
	
	ParticleControl	*tmpControl = new ParticleControl();
	newScene->AddParticleSystem(tmpControl);
	LightGroup	*tmpLightGroup;
	tmpLightGroup = new LightGroup();
	newScene->AddLightGroup(tmpLightGroup);
	if (g_rendermanOpts)
		*g_rendermanOpts = '\0';
		
	delete [] g_rendName;
	g_rendName = new char[5];
	strcpy(g_rendName, "flow");
	
	g_lightGroupWidget->blockSignals(true);
	g_particleControlWidget->blockSignals(true);
	
	g_sceneWidget->SetScenePointer(newScene);
	g_scene = newScene;
	g_particleControlWidget->SceneChange();
	g_globalCam->SetFov(40.0);
	g_globalCam->SetTrans (-4.0, 2.0, 5.0);
	g_globalCam->SetTarget (0.0, 0.5, 0.0);
	g_globalCam->ProjectGL();
	//g_camGLPort->updateGLIfNecessary();
	frameNumberEdit->setText("0");
	setFrameCount();
	g_randSeed = 0;
#ifndef WIN32
	srand48(g_randSeed);
#endif
	
	g_lightGroupWidget->blockSignals(false);
	g_particleControlWidget->blockSignals(false);

	delete oldScene;
	GL_ERROR_MSG("MainFlow::NewScene -- leave");
}

//--------------------------------------------------------------------
bool	MainFlow::LoadScene(const char *filename)
{
	warning("------------------------------------------------------------");
	warning ("---- Loading scene: %s", filename); 
	
	bool	retVal = false;
	FILE	*infile;
	MBAScene	*newScene;
	MBAScene	*oldScene;
	
	GlPort::disableGLDraw(true);
	GL_ERROR_MSG("MainFlow::LoadScene -- enter");
	//FIXME -  Re-initialize GL stuff, disable all lights, etc.
	
	infile = fopen(filename, "r");
	if (infile)
	{
		newScene = new MBAScene;
		g_particleControlWidget->SetParticleControl((ParticleControl*)0);
		
		newScene->loadFromFile(infile);
		
		fclose(infile);
		
		oldScene = g_scene;
		g_scene = newScene;
		g_sceneWidget->SetScenePointer(g_scene);
		delete oldScene;
		frameNumberEdit->setText("0");
		setFrameCount();
#ifndef WIN32
		srand48(g_randSeed);
#endif
		retVal = true;
	}
	else
		retVal = false;
		
	GlPort::disableGLDraw(false);
	GL_ERROR_MSG("MainFlow::LoadScene -- leave");
	
	return retVal;
}

//--------------------------------------------------------------------
bool	MainFlow::SaveScene(const char *filename)
{
	FILE	*outfile;
	
	outfile = fopen(filename, "w");
	
	if(outfile)
	{
		warning ("---- Saving scene: %s", filename); 
		/*status = */g_scene->saveToFile(outfile);
//		g_particleControlWidget->SaveParticleSystem(outfile);
	
		fclose (outfile);
		return true;/*status;*/
	}
	return false;
}

//----------------------------------------------------------
void	MainFlow::ResetFrameCount()
{
	g_frame = 0;
	g_camGLPort->ResetCamList();
	g_camGLPort->updateGLIfNecessary();
}

//----------------------------------------------------------
void	MainFlow::setFrameCount()
{
	static int	goingToFrame = false;
	if(!goingToFrame)
	{
		goingToFrame = true;
	
		int 	newFrame;
		QCString	newFrameString;
		
		newFrameString = frameNumberEdit->text();
		newFrame = newFrameString.toInt();

		g_scene->GotoFrame(newFrame);

		g_glWidget->setFocus();
		g_camGLPort->update();
		goingToFrame = false;
	}
}

//----------------------------------------------------------
void	MainFlow::setMoveSpeed()
{
	float	newSpeed;
	QCString	newSpeedString;

	newSpeedString = moveSpeedEdit->text();
	newSpeed = newSpeedString.toFloat();
	g_moveSpeed = newSpeed;
	warning("MainFlow::setMoveSpeed() %s %f", (const char*)newSpeedString,	newSpeed);
	g_glWidget->setFocus();

}

//----------------------------------------------------------
void	MainFlow::setTargetFPS()
{
	int	newFPS;
	QCString	newFPSString;

	newFPSString = targetFPSEdit->text();
	newFPS = newFPSString.toInt();
	g_fps = newFPS;
	animationBut->setOn(false);
	g_glWidget->setFocus();
	warning("MainFlow::setTargetFPS() %s %d", (const char*)newFPSString,	newFPS);
}

//----------------------------------------------------------
void	MainFlow::Clear()
{
	
//	g_particleControlWidget->ResetSystem();
#ifndef WIN32
	srand48(g_randSeed);
#endif
	g_scene->ResetParticleSystems();
	g_camGLPort->ClearCamList();
	g_camGLPort->updateGLIfNecessary();
	g_frame = 0;
}


//----------------------------------------------------------
void	MainFlow::writeGLUTCode()
{
	g_scene->writeCode();
}

//----------------------------------------------------------
void	MainFlow::enableCropWindow(bool )
{
	
}

//----------------------------------------------------------
void	MainFlow::ExtraFunction(bool status)
{
	warning("extra function %d", status);
//	if(g_camGLPort)
//	{
////		g_camGLPort->SetMouseMode((int)status);
//		g_glWidget->setFixedSize(642, 482);
//	}
}

////----------------------------------------------------------
//void	MainFlow::ExtraFunction(bool status)
//{
//	if(status)
//	{
//		removeToolBar(mainToolBar);
//		addToolBar(mainToolBar, 0, QMainWindow::Right, true);
//	}
//	else
//	{
//		removeToolBar(mainToolBar);
//		addToolBar(mainToolBar, 0, QMainWindow::Top, true);
//	}
//	
//}

////----------------------------------------------------------
//void	MainFlow::ExtraFunction(bool status)
//{
//	warning("extra function %d", status);
//	if (status)
//	{
//		g_glWidget->setFixedSize(512, 384);
////		g_glWidget->setMinimumSize(10, 10);
////		g_glWidget->resize(640, 480);
//	}
//	else
//	{
//		g_glWidget->setMinimumSize(10, 10);
//	}
//}

