/**
 * mainFlow.h
 * Copyright 1999 by Mark B. Allan
 * 
 * main widget w/ menubar, etc.
 */
#include <qapp.h> 
#include <qwidget.h> 
#include <qmenubar.h> 
#include <qpopupmenu.h> 
#include <qlcdnumber.h> 
#include <qfiledialog.h> 
#include <qvalidator.h> 

#include "define.h"
#include "mbaMainWindow.h"
#include "mwidgets.h"
//#include "particleControl.h"
//#include "glPortController.h"

class	LightGroupWidget;		
class	RenderPanelTabDialog;	
class	PrefDialog;				
class	GLWidgetOrtho;			
/**
 * Main viewing window that has the menu bar, perspective view, etc.
 */
//==============================================================================
class MainFlow : public MbaMainWindow
{
	Q_OBJECT
	
public:
	MainFlow (QWidget* parent = 0, const char* name = 0, 
				ToolBarDock buttonBarDock = Top, ToolBarDock infoBarDock = Bottom );
	~MainFlow();
	void	Init();
	QToolBar	*GetToolBar(int toolBarNum);
	
	bool	setToolBar(int num, QMainWindow::ToolBarDock dock, bool newline = true);
	
	GLWidgetOrtho	*getGlOrthoPort(int index)	{	if(index < NUM_ORTHO_PORTS && index >= 0) return orthoWidgets[index]; else return 0; }
	
public slots:
	void	RenderFrame();

protected slots:
	void	DUMMYslot();
	
	void	iconifyAll();
	void 	ShowPreferences();
	void	LoadSceneDialog();
	void	SaveSceneDialog();
	void	SaveThisScene();
	void	NewScene();
	bool	LoadScene(const char *filename);
	bool	SaveScene(const char *filename);
	void	writeGLUTCode();
	void	enableCropWindow(bool);
	
	void	showAboutDialog();
	void	showGlOrtho(int num);
	void	showGlOrtho1();
	void	showGlOrtho2();
	void	showGlOrtho3();
	void	showGlOrtho4();
	
	void	Quit();
	
	void	setRenderSize080();
	void	setRenderSize160();
	void	setRenderSize320();
	void	setRenderSize640();
	void	setRenderSize800();
	void	setRenderSize1024();
	
	void	setRenderRatio4x3();
	void	setRenderRatio16x9();
	
//	void	setRenderSizeFullGL();
//	void	setRenderSizeHalfGL();
	
	void	setSamples(int a);
	void	setSamples11();
	void	setSamples22();
	void	setSamples33();
	void	setSamples44();
		
	void 	ResetFrameCount();
	void 	Clear();
	void 	ExtraFunction(bool);
	//void	ToggleMoveMode(bool on);
	void	setFrameCount();
	void	setMoveSpeed();
	void	setTargetFPS();
	
	void	showAndRaiseParticleControlFrame();
	void	showAndRaiseLightGroupWidgetFrame();
	void	showAndRaiseRenderOptions();
	void	showAndRaiseSurfaceEditWidget();
	void	showAndRaisePolyTweakWidget();
	void	showAndRaiseSceneWidget();
	
protected:
	void	closeEvent ( QCloseEvent *);
	void	setRenderItemsUnchecked();
protected:
	QPopupMenu	*fileMenu;
	QPopupMenu	*editMenu;
	QPopupMenu	*viewMenu;
	QPopupMenu	*rendMenu;
	QPopupMenu	*helpMenu;
	
//	QFileDialog	*saveLoadPSDialog;
//	QFileDialog	*saveLoadSceneDialog;
	QCString	currentSceneFile;
	QCString	currentSceneDir;
	
	LightGroupWidget		*lightGroupWidget;
	RenderPanelTabDialog	*renderOptions;
	PrefDialog				*prefDialog;
	GLWidgetOrtho			*orthoWidgets[NUM_ORTHO_PORTS];
	
	QToolBar	*mainToolBar;
	QToolBar	*lcdToolBar;
	
	MToolButton	*writeRIBBut;
	MToolButton	*clearCamBut;
	MToolButton	*resetFrameBut;
	MToolButton	*animationBut;
	MToolButton	*extraButton;
	MToolButton	*targetLockBut;
	
	int		rendID4x3;
	int		rendID16x9;
	int		rendID080;
	int		rendID160;
	int		rendID320;
	int		rendID640;
	int		rendID800;
	int		rendID1024;
	int		rendIDS11;
	int		rendIDS22;
	int		rendIDS33;
	int		rendIDS44;
//	int		rendIDfullGL;
//	int		rendIDhalfGL;
	int		numViews;
	
	MLineEdit		*frameNumberEdit;
	QLabel			*xMouseLabel;
	QLabel			*yMouseLabel;
	QLabel			*fpsLabel;
	
	QLabel			*moveSpeedLabel;	
	MLineEdit		*moveSpeedEdit;	
	MLineEdit		*targetFPSEdit;	
};
