/**
 * mbaCurve.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * animation curve
 */
#include "mbaCurve.h"

//==============================================================================
KeyFrame::KeyFrame(int fn, float v)
{
	frameNum = fn;
	value = v;
}

//==============================================================================
MBACurve::MBACurve(int ident)
{
	id = ident;
	keyFrameList.setAutoDelete(true);
	keyFrameList.insert(0, new KeyFrame);
	loop = true;
}

MBACurve::~MBACurve()
{
}

//--------------------------------------------------------------------
float	MBACurve::val(int frameNum)
{
	if(keyFrameList.count() > 0)
	{
		if (loop)
			frameNum = frameNum%(int)keyFrameList.count();
			
		curr = keyFrameList.current();
		if (!curr)
			keyFrameList.first();
		
		//Sequential access
		if (curr->frameNum == frameNum)
			return curr->value;
		else if (curr->frameNum < frameNum && curr == keyFrameList.getLast())
			return curr->value;// we're past the last keyframe, so hold value.

		next = keyFrameList.next(); // get next node. 
		keyFrameList.prev();		// No getNext() unfortunately.
		if (next)
		{
			if( frameNum < next->frameNum && frameNum > curr->frameNum )
			{
				return Interpolate(frameNum);
			}
			else if ( next->frameNum == frameNum )
			{
				keyFrameList.next();	
				return next->value;
			}
		}
		
		//Random access - totally weak, but this is a quick implementation
		curr = keyFrameList.first();
		if (!curr)
			return 0.0;
		while(curr->frameNum > frameNum)
		{
			if (curr == keyFrameList.getLast())
				return curr->value;
			curr = keyFrameList.next();
		}		
		if (curr->frameNum == frameNum)
			return curr->value;
		next = keyFrameList.next(); // get next node. 
		keyFrameList.prev();		// No getNext() unfortunately.
		
		if( frameNum < next->frameNum && frameNum > curr->frameNum )
		{
			return Interpolate(frameNum);
		}
		else if ( next->frameNum == frameNum )
		{
			keyFrameList.next();	
			return next->value;
		}

	}
	
	return 0.0;
}

//--------------------------------------------------------------------
float	MBACurve::Interpolate(int frameNum)
{
	// do some interpolation
	float	spread = (float)(next->frameNum - curr->frameNum);
	float	f = ((float)(frameNum - curr->frameNum))/spread;
	float	interp = (1.0-f)*curr->value + f*next->value;
	return interp;
}
