/**
 * mbaCurveAssignDialog.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * finds all things a curve could be attached to
 */
#include "mbaCurveAssignDialog.h"

#include "extern.h"
#include "mbaScene.h"
#include "particleControl.h"
#include "pAction.h"
#include "pDomain.h"
#include "mbaCurveCollection.h"

//==============================================================================
MBACurveAssignDialog::MBACurveAssignDialog(QWidget *parent, const char* name, bool modal)
	: QDialog(parent, name, modal)
{
	lightGroupRoot		= 0;
	cameraRoot			= 0;
	particleControlRoot	= 0;
	
	sceneTree = new QListView(this);
	sceneTree->addColumn("Scene Tree");
	sceneTree->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	sceneTree->setRootIsDecorated(true);
	sceneTree->setSorting(-1, false);
	
	acceptBut = new QPushButton(this);
	acceptBut->setText("&Accept");
	acceptBut->setFixedSize(70, 25);
	acceptBut->setEnabled(false);
	
	cancelBut = new QPushButton(this);
	cancelBut->setText("&Cancel");
	cancelBut->setFixedSize(70, 25);
	
	//-- Connect
	QObject::connect(acceptBut, SIGNAL(clicked()), SLOT(Accepted()) );
	QObject::connect(cancelBut, SIGNAL(clicked()), SLOT(reject()) );
	QObject::connect(sceneTree, SIGNAL(selectionChanged(QListViewItem *)), SLOT(TreeSelectionChanged(QListViewItem *)) );

	//-- Layout
	topVLayout	= new QVBoxLayout(this, 5, 5);
	butLayout	= new QHBoxLayout(5);
	
	topVLayout->addWidget(sceneTree, 1);
	topVLayout->addLayout(butLayout, 0);
		butLayout->addStretch(3);
		butLayout->addWidget(cancelBut,0);
		butLayout->addWidget(acceptBut,0);
	
	topVLayout->activate();
	
	resize(250, 400);
	
	if (g_scene)
		GenerateSceneTree();
	else
		reject();
}

//--------------------------------------------------------------------
MBACurveAssignDialog::~MBACurveAssignDialog()
{
}

//--------------------------------------------------------------------
void	MBACurveAssignDialog::GenerateSceneTree()
{
	int i;
	int ii;
	int iii;
	int iiii;
	MBAListViewFloat	*tmpListItem_i;
	MBAListViewFloat	*tmpListItem_ii;
	MBAListViewFloat	*tmpListItem_iii;
	MBAListViewFloat	*tmpListItem_iiii;
	
	if (g_scene->numLightGroups() > 0)
	{
	
	}
	if (g_scene->numCameras() > 0)
	{
	
	}
	if (g_scene->numParticleSystems() > 0)
	{
		ParticleControl	*partCtrl;
		FlowParticleAction	*pAction;
		
		particleControlRoot = new QListViewItem(sceneTree, "Particle Systems");
		particleControlRoot->setOpen(true);
		for (i = g_scene->numParticleSystems()-1; i >= 0; i--)
		{
			tmpListItem_i = new MBAListViewFloat(particleControlRoot, (partCtrl = g_scene->ParticleSystemAt(i))->Name(), 0 );
			tmpListItem_i->setOpen(true);
			for (ii = partCtrl->numActions()-1; ii >= 0; ii--)
			{
				tmpListItem_ii = new MBAListViewFloat(tmpListItem_i, (pAction = partCtrl->actionList.at(ii))->Name(), 0 );
				for (iii = (int)pAction->NumDomains()-1; iii >= 0; iii--)
				{
					tmpListItem_iii = new MBAListViewFloat(tmpListItem_ii, pAction->DomainName(iii), 0 );
					for (iiii = pAction->Domain(iii)->numFloats()-1; iiii >= 0 ; iiii--)
					{
						tmpListItem_iiii = new MBAListViewFloat(tmpListItem_iii, pAction->Domain(iii)->FullFloatName(iiii), pAction->Domain(iii)->Float(iiii) );
					}
				}
				for (iii = (int)pAction->numFloats()-1; iii >= 0 ; iii--)
				{
					tmpListItem_iii = new MBAListViewFloat(tmpListItem_ii, pAction->FloatName(iii), pAction->Float(iii) );
				}
			}
		}
	}
	
}

//--------------------------------------------------------------------
void	MBACurveAssignDialog::TreeSelectionChanged(QListViewItem *newItem)
{
	if (newItem->childCount() == 0)
		acceptBut->setEnabled(true);
	else
		acceptBut->setEnabled(false);
}

//--------------------------------------------------------------------
void	MBACurveAssignDialog::Accepted()
{
	static int	animation = 1;
	if (sceneTree->currentItem())
		if (sceneTree->currentItem()->childCount() == 0)
		{
			MBAListViewFloat *curFloat = (MBAListViewFloat*)sceneTree->currentItem();
			curFloat->pFloatPointer->SetAnimation( animation );
			animation++;
			if (animation == g_curveControl->NumCurves())
				animation = 1;
			accept();
			return;
		}
	reject();
}

//==============================================================================
MBAListViewFloat::MBAListViewFloat(QListViewItem *parent, const char* name, PFloat *pFp)
	:  QListViewItem(parent, name)
{
	pFloatPointer = pFp;
}

MBAListViewFloat::~MBAListViewFloat()
{
}



