/**
 * mbaGLSurface.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * wraps GL material calls
 */
#include "mbaGLSurface.h"

#include "compatibility.h"
#include <stdlib.h>

//====================================================================
MBA_GLSurface::MBA_GLSurface()
{
	enabled = true;
	
	diff[0] = 0.5;
	diff[1] = 0.5;
	diff[2] = 0.5;
	diff[3] = 1.0;
	
	spec[0] = 0.5;
	spec[1] = 0.5;
	spec[2] = 0.5;
	spec[3] = 1.0;
	
	ambt[0] = 0.0;
	ambt[1] = 0.0;
	ambt[2] = 0.0;
	ambt[3] = 1.0;
	
	emis[0] = 0.0;
	emis[1] = 0.0;
	emis[2] = 0.0;
	emis[3] = 1.0;
	
	rough	= 0.8;
}

//--------------------------------------------------------------------
MBA_GLSurface::MBA_GLSurface(MBA_GLSurface& in)
{
	enabled = in.enabled;
	diff[0] = in.diff[0];
	diff[1] = in.diff[1];
	diff[2] = in.diff[2];
	diff[3] = in.diff[3];

	spec[0] = in.spec[0];
	spec[1] = in.spec[1];
	spec[2] = in.spec[2];
	spec[3] = in.spec[3];

	ambt[0] = in.ambt[0];
	ambt[1] = in.ambt[1];
	ambt[2] = in.ambt[2];
	ambt[3] = in.ambt[3];

	emis[0] = in.emis[0];
	emis[1] = in.emis[1];
	emis[2] = in.emis[2];
	emis[3] = in.emis[3];

	rough	= in.rough;
}

//--------------------------------------------------------------------
MBA_GLSurface::~MBA_GLSurface()
{
}

//--------------------------------------------------------------------
MBA_GLSurface& MBA_GLSurface::operator = (MBA_GLSurface& in)
{
	enabled = in.enabled;
	diff[0] = in.diff[0];
	diff[1] = in.diff[1];
	diff[2] = in.diff[2];
	diff[3] = in.diff[3];

	spec[0] = in.spec[0];
	spec[1] = in.spec[1];
	spec[2] = in.spec[2];
	spec[3] = in.spec[3];

	ambt[0] = in.ambt[0];
	ambt[1] = in.ambt[1];
	ambt[2] = in.ambt[2];
	ambt[3] = in.ambt[3];

	emis[0] = in.emis[0];
	emis[1] = in.emis[1];
	emis[2] = in.emis[2];
	emis[3] = in.emis[3];

	rough	= in.rough;
	return *this;
}	

//--------------------------------------------------------------------
bool	MBA_GLSurface::LoadFromFile(FILE *infile)
{
	char	buffer[128];
	float	tmp0;
	float	tmp1;
	float	tmp2;
	float	tmp3;

	enabled = true;
//FIXME - No Error checking is being done. Do it at some point...
	fgets(buffer, 128, infile);
	sscanf(buffer, "%f %f %f %f", &tmp0, &tmp1, &tmp2, &tmp3);
	SetDiffuse(tmp0, tmp1, tmp2, tmp3);
	
	fgets(buffer, 128, infile);
	sscanf(buffer, "%f %f %f %f", &tmp0, &tmp1, &tmp2, &tmp3);
	SetSpecular(tmp0, tmp1, tmp2, tmp3);
	
	fgets(buffer, 128, infile);
	sscanf(buffer, "%f %f %f %f", &tmp0, &tmp1, &tmp2, &tmp3);
	SetAmbient(tmp0, tmp1, tmp2, tmp3);
	
	fgets(buffer, 128, infile);
	sscanf(buffer, "%f %f %f %f", &tmp0, &tmp1, &tmp2, &tmp3);
	SetEmission(tmp0, tmp1, tmp2, tmp3);
	
	fgets(buffer, 128, infile);
	tmp0 = atof(buffer);
	SetRoughness(tmp0);
	
	return true;
}

//--------------------------------------------------------------------
void	MBA_GLSurface::SaveToFile(FILE *outfile, int tabs)
{
	int		i;
	char	buffer[128];
	
	char	*walker = buffer;
	for (i = 0; i < tabs; i++)
	{	*walker = '\t';	*(walker+1) = '\0';	walker++;	}

	float	*pTmpFloat;
	sprintf(walker, "GL_SURFACE\n");
	fputs(buffer, outfile);
	pTmpFloat = Diffuse();
	sprintf(walker, "\t%f %f %f %f\n", pTmpFloat[0], pTmpFloat[1], pTmpFloat[2], pTmpFloat[3] ); 
	fputs(buffer, outfile);
	pTmpFloat = Specular();
	sprintf(walker, "\t%f %f %f %f\n", pTmpFloat[0], pTmpFloat[1], pTmpFloat[2], pTmpFloat[3] ); 
	fputs(buffer, outfile);
	pTmpFloat = Ambient();
	sprintf(walker, "\t%f %f %f %f\n", pTmpFloat[0], pTmpFloat[1], pTmpFloat[2], pTmpFloat[3] ); 
	fputs(buffer, outfile);
	pTmpFloat = Emission();
	sprintf(walker, "\t%f %f %f %f\n", pTmpFloat[0], pTmpFloat[1], pTmpFloat[2], pTmpFloat[3] ); 
	fputs(buffer, outfile);
	sprintf(walker, "\t%f\n", Roughness() );
	fputs(buffer, outfile);
}




