/**
 * mbaGLSurface.h
 * Copyright 1999 by Mark B. Allan
 * 
 * wraps GL material calls
 */
#ifndef mbaGLSurface_h
#define mbaGLSurface_h

#include <stdio.h>

#include "pFloat.h"

/**
 * 
 */
class MBA_GLSurface
{
public:
	MBA_GLSurface();
	MBA_GLSurface(MBA_GLSurface& in);
	~MBA_GLSurface();
	MBA_GLSurface&	operator = (MBA_GLSurface& in);
	
	bool	LoadFromFile(FILE *infile);
	void	SaveToFile(FILE *outfile, int tabs);
	
	inline void	SetDiffuse (float r, float g, float b)			{   diff[0] = r; diff[1] = g; diff[2] = b; }
	inline void	SetSpecular(float r, float g, float b)			{   spec[0] = r; spec[1] = g; spec[2] = b; }
	inline void	SetAmbient (float r, float g, float b)			{   ambt[0] = r; ambt[1] = g; ambt[2] = b; }
	inline void	SetEmission(float r, float g, float b)			{   emis[0] = r; emis[1] = g; emis[2] = b; }
	inline void	SetDiffuse (float r, float g, float b, float a)	{   diff[0] = r; diff[1] = g; diff[2] = b; diff[3] = a; }
	inline void	SetSpecular(float r, float g, float b, float a)	{   spec[0] = r; spec[1] = g; spec[2] = b; spec[3] = a; }
	inline void	SetAmbient (float r, float g, float b, float a)	{   ambt[0] = r; ambt[1] = g; ambt[2] = b; ambt[3] = a; }
	inline void	SetEmission(float r, float g, float b, float a)	{   emis[0] = r; emis[1] = g; emis[2] = b; emis[3] = a; }
	inline void	SetRoughness(float s)	{ rough = s;	}
	inline void	SetAlpha(float a)		{ diff[3] = a; spec[3] = a; ambt[3] = a; emis[3] = a; }
	
	inline	float	*Diffuse()	{	return diff;	}
	inline	float	*Specular()	{	return spec;	}
	inline	float	*Ambient()	{	return ambt;	}
	inline	float	*Emission()	{	return emis;	}
	inline	float	Roughness()	{	return rough;	}
	inline	int		Shininess()	{	return 255-(int)(rough*255.0);	}
	inline	float	Alpha()		{	return	(diff[3]+spec[3]+ambt[3]+emis[3])/4.0; }

	protected:
	float	diff[4];
	float	spec[4];
	float	ambt[4];
	float	emis[4];
	float	rough;
	bool	enabled;
};


#endif // mbaGLSurface_h




