/**
 * mbaGLSurfaceWidget.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * edit GL material
 */
#include "mbaGLSurfaceWidget.h"

#include "extern.h"
#include "colorButton.h"

//====================================================================
MBA_GLSurfaceWidget::MBA_GLSurfaceWidget(QWidget* parent, const char* name)
	: QFrame(parent, name)
{
	glSurface = 0;
//	setFont(*g_font_MB);
	
	diffLabel = new QLabel(this);
	diffLabel->setText("diffuse");
	diffLabel->setAlignment(AlignCenter);
	specLabel = new QLabel(this);
	specLabel->setText("specular");
	specLabel->setAlignment(AlignCenter);
	ambtLabel = new QLabel(this);
	ambtLabel->setText("ambient");
	ambtLabel->setAlignment(AlignCenter);
	emisLabel = new QLabel(this);
	emisLabel->setText("emission");
	emisLabel->setAlignment(AlignCenter);
	
	diffColor = new ColorButton (this);
	diffColor->setFixedWidth(g_numInputWidth);
	diffColor->setEnabled(true);
	specColor = new ColorButton (this);
	specColor->setFixedWidth(g_numInputWidth);
	specColor->setEnabled(true);
	ambtColor = new ColorButton (this);
	ambtColor->setFixedWidth(g_numInputWidth);
	ambtColor->setEnabled(true);
	emisColor = new ColorButton (this);
	emisColor->setFixedWidth(g_numInputWidth);
	emisColor->setEnabled(true);
	
	diffColor->setColor(1.0, 1.0, 1.0);
	specColor->setColor(1.0, 1.0, 1.0);
	ambtColor->setColor(1.0, 1.0, 1.0);
	emisColor->setColor(1.0, 1.0, 1.0);
	
	alphaInput = new NumInput (NumInput::Double, this);
	alphaInput->SetLabel("alpha");
	alphaInput->SetLabelWidth(g_numLabelWidth);
	alphaInput->SetTotalWidth(g_numInputWidth);
	
	roughInput = new NumInput (NumInput::Double, this);
	roughInput->SetLabel("roughness");
	roughInput->SetLabelWidth(g_numLabelWidth);
	roughInput->SetTotalWidth(g_numInputWidth);
	
	//----------------------------------
	QObject::connect(diffColor,	SIGNAL(RGBChanged(float, float, float)), SLOT(UpdateDiff(float, float, float)) );
	QObject::connect(specColor,	SIGNAL(RGBChanged(float, float, float)), SLOT(UpdateSpec(float, float, float)) );
	QObject::connect(ambtColor,	SIGNAL(RGBChanged(float, float, float)), SLOT(UpdateAmbt(float, float, float)) );
	QObject::connect(emisColor,	SIGNAL(RGBChanged(float, float, float)), SLOT(UpdateEmis(float, float, float)) );
	QObject::connect(roughInput,SIGNAL(ValueChanged(float)), SLOT(UpdateRough(float)) );
	QObject::connect(alphaInput,SIGNAL(ValueChanged(float)), SLOT(UpdateAlpha(float)) );
//	QObject::connect(compCombo,	SIGNAL(activated(int)), SLOT(SwitchComponent(int)) );

	//----------------------------------
	topHLayout = new QHBoxLayout(this, 10, 5, "topHLayout");
	col0Layout = new QVBoxLayout(0, "col0Layout");
	col1Layout = new QVBoxLayout(0, "col1Layout");
	
	topHLayout->addLayout(col0Layout, 0);
		col0Layout->addWidget(diffLabel,  0);
		col0Layout->addWidget(diffColor,  0);
		col0Layout->addSpacing(5);
		col0Layout->addWidget(ambtLabel,  0);
		col0Layout->addWidget(ambtColor,  0);
		col0Layout->addSpacing(5);
		col0Layout->addStretch(1);
		col0Layout->addWidget(roughInput, 0);
	topHLayout->addStretch(1);
	topHLayout->addLayout(col1Layout, 0);
		col1Layout->addWidget(specLabel,  0);
		col1Layout->addWidget(specColor,  0);
		col1Layout->addSpacing(5);
		col1Layout->addWidget(emisLabel,  0);
		col1Layout->addWidget(emisColor,  0);
		col1Layout->addStretch(1);
		col1Layout->addWidget(alphaInput, 0);
	topHLayout->addStretch(4);
	
	topHLayout->activate();
	
	SetGLSurfacePointer(0);
	
}

//--------------------------------------------------------------------
MBA_GLSurfaceWidget::~MBA_GLSurfaceWidget()
{
}

//--------------------------------------------------------------------
void MBA_GLSurfaceWidget::UpdateRough(float in)
{
	if (glSurface)
	{
		glSurface->SetRoughness(in);
	}
	emit Updated();
}

//--------------------------------------------------------------------
void MBA_GLSurfaceWidget::UpdateAlpha(float in)
{
	if (glSurface)
	{
		glSurface->SetAlpha(in);
	}
	emit Updated();
}

//--------------------------------------------------------------------
void MBA_GLSurfaceWidget::UpdateDiff(float r, float g, float b)
{
	if (glSurface)
	{
		glSurface->SetDiffuse(r, g, b);
	}
	emit Updated();
}

//--------------------------------------------------------------------
void MBA_GLSurfaceWidget::UpdateSpec(float r, float g, float b)
{
	if (glSurface)
	{
		glSurface->SetSpecular(r, g, b);
	}
	emit Updated();
}

//--------------------------------------------------------------------
void MBA_GLSurfaceWidget::UpdateAmbt(float r, float g, float b)
{
	if (glSurface)
	{
		glSurface->SetAmbient(r, g, b);
	}
	emit Updated();
}

//--------------------------------------------------------------------
void MBA_GLSurfaceWidget::UpdateEmis(float r, float g, float b)
{
	if (glSurface)
	{
		glSurface->SetEmission(r, g, b);
	}
	emit Updated();
}

//--------------------------------------------------------------------
void	MBA_GLSurfaceWidget::SetGLSurfacePointer(MBA_GLSurface *inP)
{
	glSurface = inP;
	if (glSurface)
	{
		//if (!isEnabled())
			setEnabled(true);
		diffColor->setColor(glSurface->Diffuse());
		specColor->setColor(glSurface->Specular());
		ambtColor->setColor(glSurface->Ambient());
		emisColor->setColor(glSurface->Emission());
		roughInput->SetValueSolid(glSurface->Roughness());
		alphaInput->SetValueSolid(glSurface->Alpha());
	}
	else
	{
//		if (isEnabled())
			setEnabled(false);
	}
}

//--------------------------------------------------------------------
void	MBA_GLSurfaceWidget::Copy()
{
	if(glSurface)
	{
		copySurface = *glSurface;
	}
}

//--------------------------------------------------------------------
void	MBA_GLSurfaceWidget::Paste()
{
	if(glSurface)
	{
		*glSurface = copySurface;
		SetGLSurfacePointer(glSurface);
	}
	emit Updated();
}

//--------------------------------------------------------------------
void	MBA_GLSurfaceWidget::setEnabled(bool enable)
{
	if (enable && glSurface)
	{
		diffColor->setEnabled(true);
		specColor->setEnabled(true);
		ambtColor->setEnabled(true);
		emisColor->setEnabled(true);
		roughInput->setEnabled(true);
		alphaInput->setEnabled(true);
	}
	else
	{
		diffColor->setEnabled(false);
		specColor->setEnabled(false);
		ambtColor->setEnabled(false);
		emisColor->setEnabled(false);
		roughInput->setEnabled(false);
		alphaInput->setEnabled(false);
	}
}
		

