/**
 * mbaLight.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * light class for GL and RM 
 */
#include "mbaLight.h"
#include <ri.h>
#include <qobject.h>
#include "glPrimitives.h"

#include "mmath.h"

#include "extern.h"
#include "define.h"

/**
 * Constructor. If infile exists, read light from file.
 */
//====================================================================
MBALight::MBALight(FILE* infile)
{
	glLightNum	= (GLenum)-1;
	lightName.sprintf("light");
	from[0]		= 1.0;
	from[1]		= 1.0;
	from[2]		= 1.0;
	from[3]		= 1.0;
	to[0]		= 0.0;
	to[1]		= 0.0;
	to[2]		= 0.0;
	to[3]		= 1.0;
	angle		= 45.0;
	delta		= 10.0;
	distrib		= 1.0;
	color[0]	= 1.0;
	color[1]	= 1.0;
	color[2]	= 1.0;
	color[3]	= 1.0;
	intensity	= 1.0;
	glColor[0]	= 1.0;
	glColor[1]	= 1.0;
	glColor[2]	= 1.0;
	glColor[3]	= 1.0;
	castsShadows = false;
	
	drawGLFlag = true;
	
	attenConstant	= 0.0;
	attenLinear		= 0.05;
	attenQuadratic	= 0.09;
	lightType = Local;
	
	if (infile)
		ReadLight(infile);      
	
	UpdateVariables();
}

//--------------------------------------------------------------------
MBALight::~MBALight()
{
	if((int)glLightNum != -1)
		glDisable(glLightNum);
}

/**
 * copy constructor
 */
//--------------------------------------------------------------------
MBALight::MBALight(MBALight &in)
{
	glLightNum	= in.glLightNum;
	lightType	= in.lightType;
	lightName	= in.lightName;
	from[0]		= in.from[0];
	from[1]		= in.from[1];
	from[2]		= in.from[2];
	from[3]		= in.from[3];
	to[0]		= in.to[0];
	to[1]		= in.to[1];
	to[2]		= in.to[2];
	angle		= in.angle;
	delta		= in.delta;
	distrib		= in.distrib;
	color[0]	= in.color[0];
	color[1]	= in.color[1];
	color[2]	= in.color[2];
	intensity	= in.intensity;
	glColor[0]	= in.glColor[0];
	glColor[1]	= in.glColor[1];
	glColor[2]	= in.glColor[2];
	glColor[3]	= in.glColor[3];
	enabled		= in.enabled;
	attenConstant	= in.attenConstant;
	attenLinear		= in.attenLinear;
	attenQuadratic	= in.attenQuadratic;
	castsShadows	= in.castsShadows;
	drawGLFlag		= in.drawGLFlag;
	
	UpdateVariables();
}

//--------------------------------------------------------------------
MBALight&	MBALight::operator = (MBALight& in)
{
	glLightNum	= in.glLightNum;
	lightType	= in.lightType;
	lightName	= in.lightName;
	from[0]		= in.from[0];
	from[1]		= in.from[1];
	from[2]		= in.from[2];
	from[3]		= in.from[3];
	to[0]		= in.to[0];
	to[1]		= in.to[1];
	to[2]		= in.to[2];
	angle		= in.angle;
	delta		= in.delta;
	distrib		= in.distrib;
	color[0]	= in.color[0];
	color[1]	= in.color[1];
	color[2]	= in.color[2];
	intensity	= in.intensity;
	glColor[0]	= in.glColor[0];
	glColor[1]	= in.glColor[1];
	glColor[2]	= in.glColor[2];
	glColor[3]	= in.glColor[3];
	enabled		= in.enabled;
	attenConstant	= in.attenConstant;
	attenLinear		= in.attenLinear;
	attenQuadratic	= in.attenQuadratic;
	castsShadows	= in.castsShadows;
	drawGLFlag		= in.drawGLFlag;
	
	UpdateVariables();
	
	return *this;
}

//--------------------------------------------------------------------
GLenum	MBALight::setGlLightNum(int in) 
{
	updated_v = true;
	bool	setup = false;
	if((int)glLightNum == -1 && in != -1)
		setup = true;
	switch(in)
	{
		case 0:
			glLightNum = GL_LIGHT0;
			break;
		case 1:
			glLightNum = GL_LIGHT1;
			break;
		case 2:
			glLightNum = GL_LIGHT2;
			break;
		case 3:
			glLightNum = GL_LIGHT3;
			break;
		case 4:
			glLightNum = GL_LIGHT4;
			break;
		case 5:
			glLightNum = GL_LIGHT5;
			break;
		case 6:
			glLightNum = GL_LIGHT6;
			break;
		case 7:
			glLightNum = GL_LIGHT7;
			break;
	}
	setAttenuation();
	setLightType(lightType);
	
	return glLightNum;
}

//--------------------------------------------------------------------
void	MBALight::SetName(const char* newName) 
{
	updated_v = true;
	lightName = newName;
}

//--------------------------------------------------------------------
void	MBALight::setColor(float r, float g, float b) 
{	
	updated_v = true;
	color[0] = r;	color[1] = g;	color[2] = b;
	glColor[0] = color[0]*intensity;
	glColor[1] = color[1]*intensity;
	glColor[2] = color[2]*intensity;
	glColor[3] = 1.0;
}

//--------------------------------------------------------------------
void	MBALight::setIntensity(float in) 
{	
	updated_v = true;
	intensity = in;
	glColor[0] = color[0]*intensity;
	glColor[1] = color[1]*intensity;
	glColor[2] = color[2]*intensity;
	glColor[3] = 1.0;
}
	
//--------------------------------------------------------------------
void	MBALight::setAttenuation(float c, float l, float q)
{
	updated_v = true;
	if (c >= 0.0)
		attenConstant = c;
	if (l >= 0.0)
		attenLinear = l;
	if (q >= 0.0)
		attenQuadratic = q;
	setAttenuation();
}
	
//--------------------------------------------------------------------
void	MBALight::setAttenuation()
{
	if((int)glLightNum != -1)
	{
		glLightf (glLightNum, GL_CONSTANT_ATTENUATION,	attenConstant);        	
		glLightf (glLightNum, GL_LINEAR_ATTENUATION,	attenLinear);        	
		glLightf (glLightNum, GL_QUADRATIC_ATTENUATION,	attenQuadratic);
	}
}
	
//--------------------------------------------------------------------
void	MBALight::setEnabled(bool status, bool onlyFlag)
{
	enabled = status;
	if( !onlyFlag && (int)glLightNum != -1 )
	{
		updated_v = true;
		if (enabled)
			glEnable(glLightNum);
		else
			glDisable(glLightNum);
	}
}

//--------------------------------------------------------------------
void	MBALight::UpdateVariables()
{
	glInfinite[0] = from[0]-to[0];
	glInfinite[1] = from[1]-to[1];
	glInfinite[2] = from[2]-to[2];
	glInfinite[3] = 0.0;
	
	glSpot[0] = -glInfinite[0];
	glSpot[1] = -glInfinite[1];
	glSpot[2] = -glInfinite[2];
	
	setLightType(lightType);
}

//--------------------------------------------------------------------
void	MBALight::illuminateGL(int frame)
{
	if((int)glLightNum != -1)
	{
		frame = frame;
		glLightfv (glLightNum, GL_AMBIENT,	glColor);
		glLightfv (glLightNum, GL_DIFFUSE,	glColor);
		glLightfv (glLightNum, GL_SPECULAR,	glColor);
		switch(lightType)
		{
			case Ambient:
				glLightfv (glLightNum, GL_POSITION,	from);
				break;
			case Infinite:
				UpdateVariables();
				glLightfv (glLightNum, GL_POSITION,	glInfinite);
				break;
			case Local:
				glLightfv (glLightNum, GL_POSITION,	from);
				break;
			case Spot:
				UpdateVariables();
				glLightfv (glLightNum, GL_POSITION,	from);
				glLightf  (glLightNum, GL_SPOT_CUTOFF,	angle);
				glLightf  (glLightNum, GL_SPOT_EXPONENT, distrib);
				glLightfv (glLightNum, GL_SPOT_DIRECTION, glSpot);
				break;
		}
	}		
}

//--------------------------------------------------------------------
void	MBALight::drawGL(int )
{
	float	size = intensity*0.1;
	if(enabled && drawGLFlag)
	{
		if (GL__lighting)
			glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,	color);
		else
			glColor3fv(color);
		switch(lightType)
		{
			case Ambient:
				break;
			case Infinite:
				glPushMatrix();
				glTranslatef (from[0], from[1], from[2]);
				RglSphere(size, -size, size, 360.0, 4);
				glPopMatrix();
				glBegin(GL_LINES);
				glVertex3fv(from);
				glVertex3fv(to);
				glEnd();
				break;
			case Local:	
				glPushMatrix();
				glTranslatef (from[0], from[1], from[2]);
				RglSphere(size, -size, size, 360.0, 6);
				glPopMatrix();
				break;
			case Spot:
				MVertex	d(to[0]-from[0], to[1]-from[1], to[2]-from[2]);
				MMatrix	mat;
				mat.pointAt(d);
				mat.setTranslate(from[0], from[1], from[2]);
				glPushMatrix();
					glMultMatrixf(mat.mat());
					glTranslatef(0.0f, 0.0f, 1.0f);
					RglCone(-1.0, sin(D2RAD(angle)), -360.0, 8);
				glPopMatrix();
//				d = d.normalize();
//				glBegin(GL_LINES);
//				glVertex3fv(from);
//				glVertex3f(d.X()+from[0], d.Y()+from[1], d.Z()+from[2]);
//				glEnd();
				break;
		}
	}
	
}

//--------------------------------------------------------------------
void	MBALight::illuminateRIB(int frame)
{
	frame = frame;
	float	intensityRibPoint = intensity*13.0;	// arbitrary value that works pretty well
	float	angle_rm = (angle*2*M_PI)/360.0;
	float	delta_rm = (delta*2*M_PI)/360.0;
	
	char	*on		= "on";
	char	*off	= "off";
	if (g_allowShadows)
	{
		if (castsShadows)
			RiAttribute ("light", "shadows", &on, RI_NULL);
		else
			RiAttribute ("light", "shadows", &off, RI_NULL);
	}
	if (enabled)
	{
		from[2]	= -from[2];	//flip Z for compatibility
		to[2]	= -to[2]; 
		switch(lightType)
		{
			case Ambient:
				break;
			case Infinite:
				RiLightSource(	"distantlight",	
								"intensity", &intensity,
								"lightcolor", color,
								"from", from, 
								"to", to, 
								RI_NULL);
				break;
			case Local:
				RiLightSource(	"pointlight",	
								"intensity", &intensityRibPoint,
								"lightcolor", color,
								"from", from, 
								RI_NULL);
				break;
			case Spot:
				RiLightSource(	"spotlight", 
								"intensity", &intensityRibPoint, 
								"lightcolor", color,
								"from", from, 
								"to", to, 
								"coneangle", &angle_rm, 
								"conedeltaangle", &delta_rm, 
								"beamdistribution", &distrib,
								RI_NULL);
				break;
		}
		from[2]	= -from[2];	//flip it back
		to[2]	= -to[2]; 
	}
}

//--------------------------------------------------------------------
void	MBALight::drawRIB(int frame)
{
	frame = frame;
	float	size = intensity*0.1;
	if(enabled)
	{
		// How to intelligently put a shader here?
		RiTransformBegin();
			RiTranslate (from[0], from[1], -from[2]);
			RiSphere(size, -size, size, 360.0, RI_NULL);
		RiTransformEnd();
	}
}

//--------------------------------------------------------------------
void	MBALight::setLightType(LightTypeEnum type)
{

	updated_v = true;
	lightType = type;
	if((int)glLightNum != -1)
	{
		switch(lightType)
		{
			case Ambient:
				glLightf(glLightNum, GL_SPOT_CUTOFF, 180.0);
				break;
			case Infinite:
				setAttenuation (1.00, 0.00, 0.00);
				glLightf(glLightNum, GL_SPOT_CUTOFF, 180.0);
				from[3] = 0.0;
				break;
			case Local:	
				setAttenuation (0.00, 0.05, 0.09);
				glLightf(glLightNum, GL_SPOT_CUTOFF, 180.0);
				from[3] = 1.0;
				break;
			case Spot:	
				setAttenuation (0.00, 0.05, 0.10);
				glLightf(glLightNum, GL_SPOT_CUTOFF, angle);
				from[3] = 1.0;
				break;
		}
	}
}

//--------------------------------------------------------------------
bool	MBALight::WriteLight(FILE* outfile, int tabs)
{
	int		i;
	char	buffer[256];
	char	*walker = buffer;
	
	for (i = 0; i < tabs; i++)
	{	*walker = '\t';	*(walker+1) = '\0';	walker++;	}
	
	sprintf (walker, "Light\n");
	fputs(buffer, outfile);
	sprintf (walker, "{\n");
	fputs(buffer, outfile);
	sprintf (walker, "\t%s\n", (const char*)lightName);
	fputs(buffer, outfile);
	sprintf (walker, "\t%d\n", (int)lightType);
	fputs(buffer, outfile);
	sprintf (walker, "\t%f\n", from[0]);
	fputs(buffer, outfile);
	sprintf (walker, "\t%f\n", from[1]);
	fputs(buffer, outfile);
	sprintf (walker, "\t%f\n", from[2]);
	fputs(buffer, outfile);
	sprintf (walker, "\t%f\n", to[0]);
	fputs(buffer, outfile);
	sprintf (walker, "\t%f\n", to[1]);
	fputs(buffer, outfile);
	sprintf (walker, "\t%f\n", to[2]);
	fputs(buffer, outfile);
	sprintf (walker, "\t%f\n", angle);
	fputs(buffer, outfile);
	sprintf (walker, "\t%f\n", delta);
	fputs(buffer, outfile);
	sprintf (walker, "\t%f\n", distrib);
	fputs(buffer, outfile);
	sprintf (walker, "\t%f\n", color[0]);
	fputs(buffer, outfile);
	sprintf (walker, "\t%f\n", color[1]);
	fputs(buffer, outfile);
	sprintf (walker, "\t%f\n", color[2]);
	fputs(buffer, outfile);
	sprintf (walker, "\t%f\n", intensity);
	fputs(buffer, outfile);
	sprintf (walker, "\t%d\n", (int)enabled );
	fputs(buffer, outfile);
	sprintf (walker, "\t%d\n", (int)castsShadows );
	fputs(buffer, outfile);
	sprintf (walker, "\t%d\n", (int)drawGLFlag );
	fputs(buffer, outfile);
	sprintf (walker, "}\n");
	fputs(buffer, outfile);

	return true;
}

//--------------------------------------------------------------------
bool	MBALight::ReadLight(FILE* infile)
{
	updated_v = true;
	char	buffer[256];
	QCString	tmpString;
	int		tmpi;
	float	tmpf[4];
	int		count = -1;
	char	tmpBuffer[128];
	
	GL_ERROR_MSG("MBALight::ReadLight -- enter");
	while(fgets(buffer, 256, infile))
	{
		tmpString = buffer; tmpString = tmpString.simplifyWhiteSpace();
		if(tmpString == "{" && count == -1)
		{
			count = 0;
			continue;
		}
		if(tmpString == "{" && count != -1)
			return false;
		if(tmpString == "}")
			break;
		
		switch(count)
		{
			case 0:
				//--Light name
				lightName = (const char*)tmpString;
				break;
			case 1:
				//--Light type
				tmpi = tmpString.toInt();
				setLightType((LightTypeEnum)tmpi);
				break;
			case 2:
				//--From x, y, z
				tmpf[0] = tmpString.toFloat();
				break;
			case 3:
				tmpf[1] = tmpString.toFloat();
				break;
			case 4:
				tmpf[2] = tmpString.toFloat();
				setFrom( tmpf[0], tmpf[1], tmpf[2] );
				break;
			case 5:
				//--To x, y, z
				tmpf[0] = tmpString.toFloat();
				break;
			case 6:
				tmpf[1] = tmpString.toFloat();
				break;
			case 7:
				tmpf[2] = tmpString.toFloat();
				setTo ( tmpf[0], tmpf[1], tmpf[2] );
				break;
			case 8:
				//-- Spot parameters
				angle = tmpString.toFloat();
				break;
			case 9:
				delta = tmpString.toFloat();
				break;
			case 10:
				distrib = tmpString.toFloat();
				break;
			case 11:
				//-- color
				tmpf[0] = tmpString.toFloat();
				break;
			case 12:
				tmpf[1] = tmpString.toFloat();
				break;
			case 13:
				tmpf[2] = tmpString.toFloat();
				break;
			case 14:
				tmpf[3] = tmpString.toFloat();
				setIntensity( tmpf[3] );
				setColor( tmpf[0], tmpf[1], tmpf[2] );
				break;
			case 15:
				//-- enabled
				tmpi = tmpString.toInt();
				setEnabled((bool)tmpi);
				break;
			case 16:
				//-- casts shadows
				castsShadows = (bool)tmpString.toInt();
				break;
			case 17:
				//-- draw light
				drawGLFlag = (bool)tmpString.toInt();
				break;
		}
		sprintf(tmpBuffer, "    MBALight::ReadLight count = %d", count);
		GL_ERROR_MSG(tmpBuffer);
		count++;
	}
	GL_ERROR_MSG("MBALight::ReadLight -- leave");
	return true;
}

