/**
 * mbaLight.h
 * Copyright 1999 by Mark B. Allan
 * 
 * light class for GL and RM 
 */
#ifndef mbaLight_h
#define mbaLight_h

#include "compatibility.h"
#include <GL/gl.h>
#include <stdio.h>

#include <qstring.h>

/**
 * Light. Point, directional, and spot. Light class for GL and RM 
 */
//==============================================================================
class MBALight
{
public:
	MBALight( FILE* infile = 0); 
		//FIXME - move the GL light num thing to the lightGroup class... 
	MBALight( MBALight &in);
	~MBALight();
	MBALight&	operator = (MBALight& in);
	

	enum LightTypeEnum
	{
		Ambient, Infinite, Local, Spot
	};
	
	void	setEnabled(bool status, bool onlyFlag = false);
	inline	bool	isEnabled()		{	return enabled;	}
	
	inline	void	setUpdated(bool status)	{	updated_v = status;	}
	inline	bool	updated()				{	return updated_v;	}
	
	void	illuminateGL(int frame);
	void	drawGL(int frame);
	void	illuminateRIB(int frame);
	void	drawRIB(int frame);
	inline	bool	SetDrawLight(bool in)	{	return drawGLFlag = in;	}
	inline	bool	DrawLight()				{	return drawGLFlag;	}

	void	setLightType(LightTypeEnum type);
	inline	LightTypeEnum	LightType()	{	return lightType;	}

	inline	const char*	Name()		{	return (const char*)lightName;	}	
	void	SetName(const char* newName);

	void	setColor(float r, float g, float b);
	void	setIntensity(float in);
	
	inline	void	setFrom  (float x, float y, float z)	{	from[0] = x;	from[1] = y;	from[2] = z;	updated_v = true;	}
	inline	void	setFromX (float in)			{	from[0] = in;	updated_v = true;	}
	inline	void	setFromY (float in)			{	from[1] = in;	updated_v = true;	}	
	inline	void	setFromZ (float in)			{	from[2] = in;	updated_v = true;	}	
	inline	void	setTo	(float x, float y, float z)	{	to[0] = x;	to[1] = y;	to[2] = z;	updated_v = true;	}
	inline	void	setToX	(float in)			{	to[0] = in;	updated_v = true;		}
	inline	void	setToY	(float in)			{	to[1] = in;	updated_v = true;		}	
	inline	void	setToZ	(float in)			{	to[2] = in;	updated_v = true;		}	
	inline	void	setAngle	(float in)		{	angle = in;			updated_v = true;	}	
	inline	void	setDelta	(float in)		{	delta = in;			updated_v = true;	}	
	inline	void	setDistrib	(float in)		{	distrib = in;		updated_v = true;	}	
	inline	void	setCastsShadows (bool in)	{	castsShadows = in;	updated_v = true;	}
	inline	void	setDrawGL (bool in)			{	drawGLFlag = in;	updated_v = true;	}
	
	GLenum	setGlLightNum(int in);
	inline	GLenum	getGlLightNum()		{	return glLightNum;	}
	inline	float	getIntensity()		{	return intensity;	}
	inline	bool	getCastsShadows()	{	return castsShadows;}
	inline	bool	drawGLEnabled()		{	return drawGLFlag;	}
	
	inline	float	getColorR()		{	return color[0];	}
	inline	float	getColorG()		{	return color[1];	}
	inline	float	getColorB()		{	return color[2];	}
	
	inline	const float	*getFrom()		{	return from;	}
	inline	float	getFromX()			{	return from[0];	}
	inline	float	getFromY()			{	return from[1];	}
	inline	float	getFromZ()			{	return from[2];	}
	inline	const float	*getTo()		{	return to;		}
	inline	float	getToX()			{	return to[0];	}
	inline	float	getToY()			{	return to[1];	}
	inline	float	getToZ()			{	return to[2];	}
	inline	float	getAngle()			{	return angle;	}
	inline	float	getDelta()			{	return delta;	}
	inline	float	getDistrib()		{	return distrib;	}

	void	setAttenuation(float c, float l, float q);
	void	setAttenuation();
	
	bool	ReadLight(FILE* infile);
	bool	WriteLight(FILE* outfile, int tabs = 0);
	
	void	UpdateVariables();
	
protected:
	LightTypeEnum	lightType;
	QCString	lightName;
	float	from[4];
	float	to[4];
	float	angle;
	float	delta;
	float	distrib;
	float	color[4];
	float	intensity;
	float	glColor[4];
	bool	enabled;
	bool	updated_v;
	float	attenConstant;
	float	attenLinear;
	float	attenQuadratic;
	bool	castsShadows;
	bool	drawGLFlag;
	
	float	glInfinite[4];
	float	glSpot[4];
	
	GLenum	glLightNum;
	
};

#endif
