/**
 * mbaRMShaderStringWidget.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * popup widget for editing string values of RM shaders
 */
#include "mbaRMShaderStringWidget.h"

#include <qpixmap.h>
#include <qtooltip.h>
#include <qstringlist.h>

#include "extern.h"

//==============================================================================
MbaRMShaderStringWidget::MbaRMShaderStringWidget(const char *strVal, const char* dir, QWidget *parent, const char* name)
	: QDialog(parent, name, true)
{
	setCaption("flow::shader string edit");
	setIcon(QPixmap((const char**)flowIcon_xpm));
	lastDir = dir;
	
	setFont(*g_font_MB);
	
	stringLabel = new QLabel(this);
	stringLabel->setText("string value:");
	stringLabel->setAlignment( AlignBottom | AlignLeft );
	
	stringEdit = new QLineEdit(this);
	if (strVal)
		stringEdit->setText(strVal);
	stringEdit->setFont(*g_font_M);
	
	butOk = new QPushButton(this);
	butOk->setText("Ok");
	
	butCancel = new QPushButton(this);
	butCancel->setText("Cancel");
	
	showFile = new QPushButton(this);
	showFile->setPixmap( *pix_file );
	int	tmpHeight = butCancel->sizeHint().height();
	showFile->setFixedSize(tmpHeight, tmpHeight);
	
	QStringList	filterList;
	filterList.append("TIFF files (*.tif)");
	//filterList.append("JPEG files (*.jpg)");
	filterList.append("All files (*)");
	
	fileDialog = new ImageFileDialog(this, 0, true);
	fileDialog->resize(fileDialog->width(), (4*g_dskTopH)/5);
	fileDialog->setDir(QDir(dir));
//	warning("dir = %s", dir);
	//fileDialog->setMode(QFileDialog::ExistingFile);
	fileDialog->setFilters(filterList);
	if (QFile::exists(strVal))
		fileDialog->setSelection(strVal);
	fileDialog->hide();
	
	butOk->setDefault(true);
	
	if(g_toolTipsEnabled)
	{
		QToolTip::add(showFile,	"browse for texture file");
	}
	
	
	//-- Connect -----------------------
	QObject::connect(fileDialog, SIGNAL(dirEntered(const QString&)), stringEdit, SLOT(setText(const QString&)) );
	QObject::connect(fileDialog, SIGNAL(fileSelected(const QString&)), stringEdit, SLOT(setText(const QString&)) );
	QObject::connect(showFile, SIGNAL(clicked()), SLOT(ShowFileDialog()) );
	
	QObject::connect(butCancel, SIGNAL(clicked()), SLOT(reject()) );
	QObject::connect(butOk, SIGNAL(clicked()), SLOT(accept()) );
	
	//-- Layout ------------------------
	topVLayout = new QVBoxLayout(this, 5, 5);
	butHLayout = new QHBoxLayout(5);
	
	topVLayout->addStretch(2);
	topVLayout->addWidget(stringLabel, 0);
	topVLayout->addWidget(stringEdit, 0);
	topVLayout->addLayout(butHLayout, 0);
		butHLayout->addWidget(showFile, 0);
		butHLayout->addStretch(1);
		butHLayout->addWidget(butCancel, 0);
		butHLayout->addWidget(butOk, 0);
	topVLayout->addStretch(1);
//	topVLayout->addSpacing(10);
//	topVLayout->addWidget(fileFrame, 1);
	
	topVLayout->activate();
	
	if (parent)
		resize(parent->width()+20, 250);
	
}

//--------------------------------------------------------------------
MbaRMShaderStringWidget::~MbaRMShaderStringWidget()
{
}

//--------------------------------------------------------------------
void MbaRMShaderStringWidget::ShowFileDialog()
{
	if (QFile::exists(stringEdit->text()) )
		fileDialog->setSelection(stringEdit->text());
	if (fileDialog->exec() == Accepted)
	{
		lastDir = fileDialog->dir()->path();
		accept();
	}
}

