/**
 * mbaRMShaderWidget.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * widget to edit RM surface and displacement shaders
 */
#include "mbaRMShaderWidget.h"

#ifndef WIN32
	#include <unistd.h>
	#include <sys/types.h>
	#include <sys/wait.h>
#endif

#include <qdir.h> 
#include <qfile.h> 
#include <qjpegio.h>
#include <qtooltip.h>
#include <qstringlist.h>
#include <qfiledialog.h>
#include <qmessagebox.h>

#include "extern.h"
#include "mbaRMShaderStringWidget.h"
#include "colorButton.h"

//====================================================================
MBA_RMShaderWidget::MBA_RMShaderWidget(MBA_RMShader::ShaderTypeEnum st, QWidget* parent, const char* name )
	: QFrame(parent, name)
{
	shader			= 0;
	shaderType		= st;
	numVariations	= 0;
	shader 			= 0;
	ribFileInt		= 0;
	lastTextureDir	= ".";
	visSetOK		= true;
		
//	setFont(*g_font_MB);
	
	for (int i = 0; i < NUM_SHADER_PROCS; i++)
		pid_shader_wait[i] = 0;
		
	shaderList.setAutoDelete(true);
	previewNameList.setAutoDelete(true);
	previewFilenameList.setAutoDelete(true);
	
	switch (shaderType)
	{
		case MBA_RMShader::Surface:
			variationsDir = g_shaderSurfDir;
			variationsDir+= "/Variations/";
			break;
		case MBA_RMShader::Displacement:
			variationsDir = g_shaderDispDir;
			variationsDir+= "/Variations/";
			break;
		case MBA_RMShader::Undefined:
			warning ("MBA_RMShaderWidget::MBA_RMShaderWidget - Yer hosed.");
			break;
	}
			
	availableList = new QListBox(this);
	availableList->setMinimumWidth(g_numInputWidth);
	availableList->setFont(*g_font_M);

	qInitJpegIO();	// start up Qt JPEG stuff
	
	previewList = new ShaderPreview (g_numPreviewCols, this);
	
	saveButton = new MToolButton(this);
	saveButton->setText("&save");
	saveButton->setAutoRaise(true);
	updtButton = new MToolButton(this);
	updtButton->setText("&update");
	updtButton->setAutoRaise(true);
	testButton = new MToolButton(this);
	testButton->setText("&test");
	testButton->setAutoRaise(true);
	
	saveButton->setEnabled(false);
	updtButton->setEnabled(false);
	
	switch(shaderType)
	{
		case MBA_RMShader::Surface:
			shadowShadeLabel = new QLabel(this);
			shadowShadeLabel->setText("shadow value");
			shadowShadeLabel->setAlignment( AlignBottom | AlignHCenter );
			shadowShadeLabel->setFont(*g_font_S);
			shadowShadeCombo = new QComboBox(this);
			shadowShadeCombo->setFixedHeight(g_comboHeight);
			shadowShadeCombo->insertItem(" none  ");
			shadowShadeCombo->insertItem(" opaque  ");
			shadowShadeCombo->insertItem(" Os  ");
			shadowShadeCombo->insertItem(" shade  ");
			visibilityLabel = new QLabel(this);
			visibilityLabel->setText("visibility");
			visibilityLabel->setAlignment( AlignBottom | AlignHCenter );
			visibilityLabel->setFont(*g_font_S);
			visCamBut = new MToolButton(this);
			visCamBut->setText("cam");
			visCamBut->setToggleButton(true);
			visCamBut->setAutoRaise(true);
			visReflBut = new MToolButton(this);
			visReflBut->setText("refl");
			visReflBut->setToggleButton(true);
			visReflBut->setAutoRaise(true);
			visShadBut = new MToolButton(this);
			visShadBut->setText("shad");
			visShadBut->setToggleButton(true);
			visShadBut->setAutoRaise(true);
			break;
		case MBA_RMShader::Displacement:
			trueDisplaceCheck = new QCheckBox(this);
			trueDisplaceCheck->setText("true displace");
			displaceBoundInput = new NumInput(NumInput::Double, this);
			displaceBoundInput->SetLabel("bound");
			displaceBoundInput->SetLabelWidth(g_numLabelWidth);
			displaceCoordCombo = new QComboBox(this);
			displaceCoordCombo->setFixedHeight(g_comboHeight);
			displaceCoordCombo->insertItem("  camera  ");
			displaceCoordCombo->insertItem("  current  ");
			displaceCoordCombo->insertItem("  null  ");
			displaceCoordCombo->insertItem("  object  ");
			displaceCoordCombo->insertItem("  screen  ");
			displaceCoordCombo->insertItem("  shader  ");
			displaceCoordCombo->insertItem("  world  ");
			break;
		case MBA_RMShader::Undefined:
			break;
	}
	
	floatCombo = new QComboBox(this);
	floatCombo->setFixedHeight(g_comboHeight);
	floatCombo->setFixedWidth(g_numInputWidth);
	floatCombo->insertItem("floatCombo");
	floatInput = new NumInput (NumInput::Double, this);
	floatInput->SetLabel("float");
	floatInput->SetLabelWidth(g_numLabelWidth);
	floatInput->SetTotalWidth(g_numInputWidth);
	
	colorCombo = new QComboBox(this);
	colorCombo->setFixedHeight(g_comboHeight);
	colorCombo->setFixedWidth(g_numInputWidth);
	colorCombo->insertItem("colorCombo");
	colorButton = new ColorButton (this);
	colorButton->setFixedWidth(g_numInputWidth);
	colorButton->setEnabled(true);
	colorButton->setColor(1.0, 1.0, 1.0);
	
	pointCombo = new QComboBox(this);
	pointCombo->setFixedHeight(g_comboHeight);
	pointCombo->setFixedWidth(g_numInputWidth);
	pointCombo->insertItem("pointCombo");
	pointInput = new NumInTriplef(this);
		
	stringCombo = new QComboBox(this);
	stringCombo->setFixedHeight(g_comboHeight);
	stringCombo->setFixedWidth(g_numInputWidth);
	stringCombo->insertItem("stringCombo");
	stringEdit = new MLineEdit(this);
	stringEdit->setMinimumSize(20, 20);
	stringEdit->setFixedWidth(g_numInputWidth-20);
	
	stringFileButton = new MToolButton(this);
	stringFileButton->setText("e");
	stringFileButton->setFixedWidth(20);
	stringFileButton->setAutoRaise(true);
	
	if(g_toolTipsEnabled)
	{
		QToolTip::add(stringFileButton, "extended edit widget for strings");
		QToolTip::add(testButton, "test current shader values");
		QToolTip::add(saveButton, "append current shader\nvalues to preview list");
		QToolTip::add(updtButton, "update selected preview\nto current values");
		
		switch(shaderType)
		{
			case MBA_RMShader::Surface:
				QToolTip::add(visCamBut,	"surface visible to camera rays");
				QToolTip::add(visReflBut,	"surface visible to reflection rays");
				QToolTip::add(visShadBut,	"surface visible to shadow rays");
				break;
			case MBA_RMShader::Displacement:
				break;
			case MBA_RMShader::Undefined:
				break;
		}
	}
	
	
	//----------------------------------
	QObject::connect(availableList,	SIGNAL(highlighted(const QString&)), SLOT(ShaderSelected(const QString&)) );
	QObject::connect(previewList,	SIGNAL(selected(int)), SLOT(PreviewSelected(int)) );
	
	QObject::connect(floatInput,	SIGNAL(ValueChanged(float)),	SLOT(SetFloat(float)) );
	QObject::connect(colorButton,		SIGNAL(RGBChanged(float, float, float)), SLOT(setColor(float, float, float)) );
	QObject::connect(pointInput, 	SIGNAL(Value0Changed(float)), SLOT(SetPoint0(float)) );
	QObject::connect(pointInput, 	SIGNAL(Value1Changed(float)), SLOT(SetPoint1(float)) );
	QObject::connect(pointInput, 	SIGNAL(Value2Changed(float)), SLOT(SetPoint2(float)) );
	//QObject::connect(stringEdit, 	SIGNAL(textChanged(const char*)), SLOT(SetString(const char*)) );
	QObject::connect(stringEdit, 	SIGNAL(returnPressed()), SLOT(StringEntered()) );

	QObject::connect(floatCombo, 	SIGNAL(activated(int)), SLOT(SwitchFloats(int)) );
	QObject::connect(colorCombo, 	SIGNAL(activated(int)), SLOT(SwitchColors(int)) );
	QObject::connect(pointCombo, 	SIGNAL(activated(int)), SLOT(SwitchPoints(int)) );
	QObject::connect(stringCombo, 	SIGNAL(activated(int)), SLOT(SwitchStrings(int)) );
	QObject::connect(stringFileButton, 	SIGNAL(clicked()),	SLOT(OpenStringDialog()) );
	
	QObject::connect(testButton, 	SIGNAL(clicked()), SLOT(TestShader()) );
	QObject::connect(saveButton, 	SIGNAL(clicked()), SLOT(AppendShaderPreview()) );
	QObject::connect(updtButton, 	SIGNAL(clicked()), SLOT(UpdateShaderPreview()) );
	
	switch (shaderType)
	{
		case MBA_RMShader::Surface:
			QObject::connect(shadowShadeCombo,SIGNAL(activated(int)), SLOT(SwitchShadowVal(int)) );
			QObject::connect(visCamBut, SIGNAL(toggled(bool)), SLOT(SetVisibility(bool)) );
			QObject::connect(visReflBut,SIGNAL(toggled(bool)), SLOT(SetVisibility(bool)) );
			QObject::connect(visShadBut,SIGNAL(toggled(bool)), SLOT(SetVisibility(bool)) );
			break;
		case MBA_RMShader::Displacement:
			QObject::connect(trueDisplaceCheck, SIGNAL(toggled(bool)),		SLOT( ToggleTrueDisplace(bool)) );
			QObject::connect(displaceBoundInput, SIGNAL(ValueChanged(float)), SLOT( SetDisplaceBound(float)) );
			QObject::connect(displaceCoordCombo, SIGNAL(activated(int)),	SLOT( SwitchDisplaceCoord(int)) );
			break;
		case MBA_RMShader::Undefined:
			break;
	}
	
	//----------------------------------
//	topVLayout = new QVBoxLayout(this, 10, 5, "topVLayout");
	topHLayout = new QHBoxLayout(this, 10, 5, "topHLayout");
	butHLayout = new QHBoxLayout(0, "butHLayout" );
	listLayout = new QVBoxLayout(0, "listLayout" );
	prevLayout = new QVBoxLayout(0, "prevLayout" );
	col1Layout = new QVBoxLayout(0, "col1Layout" );
	stringHLayout = new QHBoxLayout(0, "stringHLayout" );
	if(shaderType == MBA_RMShader::Surface)
		visLayout = new QHBoxLayout(1, "visibilityLayout");
	
	topHLayout->addLayout(listLayout, 1);
		listLayout->addWidget(availableList, 2);
		switch(shaderType)
		{
			case MBA_RMShader::Surface:
				listLayout->addWidget( shadowShadeLabel, 0 );
				listLayout->addWidget( shadowShadeCombo, 0 );
				listLayout->addWidget( visibilityLabel, 0 );
				listLayout->addLayout( visLayout, 0);
					visLayout->addWidget( visCamBut );
					visLayout->addWidget( visReflBut );
					visLayout->addWidget( visShadBut );
				break;
			case MBA_RMShader::Displacement:
				listLayout->addSpacing(5);
				listLayout->addWidget( trueDisplaceCheck, 0 );
				listLayout->addSpacing(2);
				listLayout->addWidget( displaceBoundInput, 0 );
				listLayout->addSpacing(5);
				listLayout->addWidget( displaceCoordCombo, 0 );
				break;
			case MBA_RMShader::Undefined:
				break;
		}
	topHLayout->addLayout(prevLayout, 0);
		prevLayout->addLayout(butHLayout, 0);
			butHLayout->addWidget(saveButton, 1);
			butHLayout->addWidget(updtButton, 1);
			butHLayout->addWidget(testButton, 1);
		prevLayout->addSpacing(5);
		prevLayout->addWidget(previewList, 1);
	topHLayout->addLayout(col1Layout, 0);
		col1Layout->addWidget(floatCombo, 0);
		col1Layout->addWidget(floatInput, 0);
		col1Layout->addSpacing(10);
		col1Layout->addWidget(colorCombo, 0);
		col1Layout->addWidget(colorButton,  0);
		col1Layout->addSpacing(10);
		col1Layout->addWidget(pointCombo, 0);
		col1Layout->addWidget(pointInput, 0);
		col1Layout->addSpacing(10);
		col1Layout->addWidget(stringCombo);
		col1Layout->addLayout(stringHLayout, 0);
			stringHLayout->addWidget(stringEdit);
			stringHLayout->addWidget(stringFileButton);
		col1Layout->addStretch(1);
	
	PopulateShaderList();

	if((int)availableList->count() > 0)
		availableList->setCurrentItem(0);
	else
		;//Warning here about no installed shaders...
	
	setEnabled(false);	
}

//--------------------------------------------------------------------
MBA_RMShaderWidget::~MBA_RMShaderWidget()
{
}

//--------------------------------------------------------------------
int		MBA_RMShaderWidget::CheckForZombies()
{
	int	available = -1;
#ifndef WIN32
	int	i;
	int	status;
	for (i = 0; i < NUM_SHADER_PROCS; i++)
	{
		if (pid_shader_wait[i])	// check for any g_rendermanExec zombies...
		{
			status = waitpid(pid_shader_wait[i], (int*)0, WNOHANG);
			if (status > 0)
				pid_shader_wait[i] = 0;
			else if (status < 0)
				warning ("ERROR checking child PID in MBA_RMShaderWidget::CheckForZombies()");
		}
		if (!pid_shader_wait[i])
			available = i;
	}
#endif
	return available;
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::TestShader()
{
	TestShader(0);
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::TestShader(MBA_RMShader *combine)
{
	int	available;
	
	available = CheckForZombies();
	if (available < 0)
	{
		QCString	tmpMssg;
		tmpMssg.sprintf("PID table for shader previews is full (%d allowed).\nClose some preview windows and\ntry again.", NUM_SHADER_PROCS);
		QMessageBox::warning(0, 0, tmpMssg, "Ok");
	}
	else
	{
		pid_shader_wait[available] = WriteRIB(true, 0, combine);
	}
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::Copy()
{
	if (shader)
		copyShader = *shader;
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::Paste()
{
	if (shader)
	{
		*shader = copyShader;
		SetRMShaderPointer(shader);
	}
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::setEnabled(bool enable)
{
	//??? Is it a good idea to have enable==true do nothing?
	
	enable = enable;
	if (enable)
	{
		//MBA_RMShaderWidget::SetupEditWidgets() handles this...
	}
	else
	{
		EnableFloats(false);
		EnableColors(false);
		EnablePoints(false);
		EnableStrings(false);
		EnableExtras(false);
		EnableButtons(false);
		availableList->setEnabled(false);
		previewList->setEnabled(false);
	}
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::EnableButtons(bool enable)
{
		testButton->setEnabled(enable);
		saveButton->setEnabled(enable);
		updtButton->setEnabled(enable);
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::EnableExtras(bool enable)
{
	switch (shaderType)
	{
		case MBA_RMShader::Surface:
			shadowShadeCombo->setEnabled(enable);
			visCamBut->setEnabled(enable);
			visReflBut->setEnabled(enable);
			visShadBut->setEnabled(enable);
			if(!enable)
			{	
				visSetOK = false;
				visCamBut->setOn(false);
				visReflBut->setOn(false);
				visShadBut->setOn(false);
				visSetOK = true;
			}
			break;
		case MBA_RMShader::Displacement:
			trueDisplaceCheck->setEnabled(enable);
			if (trueDisplaceCheck->isChecked())
			{
				displaceBoundInput->setEnabled(enable);
				displaceCoordCombo->setEnabled(enable);
			}
			else
			{
				displaceBoundInput->setEnabled(false);
				displaceCoordCombo->setEnabled(false);
			}
			break;
		case MBA_RMShader::Undefined:
			break;
	}
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::EnableFloats(bool enable)
{
	floatCombo->setEnabled(enable);
	floatInput->setEnabled(enable);
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::EnableColors(bool enable)
{
	colorCombo->setEnabled(enable);
	colorButton->setEnabled(enable);
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::EnablePoints(bool enable)
{
	pointCombo->setEnabled(enable);
	pointInput->setEnabled(enable);
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::EnableStrings(bool enable)
{
	stringCombo->setEnabled(enable);
	stringEdit->setEnabled(enable);
	stringFileButton->setEnabled(enable);
	if(!enable)
		stringEdit->setText("-");
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::SwitchShadowVal(int in)
{
	if (shader)
		shader->SetShadowVal( (MBA_RMShader::ShadowTypeEnum)in );
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::ToggleTrueDisplace(bool in)
{
	if(shader)
	{
		shader->SetTrueDisplacement(in);
		if (in)
		{
			displaceBoundInput->setEnabled(true);
			displaceCoordCombo->setEnabled(true);
		}
		else
		{
			displaceBoundInput->setEnabled(false);
			displaceCoordCombo->setEnabled(false);
		}
	}
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::SetDisplaceBound(float in)
{
	if(shader)
		shader->SetDisplaceBound(in);
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::SwitchDisplaceCoord(int in)
{
	if (shader)
		shader->SetDisplaceCoord( (MBA_RMShader::CoordSystemEnum)in );
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::SwitchFloats (int index)
{
	if(shader)
	{
		floatInput->SetValueSolid( shader->floatsAt(index) );
		floatInput->SetFocus();
	}
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::SwitchColors (int index)
{
	QCString	tmpString;
	int i = index;
	if(shader)
	{
		if (shader->Color())
			i--;
		if (shader->Opacity())
			i--;
		if ( (tmpString = colorCombo->text(index)) == "color" )
		{
//			warning("*(shader->Color()) R(%f) G(%f) B(%f) ", shader->Color()->R(), shader->Color()->G(), shader->Color()->B( ) ); 
			colorButton->setColor( *(shader->Color()) );
		}
		else if ( (tmpString = colorCombo->text(index)) == "opacity" )
		{
			colorButton->setColor( *(shader->Opacity()) );
		}
		else
		{
			colorButton->setColor(shader->colorsAt(i) );
		}
		//colorButton->setFocus();
	}
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::SwitchPoints (int index)
{
	if(shader)
	{
		MVertex	&pt = shader->pointsAt(index);
		pointInput->SetValueAll(pt.X(), pt.Y(), pt.Z());
	}
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::SwitchStrings (int index)
{
	index = index;
	if(shader)
	{
		stringEdit->setText(shader->stringsAt(index) );
	}
}


//--------------------------------------------------------------------
void	MBA_RMShaderWidget::SetFloat(float val)
{
	if (shader)
		shader->SetFloat(floatCombo->currentItem(), val);
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::setColor(float r, float g, float b)
{
	GColor	tmpColor(r, g, b);
//	if (shader)
//		shader->setColor(colorCombo->currentItem(), tmpColor);
//		
	QCString	tmpString;
	int i = colorCombo->currentItem();
	if(shader)
	{
		if (shader->Color())
			i--;
		if (shader->Opacity())
			i--;
		if ( (tmpString = colorCombo->currentText()) == "color" )
			shader->setColor( tmpColor );
		else if ( (tmpString = colorCombo->currentText()) == "opacity" )
			shader->SetOpacity( tmpColor );
		else	
			shader->setColor( i, tmpColor );
	}
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::SetPoint0(float val)
{
	if (shader)
		shader->SetPoint0(colorCombo->currentItem(), val);
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::SetPoint1(float val)
{
	if (shader)
		shader->SetPoint1(colorCombo->currentItem(), val);
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::SetPoint2(float val)
{
	if (shader)
		shader->SetPoint2(colorCombo->currentItem(), val);
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::StringEntered()
{
	SetString(stringEdit->text());
	availableList->setFocus();		//after return is pressed, throw focus to list box
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::SetString(const char *val)
{
	if (shader)
		shader->SetString(stringCombo->currentItem(), val);
}

	
//--------------------------------------------------------------------
void	MBA_RMShaderWidget::PopulateShaderList()
{
//	QStrList	tempFiles;
//	int			j;
//	int			dot;
//	QCString		noExtension;
	
	shaderList.clear();

	switch (shaderType)
	{
		case MBA_RMShader::Surface:
			shaderList = g_surfaceShadersList;
			break;
		case MBA_RMShader::Displacement:
			shaderList = g_displacementShadersList;
			break;
		case MBA_RMShader::Undefined:
			warning ("MBA_RMShaderWidget::PopulateShaderList - Yer hosed.");
			break;
	}

	availableList->clear();
	availableList->insertStrList(&shaderList);
}

//--------------------------------------------------------------------
void	MBA_RMShaderWidget::CreatePreviewTable(const char *name, bool force)
{
	static QCString	lastName;
	QCString			foundFilename;
	QCString			tempFilename;
	QCString			tempString;
	QDir			varDir;
	int				i;
	int				j;
	int				k;
	bool			ok;

	if (!force)
	{
		if (lastName == name)	// Prevent uneccessary reloading
			return;
		else
			lastName = name;
	}
		
	previewList->clear();
	previewNameList.clear();
	
	varDir = QDir ((const char*)variationsDir, "*.rsi", QDir::Name, QDir::Files|QDir::Readable );
	if (varDir.exists())
	{
		previewFilenameList.clear();
		
		QStringList	entries(varDir.entryList());
		j = 0;
		QStringList::Iterator	it;
		for (i = 0; i < (int)entries.count(); i++)
		{
//			tempFilename	= entries.at(i);
			it = entries.at(i);
			tempFilename = (*it).copy();
			tempString		= name;
			tempString		+= "__";
			if ( tempFilename.find(tempString) == 0)
			{
				j++;
				tempString = (const char*)variationsDir;
				tempString += (const char*)tempFilename;
				previewFilenameList.append((const char*)tempString);
				foundFilename = tempFilename;
				k = foundFilename.findRev('.');
				foundFilename.truncate(k);
				previewNameList.append((const char*)foundFilename);
			}
		}
		
		if (j > 0)	// if there are any variations, get the last number
		{
			k = foundFilename.findRev("__");
			foundFilename = foundFilename.remove(0, k+2);
			numVariations = foundFilename.toInt(&ok) + 1;
			if (!ok)
			{
				warning ("CreatePreviewTable - numVariations not set correctly! foundFilename = \"%s\"", (const char*)foundFilename);
				numVariations = -1;
			}
		}
		else
			numVariations = 0;
		
		previewList->fillTable(variationsDir, previewNameList);
	}
	else
		warning("ERROR: Looking for variations in %s - dir does not exist.", (const char*)variationsDir);
}

//------------------------------------------------
void	MBA_RMShaderWidget::PreviewSelected(int index)
{
	QCString	previewName;
	QCString	previewNumber;
	FILE	*shaderFile;
	
	if (shader)
	{
		previewName = (const char*)variationsDir;
		previewName	+= availableList->text(availableList->currentItem());
		previewNumber.sprintf("__%03d.rsi", index);
		previewName	+= previewNumber;
		
		MBA_RMShader	*tmpShader = 0;
		shaderFile = fopen((const char*)previewName, "r");
		if (shaderFile)
		{	
			tmpShader = new MBA_RMShader(shaderFile);
			*shader = *tmpShader;
			delete tmpShader;
			fclose(shaderFile);
		}
		else
			warning("Could not open variation file \n\t\"%s\"", (const char*)previewName);
		SetupEditWidgets();
	}
}

//------------------------------------------------
void MBA_RMShaderWidget::ShaderSelected(const QString &name)
{
	ShaderSelected((const char*)name);
}
//------------------------------------------------
void MBA_RMShaderWidget::ShaderSelected(const char* name)
{
	CheckForZombies();	// Just for the hell of it, clear up any zombies here...
	
	if (shader)
	{
		MBA_RMShader	*tmpShader;
		tmpShader = new MBA_RMShader(name);
		if (tmpShader->ShaderType() == shaderType)
			*shader = *tmpShader;
		else
			return;
		SetupEditWidgets();
	}
	CreatePreviewTable(name);
}

//------------------------------------------------
void MBA_RMShaderWidget::SetupEditWidgets()
{
	int i;
	int	tmp;
	
	
	if (shader)
	{
		testButton->setEnabled(true);
		saveButton->setEnabled(true);
		updtButton->setEnabled(true);
		
		tmp = floatCombo->currentItem();
		floatCombo->clear();
		if (shader->numFloats())
		{
			EnableFloats(true);
			for (i = 0; i < shader->numFloats(); i++)
				floatCombo->insertItem(shader->floatsNameAt(i));
			if (tmp >= floatCombo->count())
				tmp = 0;
			floatCombo->setCurrentItem(tmp);
			SwitchFloats(tmp);
		}
		else EnableFloats(false);

		tmp = colorCombo->currentItem();
		colorCombo->clear();
		if (shader->numColors() || shader->Color() || shader->Opacity())
		{
			EnableColors(true);
			if (shader->Color())
				colorCombo->insertItem("color");
			if (shader->Opacity())
				colorCombo->insertItem("opacity");
			for (i = 0; i < shader->numColors(); i++)
				colorCombo->insertItem(shader->colorsNameAt(i));
			if (tmp >= colorCombo->count())
				tmp = 0;
			colorCombo->setCurrentItem(tmp);
			SwitchColors(tmp);
		}
		else EnableColors(false);

		tmp = pointCombo->currentItem();
		pointCombo->clear();
		if (shader->numPoints())
		{
			EnablePoints(true);
			for (i = 0; i < shader->numPoints(); i++)
				pointCombo->insertItem(shader->pointsNameAt(i));
			if (tmp >= pointCombo->count())
				tmp = 0;
			pointCombo->setCurrentItem(tmp);
			SwitchPoints(tmp);
		}
		else EnablePoints(false);

		tmp = stringCombo->currentItem();
		stringCombo->clear();
		if (shader->numStrings())
		{
			EnableStrings(true);
			for (i = 0; i < shader->numStrings(); i++)
				stringCombo->insertItem(shader->stringsNameAt(i));
			if (tmp >= stringCombo->count())
				tmp = 0;
			stringCombo->setCurrentItem(tmp);
			SwitchStrings(tmp);
		}
		else EnableStrings(false);
		
		EnableExtras(true);
		switch (shaderType)
		{
			case MBA_RMShader::Surface:
				shadowShadeCombo->setCurrentItem(shader->ShadowVal());
				visSetOK = false;
				visCamBut->setOn (shader->VisCamera());
				visReflBut->setOn(shader->VisReflect());
				visShadBut->setOn(shader->VisShadow());
				visSetOK = true;
				break;
			case MBA_RMShader::Displacement:
				trueDisplaceCheck->setChecked(shader->TrueDisplacement());
				displaceCoordCombo->setCurrentItem(shader->DisplaceCoord());
				displaceBoundInput->SetValueSolid(shader->DisplaceBound());
				break;
			case MBA_RMShader::Undefined:
				break;
		}
	}
	else
	{
		setEnabled(false);
	}
}

//-----------------------------------------------------------------------------------
void MBA_RMShaderWidget::SetRMShaderPointer(MBA_RMShader *in)
{
	int	index;

	shader = in;
	if (in)
	{
		const char* name = (const char*)shader->Name();
		if ( (index = shaderList.find(name)) >= 0 )
		{
			availableList->setEnabled(true);
			EnableButtons(true);
			SetupEditWidgets();
			shader = 0;	// avoid initializing the editable shader to default
			availableList->setCurrentItem(index);
			availableList->centerCurrentItem();
			shader = in;
//			shaderLabel->setText(name);
			CreatePreviewTable(name);
			previewList->setEnabled(true);
		}
		else
		{
			EnableFloats(false);
			EnableColors(false);
			EnablePoints(false);
			EnableStrings(false);
			EnableExtras(false);
			EnableButtons(false);
			availableList->setEnabled(false);
			previewList->setEnabled(false);
			QCString	tmpMssg;
			tmpMssg.sprintf("WARNING: Shader \"%s\" is not available.", name);
			if (QMessageBox::warning(this, 0, tmpMssg, "Ignore", "Default") == 1)
			{
				MBA_RMShader	tmpShader(shader->ShaderType());
				*shader = tmpShader;
				SetRMShaderPointer(shader);
			}
		}
	}
	else
	{
		EnableFloats(false);
		EnableColors(false);
		EnablePoints(false);
		EnableStrings(false);
		EnableExtras(false);
		EnableButtons(false);
		availableList->setEnabled(false);
		previewList->setEnabled(false);
	}
}
	
//-----------------------------------------------------------------------------------
void MBA_RMShaderWidget::OpenStringDialog()
{
	MbaRMShaderStringWidget	stringWidget(stringEdit->text(), lastTextureDir, this);
	
	switch(stringWidget.exec())
	{
		case QDialog::Accepted:
			lastTextureDir = stringWidget.LastDir();
			stringEdit->setText(stringWidget.StringValue());
			SetString(stringWidget.StringValue());
			break;
		case QDialog::Rejected:
			break;
	}
}

//-----------------------------------------------------------------------------------
void	MBA_RMShaderWidget::SetVisibility(bool)
{
	if(shader && visSetOK)
	{
		int	tmpVis = 0;
		if(visCamBut->isOn())
			tmpVis += 1;
		if(visReflBut->isOn())
			tmpVis += 2;
		if(visShadBut->isOn())
			tmpVis += 4;
		warning("MBA_RMShaderWidget::SetVisibility %d", tmpVis);
		shader->SetVisibility(tmpVis);
	}
}

