/**
 * mbaRMShaderWidget.h
 * Copyright 1999 by Mark B. Allan
 * 
 * widget to edit RM surface and displacement shaders
 */
#ifndef mbaRMShaderWidget_h
#define mbaRMShaderWidget_h

#include <qwidget.h>
#include <qlayout.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qstrlist.h>
#include <qcheckbox.h>

#include "mwidgets.h"
#include "mbaRMShader.h"
#include "numInput.h"
#include "numInTriplef.h"
#include "shaderPreviewList.h"

#define	NUM_SHADER_PROCS 25
class ColorButton;

/**
 * 
 */
//====================================================================
class MBA_RMShaderWidget	: public QFrame
{
	Q_OBJECT
	
public:
	MBA_RMShaderWidget(MBA_RMShader::ShaderTypeEnum, QWidget* parent = 0, const char* name = 0);
	~MBA_RMShaderWidget();
	
	void	Copy();
	void	Paste();
	
	void	SetRMShaderPointer(MBA_RMShader *in);
	void	PopulateShaderList();

public slots:
	void	TestShader(MBA_RMShader *combine = 0);
	void	TestShader();
	void	setEnabled(bool);

protected slots:
	void	ShaderSelected(const QString &name);
	void	ShaderSelected(const char* name);
	void	PreviewSelected(int index);
	
	void	SetupEditWidgets();
	
	void	SwitchFloats (int index);
	void	SwitchColors (int index);
	void	SwitchPoints (int index);
	void	SwitchStrings (int index);
	
	void	SetFloat(float val);
	void	setColor(float r, float g, float b);
	void	SetPoint0(float val);
	void	SetPoint1(float val);
	void	SetPoint2(float val);
	void	SetString(const char *val);
	void	StringEntered();
	void	OpenStringDialog();
	
	void	SwitchShadowVal(int index);
	
	void	ToggleTrueDisplace(bool);
	void	SetDisplaceBound(float);
	void	SwitchDisplaceCoord(int);
	
	void	AppendShaderPreview();
	void	UpdateShaderPreview();
	
	void	SetVisibility(bool);
	
protected:
	void	EnableFloats(bool);
	void	EnableColors(bool);
	void	EnablePoints(bool);
	void	EnableStrings(bool);
	void	EnableExtras(bool);
	void	EnableButtons(bool);
	void	InitReptileDirs();
	void	CreatePreviewTable(const char *name, bool force = false);
	int		WriteRIB (bool display = true, const char *filename = 0, MBA_RMShader *combine = 0);	// returns rendrib PID
	void	SaveCurrentShaderPreview (int update);
	int		CheckForZombies();

protected:
	MBA_RMShader::ShaderTypeEnum	shaderType;
	
	QVBoxLayout	*topVLayout;
	QHBoxLayout	*topHLayout;
	QHBoxLayout	*butHLayout;
	QHBoxLayout	*stringHLayout;
	QVBoxLayout	*listLayout;
	QVBoxLayout	*prevLayout;
	QVBoxLayout	*col1Layout;
	QHBoxLayout	*visLayout;
	
	QListBox	*availableList;
	ShaderPreview *previewList;
	
	MToolButton	*testButton;
	MToolButton	*saveButton;
	MToolButton	*updtButton;
	
	MToolButton	*stringFileButton;
	
//	QLabel		*shaderLabel;
	
	QComboBox	*floatCombo;
	QComboBox	*colorCombo;
	QComboBox	*pointCombo;
	QComboBox	*stringCombo;
	
	QLabel		*shadowShadeLabel;
	QComboBox	*shadowShadeCombo;
	
	QLabel		*visibilityLabel;
	MToolButton	*visCamBut;
	MToolButton	*visReflBut;
	MToolButton	*visShadBut;
	bool		visSetOK;
	
	QCheckBox	*trueDisplaceCheck;
	QComboBox	*displaceCoordCombo;
	NumInput	*displaceBoundInput;
	
	NumInput		*floatInput;
	ColorButton		*colorButton;
	MLineEdit		*stringEdit;
	NumInTriplef	*pointInput;	
	
	int			ribFileInt;			// keep track of rendrib filename (reptile1.rib, etc.)
	int			numVariations;
	QStrList	shaderList;
	QStrList	previewNameList;
	QStrList	previewFilenameList;
	QCString	ribFilename;
	QCString	tifFilename;
	QCString	variationsDir;	// dir to find shader variations and jpegs
	QCString	lastTextureDir;
	
	MBA_RMShader	*shader;
	MBA_RMShader	copyShader;
	
	int		pid_shader_wait[NUM_SHADER_PROCS];
};

#endif // MBA_RMShaderWidget_h


