/**
 * mbaRMShaderWidgetSaveUpdate.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * IO routines for RMShaderWidget
 */
#include "mbaRMShaderWidget.h"

#include <qapp.h>
#include <qmessagebox.h>

#include "extern.h"

#ifndef WIN32	
	#include <unistd.h>
	#include <sys/wait.h>
#endif

#include <ri.h>
#include "forkVector.h"

//--------------------------------------------------------------------
int MBA_RMShaderWidget::WriteRIB (bool display, const char *tmpFilename, MBA_RMShader *combine )
{
	static		int		ret;
	RtPoint 	originPos 			= {	0.0, 0.0, 0.0	};
	char		ribchar[256];
	char		tifchar[256];
	QCString	tifFilename;
	
	// setup background patch...
	RtPoint		blBkGnd[2][2];
	double		tmpDbl = 41.6;
	blBkGnd[0][0][0] =  tmpDbl;	blBkGnd[0][0][1] =  tmpDbl;	blBkGnd[0][0][2] = 5;
	blBkGnd[0][1][0] = -tmpDbl;	blBkGnd[0][1][1] =  tmpDbl;	blBkGnd[0][1][2] = 5;
	blBkGnd[1][0][0] =  tmpDbl;	blBkGnd[1][0][1] = -tmpDbl;	blBkGnd[1][0][2] = 5;
	blBkGnd[1][1][0] = -tmpDbl;	blBkGnd[1][1][1] = -tmpDbl;	blBkGnd[1][1][2] = 5;
	
	if(tmpFilename)
		tifFilename.sprintf("%s", tmpFilename);
	else
		tifFilename.sprintf ("RIB/shader%02d.tif", ribFileInt);	
	ribFilename.sprintf ("RIB/shader%02d.rib", ribFileInt);
	ribFileInt = ++ribFileInt % 10;
	
	strcpy(ribchar, (const char*)ribFilename);	// ribout wants these things as char*, not const char*
	FILE	*testRibname;
	testRibname = fopen(ribchar, "w");
	if (!testRibname)
	{
		QCString	tmpMssg;
		tmpMssg.sprintf("ERROR!\n\nCould not open\n\"%s\"\nfor write.\n\nCheck file permissions and disk space.", ribchar);
		QMessageBox::critical(0, "f l o w", tmpMssg, "Ok");
		return -1;
	}
	fclose(testRibname);
	RiBegin(ribchar);
		RiArchiveRecord ("comment", "RIB file generated by the Reptile Labour Project", RI_NULL );
		
		RtFloat	rFOV[1]	= { 30.0 };
		RiProjection ("perspective", "fov", rFOV, RI_NULL);
		RiTranslate	( 0.0f, 0.0f, 4.1f);

		shader->DeclareVariables();
		if (combine)
			combine->DeclareVariables();

		RiSides(2);
		RiPixelSamples (1, 1);
		if (display)
			RiFormat(g_previewRenderSize, g_previewRenderSize, 1.0);
		else
			RiFormat (g_previewTableCellSize, g_previewTableCellSize, 1.0);	//size to fit into preview table
		
		strcpy (tifchar, (const char*)tifFilename);
		if(display)
			RiDisplay (tifchar, RI_FRAMEBUFFER, RI_RGB, RI_NULL);
		else
			RiDisplay (tifchar, RI_FILE, RI_RGB, RI_NULL);

		float	light1Pos[3] =  { -7.5f,  5.0f,  -5.0f };
		float	light2Pos[3] = 	{  2.5f, 25.0f,  12.5f };
		float	light3Pos[3] = 	{  0.2f, -0.6f,  -0.2f };
		float	light0Intensity	= 0.1f;
		float	light1Intensity	= 100.0f;
		float	light2Intensity	= 450.0f;
		float	light3Intensity	= 0.4f;
		
		RiWorldBegin();
			RiLightSource("ambientlight",	"intensity",&light0Intensity,
											RI_NULL);
			RiLightSource("pointlight",		"intensity",&light1Intensity,
											"from",		light1Pos, 
											RI_NULL);
			RiLightSource("pointlight",		"intensity",&light2Intensity,
											"from",		light2Pos, 
											RI_NULL);
			RiLightSource("distantlight",	"intensity",&light3Intensity,
											"from",		light3Pos, 
											"to", 		originPos, 
											RI_NULL);
			
			float	bkrep = 96.0;
			RiDeclare("repeats", "uniform float");
			RiSurface("bkgndStripes", "repeats", &bkrep, RI_NULL);
			RiPatch (RI_BILINEAR, RI_P, (RtPointer)blBkGnd, RI_NULL);
			RiSurface("plastic", RI_NULL);
			
			//Write out shader info
			if (combine)
				combine->Activate();
			shader->Activate();
			
			//Rotate sphere into good location
			RiRotate ( -20.0, 1.0, 0.0, 0.0);
			RiRotate (  20.0, 0.0, 1.0, 0.0);
			RiRotate ( -90.0, 1.0, 0.0, 0.0);
			RiSphere (1.0, -1.0, 1.0, 360.0, RI_NULL);
			
		RiWorldEnd();
	RiEnd();
	
#ifndef WIN32
	if(g_hasRendermanExec)
	{
		// don't want to use forkVector because opts may contain -radio which screws shader previews up
		//ForkVector	forkVector;
		//FIXME -  NO ZOMBIES
		//FIXME -  set up PID table for these...
		//FIXME -  NO ZOMBIES
		
		QCString	rendermanExec = g_rendermanExec;
		int i = rendermanExec.findRev('/');
		if(i >= 0)
			rendermanExec.remove(0, i+1);
		
		char interlaceVal[8];
		sprintf(interlaceVal, "%d", g_rendInterlace);
		
		qApp->syncX();
		if( (ret = fork()) == 0 )
		{
			QApplication::exit(0);
			//execv(g_rendermanExec, forkVector.createRenderVector((const char*)ribFilename, display) );
			if(display && g_rendInterlace)
				execl(g_rendermanExec, (const char*)rendermanExec, "-d", interlaceVal, (const char*)ribFilename, NULL);
			else
				execl(g_rendermanExec, (const char*)rendermanExec, (const char*)ribFilename, NULL);
		}
	}
	return ret;
#else
	return -1;
#endif
}		
	
//-------------------------------------------------------------------
void	MBA_RMShaderWidget::AppendShaderPreview()
{
	SaveCurrentShaderPreview(false);
}
void	MBA_RMShaderWidget::UpdateShaderPreview()
{
	SaveCurrentShaderPreview(true);
}

//-------------------------------------------------------------------
void MBA_RMShaderWidget::SaveCurrentShaderPreview (int update)
{
	int		i;
	QString	infoFilename;
	QString	jpgFilename;
	QString	jpgConvertName;
	QString	previewName;
	
	if( !(g_hasRendermanExec || g_hasTiffToJpegExec) )
		return;
	
	if (shader)
	{
		switch (update)
		{
			case 0: //Save new...
					infoFilename.sprintf("%s%s__%03d.rsi", (const char*)variationsDir, shader->Name(), numVariations);
					tifFilename.sprintf ("%s%s__%03d.tif", (const char*)variationsDir, shader->Name(), numVariations);
					jpgFilename.sprintf ("%s%s__%03d.jpg", (const char*)variationsDir, shader->Name(), numVariations);
					previewName.sprintf ("%s__%03d", shader->Name(), numVariations);
				break;
			case 1: //Update current...
				if ( (i = previewList->currentItem()) >= 0 )
				{
					infoFilename.sprintf("%s%s.rsi", (const char*)variationsDir, previewNameList.at(i));
					tifFilename.sprintf ("%s%s.tif", (const char*)variationsDir, previewNameList.at(i));
					jpgFilename.sprintf ("%s%s.jpg", (const char*)variationsDir, previewNameList.at(i));
					previewName = (const char*)previewNameList.at(i);
				}
				else
				{
					warning ("ERROR: No preview item selected. Saving as new.");
					infoFilename.sprintf("%s%s__%03d.rsi", (const char*)variationsDir, shader->Name(), numVariations);
					tifFilename.sprintf ("%s%s__%03d.tif", (const char*)variationsDir, shader->Name(), numVariations);
					jpgFilename.sprintf ("%s%s__%03d.jpg", (const char*)variationsDir, shader->Name(), numVariations);
					previewName.sprintf ("%s__%03d", shader->Name(), numVariations);
				}
				break;
		}

		//----------------------------------Render example shader 
		FILE	*infoFile;
		infoFile = fopen((const char*)infoFilename, "w");
		if (!infoFile)
		{	
			QMessageBox::critical( this, "f l o w", "ERROR:\nCould not open\n\"%s\"\nfor write.\n\nCheck file permissions and disk space.", "Ok");
			return;
		}
		else
		{
			shader->SaveInfo(infoFile);
			fclose(infoFile);
		}
		int rendermanProcess = WriteRIB(false, (const char*)tifFilename);	
		if (rendermanProcess < 0)
			return;

#ifndef WIN32
		ForkVector	forkVector;
		int		ret;
		waitpid (rendermanProcess, (int*)0, WUNTRACED);

		// spawn off an imageMagick 'convert' process to convert the tiff to a jpg 
		jpgConvertName = "JPEG:"+jpgFilename;
		qApp->syncX();
		if( (ret = fork()) == 0 )
		{
			//warning("%s %s %s", g_tiffToJpegExec, (const char*)tifFilename, (const char*)jpgFilename);
			QApplication::exit(0);
			//char	cropString[32];
			//sprintf(cropString, "%dx%d+0+0", g_previewTableCellSize, g_previewTableCellSize );
			//execlp( g_tiffToJpegExec, g_tiffToJpegExec, "-crop", cropString, (const char*)tifFilename, (const char*)jpgFilename, 0 );
			execv( g_tiffToJpegExec, forkVector.createConvertVector((const char*)tifFilename, (const char*)jpgConvertName) );
		}
		waitpid (ret, (int*)0, WUNTRACED);
		remove((const char*)tifFilename);

		// update preview table
		i = previewList->currentItem();
		
		if (update)
		{
			QCString	tmpString;
			tmpString = (const char*)jpgFilename;
			int	baseI;
			baseI = tmpString.findRev('/');
			if (baseI > 0)
				tmpString.remove(0, baseI+1);
			baseI = tmpString.findRev('.');
			if (baseI > 0)
				tmpString.truncate(baseI);
			//warning("tmpString \"%s\"", (const char*)tmpString);	// prepare key for pixmap cache in preview table
			previewList->replace (i, QPixmap((const char*)jpgFilename, "JPEG"), (const char*)tmpString);
		}
		else
		{
			previewNameList.append((const char*)previewName);		// add name to preview list
			previewFilenameList.append((const char*)infoFilename);	// add name to preview list
			previewList->replace (-1, QPixmap((const char*)jpgFilename, "JPEG"));
			numVariations++;
		}
#endif
	}
}	


