/**
 * mbaScene.h
 * Copyright 1999 by Mark B. Allan
 * 
 * contains and controls the scene
 */
#ifndef mbaScene_h
#define mbaScene_h

#include <qlist.h>

#include "basetypes.h"

#include "lightGroup.h"
#include "camera.h"
#include "particleControl.h"

/**
 * handles the scene
 */
//====================================================================
class MBAScene
{
public:
	MBAScene();
	~MBAScene();

	void	clear();
	void	saveToFile(FILE *outfile);
	void	loadFromFile(FILE *infile);
	
	void	reset();
	void	execute(int frame);
	void	drawGL(int frame);
	
	bool	writeCode();
	void	writeCode_VARIABLE_DECLARE (FILE *outfile);
	void	writeCode_FUNCTION_DECLARE (FILE *outfile);
	void	writeCode_VARIABLE_INIT (FILE *outfile);
	void	writeCode_EXEC (FILE *outfile);
	void	writeCode_FUNCTIONS (FILE *outfile);
	
	inline int	numLightGroups()	{	if (lightGroupList)	return (int)lightGroupList->count();	else return 0;	}
	inline int	numCameras()		{	if (cameraList)		return (int)cameraList->count();		else return 0;	}
	inline int	numParticleSystems(){	if (particleList)	return (int)particleList->count();		else return 0;	}
	
	inline void	setBackgroundColor (float r, float g, float b)	{	backgroundColor->SetRGB(r, g, b);	}
	
	void	AddLightGroup(LightGroup* lg, int index = -1);
	void	AddParticleSystem(ParticleControl* pc, int index = -1);
	
	LightGroup*			ActiveLightGroup();
	CameraView*			ActiveCamera();
	ParticleControl*	ActiveParticleSystem();
	
	int		setActiveParticleSystem(int);
	
	void	RemoveLightGroup	(int index);
	void	RemoveParticleSystem(int index);
	
	bool	GotoFrame(int f);
	
	LightGroup*			LightGroupAt(int);
	CameraView*			CameraAt(int);
	ParticleControl*	ParticleSystemAt(int);
	
	inline	GColor*			BackgroundColor()	{	return backgroundColor;	}
	void	SetBkGndColorPointer(GColor* in)	{	if(backgroundColor) delete backgroundColor; backgroundColor = in;	}
	
	// . . . . . . . . . . . .
	void	ResetParticleSystems();
	
	inline	QString		&getCodeConfigFile() {	return codeConfigFile;	}
	inline	QStringList	&getCodeFiles()		 {	return codeFiles;	}
	inline	QString		&getCodeOutDir()	 {	return codeOutDir;	}
	bool	loadCodeConfig(const QString &filename);
	bool	saveCodeConfig(const QString &filename);
	
protected:
	QList<LightGroup>		*lightGroupList;
	QList<CameraView>		*cameraList;
	QList<ParticleControl>	*particleList;
	
	int	activeLightGroup;
	int	activeCamera;
	int	activeParticleSystem;
	
	GColor		*backgroundColor;
	
	bool	glCullFace;
	bool	glBlend;
	bool	glTwoSideLight;

	QString		codeConfigFile;
	QStringList	codeFiles;
	QString		codeOutDir;
	
//	float	ribRendSamples[2];
//	bool	ribMotionBlur;
//	bool	ribAllowShadows;
//	bool	ribAllowTrueDisplace;
//	int		ribNumBlurSteps;
//	int		ribRendW;
//	int		ribRendH;
};

#endif
