/**
 * mbaSceneTreeWidget.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * tree widget that displays the scene
 */
#include "mbaSceneTreeWidget.h"
#include "mbaSceneTreePartItem.h"


#include "extern.h"
#include "camera.h"
#include "lightGroup.h"
#include "particleControl.h"
#include "particleControlWidget.h"
#include "mbaScene.h"

//--------------------------------------------------------------------
MBASceneTreeWidget::MBASceneTreeWidget(MBAScene* s, QWidget *parent, const char* name)
	: QListView(parent, name)
{
	addColumn("Scene");
	
	scene 				= 0;
	cameras				= 0;
	lightGroups			= 0;
	particleControls	= 0;
	
	setSorting(-1, false);
	setColumnWidthMode(0, QListView::Maximum);
	
	setFrameStyle(QFrame::Panel | QFrame::Sunken);
	
//	if( s )
	if( 0 )
		SetScenePointer(s);
	else
	{
		particleControls	= new QCheckListItem(this, "ParticleSystems");
		lightGroups			= new QCheckListItem(this, "LightGroups");
		cameras				= new QCheckListItem(this, "Cameras");
	}
	
	QObject::connect( this, SIGNAL(doubleClicked(QListViewItem*)), SLOT(DoubleClicked(QListViewItem*)) );
	QObject::connect( this, SIGNAL(selectionChanged(QListViewItem*)), SLOT(SelectionChanged(QListViewItem*)) );
}

MBASceneTreeWidget::~MBASceneTreeWidget()
{
}

//--------------------------------------------------------------------
void	MBASceneTreeWidget::SelectionChanged(QListViewItem* inItem)
{
	warning("SelectionChanged"); 
	static	QCString	cameraString("Cameras");
	static	QCString	lightString	("LightGroups");
	static	QCString	particleString("ParticleSystems");
	
	if(inItem)
	{
		QListViewItem	*current;
		QCheckListItem	*parent;
		QCheckListItem	*item = (QCheckListItem*)inItem;
		parent = (QCheckListItem*)item->parent();
		if(parent)
		{
			if (item->type() == QCheckListItem::RadioButton)
			{
				if (cameraString == (const char*)parent->text())
				{ 
					warning("Camera selected");
				}
				else if (lightString ==  (const char*)parent->text())
				{
					warning("Light group selected");
				}
			}
			else if (item->type() == QCheckListItem::CheckBox)
			{
				int i = 0;
				if (particleString ==  (const char*)parent->text())
				{
					current = parent->firstChild();
					//warning("ParticleSystem selected");
					while (current)
					{
						// I need some way of uniquely identifying the systems...
						if (current == inItem)
						{
							item->setOn(!item->isOn());
							//warning("found it at %d", i);
	//  					ParticleControl *partCtrl = 0;
	//  					partCtrl = g_scene->ParticleSystemAt(i);
	//  					g_particleControlWidget->SetParticleControl(partCtrl);
							g_particleControlWidget->SetParticleControl(i);
							break;
						}
						current = current->nextSibling();
						i++;
					}
				}
			}
		}
		else
			warning("MBASceneTreeWidget::SelectionChanged(QListViewItem* inItem)\n --Why don't we have a 'parent' here?"); 
	}
}

//--------------------------------------------------------------------
void	MBASceneTreeWidget::DoubleClicked(QListViewItem* item)
{
	static	int	count = 0;
	count++;
	QCString newName;
	newName.setNum(count);
	QCString	tmpString;
	warning("double click");
	item->setOpen(!item->isOpen());
	tmpString = item->text(0);
	if (tmpString == "ParticleSystems")
	{
		ParticleControl	*newPControl;
		newPControl = new ParticleControl();
		newPControl->SetName(newName);
//		g_particleControlWidget->SetParticleControl(newPControl);
		g_scene->AddParticleSystem(newPControl);
		SetScenePointer(g_scene);
		g_particleControlWidget->SceneChange();
		warning("create new particle system");
	}
}

//--------------------------------------------------------------------
void	MBASceneTreeWidget::SetScenePointer(MBAScene* s)
{
	scene = s;	// Don't delete the old one - maybe useful later for multiple scenes in project
	
	clear();

	int i;
	QCheckListItem				*tmpItem = 0;
	MBASceneTreePartItemCheck	*tmpPartItem = 0;
	if(scene)
	{
		particleControls	= new QCheckListItem(this, "ParticleSystems");
		lightGroups			= new QCheckListItem(this, "LightGroups");
		cameras				= new QCheckListItem(this, "Cameras");
		if (scene->numCameras() > 0)
		{
			for (i = 0; i < scene->numCameras(); i++)
			{
				tmpItem = new QCheckListItem( cameras, scene->CameraAt(i)->name(), QCheckListItem::RadioButton );
			}
			cameras->setOpen(true);
		}
		else
			cameras = 0;
		if (scene->numLightGroups() > 0)
		{
			for (i = scene->numLightGroups()-1; i >= 0; i--)
			{
				tmpItem = new QCheckListItem( lightGroups, scene->LightGroupAt(i)->Name(), QCheckListItem::RadioButton );
			}
			lightGroups->setOpen(true);
		}
		else
			lightGroups = 0;
		if (scene->numParticleSystems() > 0)
		{
			int	lastPSys = scene->numParticleSystems()-1;
			for (i = lastPSys; i >= 0; i--)
			{
//				tmpItem = new QCheckListItem( particleControls, scene->ParticleSystemAt(i)->Name(), QCheckListItem::CheckBox );
				tmpPartItem = new MBASceneTreePartItemCheck( i, particleControls, scene->ParticleSystemAt(i)->Name() );
				tmpPartItem->setOn(scene->ParticleSystemAt(i)->isEnabled());
			}
			particleControls->setOpen(true);
		}
		else
			particleControls = 0;
	}
}
	
