/**
 * mbaSceneWidget.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * scene widget
 */
#include "mbaSceneWidget.h"
#include "extern.h"

#include <qkeycode.h>
#include <qtooltip.h>
#include <qlabel.h>

#include "extern.h"
#include "define.h"
#include "lightGroupWidget.h"
#include "mbaCurveAssignDialog.h"
#include "particleControlWidget.h"

//#include "Pixmaps/but_close.xpm"
//#include "Pixmaps/flowIcon.xpm"

//==============================================================================
MBASceneWidgetMain::MBASceneWidgetMain(MBAScene *scene, QWidget *parent, const char* name)
	: QWidget(parent, name)
{		
	sceneTreeWidget = new MBASceneTreeWidget(scene, this);
	sceneTreeWidget->setRootIsDecorated(true);
	
	curveAssignBut = new QPushButton(this);
	curveAssignBut->setText("curve connect");
	curveAssignBut->setFixedHeight(25);
//	curveAssignBut->setEnabled(false);
	
	//-- Connect
	QObject::connect( curveAssignBut, SIGNAL(clicked()), SLOT(ShowCurveAssign()) );
	
	//-- Layout
	topVLayout = new QVBoxLayout(this, 5, 5);
	
	topVLayout->addWidget(curveAssignBut);
	topVLayout->addWidget(sceneTreeWidget, 1);
	
	topVLayout->activate();	
}

MBASceneWidgetMain::~MBASceneWidgetMain()
{
}

//----------------------------------------------------------
void MBASceneWidgetMain::ShowCurveAssign()
{
	MBACurveAssignDialog *assignDialog;
	assignDialog = new MBACurveAssignDialog(this);
	assignDialog->exec();
	switch(assignDialog->result())
	{
		case QDialog::Accepted:
			warning("assignDialog accepted");
			break;
		case QDialog::Rejected:
			warning("assignDialog rejected");
			break;
	}
	delete assignDialog;
}

void	MBASceneWidgetMain::SetScenePointer(MBAScene *in)
{	
	sceneTreeWidget->SetScenePointer(in);
	
	LightGroupWidget* lightGroupWidget = g_lightGroupWidgetFrame->GetLightGroupWidget();
	if(in->numLightGroups() > 0)
	{
		lightGroupWidget->SetLightGroupPointer(in->LightGroupAt(0));
	}
	else
		lightGroupWidget->SetLightGroupPointer((LightGroup*)0);
	if(in->numParticleSystems() > 0)
	{
		g_particleControlWidget->SetParticleControl(in->ParticleSystemAt(0));
		g_particleControlWidget->SceneChange();
	}
	else
		g_particleControlWidget->SetParticleControl((ParticleControl*)0);
}

//==============================================================================
MBASceneWidget::MBASceneWidget(MBAScene *scene, QWidget *parent, const char* name, ToolBarDock buttonBarDock )
	: MbaMainWindow(parent, name)
{
	setCaption("flow::Scene Control");
	setIcon(QPixmap((const char**)flowIcon_xpm));	
	
	mainWidget = new MBASceneWidgetMain(scene, this);
	setCentralWidget(mainWidget);
	setRightJustification(true);
	
	toolBar = new QToolBar(0, this, buttonBarDock);
	
	if(g_winStyleToolBars)
	{
		toolBar->setStyle(winStyle);
	}
	
	QLabel	*tmpStretch = 0;
	
	closeBut = new MToolButton(toolBar);
	closeBut->setAccel(ALT+'Q');
	closeBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	closeBut->setUsesBigPixmap(true);
	closeBut->setIconSet(*icoSet_close);
	
	tmpStretch = new MLabel(toolBar);
	tmpStretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	
	refreshBut = new MToolButton(toolBar);
	refreshBut->setAccel(ALT+'R');
	refreshBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	refreshBut->setUsesBigPixmap(true);
	refreshBut->setText("R");
	refreshBut->setEnabled(false);
	
	tmpStretch = new MLabel(toolBar);
	tmpStretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	
	toolBar->setStretchableWidget(tmpStretch);
	
	if(g_toolTipsEnabled)
	{
		QToolTip::add(closeBut,		"close\nalt-Q");
		QToolTip::add(refreshBut,	"refresh Scene Info\nalt-R");
	}
	
	//-- Connect
	QObject::connect(closeBut, SIGNAL(clicked()), SLOT(hide()) );
	
}

MBASceneWidget::~MBASceneWidget()
{
}

//----------------------------------------------------------
bool MBASceneWidget::setToolBar(int num, QMainWindow::ToolBarDock dock, bool newline)
{
	switch(num)
	{
		case 0:
			removeToolBar(toolBar);
			addToolBar(toolBar, 0, dock, newline);
			return true;
		default:
			return false;
	}
}

