/**
 * mbaSurfaceTabDialog.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * topmost surface properties widget, w/ tabs for Surface, Displacement & GL
 */
#include "mbaSurfaceTabDialog.h"
#include "extern.h"
#include "define.h"

#include <string.h>

#include <qtooltip.h>
#include <qkeycode.h>
#include <qmessagebox.h>

//==================================================================================================
MBASurfaceTabDialog::MBASurfaceTabDialog(QWidget *parent, const char* name, ToolBarDock buttonBarDock)
	: MbaMainWindow(parent, name)
{		
	setCaption("flow::Surface Properties");
	setIcon(QPixmap((const char**)flowIcon_xpm));
	ppRMSurface		= 0;
	ppRMDisplace	= 0;
	currentTab		= SurfaceTab;
	
	glSurfaceWidget		= new MBA_GLSurfaceWidget(this, "SurfaceTab_GLSurface");
	rmSurfaceWidget		= new MBA_RMShaderWidget(MBA_RMShader::Surface, this, "SurfaceTab_RMSurface");
	rmDisplaceWidget	= new MBA_RMShaderWidget(MBA_RMShader::Displacement, this, "SurfaceTab_RMDisplace");

	rmSurfaceWidget->setFrameStyle ( QFrame::Sunken | QFrame::WinPanel );
	rmSurfaceWidget->setLineWidth(2);
	rmDisplaceWidget->setFrameStyle ( QFrame::Sunken | QFrame::WinPanel );
	rmDisplaceWidget->setLineWidth(2);
	glSurfaceWidget->setFrameStyle ( QFrame::Sunken | QFrame::WinPanel );
	glSurfaceWidget->setLineWidth(2);
	
	surfaceTabText		= "sur&face";
	displaceTabText	= "&displacement";
	openGLTabText		= "open&GL";
	tabDialog = new MTabWidget(this);
	tabDialog->addTab(rmSurfaceWidget, surfaceTabText);
	tabDialog->addTab(rmDisplaceWidget, displaceTabText);
	tabDialog->addTab(glSurfaceWidget, openGLTabText);
	
	manipToolBar = new QToolBar(0, this, buttonBarDock, true);
	dockList.append(new QMainWindow::ToolBarDock(buttonBarDock));
	
	if(g_winStyleToolBars)
	{
		manipToolBar->setStyle(winStyle);
	}
	
	setCentralWidget(tabDialog);
	setRightJustification(true);
	
	closeBut = new MToolButton(manipToolBar);
	closeBut->setAccel(ALT+'Q');
	closeBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	closeBut->setUsesBigPixmap(true);
	closeBut->setIconSet(*icoSet_close);
	
	MLabel *stretch;
	
	stretch = new MLabel(manipToolBar);
	stretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
//	stretch->setFixedSize(ICON_SPACER, ICON_SPACER);
	
	disableBut = new MToolButton(manipToolBar);
	disableBut->setAccel(ALT+'E');
	disableBut->setToggleButton(true);
	disableBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	disableBut->setUsesBigPixmap(true);
	disableBut->setIconSet(*icoSet_dis);
	
	stretch = new MLabel(manipToolBar);
	stretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
//	stretch->setFixedSize(ICON_SPACER, ICON_SPACER);
	
	copyBut = new MToolButton(manipToolBar);
	copyBut->setAccel(CTRL+'C');
	copyBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	copyBut->setUsesBigPixmap(true);
	copyBut->setIconSet(*icoSet_copy);

	
	pasteBut = new MToolButton(manipToolBar);
	pasteBut->setAccel(CTRL+'V');
	pasteBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	pasteBut->setUsesBigPixmap(true);
	//pasteBut->setText("paste");
	pasteBut->setIconSet(*icoSet_paste);
	
	addBut = new MToolButton(manipToolBar);
	addBut->setAccel(ALT+'A');
	addBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	addBut->setUsesBigPixmap(true);
	addBut->setIconSet(*icoSet_add);
	
	delBut = new MToolButton(manipToolBar);
	delBut->setAccel(ALT+'D');
	delBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	delBut->setUsesBigPixmap(true);
	delBut->setIconSet(*icoSet_del);
	
	stretch = new MLabel(manipToolBar);
	stretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
//	stretch->setFixedSize(ICON_SPACER, ICON_SPACER);
	
	combineTest = new MToolButton(manipToolBar);
//	combineTest->setAccel(SHIFT+ALT+'T');
	combineTest->setAccel(ALT+SHIFT+'T');
	combineTest->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	combineTest->setUsesBigPixmap(true);
	combineTest->setIconSet(*icoSet_shade);
	
	stretch = new MLabel(manipToolBar);
	stretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	
	manipToolBar->setStretchableWidget(stretch);
	
	if(g_toolTipsEnabled)
	{
		QToolTip::add(closeBut,		"close (alt-Q)");
		QToolTip::add(disableBut,	"enable/disable (alt-E)");
		QToolTip::add(copyBut,		"copy (ctrl-C)");
		QToolTip::add(pasteBut,		"paste (ctrl-V)");
		QToolTip::add(addBut,		"add (alt-A)");
		QToolTip::add(delBut,		"delete (alt-D)");
		QToolTip::add(combineTest,	"test surface+displacement (alt-shift-T)");
	}
		
	//----------------------------------
	QObject::connect(closeBut, SIGNAL(clicked()), SLOT(hide()) );
	QObject::connect(disableBut, SIGNAL(toggled(bool)), SLOT(EnableProperty(bool)) );
	QObject::connect(copyBut, SIGNAL(clicked()), SLOT(Copy()) );
	QObject::connect(pasteBut, SIGNAL(clicked()), SLOT(Paste()) );
	QObject::connect(addBut, SIGNAL(clicked()), SLOT(add()) );
	QObject::connect(delBut, SIGNAL(clicked()), SLOT(Delete()) );
	
	QObject::connect(combineTest, SIGNAL(clicked()), SLOT(TestCombined()) );
	
	QObject::connect(tabDialog, SIGNAL(selected(const QString&)), SLOT(TabSelected(const QString&)) );
	
	QObject::connect(glSurfaceWidget, SIGNAL(Updated()), SIGNAL(Updated()) );
	
	//----------------------------------
	setDockEnabled( QMainWindow::TornOff, true);
	setDockEnabled( QMainWindow::Top	, true);
	setDockEnabled( QMainWindow::Bottom	, true); 
	setDockEnabled( QMainWindow::Right	, true);
	setDockEnabled( QMainWindow::Left	, true);
}

MBASurfaceTabDialog::~MBASurfaceTabDialog()
{
}

void	MBASurfaceTabDialog::SetGLSurfacePointer(MBA_GLSurface *in)	
{	
	glSurfaceWidget->SetGLSurfacePointer(in);
	
	if(currentTab == OpenGLTab)
	{
		SetDisableButton();
		SetButtonsEnabled();
	}
}

//--------------------------------------------------------------------
QToolBar*	MBASurfaceTabDialog::GetToolBar(int toolBarNum)
{
	switch(toolBarNum)
	{
		case 0:
			return manipToolBar;
		default:
			return 0;
	}
}

//----------------------------------------------------------
bool MBASurfaceTabDialog::setToolBar(int num, QMainWindow::ToolBarDock dock, bool newline)
{
	switch(num)
	{
		case 0:
			removeToolBar(manipToolBar);
			addToolBar(manipToolBar, 0, dock, newline);
			*dockList.at(0) = dock;
			return true;
		default:
			return false;
	}
}

//--------------------------------------------------------------------
void	MBASurfaceTabDialog::SetRMSurfacePointerPointer(MBA_RMShader **in, bool addDelOK)
{
	okToAddDelSurface = addDelOK;
	ppRMSurface = in;
	if (ppRMSurface)
		rmSurfaceWidget->SetRMShaderPointer(*ppRMSurface);
	else
		rmSurfaceWidget->SetRMShaderPointer(0);
		
	if(currentTab == SurfaceTab)
	{
		SetDisableButton();
		SetButtonsEnabled();
	}
}
	
//--------------------------------------------------------------------
void	MBASurfaceTabDialog::SetRMDisplacePointerPointer(MBA_RMShader **in, bool addDelOK)
{
	okToAddDelDisplace = addDelOK;
	ppRMDisplace = in;
	if (ppRMDisplace)
		rmDisplaceWidget->SetRMShaderPointer(*ppRMDisplace);
	else
		rmDisplaceWidget->SetRMShaderPointer(0);
	
	if(currentTab == DisplaceTab)
	{
		SetDisableButton();
		SetButtonsEnabled();
	}
}
	
//--------------------------------------------------------------------
void	MBASurfaceTabDialog::EnableProperty(bool disable)
{
	switch(currentTab)
	{
		case SurfaceTab:
			if (ppRMSurface)
				if (*ppRMSurface)
					(*ppRMSurface)->setEnabled(!disable);
			surfaceDisabled = disable;
			break;
		case DisplaceTab:
			if (ppRMDisplace)
				if(*ppRMDisplace)
					(*ppRMDisplace)->setEnabled(!disable);
			displaceDisabled = disable;
			break;
		case OpenGLTab:
			openGLDisabled = disable;
			break;
	}
}

//--------------------------------------------------------------------
void	MBASurfaceTabDialog::TabSelected(const QString& text)
{
//	warning("MBASurfaceTabDialog::TabSelected( %s )", text);
	if (strcmp((const char*)text, surfaceTabText) == 0)
		currentTab = SurfaceTab;
	else if (strcmp((const char*)text, displaceTabText) == 0)
		currentTab = DisplaceTab;
	else if (strcmp((const char*)text, openGLTabText) == 0)
		currentTab = OpenGLTab;
	
	SetDisableButton();
	SetButtonsEnabled();
}
	
//--------------------------------------------------------------------
void	MBASurfaceTabDialog::TestCombined()
{
	if (ppRMSurface && *ppRMSurface)
	{
		if (ppRMDisplace && *ppRMDisplace)
			rmSurfaceWidget->TestShader(*ppRMDisplace);
		else
			rmSurfaceWidget->TestShader(0);
	}
	else if (ppRMDisplace && *ppRMDisplace)
	{
		rmDisplaceWidget->TestShader(0);
	}
			
}

//--------------------------------------------------------------------
void	MBASurfaceTabDialog::SetDisableButton()
{
	switch(currentTab)
	{
		case SurfaceTab:
			if (ppRMSurface)
			{
				disableBut->setEnabled(true);
				if (*ppRMSurface)
					disableBut->setOn(!(*ppRMSurface)->Enabled());
			}
			else
				disableBut->setEnabled(false);
			break;
		case DisplaceTab:
			if (ppRMDisplace)
			{
				disableBut->setEnabled(true);
				if(*ppRMDisplace)
					disableBut->setOn(!(*ppRMDisplace)->Enabled());
			}
			else
				disableBut->setEnabled(false);
			break;
		case OpenGLTab:
			disableBut->setOn(false);
			disableBut->setEnabled(false);
			break;
	}
}

//--------------------------------------------------------------------
void	MBASurfaceTabDialog::SetButtonsEnabled()
{
	switch(currentTab)
	{
		case SurfaceTab:
//			copyBut->setEnabled(okToAddDelSurface);
//			pasteBut->setEnabled(okToAddDelSurface);
			copyBut->setEnabled(true);
			pasteBut->setEnabled(true);
			addBut->setEnabled(okToAddDelSurface);
			delBut->setEnabled(okToAddDelSurface);
			break;
		case DisplaceTab:
//			copyBut->setEnabled(okToAddDelDisplace);
//			pasteBut->setEnabled(okToAddDelDisplace);
			copyBut->setEnabled(true);
			pasteBut->setEnabled(true);
			addBut->setEnabled(okToAddDelDisplace);
			delBut->setEnabled(okToAddDelDisplace);
			break;
		case OpenGLTab:
			if(glSurfaceWidget->isValid())
			{
				copyBut->setEnabled(true);
				pasteBut->setEnabled(true);
			}
			else
			{
				copyBut->setEnabled(false);
				pasteBut->setEnabled(false);
			}
			addBut->setEnabled(false);
			delBut->setEnabled(false);
			break;
	}
	if( okToAddDelDisplace || okToAddDelSurface )
		combineTest->setEnabled(true);
	else
		combineTest->setEnabled(false);
	
}

//--------------------------------------------------------------------
void	MBASurfaceTabDialog::Paste()
{
	switch(currentTab)
	{
		case SurfaceTab:
			rmSurfaceWidget->Paste();
			break;
		case DisplaceTab:
			rmDisplaceWidget->Paste();
			break;
		case OpenGLTab:
			glSurfaceWidget->Paste();
			break;
		default:
			warning("Error MBASurfaceTabDialog::currentTab is set to illegal value of %d", currentTab);
	}
}

//--------------------------------------------------------------------
void	MBASurfaceTabDialog::Copy()
{
	switch(currentTab)
	{
		case SurfaceTab:
			rmSurfaceWidget->Copy();
			break;
		case DisplaceTab:
			rmDisplaceWidget->Copy();
			break;
		case OpenGLTab:
			glSurfaceWidget->Copy();
			break;
		default:
			warning("Error MBASurfaceTabDialog::currentTab is set to illegal value of %d", currentTab);
	}
}
	
//--------------------------------------------------------------------
void	MBASurfaceTabDialog::add()
{
	switch(currentTab)
	{
		case SurfaceTab:
			if ( okToAddDelSurface )
				if (ppRMSurface)
					if ( !(*ppRMSurface) )
					{
						*ppRMSurface = new MBA_RMShader(MBA_RMShader::Surface);
						rmSurfaceWidget->SetRMShaderPointer(*ppRMSurface);
					}
			break;
		case DisplaceTab:
			if ( okToAddDelDisplace )
				if (ppRMDisplace)
					if ( !(*ppRMDisplace) )
					{
						*ppRMDisplace = new MBA_RMShader(MBA_RMShader::Displacement);
						rmDisplaceWidget->SetRMShaderPointer(*ppRMDisplace);
					}
			break;
		case OpenGLTab:
			break;
		default:
			warning("Error MBASurfaceTabDialog::currentTab is set to illegal value of %d", currentTab);
	}
}

//--------------------------------------------------------------------
void	MBASurfaceTabDialog::Delete()
{
	switch(currentTab)
	{
		case SurfaceTab:
			if ( okToAddDelSurface )
				if (ppRMSurface)
					if ( *ppRMSurface )
					{	
						switch( QMessageBox::warning( this, "confirm delete", "delete surface shader?", "ok", "cancel" )) 
						{
							case 0:
								rmSurfaceWidget->SetRMShaderPointer(0);
								delete *ppRMSurface;
								*ppRMSurface = 0;
								break;
						}
					}
			break;
		case DisplaceTab:
			if ( okToAddDelDisplace )
				if (ppRMDisplace)
					if ( *ppRMDisplace )
					{
						switch( QMessageBox::warning( this, "confirm delete", "delete displacement shader?", "ok", "cancel" )) 
						{
							case 0:
								rmDisplaceWidget->SetRMShaderPointer(0);
								delete *ppRMDisplace;
								*ppRMDisplace = 0;
								break;
						}
					}
			break;
		case OpenGLTab:
			break;
		default:
			warning("Error MBASurfaceTabDialog::currentTab is set to illegal value of %d", currentTab);
	}
		
}

//--------------------------------------------------------------------
void	MBASurfaceTabDialog::reloadShaders()
{
	rmDisplaceWidget->SetRMShaderPointer(0);
	rmSurfaceWidget->SetRMShaderPointer(0);
	rmSurfaceWidget->PopulateShaderList();
	rmDisplaceWidget->PopulateShaderList();
//	rmSurfaceWidget->setEnabled(false);
//	rmDisplaceWidget->setEnabled(false);
}

