/**
 * numInTriple.cpp
 * Copyright 1999 by Mark B. Allan
 * 
 * three numerical inputs side-by-side
 */
#include "numInTriplef.h"
#include "numInputValid.h"
#include <limits.h>
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif

#include <qevent.h>
#include <qapp.h>
#include <qclipboard.h>
#include <qtimer.h>
 
#include "extern.h" 

static int totalWidth = g_numInputWidth;

//=======================================================================
//											NumInTriplef
//=======================================================================
NumInTriplef::NumInTriplef ( QWidget* parent, const char* name )
	: QWidget (parent, name)
{
	totalWidth		= g_numInputWidth;
	height			= 20;
	
	val0	= 0.0;
	val1	= 0.0;
	val2	= 0.0;
	
	int	editSize = totalWidth/3;
	
	input0Edit	= new NumInTriplefEdit (0, this);
	input0Edit->setMaxLength(5);
	input0Edit->setFont(*g_font_M);
	height = input0Edit->sizeHint().height();
	input0Edit->setFixedSize(editSize, height-2);
	input1Edit	= new NumInTriplefEdit (1, this);
	input1Edit->setMaxLength(5);
	input1Edit->setFont(*g_font_M);
	input1Edit->setFixedSize(editSize, height-2);
	input2Edit	= new NumInTriplefEdit (2, this);
	input2Edit->setMaxLength(5);
	input2Edit->setFont(*g_font_M);
	input2Edit->setFixedSize(editSize, height-2);
	
	
	//-- Connect -----------------------
	QObject::connect(input0Edit, SIGNAL(ValueChanged(float)), this, SIGNAL(Value0Changed(float)) );
	QObject::connect(input1Edit, SIGNAL(ValueChanged(float)), this, SIGNAL(Value1Changed(float)) );
	QObject::connect(input2Edit, SIGNAL(ValueChanged(float)), this, SIGNAL(Value2Changed(float)) );

	input0Edit->move(0*editSize, 0);
	input1Edit->move(1*editSize, 0);
	input2Edit->move(2*editSize, 0);
	
	indicate0 = new QFrame(this);
	indicate1 = new QFrame(this);
	indicate2 = new QFrame(this);
	indicate0->setFixedSize(editSize, 2);
	indicate1->setFixedSize(editSize, 2);
	indicate2->setFixedSize(editSize, 2);
	indicate0->move(0*editSize, height-2);
	indicate1->move(1*editSize, height-2);
	indicate2->move(2*editSize, height-2);
	SetIndicator(-1);
	
	setFixedSize(totalWidth, height);
	setEnabled(true);
//	indicate0->setFocusPolicy(QWidget::StrongFocus);
//	setFocusPolicy(QWidget::StrongFocus);
}

NumInTriplef::~NumInTriplef()
{
}

//-----------------------------------------------------------------------
void	NumInTriplef::SetIndicator (int index)	
{ 
	indicate0->setBackgroundColor(palette().normal().background());
	indicate1->setBackgroundColor(palette().normal().background());
	indicate2->setBackgroundColor(palette().normal().background());
	switch(index)
	{
		case 0:
			indicate0->setBackgroundColor(QColor(255, 0, 0));
			break;
		case 1:
			indicate1->setBackgroundColor(QColor(255, 0, 0));
			break;
		case 2:
			indicate2->setBackgroundColor(QColor(255, 0, 0));
			break;
	}	
}

//-----------------------------------------------------------------------
void	NumInTriplef::SetHeight (int h)	
{ 
	setFixedHeight(h);
	input0Edit->setFixedHeight(h);
}

///-----------------------------------------------------------------------
void	NumInTriplef::SetTotalWidth (int w)	
{ 
	totalWidth = w;
	int	editSize = totalWidth/3;
	input0Edit->setFixedSize(editSize, height);
	input1Edit->setFixedSize(editSize, height);
	input2Edit->setFixedSize(editSize, height);
	input0Edit->move(0*editSize, 0);
	input1Edit->move(1*editSize, 0);
	input2Edit->move(2*editSize, 0);
	setFixedWidth(w); 
}

//-----------------------------------------------------------------------
void	NumInTriplef::setEnabled (bool status)
{
	active = status;
	input0Edit->setEnabled(status);
	input1Edit->setEnabled(status);
	input2Edit->setEnabled(status);
	if (status)
	{
		input0Edit->SetValue(val0);
		input1Edit->SetValue(val1);
		input2Edit->SetValue(val2);
	}
}

//-----------------------------------------------------------------------
float	NumInTriplef::SetValueSolid (int index, float in)
{
	switch(index)
	{
		case 0:
			val0	= in;
			input0Edit->SetValue (val0);
			return val0;
		case 1:
			val1	= in;
			input1Edit->SetValue (val1);
			return val1;
		case 2:
			val2	= in;
			input2Edit->SetValue (val2);
			return val2;
	}
	return -1.0;
}

//-----------------------------------------------------------------------
void	NumInTriplef::SetValueAll (float x, float y, float z)
{
	val0	= x;
	val1	= y;
	val1	= z;
	input0Edit->SetValue (x);
	input1Edit->SetValue (y);
	input2Edit->SetValue (z);
}

//-----------------------------------------------------------------------
float	NumInTriplef::SetValue0 (float in)
{
	if ( in != val0 )
	{
		val0	= in;
		input0Edit->SetValue (val0);
		emit Value0Changed (val0);
	}
	return val0;
}

//-----------------------------------------------------------------------
float	NumInTriplef::SetValue1 (float in)
{
	if ( in != val1 )
	{
		val1	= in;
		input1Edit->SetValue (val1);
		emit Value1Changed (val1);
	}
	return val1;
}

//-----------------------------------------------------------------------
float	NumInTriplef::SetValue2 (float in)
{
	if ( in != val2 )
	{
		val2	= in;
		input2Edit->SetValue (val2);
		emit Value2Changed (val2);
	}
	return val2;
}

//=======================================================================
//											NumInTriplef Line Edit
//=======================================================================
NumInTriplefEdit::NumInTriplefEdit ( int in, QWidget* parent, const char* name )
	: QLineEdit ( parent, name )
{
	longInput = false;
	index = in;
	xloc = (totalWidth/3)*index;
	lastGoodValue = 0.0;

	setEchoMode( QLineEdit::Normal );
	setFrame( true );
	
	currentValidator = new NumInputDblValidator (this);
	setValidator ( currentValidator );	
	setText( "0" );

	QObject::connect (this, SIGNAL(returnPressed()), SLOT(InputComplete()) );
}

NumInTriplefEdit::~NumInTriplefEdit()
{
}

//-----------------------------------------------------------------------
void	NumInTriplefEdit::mousePressEvent(QMouseEvent *ev)
{
	if(ev->button() == RightButton)
	{
		if(!longInput)
		{
			longInput = true;
			move(0, 0);
			setFixedWidth(totalWidth);
			setMaxLength(10);
			raise();
			((NumInTriplef*)parent())->SetIndicator(index);
		}
		else
		{	
			longInput = false;
			move((int)xloc, 0);
			setFixedWidth(totalWidth/3);
			setMaxLength(5);
			((NumInTriplef*)parent())->SetIndicator(-1);
		}
	}
}

//-----------------------------------------------------------------------
void	NumInTriplefEdit::focusInEvent ( QFocusEvent *fE )
{
	QLineEdit::focusInEvent(fE);
	QCString	tmpString;
	tmpString.setNum(lastGoodValue);
	setText(tmpString);
}

//-----------------------------------------------------------------------
void	NumInTriplefEdit::focusOutEvent ( QFocusEvent *fE )
{
	InputComplete();
	QLineEdit::focusOutEvent(fE);
	if(longInput)
	{
		move((int)xloc, 0);
		setFixedWidth(totalWidth/3);
		setMaxLength(5);
		((NumInTriplef*)parent())->SetIndicator(-1);
		longInput = false;
	}
}

//-----------------------------------------------------------------------
void	NumInTriplefEdit::setEnabled (bool status)
{
	QCString tempString;
	
	active = status;
	if (status)
	{
		tempString.setNum(lastGoodValue);
		setText(tempString);
	}
	else
	{
		setText ("-");
	}	
	QLineEdit::setEnabled (status);
}

//-----------------------------------------------------------------------
void	NumInTriplefEdit::InputComplete ()
{
	QCString	tmpString;
	float	tmpDbl;
	bool	ok;
		
	tmpString = text();
//	if (tmpString.find("-") == 0)
//		return;
	tmpDbl = tmpString.toFloat(&ok);
	if (ok)
	{
		//warning("NumInTriplefEdit::InputComplete () -- OK");
		lastGoodValue = tmpDbl;
	}
	else
		warning("NumInTriplefEdit::InputComplete () -- not OK (\"%s\"", (const char*)text());
	
	tmpString.setNum(lastGoodValue);
	setText (tmpString);
	clearFocus();
//	((QWidget*)parent())->setFocus();
	emit ValueChanged(lastGoodValue);
}
	
//-----------------------------------------------------------------------
void	NumInTriplefEdit::SetValue (float in)
{
	lastGoodValue = in;
	QCString	tmp;
	tmp.setNum(in);
	setText (tmp);
}
