/**
 * numInput.h
 * Copyright 1999 by Mark B. Allan
 * 
 * numerical input w/ 'spinbox' label
 */
#ifndef NumInput_included
#define NumInput_included

#include "mwidgets.h"
#include <qvalidator.h> 
#include <qlineedit.h>
#include <qlayout.h>
#include <qlabel.h>

//#define		NUM_INPUT_INT	0
//#define		NUM_INPUT_DBL	1

class NumInputEdit;
class NumInputLabel;

/**
 * num input widget
 */
//=======================================================================
class NumInput : public QWidget
{
    Q_OBJECT
public:

    NumInput	( int type = Double, QWidget* parent = 0, const char* name = 0);
    ~NumInput	();
	
	enum NumInputType { Int, Double, Float };
	
public:
	inline	void		UpdateCurrentValue(double in)	{	currentValue = in;	}
	inline	void		IntifyCurrentValue()			{	currentValue = (double)(int)currentValue;	}
	inline	int			GetIntValue()	{	if(active)return currentIntValue;	else return 0;	}
	inline	double		GetDblValue()	{	if(active)return currentValue	;	else return 0.0;}
	inline	int			intVal()		{	if(active)return currentIntValue;	else return 0;	}
	inline	double		dblVal()		{	if(active)return currentValue;		else return 0.0;}

	void		add(double in);
	
	void		SetType (int t);
	void		SetLabel (const char *);
	void		SetLabelWidth (int w);
	void		SetTotalWidth (int w);
	void		SetHeight(int h);
	void		InputComplete();
	
	void		SetFocus();
	
public slots:
	float		SetValue (float);
	float		SetValueSolid (float);
	double		SetValue (double);
	double		SetValueSolid (double);
	int			SetValue (int);
	int			SetValueSolid (int);
	void		setEnabled (bool);
	void		setButtonMode(bool);
	void		toggleButtonMode();
	
signals:
	void		ValueChanged(int);
	void		ValueChanged(float);
	void		ValueChanged(double);
	void		Updated();
	
protected:
	void		resizeEvent(QResizeEvent *);
	
protected:
	bool			active;
	NumInputEdit	*inputEdit;
	NumInputLabel	*inputLabel;
	MToolButton		*inputButton;
	int				labelWidth;
	int				editWidth;
	int				editMinWidth;
	int				height;
	
	double			currentValue;
	int				currentIntValue;
	int				type;
	
	bool			buttonMode;
};
	
/**
 * num input label
 */
//=======================================================================
class NumInputLabel : public QLabel
{
	Q_OBJECT

public:
	NumInputLabel	( NumInput* p, const char* name = 0 );
	~NumInputLabel	();
	

	void			SetType (int t)		{	type = t;	}
	void			SetActive (bool);
	
	QCString			lastText;

protected:
	void			focusInEvent ( QFocusEvent * );
	
	void 			mousePressEvent  (QMouseEvent *msEv);
	void 			mouseMoveEvent  (QMouseEvent *msEv);
	void 			mouseReleaseEvent  (QMouseEvent *msEv);
	
protected:
	NumInput	*numInput;
	bool	active;
	int		lastXpos;
	int		type;
	int		moved;
	int		triggered;
	
	int		ctrlKey;
	int		altKey;
	int		shftKey;
	
//	QPoint	mousePos;
//	int		xOrig;
};
	
/**
 * num input line edit
 */
//=======================================================================
class NumInputEdit : public QLineEdit
{
    Q_OBJECT
	
public:

    NumInputEdit	( QWidget* parent = NULL, const char* name = NULL );
    ~NumInputEdit	();
	
	double		GetLastGoodValue ();
	void		SetType (int t);
	void		SetActive (bool);
	void		focusOutEvent ( QFocusEvent * );

	void		focusInEvent ( QFocusEvent * );
	
signals:
	void		ValueChanged(int);
	void		ValueChanged(double);
	void		ValueChanged(float);
	
public slots:
	void		SetValue (int);
	void		SetValue (double);
	void		SetValue (float);

protected slots:
	void		InputComplete();

protected:
	bool		active;
	double		lastGoodValue;
	double		i;
	int			type;
	QValidator	*currentValidator;
};


#endif // NumInput_included
