#include "numInputValid.h"
#include "numInput.h"
#include <limits.h>

//-----------------------------------------------------------------------
//											Line Edit Validator	: Double
//-----------------------------------------------------------------------
NumInputDblValidator::NumInputDblValidator (QWidget* parent, const char* name )
	: QValidator (parent, name)
{
	max =  (INT_MAX-1);
	min = -(INT_MAX-1);
}

QValidator::State	NumInputDblValidator::validate ( QString & input, int & ) const
{
	bool	ok;
	double	tmp = input.toDouble( &ok );
	// don't block input if field is empty
	if ( input.length() == 0 )
		return		QValidator::Valid;
	// don't block input if first character is a '-' and it is the only character
	else if ( input.find('-') == 0 && input.length() == 1 )
 		return QValidator::Valid;
	// don't block input if first character is a '.' and it is the only character or if first characters are '-.'
	else if ( (input.find('.') == 0 && input.length() == 1 ) ||
			  (input.find('-') == 0 && input.find('.') == 1 && input.length() == 2 ) )
 		return QValidator::Valid;
	// do check on the number
	else if ( !ok )
		return	QValidator::Invalid;
	else if ( tmp < min || tmp > max )
		return	QValidator::Valid;
	else
		return	QValidator::Acceptable;
}

NumInputDblValidator::~NumInputDblValidator()
{
}

//------------------------------------------------------------------------
void	NumInputDblValidator::fixup(QString & input) const
{
	input.setNum ( ((NumInputEdit*)parent())->GetLastGoodValue() );
}

//------------------------------------------------------------------------
void	NumInputDblValidator::SetMax(double in)
{
	max = in;
}
void	NumInputDblValidator::SetMin(double in)
{
	min = in;
}

//-----------------------------------------------------------------------
//											Line Edit Validator	: Integer
//-----------------------------------------------------------------------
NumInputIntValidator::NumInputIntValidator (QWidget* parent, const char* name )
	: QValidator (parent, name)
{
    max =  (INT_MAX-1);
    min = -(INT_MAX-1);
}

QValidator::State	NumInputIntValidator::validate ( QString & input, int & ) const
{
	bool ok;
	int tmp = input.toInt ( &ok );
	// don't block input if field is empty 
	if ( input.length() == 0 )
		return		QValidator::Valid;
	// don't block input if first character is a '-' and it is the only character
	else if ( input.find('-') == 0 && input.length() == 1 )
 		return QValidator::Valid;
	// do check on the number
	else if ( !ok )
		return	QValidator::Invalid;
	else if ( tmp < min || tmp > max )
		return	QValidator::Valid;
	else
		return	QValidator::Acceptable;
}

NumInputIntValidator::~NumInputIntValidator()
{
}

//------------------------------------------------------------------------
void	NumInputIntValidator::fixup(QString & input) const
{
	input.setNum ( (int)((NumInputEdit*)parent())->GetLastGoodValue() );
}

//------------------------------------------------------------------------
void	NumInputIntValidator::SetMax(int in)
{
	max = in;
}
void	NumInputIntValidator::SetMin(int in)
{
	min = in;
}


