#include "pAction.h"

#include "glPrimitives.h"
#include "compatibility.h"
#include "GL/gl.h"
#include <particle/general.h>
#include <math.h>
#include "extern.h"
#include "ri.h"
#include "noise.h"
#include "pPolygonObject.h"

static float temp = 0.0;

int FlowParticleAction::icount = 0;

extern _ParticleState _ps;

float FlowParticleAction::dtAdjust = 1.0;

/**
 * default constructor
 */
//==============================================================================
FlowParticleAction::FlowParticleAction(	)
{
	icount++;
	int	i;
	
	glDomain	= 0;
	attributesChanged = false;
//FIXME -  temporary...
	glDomain = new GLDomain();
	glListOnce = true;
//FIXME - 
	glSurf		= 0;
	rmSurf		= 0;
	rmDisp		= 0;
	baseType	= P_Undefined;
	type		= Undefined;
	
	for (i = 0; i < PA_NUM_FLOATS; i++)
	{
		floats[i] = 0.0f;
	}
	for (i = 0; i < PA_NUM_DOMAINS; i++)
	{
		domains[i] = PDPoint;
	}
	for (i = 0; i < PA_NUM_BOOLS; i++)
	{
		bools[i] = false;
	}

	numFloats_v		= 0;
	numDomains_v	= 0;
	numBools_v		= 0;
	
	enabled = true;
}

FlowParticleAction::~FlowParticleAction()
{
	icount--;
	//warning("~FlowParticleAction(%d) -- %20s %20s", icount, (const char*)actionName, (const char*)actionType );
	delete glSurf;
	delete rmSurf;
	delete rmDisp;
	delete glDomain;
}

/**
 * copy constructor
 */
//--------------------------------------------------------------------
FlowParticleAction::FlowParticleAction(FlowParticleAction& old)
{
	icount++;
	int i;
	actionName	= (const char*)old.Name();
	actionType	= (const char*)old.ActionType();
	
	attributesChanged = false;
	glDomain	= 0;
	if (old.glDomain)
	{
//FIXME -  temporary...
		glDomain = new GLDomain();
		glListOnce = true;
	}
//FIXME - 	
	//--
	if (old.GLSurface())
		glSurf	= new MBA_GLSurface( *(old.GLSurface()) );
	else
		glSurf = 0;
	//--
	if (old.RMSurface())
		rmSurf	= new MBA_RMShader( *(old.RMSurface()) );
	else
		rmSurf = 0;
	//--
	if (old.RMDisplace())
		rmDisp	= new MBA_RMShader( *(old.RMDisplace()) );
	else
		rmDisp = 0;
	//--
	for (i = 0; i < PA_NUM_FLOATS; i++)
	{
		floats[i]		= *(old.Float(i));
		floatName[i]	= (const char*)old.FloatName(i);
	}
	for (i = 0; i < PA_NUM_DOMAINS; i++)
	{
		domains[i]		= *(old.Domain(i));
		domainName[i]	= (const char*)old.DomainName(i);
	}
	for (i = 0; i < PA_NUM_BOOLS; i++)
	{
		bools[i] = *(old.Bool(i));
		boolName[i]	= (const char*)old.BoolName(i);
	}
	numFloats_v		= old.numFloats();
	numBools_v		= old.NumBools();
	numDomains_v	= old.NumDomains();
	
	enabled = old.Enabled();
	
//	warning ("numFloats = %d", numFloats);
//	warning ("NumBools  = %d", numBools);
//	warning ("NumDomains= %d", numDomains);
}

bool	FlowParticleAction::SetFloat( int index, float val )
{
	if (index < (int)numFloats_v )
	{
		floats[index] = val;
		return true;
	}
	return false;
}

//--------------------------------------------------------------------
void FlowParticleAction::Print()
{
	int i;
	warning("-- FLOATS --");
	for (i = 0; i < PA_NUM_FLOATS; i++)
		warning("%-12s floats[%02d] = %f\t%d", (const char*)FloatName(i), i, (float)floats[i], floats[i].Animation());
	warning("-- DOMAINS --");
	for (i = 0; i < PA_NUM_DOMAINS; i++)
	{
		warning("%-12s domains[%02d] = %d", (const char*)DomainName(i), i, (int)domains[i].Type());
		warning("\tA0 %-12s = %f\t%d", (const char*)domains[i].FloatName(0), domains[i].PFloatA0()->Value(), domains[i].PFloatA0()->Animation() ) ;
		warning("\tA1 %-12s = %f\t%d", (const char*)domains[i].FloatName(1), domains[i].PFloatA1()->Value(), domains[i].PFloatA1()->Animation() ) ;
		warning("\tA2 %-12s = %f\t%d", (const char*)domains[i].FloatName(2), domains[i].PFloatA2()->Value(), domains[i].PFloatA2()->Animation() ) ;
		warning("\tA3 %-12s = %f\t%d", (const char*)domains[i].FloatName(3), domains[i].PFloatA3()->Value(), domains[i].PFloatA3()->Animation() ) ;
		warning("\tA4 %-12s = %f\t%d", (const char*)domains[i].FloatName(4), domains[i].PFloatA4()->Value(), domains[i].PFloatA4()->Animation() ) ;
		warning("\tA5 %-12s = %f\t%d", (const char*)domains[i].FloatName(5), domains[i].PFloatA5()->Value(), domains[i].PFloatA5()->Animation() ) ;
		warning("\tA6 %-12s = %f\t%d", (const char*)domains[i].FloatName(6), domains[i].PFloatA6()->Value(), domains[i].PFloatA6()->Animation() ) ;
		warning("\tA7 %-12s = %f\t%d", (const char*)domains[i].FloatName(7), domains[i].PFloatA7()->Value(), domains[i].PFloatA7()->Animation() ) ;
		warning("\tA8 %-12s = %f\t%d", (const char*)domains[i].FloatName(8), domains[i].PFloatA8()->Value(), domains[i].PFloatA8()->Animation() ) ;
	}
	warning("-- BOOLS --");
	for (i = 0; i < PA_NUM_BOOLS; i++)
	{
		warning("%-12s bools[%02d] = %d", (const char*)BoolName(i), i, (bool)bools[i]);
	}

}

//--------------------------------------------------------------------
void FlowParticleAction::setDomainA0(int index, float val)
{	if(index < PA_NUM_FLOATS)	domains[index].SetA0( val );	attributesChanged = true;	}
void FlowParticleAction::setDomainA1(int index, float val)
{	if(index < PA_NUM_FLOATS)	domains[index].SetA1( val );	attributesChanged = true;	}
void FlowParticleAction::setDomainA2(int index, float val)
{	if(index < PA_NUM_FLOATS)	domains[index].SetA2( val );	attributesChanged = true;	}
void FlowParticleAction::setDomainA3(int index, float val)
{	if(index < PA_NUM_FLOATS)	domains[index].SetA3( val );	attributesChanged = true;	}
void FlowParticleAction::setDomainA4(int index, float val)
{	if(index < PA_NUM_FLOATS)	domains[index].SetA4( val );	attributesChanged = true;	}
void FlowParticleAction::setDomainA5(int index, float val)
{	if(index < PA_NUM_FLOATS)	domains[index].SetA5( val );	attributesChanged = true;	}
void FlowParticleAction::setDomainA6(int index, float val)
{	if(index < PA_NUM_FLOATS)	domains[index].SetA6( val );	attributesChanged = true;	}
void FlowParticleAction::setDomainA7(int index, float val)
{	if(index < PA_NUM_FLOATS)	domains[index].SetA7( val );	attributesChanged = true;	}
void FlowParticleAction::setDomainA8(int index, float val)
{	if(index < PA_NUM_FLOATS)	domains[index].SetA8( val );	attributesChanged = true;	}

//--------------------------------------------------------------------
QCString FlowParticleAction::DisabledName()
{
	QCString tmpString = (const char*)Name();
	tmpString.prepend("( ");
	tmpString.append(" )");
	return tmpString;
}
	
//--------------------------------------------------------------------
void FlowParticleAction::Execute(int)
{
}
	
//--------------------------------------------------------------------
void FlowParticleAction::drawGL(int)
{
}
	
//--------------------------------------------------------------------
void FlowParticleAction::drawGLFill(int)
{
}
	
//--------------------------------------------------------------------
void FlowParticleAction::drawGLThin(int)
{
}
	
//--------------------------------------------------------------------
void FlowParticleAction::drawRIB(int)
{
}

//--------------------------------------------------------------------
void FlowParticleAction::SaveToFile(FILE* outfile, int tabs)
{
	int		i;
	int		j;
	bool	tmpBool;
	char	buffer[128];
	PDomain	*domain;

	
	char	*walker = buffer;
	for (i = 0; i < tabs; i++)
	{	*walker = '\t';	*(walker+1) = '\0';	walker++;	}
	
	switch(baseType)
	{
		case P_Undefined:
			return;
		case P_State:
			sprintf(walker, "\tPState %s\n", ActionType() );
			break;
		case P_Action:
			sprintf(walker, "\tPAction %s\n", ActionType() );
			break;
		case P_PolyAction:
			sprintf(walker, "\tPMetaAction %s\n", ActionType() );
			break;
		case P_Geometry:
			sprintf(walker, "\tPGeometry %s\n", ActionType() );
			break;
	}
	fputs(buffer, outfile);
		
	sprintf(walker, "\t{\n" );
	fputs(buffer, outfile);
	sprintf(walker, "\t\tEnabled	%d\n", (int)enabled);
	fputs(buffer, outfile);
	sprintf(walker, "\t\tNAME	%s\n", (const char*)Name() );
	fputs(buffer, outfile);
	for (j = 0; j < (int)numFloats(); j++)
	{
		sprintf(walker, "\t\tFLOAT	%s\n", (const char*)FloatName(j) );
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", (float)*(Float(j)), Float(j)->Animation() );
		fputs(buffer, outfile);
	}
	for (j = 0; j < (int)NumDomains(); j++)
	{
		domain = Domain(j);
		sprintf(walker, "\t\tDOMAIN	%s\n", (const char*)DomainName(j));
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%d\n", (int)domain->Type());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A0(), domain->PFloatA0()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A1(), domain->PFloatA1()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A2(), domain->PFloatA2()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A3(), domain->PFloatA3()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A4(), domain->PFloatA4()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A5(), domain->PFloatA5()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A6(), domain->PFloatA6()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A7(), domain->PFloatA7()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A8(), domain->PFloatA8()->Animation());
		fputs(buffer, outfile);
	}
	for (j = 0; j < (int)NumBools(); j++)
	{
		tmpBool = (*(Bool(j))).Val();
		sprintf(walker, "\t\tBOOL	%s\n", (const char*)BoolName(j) );
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%d\n", tmpBool );
		fputs(buffer, outfile);
	}

	if (GLSurface())
		GLSurface()->SaveToFile(outfile, tabs+2);

	if (RMSurface())
		RMSurface()->SaveToFile(outfile, tabs+2);

	if (RMDisplace())
		RMDisplace()->SaveToFile(outfile, tabs+2);

	sprintf(walker, "\t}\n" );
	fputs(buffer, outfile);
}
	
//--------------------------------------------------------------------
void FlowParticleAction::SetName(const char* in, bool /*FIXME - updateRequired*/)
{
	actionName = in;
//	if (updateRequired)
//		emit Updated();
}

void FlowParticleAction::SetFloatName(int i, const char* in, bool /*FIXME - updateRequired*/)
{
	if (i < (int)numFloats_v)
	{
		floatName[i] = in;
//		if (updateRequired)
//			emit Updated();
	}
}

void FlowParticleAction::SetDomainName(int i, const char* in, bool /*FIXME - updateRequired*/)
{
	if (i < (int)numDomains_v)
	{
		domainName[i] = in;
//		if (updateRequired)
//			emit Updated();
	}
}

void FlowParticleAction::SetBoolName(int i, const char* in, bool /*FIXME - updateRequired*/)
{
	if (i < (int)numBools_v)
	{
		boolName[i] = in;
//		if (updateRequired)
//			emit Updated();
	}
}

//--------------------------------------------------------------------
void	FlowParticleAction::WriteCode(FILE* outfile)
{
	char	buffer[128];
	
	sprintf(buffer, "//\n// \'%s::WriteCode\' has not yet been implemented.\n//\n\n", (const char*)actionType);
	fputs(buffer, outfile);
}

//--------------------------------------------------------------------
//--------------------------------------------------------------------
void	FlowParticleAction::DrawPointGL()
{}
//--------------------------------------------------------------------
void	FlowParticleAction::DrawLineGL()
{}
//--------------------------------------------------------------------
void	FlowParticleAction::DrawTriangleGL(float detail, float depth)
{
	// This needs work. How to handle clockwise/counterclockwise?
	// How to handle normal?
	detail = detail;
	depth = depth;
	
	float	vert1[3] = { domains[0].A0(), domains[0].A1(), domains[0].A2() };
	float	vert2[3] = { domains[0].A3(), domains[0].A4(), domains[0].A5() };
	float	vert3[3] = { domains[0].A6(), domains[0].A7(), domains[0].A8() };
	float	norm[3];
	
	::calcNormal(norm, vert1, vert2, vert3);
	
	glNormal3fv(norm);
	glBegin(GL_TRIANGLES);
		glVertex3f( domains[0].A0(), domains[0].A1(), domains[0].A2() );
		glVertex3f( domains[0].A3(), domains[0].A4(), domains[0].A5() );
		glVertex3f( domains[0].A6(), domains[0].A7(), domains[0].A8() );
	glEnd();
}

//--------------------------------------------------------------------
void	FlowParticleAction::DrawPlaneGL(float detail, float depth)
{
	// Get corners A, B, C and D
	float	A0[3] = {	domains[0].A0(), 
						domains[0].A1(), 
						domains[0].A2() };
	float	B0[3] = {	A0[0] + domains[0].A6(),
					 	A0[1] + domains[0].A7(),
					 	A0[2] + domains[0].A8() };
	float	C0[3] = {	B0[0] + domains[0].A3(),
						B0[1] + domains[0].A4(),
						B0[2] + domains[0].A5() };
	float	D0[3] = {	A0[0] + domains[0].A3(), 
						A0[1] + domains[0].A4(), 
						A0[2] + domains[0].A5() };

	float	p0[2][2][3];
	p0[0][0][0] = C0[0];	p0[0][0][1] = C0[1];	p0[0][0][2] = -C0[2];
	p0[0][1][0] = B0[0];	p0[0][1][1] = B0[1];	p0[0][1][2] = -B0[2];
	p0[1][1][0] = A0[0];	p0[1][1][1] = A0[1];	p0[1][1][2] = -A0[2];
	p0[1][0][0] = D0[0];	p0[1][0][1] = D0[1];	p0[1][0][2] = -D0[2];
	
	RglBilinearPatch(p0, (int)detail);
	//-- if plane has depth, draw it...
	if (depth != 0.0f)
	{
		MVertex	v1(D0[0], D0[1], D0[2]);
		MVertex v2(A0[0], A0[1], A0[2]);
		MVertex v3(B0[0], B0[1], B0[2]);
		MVertex normal;
		normal.CreateNormal(v1, v2, v3);
		normal.setX(normal.X()*depth);
		normal.setY(normal.Y()*depth);
		normal.setZ(normal.Z()*depth);
	
		float	A1[3];
		float	B1[3];
		float	C1[3];
		float	D1[3];
		A1[0] = A0[0]-normal.X();
		A1[1] = A0[1]-normal.Y();
		A1[2] = A0[2]-normal.Z();
		B1[0] = B0[0]-normal.X();
		B1[1] = B0[1]-normal.Y();
		B1[2] = B0[2]-normal.Z();
		C1[0] = C0[0]-normal.X();
		C1[1] = C0[1]-normal.Y();
		C1[2] = C0[2]-normal.Z();
		D1[0] = D0[0]-normal.X();
		D1[1] = D0[1]-normal.Y();
		D1[2] = D0[2]-normal.Z();

		p0[0][0][0] = C1[0];	p0[0][0][1] = C1[1];	p0[0][0][2] = -C1[2];
		p0[0][1][0] = D1[0];	p0[0][1][1] = D1[1];	p0[0][1][2] = -D1[2];
		p0[1][1][0] = A1[0];	p0[1][1][1] = A1[1];	p0[1][1][2] = -A1[2];
		p0[1][0][0] = B1[0];	p0[1][0][1] = B1[1];	p0[1][0][2] = -B1[2];
		float	p1[2][2][3];
		float	p2[2][2][3];
		float	p3[2][2][3];
		float	p4[2][2][3];
		p1[0][0][0] = B0[0];	p1[0][0][1] = B0[1];	p1[0][0][2] = -B0[2];
		p1[0][1][0] = B1[0];	p1[0][1][1] = B1[1];	p1[0][1][2] = -B1[2];
		p1[1][1][0] = A1[0];	p1[1][1][1] = A1[1];	p1[1][1][2] = -A1[2];
		p1[1][0][0] = A0[0];	p1[1][0][1] = A0[1];	p1[1][0][2] = -A0[2];
		p2[0][0][0] = C0[0];	p2[0][0][1] = C0[1];	p2[0][0][2] = -C0[2];
		p2[0][1][0] = C1[0];	p2[0][1][1] = C1[1];	p2[0][1][2] = -C1[2];
		p2[1][1][0] = B1[0];	p2[1][1][1] = B1[1];	p2[1][1][2] = -B1[2];
		p2[1][0][0] = B0[0];	p2[1][0][1] = B0[1];	p2[1][0][2] = -B0[2];
		p3[0][0][0] = D0[0];	p3[0][0][1] = D0[1];	p3[0][0][2] = -D0[2];
		p3[0][1][0] = D1[0];	p3[0][1][1] = D1[1];	p3[0][1][2] = -D1[2];
		p3[1][1][0] = C1[0];	p3[1][1][1] = C1[1];	p3[1][1][2] = -C1[2];
		p3[1][0][0] = C0[0];	p3[1][0][1] = C0[1];	p3[1][0][2] = -C0[2];
		p4[0][0][0] = A0[0];	p4[0][0][1] = A0[1];	p4[0][0][2] = -A0[2];
		p4[0][1][0] = A1[0];	p4[0][1][1] = A1[1];	p4[0][1][2] = -A1[2];
		p4[1][1][0] = D1[0];	p4[1][1][1] = D1[1];	p4[1][1][2] = -D1[2];
		p4[1][0][0] = D0[0];	p4[1][0][1] = D0[1];	p4[1][0][2] = -D0[2];
		
		int	sideDetail = (int)detail/2;
		RglBilinearPatch(p0, (int)detail);
		RglBilinearPatch(p1, sideDetail);
		RglBilinearPatch(p2, sideDetail);
		RglBilinearPatch(p3, sideDetail);
		RglBilinearPatch(p4, sideDetail);
	}
	
//	float	emis[4] = { 1.00, 1.00, 1.00, 1.00 };
//	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION,	emis);
//	if(g_glThick) glPointSize(5.0);
//	glBegin(GL_POINTS);
//		glVertex3f(domains[0].A0(), domains[0].A1(), domains[0].A2() );
//	glEnd();	
}

//--------------------------------------------------------------------
void	FlowParticleAction::DrawBoxGL()
{}
//--------------------------------------------------------------------
void	FlowParticleAction::DrawSphereGL(float detail)
{
	glPushMatrix();
	glTranslatef( domains[0].A0(), domains[0].A1(), domains[0].A2() );
	RglSphere( domains[0].A3(), -domains[0].A3(), domains[0].A3(), 360.0, (int)detail );
//	RglSphere( domains[0].A4(), 0.0,  domains[0].A4(), 360.0, 12 );
	glPopMatrix();
}
//--------------------------------------------------------------------
void	FlowParticleAction::DrawCylinderGL()
{}
//--------------------------------------------------------------------
void	FlowParticleAction::DrawConeGL()
{}
//--------------------------------------------------------------------
void	FlowParticleAction::DrawBlobGL()
{}


//===================================================================================
// STATE FUNCTIONS
//===================================================================================
////===================================================================================
//PS::PS( )
//	: FlowParticleAction()
//{
//	actionName = "";
//	actionType = "";
//	domainName[0] = "";
//	
//	numFloats	= 0;
//	numDomains	= 0;
//	numBools	= 0;
//}
//PS::~PS()	{}
////------------------------------------------------
//void	PS::Execute(int frame)
//{
//	frame = frame;
//}
//
//===================================================================================
FPSColor::FPSColor(PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, PFloat inFloat03 )
	: FlowParticleAction()
{
	baseType= P_State;
	type	= FlowParticleAction::PS_Color;
	
	actionName = "Color";
	actionType = "Color";
	floatName[0] = "red";
	floatName[1] = "green";
	floatName[2] = "blue";
	floatName[3] = "alpha";
	
	floats[0] = inFloat00;
	floats[1] = inFloat01;
	floats[2] = inFloat02;
	floats[3] = inFloat03;
	
	numFloats_v		= 4;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPSColor::~FPSColor()	
{}
FPSColor::FPSColor(FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_Color;
}
//------------------------------------------------
void	FPSColor::Execute(int frame)
{
	frame = frame;
	if (enabled)
		pColor( floats[0], floats[1], floats[2], floats[3] ); 
}
//--------------------------------------------------------------------
void	FPSColor::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpColor( %f, %f, %f, %f );\n\n", 
			(float)floats[0], 
			(float)floats[1],
			(float)floats[2],
			(float)floats[3] );
	fputs(buffer, outfile);
}

//===================================================================================
FPSColorDomain::FPSColorDomain(PFloat inFloat00, PDomain inDomain00 )
	: FlowParticleAction()
{
	baseType = P_State;
	type	= FlowParticleAction::PS_ColorDomain;
		
	actionName = "Color Domain";
	actionType = "ColorDomain";
	domainName[0] = "domain";
	floatName[0] = "alpha";
	
	floats[0] = inFloat00;
	domains[0] = inDomain00;
	
	numFloats_v		= 1;
	numDomains_v	= 1;
	numBools_v		= 0;
}
FPSColorDomain::~FPSColorDomain()	
{}
FPSColorDomain::FPSColorDomain(FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_ColorDomain;
}
//------------------------------------------------
void	FPSColorDomain::Execute(int frame)
{
	frame = frame;
	if (enabled)
		pColorD( floats[0], 
			domains[0].Type(),
			domains[0].A0(), 
			domains[0].A1(), 
			domains[0].A2(), 
			domains[0].A3(), 
			domains[0].A4(), 
			domains[0].A5(), 
			domains[0].A6(), 
			domains[0].A7(), 
			domains[0].A8());
}
//--------------------------------------------------------------------
void	FPSColorDomain::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpColorD( %f, \n\t\t\t(PDomainEnum)%d, \n\t\t\t%f, %f, %f, %f, %f, %f, %f, %f, %f );\n\n", 
			(float)floats[0], 
			(int)domains[0].Type(),
			(float)domains[0].A0(), 
			(float)domains[0].A1(), 
			(float)domains[0].A2(), 
			(float)domains[0].A3(), 
			(float)domains[0].A4(), 
			(float)domains[0].A5(), 
			(float)domains[0].A6(), 
			(float)domains[0].A7(), 
			(float)domains[0].A8());
	fputs(buffer, outfile);
}

//===================================================================================
FPSSize::FPSSize(PFloat inFloat00, PFloat inFloat01, PFloat inFloat02)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_Size ;
	
	actionName	= "Size";
	actionType	= "Size";
	floatName[0]	= "radius";
	floatName[1]	= "length";
	floatName[2]	= "(unused)";
	
	floats[0]	= inFloat00;
	floats[1]	= inFloat01;
	floats[2]	= inFloat02;
	
	numFloats_v		= 2;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPSSize::~FPSSize()
{}

FPSSize::FPSSize ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_Size ;
}

//------------------------------------------------
void	FPSSize::Execute(int frame)
{
	frame = frame;
	if (enabled)
		pSize(floats[0], floats[1], floats[2]);
}
//--------------------------------------------------------------------
void	FPSSize::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpSize( %f, %f, %f );\n\n", 
			(float)floats[0], 
			(float)floats[1], 
			(float)floats[2] );
	fputs(buffer, outfile);
}

//===================================================================================
FPSSizeDomain::FPSSizeDomain(PDomain inDomain00)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_SizeDomain ;
	
	actionName	= "Size Domain";
	actionType	= "SizeDomain";
	domainName[0]	= "size";
	
	domains[0]	= inDomain00;
	
	numFloats_v		= 0;
	numDomains_v	= 1;
	numBools_v		= 0;
}
FPSSizeDomain::~FPSSizeDomain()
{}

FPSSizeDomain::FPSSizeDomain ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_SizeDomain ;
}

//------------------------------------------------
void	FPSSizeDomain::Execute(int frame)
{
	frame = frame;
	if (enabled)
		pSizeD(	domains[0].Type(),
				domains[0].A0(), 
				domains[0].A1(), 
				domains[0].A2(), 
				domains[0].A3(), 
				domains[0].A4(), 
				domains[0].A5(), 
				domains[0].A6(), 
				domains[0].A7(), 
				domains[0].A8());
}
//--------------------------------------------------------------------
void	FPSSizeDomain::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpSizeD((PDomainEnum)%d, \n\t\t\t%f, %f, %f, %f, %f, %f, %f, %f, %f );\n\n", 
			(int)domains[0].Type(),
			(float)domains[0].A0(), 
			(float)domains[0].A1(), 
			(float)domains[0].A2(), 
			(float)domains[0].A3(), 
			(float)domains[0].A4(), 
			(float)domains[0].A5(), 
			(float)domains[0].A6(), 
			(float)domains[0].A7(), 
			(float)domains[0].A8());
	fputs(buffer, outfile);
}

//===================================================================================
FPSStartingAge::FPSStartingAge(PFloat inFloat00)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_StartingAge ;
	
	actionName	= "Starting Age";
	actionType	= "StartingAge";
	floatName[0]	= "age";
	
	floats[0]	= inFloat00;
	
	numFloats_v		= 1;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPSStartingAge::~FPSStartingAge()
{}

FPSStartingAge::FPSStartingAge ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_StartingAge ;
}
//------------------------------------------------
void	FPSStartingAge::Execute(int frame)
{
	frame = frame;
	if (enabled)
		pStartingAge(floats[0]);
}
//--------------------------------------------------------------------
void	FPSStartingAge::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpStartingAge( %f );\n\n", 
			(float)floats[0] );
	fputs(buffer, outfile);
}

//===================================================================================
FPSTimeStep::FPSTimeStep(PFloat inFloat00)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_TimeStep ;
	
	actionName	= "TimeStep";
	actionType	= "TimeStep";
	floatName[0]	= "dt";
	
	floats[0]	= inFloat00;
	
	numFloats_v		= 1;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPSTimeStep::~FPSTimeStep()
{}

FPSTimeStep::FPSTimeStep ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_TimeStep ;
}
//------------------------------------------------
void	FPSTimeStep::Execute(int frame)
{
	frame = frame;
	if (enabled)
		pTimeStep( floats[0]/**dtAdjust*/ );
}
//--------------------------------------------------------------------
void	FPSTimeStep::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpTimeStep( %f*timeAdj );\n\n", 
			(float)floats[0] );
	fputs(buffer, outfile);
}

//===================================================================================
FPSVelocity::FPSVelocity(PFloat inFloat00, PFloat inFloat01, PFloat inFloat02)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_Velocity ;
	
	actionName	= "Velocity";
	actionType	= "Velocity";
	floatName[0]	= "x velocity";
	floatName[1]	= "y velocity";
	floatName[2]	= "z velocity";
	
	floats[0]	= (float)inFloat00;
	floats[1]	= (float)inFloat01;
	floats[2]	= (float)inFloat02;
	
	numFloats_v		= 3;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPSVelocity::~FPSVelocity()
{}

FPSVelocity::FPSVelocity ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_Velocity ;
}
//------------------------------------------------
void	FPSVelocity::Execute(int frame)
{
	frame = frame;
	if (enabled)
		pVelocity(floats[0], floats[1], floats[2]);
}
//--------------------------------------------------------------------
void	FPSVelocity::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpVelocity( %f, %f, %f );\n\n", 
			(float)floats[0], 
			(float)floats[1], 
			(float)floats[2] );
	fputs(buffer, outfile);
}

//===================================================================================
FPSVelocityDomain::FPSVelocityDomain(PDomain& in)
	: FlowParticleAction()
{
	baseType = P_State;
	type	= FlowParticleAction::PS_VelocityDomain ;
	
	actionName = "Velocity Domain";
	actionType = "VelocityDomain";
	domainName[0] = "domain";
	
	domains[0] = in;
//	PFloat	*pTmpPFloat;
//	pTmpPFloat = domains[0].PFloatA4();
//	pTmpPFloat->SetVariable(new MBAVFloat(0.025, 0.05));
	
	numFloats_v		= 0;
	numDomains_v	= 1;
	numBools_v		= 0;
}
FPSVelocityDomain::~FPSVelocityDomain()
{}

FPSVelocityDomain::FPSVelocityDomain ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_VelocityDomain ;
}
//------------------------------------------------
void	FPSVelocityDomain::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pVelocityD(domains[0].Type(),
			domains[0].A0(), 
			domains[0].A1(), 
			domains[0].A2(), 
			domains[0].A3(), 
			domains[0].A4(), 
			domains[0].A5(), 
			domains[0].A6(), 
			domains[0].A7(), 
			domains[0].A8());
}
//--------------------------------------------------------------------
void	FPSVelocityDomain::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpVelocityD( (PDomainEnum)%d, \n\t\t\t%f, %f, %f, %f, %f, %f, %f, %f, %f );\n\n", 
			(int)domains[0].Type(),
			(float)domains[0].A0(), 
			(float)domains[0].A1(), 
			(float)domains[0].A2(), 
			(float)domains[0].A3(), 
			(float)domains[0].A4(), 
			(float)domains[0].A5(), 
			(float)domains[0].A6(), 
			(float)domains[0].A7(), 
			(float)domains[0].A8());
	fputs(buffer, outfile);
}

//===================================================================================
FPSVertex2ndPosition::FPSVertex2ndPosition(PFloat inFloat00, PFloat inFloat01, PFloat inFloat02)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_Vertex2ndPosition ;
	
	actionName	= "Vertex 2nd Position";
	actionType	= "Vertex2ndPosition";
	floatName[0]	= "x";
	floatName[1]	= "y";
	floatName[2]	= "z";
	
	floats[0]	= inFloat00;
	floats[1]	= inFloat01;
	floats[1]	= inFloat02;
	
	numFloats_v		= 3;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPSVertex2ndPosition::~FPSVertex2ndPosition()
{}
FPSVertex2ndPosition::FPSVertex2ndPosition ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_Vertex2ndPosition ;
}
//------------------------------------------------
void	FPSVertex2ndPosition::Execute(int frame)
{
	frame = frame;
	if (enabled)
		pVertexB(floats[0], floats[1], floats[2]);
}
//--------------------------------------------------------------------
void	FPSVertex2ndPosition::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpVertexB( %f, %f, %f );\n\n", 
			(float)floats[0], 
			(float)floats[1], 
			(float)floats[2] );
	fputs(buffer, outfile);
}

//===================================================================================
FPSVertex2ndPositionDomain::FPSVertex2ndPositionDomain(PDomain& in)
	: FlowParticleAction()
{
	baseType = P_State;
	type	= FlowParticleAction::PS_Vertex2ndPositionDomain ;
	
	actionName = "Vertex 2nd Position Domain";
	actionType = "Vertex2ndPositionDomain";
	domainName[0] = "domain";
	
	domains[0] = in;
	
	numFloats_v		= 0;
	numDomains_v	= 1;
	numBools_v		= 0;
}
FPSVertex2ndPositionDomain::~FPSVertex2ndPositionDomain()
{}

FPSVertex2ndPositionDomain::FPSVertex2ndPositionDomain ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_Vertex2ndPositionDomain ;
}
//------------------------------------------------
void	FPSVertex2ndPositionDomain::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pVertexBD(domains[0].Type(),
			domains[0].A0(), 
			domains[0].A1(), 
			domains[0].A2(), 
			domains[0].A3(), 
			domains[0].A4(), 
			domains[0].A5(), 
			domains[0].A6(), 
			domains[0].A7(), 
			domains[0].A8());
}
//--------------------------------------------------------------------
void	FPSVertex2ndPositionDomain::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpVertexBD( (PDomainEnum)%d, \n\t\t\t%f, %f, %f, %f, %f, %f, %f, %f, %f );\n\n", 
			(int)domains[0].Type(),
			(float)domains[0].A0(), 
			(float)domains[0].A1(), 
			(float)domains[0].A2(), 
			(float)domains[0].A3(), 
			(float)domains[0].A4(), 
			(float)domains[0].A5(), 
			(float)domains[0].A6(), 
			(float)domains[0].A7(), 
			(float)domains[0].A8());
	fputs(buffer, outfile);
}

//===================================================================================
FPSVertex2ndPositionTracks::FPSVertex2ndPositionTracks(PBool inBool00)
	: FlowParticleAction()
{
	baseType = P_State;
	type	= FlowParticleAction::PS_Vertex2ndPositionTracks ;
	
	actionName = "Vertex 2nd Position Tracks";
	actionType = "Vertex2ndPositionTracks";
	boolName[0] = "do copy";
	
	bools[0] = inBool00;
	
	numFloats_v		= 0;
	numDomains_v	= 0;
	numBools_v		= 1;
}
FPSVertex2ndPositionTracks::~FPSVertex2ndPositionTracks()
{
}
FPSVertex2ndPositionTracks::FPSVertex2ndPositionTracks ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_State;
	type	= FlowParticleAction::PS_Vertex2ndPositionTracks ;
}
//--------------------------------------------------------------------
void	FPSVertex2ndPositionTracks::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pVertexBTracks(bools[0]);
}
//--------------------------------------------------------------------
void	FPSVertex2ndPositionTracks::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpVertexBTracks( (bool)%d );\n\n", 
			(bool)bools[0]);
	fputs(buffer, outfile);
}

//********************************************************************
//===================================================================================
// ACTION FUNCTIONS **************************************************
//===================================================================================
//********************************************************************
FPABounce::FPABounce(PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, PDomain& inDomain00, PFloat inFloat03, PFloat inFloat04, PBool inBool00)
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Bounce;
		
	actionName	= "Bounce";
	actionType	= "Bounce";
	floatName[0]	= "friction";
	floatName[1]	= "resilience";
	floatName[2]	= "cutoff";
	floatName[3]	= "DrawDetail";
	floatName[4]	= "DrawDepth";
	domainName[0]	= "domain";
	boolName[0]		= "DrawSurface";
	
	glDomain = new GLDomain();
	glListOnce = true;
	
	glSurf = new MBA_GLSurface();
//	rmSurf = new MBA_RMShader();
	rmSurf = 0;
	
	floats[0]	= inFloat00;
	floats[1]	= inFloat01;
	floats[2]	= inFloat02;
	floats[3]	= inFloat03;
	floats[4]	= inFloat04;
	domains[0]	= inDomain00;
	bools[0]	= inBool00;
	
	
	numFloats_v		= 5;
	numDomains_v	= 1;
	numBools_v		= 1;
}
FPABounce::~FPABounce()
{
}
FPABounce::FPABounce ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Bounce;
	// should I set this stuff or read it from file? This may hide errors...
	floatName[0]	= "friction";
	floatName[1]	= "resilience";
	floatName[2]	= "cutoff";
	floatName[3]	= "DrawDetail";
	floatName[4]	= "DrawDepth";
	domainName[0]	= "domain";
	boolName[0]		= "DrawSurface";
	
	numFloats_v		= 5;
	numDomains_v	= 1;
	numBools_v		= 1;
}

//--------------------------------------------------------------------
void	FPABounce::Execute(int)
{
	if (enabled)
	{
		pBounce(floats[0], floats[1], floats[2], domains[0].Type(),
			domains[0].A0(), 
			domains[0].A1(), 
			domains[0].A2(), 
			domains[0].A3(), 
			domains[0].A4(), 
			domains[0].A5(), 
			domains[0].A6(), 
			domains[0].A7(), 
			domains[0].A8());
	}
}

//--------------------------------------------------------------------
void	FPABounce::drawGL(int)
{
	if (enabled)
	{
		if (bools[0])
		{
			if (!glSurf)
			{
				glSurf = new MBA_GLSurface();
				warning ("No GLSurface for \"%s\". Creating default surface", (const char*)actionName);
			}
			glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,		glSurf->Diffuse()  );
			glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,	glSurf->Specular() );
			glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,		glSurf->Ambient()  );
			glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,	glSurf->Emission() );
			glMaterialf  (GL_FRONT_AND_BACK, GL_SHININESS,	glSurf->Shininess());

			if (!GL__lighting)
			{	GL__lighting = true; glEnable(GL_LIGHTING);	}
			if ( glListOnce || attributesChanged )
			{
				//warning("glListOnce(%d), attributesChanged(%d) -- \"%s\"", glListOnce, attributesChanged, (const char*)actionName);
				glDomain->CreateListFromDomain(GLDomain::Filled, domains[0], floats[3], floats[4]);
				glListOnce = false;
			}
			glDomain->CallList();
		}
	}
}

//--------------------------------------------------------------------
void FPABounce::drawRIB(int)
{
	if (enabled)
	{
		char	idString[128];
		sprintf(idString, "--------------------%s----------> \"%s\"", (const char*)actionType, (const char*)actionName);
		RiArchiveRecord ("comment", "", RI_NULL );		
		RiArchiveRecord ("comment", idString, RI_NULL );		
		if (rmSurf)
		{
			rmSurf->DeclareVariables();
			rmSurf->Activate();
		}
		if (rmDisp)
		{
			rmDisp->DeclareVariables();
			rmDisp->Activate();
		}
		//--
		if (bools[0])
		{
			switch(domains[0].Type())
			{
				case PDPoint:
					break;
				case PDLine:
					break;
				case PDBox:
					break;
				case PDCylinder:
					break;
				case PDCone:
					break;
				case PDBlob:
					break;
				case PDPlane:
					DrawPlaneRIB(floats[4]);
					break;
				case PDSphere:
					DrawSphereRIB();
					break;
				case PDTriangle:
					break;
				case PDDisc:
					DrawDiskRIB(floats[4]);
					break;
			}
		}
	}
}

//--------------------------------------------------------------------
void	FPABounce::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpBounce( %f, %f, %f, \n\t\t\t(PDomainEnum)%d, \n\t\t\t%f, %f, %f, %f, %f, %f, %f, %f, %f );\n\n", 
			(float)floats[0], (float)floats[1], (float)floats[2], (int)domains[0].Type(),
			(float)domains[0].A0(), 
			(float)domains[0].A1(), 
			(float)domains[0].A2(), 
			(float)domains[0].A3(), 
			(float)domains[0].A4(), 
			(float)domains[0].A5(), 
			(float)domains[0].A6(), 
			(float)domains[0].A7(), 
			(float)domains[0].A8());
	fputs(buffer, outfile);
}
	
//===================================================================================
//===================================================================================
FPADamping::FPADamping(PFloat dX, PFloat dY, PFloat dZ, PFloat vLow, PFloat vHigh)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Damping ;
	
	actionName = "Damping";
	actionType = "Damping";
	floatName[0] = "damp X";
	floatName[1] = "damp Y";
	floatName[2] = "damp Z";
	floatName[3] = "low vel";
	floatName[4] = "high vel";
	
	floats[0] = dX;
	floats[1] = dY;
	floats[2] = dZ;
	floats[3] = vLow;
	floats[4] = vHigh;
	
	numFloats_v		= 5;
	numDomains_v	= 0;
	numBools_v		= 0;
}

FPADamping::~FPADamping()
{
}

FPADamping::FPADamping ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Damping ;
}
//--------------------------------------------------------------------
void FPADamping::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pDamping(floats[0], floats[1], floats[2], floats[3], floats[4]);
}
//--------------------------------------------------------------------
void	FPADamping::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpDamping( %f, %f, %f, %f, %f );\n\n", 
			(float)floats[0], 
			(float)floats[1],
			(float)floats[2],
			(float)floats[3],
			(float)floats[4] );
	fputs(buffer, outfile);
}


//===================================================================================
int FPAExplosion::explosionCount = 0;
FPAExplosion::FPAExplosion(	PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, 
							PFloat inFloat03, PFloat inFloat04, PFloat inFloat05, 
							PFloat inFloat06, PFloat beginAt, PFloat recycle )
{
	explosionCount++;
	explosionNumber = explosionCount;
	icount = 0;
	baseType = P_Action;
	type	= FlowParticleAction::PA_Explosion ;
	
	actionName = "Explosion";
	actionType = "Explosion";
	floatName[0] = "x";
	floatName[1] = "y";
	floatName[2] = "z";
	floatName[3] = "velocity";
	floatName[4] = "magnitude";
	floatName[5] = "lifetime";
	floatName[6] = "epsilon";
	floatName[7] = "BeginAt";
	floatName[8] = "Recycle";
	
	floats[0] = inFloat00;
	floats[1] = inFloat01;
	floats[2] = inFloat02;
	floats[3] = inFloat03;
	floats[4] = inFloat04;
	floats[5] = inFloat05;
	floats[6] = inFloat06;
	floats[7] = beginAt;
	floats[8] = recycle;
	
	numFloats_v		= 9;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPAExplosion::~FPAExplosion()
{}
FPAExplosion::FPAExplosion ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	icount = 0;
	explosionCount++;
	explosionNumber = explosionCount;
	baseType = P_Action;
	type	= FlowParticleAction::PA_Explosion ;
}
//--------------------------------------------------------------------
void FPAExplosion::Execute(int frame)
{
	frame = frame;
	
	if ( frame >= (int)floats[7] )
		if ( (frame-(int)floats[7])%(int)floats[8] == 0 )
			icount = 0;
		
	if (enabled)
		pExplosion( floats[0], floats[1], floats[2],
				floats[3], floats[4], floats[5], 
				floats[6], icount );
	icount++;
}
//--------------------------------------------------------------------
void	FPAExplosion::WriteCode(FILE* outfile)
{
	char	buffer[256];
	
//	warning("FPAExplosion::WriteCode -- explosionNumber = %d", explosionNumber);
//	warning("FPAExplosion::WriteCode -- explosionCount  = %d", explosionCount);
	
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tstatic int explosionAge%d   = 0;\n", explosionNumber);
	fputs(buffer, outfile);
	
	sprintf (buffer, "\tif ( frame >= %d )	{\n", (int)floats[7] );
	fputs(buffer, outfile);
	sprintf (buffer, "\t\texplosionAge%d++;\n", explosionNumber);
	fputs(buffer, outfile);

	sprintf (buffer, "\t\tif ( (frame-%d) %% %d == 0 )\n", (int)floats[7], (int)floats[8] );

	fputs(buffer, outfile);
	sprintf (buffer, "\t\t\texplosionAge%d = 0;\n", explosionNumber);
	fputs(buffer, outfile);
	
	sprintf (buffer, "\t\tpExplosion( %f, %f, %f, %f, %f, %f, %f, explosionAge%d );\n\n", 
			(float)floats[0], 
			(float)floats[1],
			(float)floats[2],
			(float)floats[3],
			(float)floats[4],
			(float)floats[5],
			(float)floats[6], explosionNumber );
	fputs(buffer, outfile);
	
	sprintf (buffer, "\t}\n" );
	fputs(buffer, outfile);
	
}

//--------------------------------------------------------------------
void	FPAExplosion::drawGL(int)
{
//FIXME - !
	glDisable(GL_LIGHTING);
	if(g_glThick) glPointSize(7.0);
	glBegin(GL_POINTS);
		glVertex3f(floats[0], floats[1], floats[2]);
	glEnd();
}

//===================================================================================
//===================================================================================
FPAFollow::FPAFollow(PFloat inFloat00, PFloat inFloat01)
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Follow ;
	
	actionName = "Follow";
	actionType = "Follow";
	floatName[0]	= "gravity";
	floatName[1]	= "epsilon";
	
	floats[0] = inFloat00;
	floats[1] = inFloat01;
	
	numFloats_v		= 3;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPAFollow::~FPAFollow()
{
}

FPAFollow::FPAFollow ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Follow ;
}
//--------------------------------------------------------------------
void	FPAFollow::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pFollow(floats[0], floats[1]);
}
//--------------------------------------------------------------------
void	FPAFollow::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpFollow( %f, %f );\n\n", 
			(float)floats[0], 
			(float)floats[1] );
	fputs(buffer, outfile);
}


//===================================================================================
//===================================================================================
FPAGravitate::FPAGravitate(PFloat inFloat00, PFloat inFloat01)
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Gravitate ;
	
	actionName = "Gravitate";
	actionType = "Gravitate";
	floatName[0] = "grav";
	floatName[1] = "epsilon";
	
	floats[0] = inFloat00;
	floats[1] = inFloat01;
	
	numFloats_v		= 2;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPAGravitate::~FPAGravitate()
{
}

FPAGravitate::FPAGravitate ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Gravitate ;
}
//--------------------------------------------------------------------
void FPAGravitate::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pGravitate(floats[0], floats[1]);
}


//===================================================================================
//===================================================================================
FPAGravity::FPAGravity(PFloat inFloat00, PFloat inFloat01, PFloat inFloat02)
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Gravity ;
	
	actionName = "Gravity";
	actionType = "Gravity";
	floatName[0]	= "X direction";
	floatName[1]	= "Y direction";
	floatName[2]	= "Z direction";
	
	floats[0] = inFloat00;
	floats[1] = inFloat01;
	floats[2] = inFloat02;
	
	numFloats_v		= 3;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPAGravity::~FPAGravity()
{
}
FPAGravity::FPAGravity ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Gravity ;
}
//--------------------------------------------------------------------
void	FPAGravity::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pGravity(floats[0], floats[1], floats[2]);
}
//--------------------------------------------------------------------
void	FPAGravity::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpGravity( %f, %f, %f );\n\n", 
			(float)floats[0], 
			(float)floats[1],
			(float)floats[2] );
	fputs(buffer, outfile);
}

//===================================================================================
//===================================================================================
FPAJet::FPAJet(	PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, 
					PFloat inFloat03, PFloat inFloat04, PFloat inFloat05 )
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Jet ;
	
	actionName = "Jet";
	actionType = "Jet";
	floatName[0] = "center x";
	floatName[1] = "center y";
	floatName[2] = "center z";
	floatName[3] = "grav";
	floatName[4] = "epsilon";
	floatName[5] = "max radius";
	
	floats[0] = inFloat00;
	floats[1] = inFloat01;
	floats[2] = inFloat02;
	floats[3] = inFloat03;
	floats[4] = inFloat04;
	floats[5] = inFloat05;
	
	numFloats_v		= 6;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPAJet::~FPAJet()
{}

FPAJet::FPAJet ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Jet ;
}

//--------------------------------------------------------------------
void FPAJet::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pJet(	floats[0], floats[1], floats[2],
				floats[3], floats[4], floats[5]);
}
//--------------------------------------------------------------------
void	FPAJet::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpJet( %f, %f, %f, %f, %f, %f );\n\n", 
			(float)floats[0], 
			(float)floats[1],
			(float)floats[2],
			(float)floats[3],
			(float)floats[4],
			(float)floats[5] );
	fputs(buffer, outfile);
}

//===================================================================================
//===================================================================================
FPAKillOld::FPAKillOld(PFloat inFloat00, PBool inBool00)
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_KillOld ;
	
	actionName = "Kill Old";
	actionType = "KillOld";
	floatName[0]	= "age limit";
	boolName[0]	= "kill less than";
	
	floats[0] = inFloat00;
	bools[0] = inBool00;
	
	numFloats_v		= 1;
	numDomains_v	= 0;
	numBools_v		= 1;
}
FPAKillOld::~FPAKillOld()
{
}

FPAKillOld::FPAKillOld ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_KillOld ;
}
//--------------------------------------------------------------------
void	FPAKillOld::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pKillOld(floats[0], bools[0]);
}
//--------------------------------------------------------------------
void	FPAKillOld::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpKillOld( %f, (bool)%d );\n\n", 
			(float)floats[0], 
			(bool)bools[0]  );
	fputs(buffer, outfile);
}

////===================================================================================
////===================================================================================
//FPAKillSlow::FPAKillSlow(PFloat inFloat00, PBool inBool00)
//	: FlowParticleAction()
//{
//	baseType = P_Action;
//	type	= FlowParticleAction::PA_KillSlow ;
//
//	actionName = "Kill Slow";
//	actionType = "KillSlow";
//	floatName[0]	= "speed limit";
//	boolName[0]	= "kill less than";
//	
//	floats[0] = inFloat00;
//	bools[0] = inBool00;
//	
//	numFloats_v		= 1;
//	numDomains_v	= 0;
//	numBools_v		= 1;
//}
//FPAKillSlow::~FPAKillSlow()
//{
//}
//
//FPAKillSlow::FPAKillSlow ( FlowParticleAction& copy)
//	: FlowParticleAction(copy)
//{
//	baseType = P_Action;
//	type	= FlowParticleAction::PA_KillSlow ;
//}
////--------------------------------------------------------------------
//void	FPAKillSlow::Execute(int frame)
//{
//	frame = frame;
//	
//	if (enabled)
//		pKillSlow(floats[0], bools[0]);
//}
////--------------------------------------------------------------------
//void	FPAKillSlow::WriteCode(FILE* outfile)
//{
//	char	buffer[256];
//	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
//	fputs(buffer, outfile);
//	sprintf (buffer, "\tpKillSlow( %f, (bool)%d );\n\n", 
//			(float)floats[0], 
//			(bool)bools[0]  );
//	fputs(buffer, outfile);
//}
//
//===================================================================================
//===================================================================================
FPAOrbitLine::FPAOrbitLine(	PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, 
					PFloat inFloat03, PFloat inFloat04, PFloat inFloat05, 
					PFloat inFloat06, PFloat inFloat07, PFloat inFloat08)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_OrbitLine ;
	
	actionName = "Orbit Line";
	actionType = "OrbitLine";
	floatName[0] = "center x";
	floatName[1] = "center y";
	floatName[2] = "center z";
	floatName[3] = "axis x";
	floatName[4] = "axis y";
	floatName[5] = "axis z";
	floatName[6] = "gravity";
	floatName[7] = "epsilon";
	floatName[8] = "max radius";
	boolName[0] = "Draw";
		
	floats[0] = inFloat00;
	floats[1] = inFloat01;
	floats[2] = inFloat02;
	floats[3] = inFloat03;
	floats[4] = inFloat04;
	floats[5] = inFloat05;
	floats[6] = inFloat06;
	floats[7] = inFloat07;
	floats[8] = inFloat08;
	bools[0] = true;
	
	numFloats_v		= 9;
	numDomains_v	= 0;
	numBools_v		= 1;
}
FPAOrbitLine::~FPAOrbitLine()
{}

FPAOrbitLine::FPAOrbitLine ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_OrbitLine ;
}

//--------------------------------------------------------------------
void FPAOrbitLine::drawGL(int)
{
	#define	CSTEPS 12
	static	float	circle[CSTEPS][2];
	static bool once = true;
	int i;
	
	if (once)
	{
		once = false;
		float	rinc = (2.0*M_PI)/((double)(CSTEPS));
		for (i = 0; i < CSTEPS; i++)
		{
			circle[i][0] = sin((float)i*rinc)*0.25;
			circle[i][1] = cos((float)i*rinc)*0.25;
		}
	}
	
	if (bools[0])
	{
		if (GL__lighting)
		{	GL__lighting = false; glDisable(GL_LIGHTING);	}

		if (enabled)
			glColor4f(0.75, 1.0, 0.75, 1.0);
		else
			glColor4f(0.5, 0.5, 0.5, 1.0);

		glLineWidth(1.0);
		//- - - - - - - - - - - - - - - - -
		glBegin(GL_LINE_LOOP);
			for (i = 0; i < CSTEPS; i++)
				glVertex3f( circle[i][0]+(float)floats[0], circle[i][1]+(float)floats[1], (float)floats[2]);
		glEnd();
		glBegin(GL_LINE_LOOP);
			for (i = 0; i < CSTEPS; i++)
				glVertex3f( (float)floats[0], circle[i][1]+(float)floats[1], circle[i][0]+(float)floats[2]);
		glEnd();
		glBegin(GL_LINE_LOOP);
			for (i = 0; i < CSTEPS; i++)
				glVertex3f(	circle[i][1]+(float)floats[0], (float)floats[1], circle[i][0]+(float)floats[2]);
		glEnd();
		//- - - - - - - - - - - - - - - - -
		glBegin(GL_LINE_LOOP);
			for (i = 0; i < CSTEPS; i++)
				glVertex3f( circle[i][0]+(float)floats[0]-(float)floats[3], circle[i][1]+(float)floats[1]-(float)floats[4], (float)floats[2]-(float)floats[5]);
		glEnd();
		glBegin(GL_LINE_LOOP);
			for (i = 0; i < CSTEPS; i++)
				glVertex3f( (float)floats[0]-(float)floats[3], circle[i][1]+(float)floats[1]-(float)floats[4], circle[i][0]+(float)floats[2]-(float)floats[5]);
		glEnd();
		glBegin(GL_LINE_LOOP);
			for (i = 0; i < CSTEPS; i++)
				glVertex3f(	circle[i][1]+(float)floats[0]-(float)floats[3], (float)floats[1]-(float)floats[4], circle[i][0]+(float)floats[2]-(float)floats[5]);
		glEnd();
		//- - - - - - - - - - - - - - - - - 
		glBegin(GL_LINE_LOOP);
			for (i = 0; i < CSTEPS; i++)
				glVertex3f( circle[i][0]+(float)floats[0]+(float)floats[3], circle[i][1]+(float)floats[1]+(float)floats[4], (float)floats[2]+(float)floats[5]);
		glEnd();
		glBegin(GL_LINE_LOOP);
			for (i = 0; i < CSTEPS; i++)
				glVertex3f( (float)floats[0]+(float)floats[3], circle[i][1]+(float)floats[1]+(float)floats[4], circle[i][0]+(float)floats[2]+(float)floats[5]);
		glEnd();
		glBegin(GL_LINE_LOOP);
			for (i = 0; i < CSTEPS; i++)
				glVertex3f(	circle[i][1]+(float)floats[0]+(float)floats[3], (float)floats[1]+(float)floats[4], circle[i][0]+(float)floats[2]+(float)floats[5]);
		glEnd();
		//- - - - - - - - - - - - - - - - -
		if(g_glThick) glLineWidth(3.0);
		glBegin(GL_LINES);
			glVertex3f(	(float)floats[0]-(float)floats[3],
						(float)floats[1]-(float)floats[4],
						(float)floats[2]-(float)floats[5]);
			glVertex3f(	(float)floats[0]+(float)floats[3],
						(float)floats[1]+(float)floats[4],
						(float)floats[2]+(float)floats[5]);
		glEnd();
	}
	#undef CSTEPS
}

//--------------------------------------------------------------------
void FPAOrbitLine::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pOrbitLine(	floats[0], floats[1], floats[2],
				floats[3], floats[4], floats[5], 
				floats[6], floats[7], floats[8] );
}
//--------------------------------------------------------------------
void	FPAOrbitLine::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpOrbitLine( %f, %f, %f, %f, %f, %f, %f, %f, %f );\n\n", 
			(float)floats[0], 
			(float)floats[1],
			(float)floats[2],
			(float)floats[3],
			(float)floats[4],
			(float)floats[5],
			(float)floats[6],
			(float)floats[7],
			(float)floats[8] );
	fputs(buffer, outfile);
}


//===================================================================================
//===================================================================================
FPAOrbitPoint::FPAOrbitPoint(	PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, 
					PFloat inFloat03, PFloat inFloat04, PFloat inFloat05)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_OrbitPoint ;
	
	actionName = "Orbit Point";
	actionType = "OrbitPoint";
	floatName[0] = "center x";
	floatName[1] = "center y";
	floatName[2] = "center z";
	floatName[3] = "gravity";
	floatName[4] = "epsilon";
	floatName[5] = "max radius";
	boolName[0] = "Draw";
	
	floats[0] = inFloat00;
	floats[1] = inFloat01;
	floats[2] = inFloat02;
	floats[3] = inFloat03;
	floats[4] = inFloat04;
	floats[5] = inFloat05;
	bools[0] = true;
	
	numFloats_v		= 6;
	numDomains_v	= 0;
	numBools_v		= 1;
}
FPAOrbitPoint::~FPAOrbitPoint()
{}

FPAOrbitPoint::FPAOrbitPoint ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_OrbitPoint ;
}

//--------------------------------------------------------------------
void FPAOrbitPoint::drawGL(int)
{
	#define	CSTEPS 12
	static	float	circle[CSTEPS][2];
	static bool once = true;
	int i;
	
	if (once)
	{
		once = false;
		float	rinc = (2.0*M_PI)/((double)(CSTEPS));
		for (i = 0; i < CSTEPS; i++)
		{
			circle[i][0] = sin((float)i*rinc)*0.25;
			circle[i][1] = cos((float)i*rinc)*0.25;
		}
	}
	
	if (bools[0])
	{
		if (GL__lighting)
		{	GL__lighting = false; glDisable(GL_LIGHTING);	}

		if(enabled)
			glColor4f(0.75, 0.75, 1.0, 1.0);
		else
			glColor4f(0.5, 0.5, 0.5, 1.0);

		if(g_glThick) glPointSize(5);
		glBegin(GL_POINTS);
			glVertex3f(floats[0], floats[1], floats[2]);
		glEnd();
		glLineWidth(1.0);
		glBegin(GL_LINE_LOOP);
			for (i = 0; i < CSTEPS; i++)
				glVertex3f(circle[i][0]+(float)floats[0], circle[i][1]+(float)floats[1], floats[2]);
		glEnd();
		glBegin(GL_LINE_LOOP);
			for (i = 0; i < CSTEPS; i++)
				glVertex3f(floats[0], circle[i][1]+(float)floats[1], circle[i][0]+(float)floats[2]);
		glEnd();
		glBegin(GL_LINE_LOOP);
			for (i = 0; i < CSTEPS; i++)
				glVertex3f(circle[i][1]+(float)floats[0], floats[1], circle[i][0]+(float)floats[2]);
		glEnd();
	}
	#undef CSTEPS
}

//--------------------------------------------------------------------
void FPAOrbitPoint::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pOrbitPoint(	floats[0], floats[1], floats[2],
				floats[3], floats[4], floats[5] );
}
//--------------------------------------------------------------------
void	FPAOrbitPoint::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpOrbitPoint( %f, %f, %f, %f, %f, %f );\n\n", 
			(float)floats[0], 
			(float)floats[1],
			(float)floats[2],
			(float)floats[3],
			(float)floats[4],
			(float)floats[5] );
	fputs(buffer, outfile);
}

//===================================================================================
//===================================================================================
FPARandomAccel::FPARandomAccel(PDomain& in)
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_RandomAccel ;
	
	actionName = "RandomAccel";
	actionType = "RandomAccel";
	domainName[0] = "domain";
	
	domains[0] = in;
	
	numFloats_v		= 0;
	numDomains_v	= 1;
	numBools_v		= 0;
}
FPARandomAccel::~FPARandomAccel()
{}

FPARandomAccel::FPARandomAccel ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_RandomAccel ;
}
//--------------------------------------------------------------------
void	FPARandomAccel::Execute(int frame)
{
	frame = frame;
	if (enabled)
		pRandomAccel(domains[0].Type(),
			domains[0].A0(), domains[0].A1(), domains[0].A2(), 
			domains[0].A3(), domains[0].A4(), domains[0].A5(), 
			domains[0].A6(), domains[0].A7(), domains[0].A8());
}
//--------------------------------------------------------------------
void	FPARandomAccel::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpRandomAccel( (PDomainEnum)%d, \n\t\t\t%f, %f, %f, %f, %f, %f, %f, %f, %f );\n\n", 
			(int)domains[0].Type(),
			(float)domains[0].A0(), (float)domains[0].A1(), (float)domains[0].A2(), 
			(float)domains[0].A3(), (float)domains[0].A4(), (float)domains[0].A5(), 
			(float)domains[0].A6(), (float)domains[0].A7(), (float)domains[0].A8());
	fputs(buffer, outfile);
}

//===================================================================================
//===================================================================================
FPARandomDisplace::FPARandomDisplace(PDomain& in)
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_RandomDisplace ;
	
	actionName = "RandomDisplace";
	actionType = "RandomDisplace";
	domainName[0] = "domain";
	
	domains[0] = in;
	
	numFloats_v		= 0;
	numDomains_v	= 1;
	numBools_v		= 0;
}
FPARandomDisplace::~FPARandomDisplace()
{}

FPARandomDisplace::FPARandomDisplace ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_RandomDisplace ;
}
//--------------------------------------------------------------------
void	FPARandomDisplace::Execute(int frame)
{
	frame = frame;
	if (enabled)
		pRandomDisplace(domains[0].Type(),
			domains[0].A0(), domains[0].A1(), domains[0].A2(), 
			domains[0].A3(), domains[0].A4(), domains[0].A5(), 
			domains[0].A6(), domains[0].A7(), domains[0].A8());
}
//--------------------------------------------------------------------
void	FPARandomDisplace::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpRandomDisplace( (PDomainEnum)%d, \n\t\t\t%f, %f, %f, %f, %f, %f, %f, %f, %f );\n\n", 
			(int)domains[0].Type(),
			(float)domains[0].A0(), (float)domains[0].A1(), (float)domains[0].A2(), 
			(float)domains[0].A3(), (float)domains[0].A4(), (float)domains[0].A5(), 
			(float)domains[0].A6(), (float)domains[0].A7(), (float)domains[0].A8());
	fputs(buffer, outfile);
}

//===================================================================================
//===================================================================================
FPARandomVelocity::FPARandomVelocity(PDomain& in)
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_RandomVelocity ;
	
	actionName = "RandomVelocity";
	actionType = "RandomVelocity";
	domainName[0] = "domain";
	
	domains[0] = in;
	
	numFloats_v		= 0;
	numDomains_v	= 1;
	numBools_v		= 0;
}
FPARandomVelocity::~FPARandomVelocity()
{}

FPARandomVelocity::FPARandomVelocity ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_RandomVelocity ;
}
//--------------------------------------------------------------------
void	FPARandomVelocity::Execute(int frame)
{
	frame = frame;
	if (enabled)
		pRandomVelocity(domains[0].Type(),
			domains[0].A0(), domains[0].A1(), domains[0].A2(), 
			domains[0].A3(), domains[0].A4(), domains[0].A5(), 
			domains[0].A6(), domains[0].A7(), domains[0].A8());
}
//--------------------------------------------------------------------
void	FPARandomVelocity::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpRandomVelocity( (PDomainEnum)%d, \n\t\t\t%f, %f, %f, %f, %f, %f, %f, %f, %f );\n\n", 
			(int)domains[0].Type(),
			(float)domains[0].A0(), (float)domains[0].A1(), (float)domains[0].A2(), 
			(float)domains[0].A3(), (float)domains[0].A4(), (float)domains[0].A5(), 
			(float)domains[0].A6(), (float)domains[0].A7(), (float)domains[0].A8());
	fputs(buffer, outfile);
}

//===================================================================================
//===================================================================================
FPARestore::FPARestore(PFloat inFloat00 )
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Restore ;
	
	actionName = "Restore";
	actionType = "Restore";
	floatName[0] = "time left";
	
	floats[0] = inFloat00;
	
	numFloats_v		= 1;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPARestore::~FPARestore()
{
}

FPARestore::FPARestore ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Restore ;
}
//--------------------------------------------------------------------
void	FPARestore::Execute(int frame)
{
	frame = frame;
	if (enabled)
		pRestore(floats[0]);
}
//--------------------------------------------------------------------
void	FPARestore::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpRestore( %f );\n\n", 
			(float)floats[0]);
	fputs(buffer, outfile);
}

//===================================================================================
//===================================================================================
FPASink::FPASink(PBool inBool00, PDomain& in)
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Sink ;
	
	actionName = "Sink";
	actionType = "Sink";
	boolName[0] = "kill inside";
	domainName[0] = "domain";
	
	bools[0] = inBool00;
	domains[0] = in;
	
	numFloats_v		= 0;
	numDomains_v	= 1;
	numBools_v		= 1;
}
FPASink::~FPASink()
{
}

FPASink::FPASink ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Sink ;
}
//--------------------------------------------------------------------
void	FPASink::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pSink(bools[0], domains[0].Type(),
			domains[0].A0(), 
			domains[0].A1(), 
			domains[0].A2(), 
			domains[0].A3(), 
			domains[0].A4(), 
			domains[0].A5(), 
			domains[0].A6(), 
			domains[0].A7(), 
			domains[0].A8());
}
//--------------------------------------------------------------------
void	FPASink::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpSink( (bool)%d, \n\t\t\t(PDomainEnum)%d, \n\t\t\t%f, %f, %f, %f, %f, %f, %f, %f, %f );\n\n", 
			(bool)bools[0],
			(int)domains[0].Type(),
			(float)domains[0].A0(), 
			(float)domains[0].A1(), 
			(float)domains[0].A2(), 
			(float)domains[0].A3(), 
			(float)domains[0].A4(), 
			(float)domains[0].A5(), 
			(float)domains[0].A6(), 
			(float)domains[0].A7(), 
			(float)domains[0].A8());
	fputs(buffer, outfile);
}


//===================================================================================
//===================================================================================
FPASinkVelocity::FPASinkVelocity(PBool inBool00, PDomain& in)
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_SinkVelocity ;
	
	actionName = "SinkVelocity";
	actionType = "SinkVelocity";
	boolName[0] = "kill inside";
	domainName[0] = "domain";
	
	bools[0] = inBool00;
	domains[0] = in;
	
	numFloats_v		= 0;
	numDomains_v	= 1;
	numBools_v		= 1;
}
FPASinkVelocity::~FPASinkVelocity()
{
}

FPASinkVelocity::FPASinkVelocity ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_SinkVelocity ;
}
//--------------------------------------------------------------------
void	FPASinkVelocity::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pSinkVelocity(bools[0], domains[0].Type(),
			domains[0].A0(), 
			domains[0].A1(), 
			domains[0].A2(), 
			domains[0].A3(), 
			domains[0].A4(), 
			domains[0].A5(), 
			domains[0].A6(), 
			domains[0].A7(), 
			domains[0].A8());
}
//--------------------------------------------------------------------
void	FPASinkVelocity::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpSinkVelocity( (bool)%d, \n\t\t\t(PDomainEnum)%d, \n\t\t\t%f, %f, %f, %f, %f, %f, %f, %f, %f );\n\n", 
			(bool)bools[0],
			(int)domains[0].Type(),
			(float)domains[0].A0(), 
			(float)domains[0].A1(), 
			(float)domains[0].A2(), 
			(float)domains[0].A3(), 
			(float)domains[0].A4(), 
			(float)domains[0].A5(), 
			(float)domains[0].A6(), 
			(float)domains[0].A7(), 
			(float)domains[0].A8());
	fputs(buffer, outfile);
}


//===================================================================================
//===================================================================================
FPASource::FPASource(PFloat inFloat00, PDomain& in)
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Source ;
	
	actionName		= "Source";
	actionType		= "Source";
	floatName[0]	= "rate";
	floatName[1]	= "DrawDetail";
	domainName[0]	= "domain";
	boolName[0]		= "DrawDomain";
	
	floats[0]	= inFloat00;
	domains[0]	= in;
	bools[0]	= true;

	numFloats_v		= 2;
	numDomains_v	= 1;
	numBools_v		= 1;
	
	glDomain = new GLDomain();
	glListOnce = true;
	
}
FPASource::~FPASource()
{
//	warning("	FPASource::~FPASource()");
}

FPASource::FPASource ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Source ;

	floatName[0]	= "rate";
	floatName[1]	= "DrawDetail";
	domainName[0]	= "domain";
	boolName[0]		= "DrawDomain";

	numFloats_v		= 2;
	numDomains_v	= 1;
	numBools_v		= 1;
}
//--------------------------------------------------------------------
void	FPASource::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
	{
		pSource(floats[0], domains[0].Type(),
			domains[0].A0(), 
			domains[0].A1(), 
			domains[0].A2(), 
			domains[0].A3(), 
			domains[0].A4(), 
			domains[0].A5(), 
			domains[0].A6(), 
			domains[0].A7(), 
			domains[0].A8());
	}
}

//--------------------------------------------------------------------
void	FPASource::drawGL(int)
{
//	if (enabled)
//	{
		if (bools[0])
		{
			if (GL__lighting)
			{	GL__lighting = false; glDisable(GL_LIGHTING);	}

			if (enabled)
				glColor4f(1.0, 1.0, 1.0, 1.0);
			else
				glColor4f(0.5, 0.5, 0.5, 1.0);

			//if ( glListOnce || attributesChanged )
			{
				//warning("glListOnce(%d), attributesChanged(%d) -- \"%s\"", glListOnce, attributesChanged, (const char*)actionName);
				glDomain->CreateListFromDomain(GLDomain::Wire, domains[0], floats[1], 0.0);
				glListOnce = false;
			}
			glDomain->CallList();
		}
//	}
}

//--------------------------------------------------------------------
void	FPASource::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpSource( %f*rateAdj, \n\t\t\t(PDomainEnum)%d, \n\t\t\t%f, %f, %f, %f, %f, %f, %f, %f, %f );\n\n", 
			(float)floats[0],
			(int)domains[0].Type(),
			(float)domains[0].A0(), 
			(float)domains[0].A1(), 
			(float)domains[0].A2(), 
			(float)domains[0].A3(), 
			(float)domains[0].A4(), 
			(float)domains[0].A5(), 
			(float)domains[0].A6(), 
			(float)domains[0].A7(), 
			(float)domains[0].A8());
	fputs(buffer, outfile);
}


//===================================================================================
//===================================================================================
FPATargetColor::FPATargetColor(	PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, 
								PFloat inFloat03, PFloat inFloat04)
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_TargetColor ;
	
	actionName = "Target Color";
	actionType = "TargetColor";
	floatName[0]	= "red";
	floatName[1]	= "green";
	floatName[2]	= "blue";
	floatName[3]	= "alpha";
	floatName[4]	= "scale";
	
	floats[0] = inFloat00;
	floats[1] = inFloat01;
	floats[2] = inFloat02;
	floats[3] = inFloat03;
	floats[4] = inFloat04;
	
	numFloats_v		= 5;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPATargetColor::~FPATargetColor()
{
}

FPATargetColor::FPATargetColor ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_TargetColor ;
}

//--------------------------------------------------------------------
void	FPATargetColor::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pTargetColor(floats[0], floats[1], floats[2], floats[3], floats[4]);
}
//--------------------------------------------------------------------
void	FPATargetColor::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpTargetColor( %f, %f, %f, %f, %f );\n\n", 
			(float)floats[0], 
			(float)floats[1],
			(float)floats[2],
			(float)floats[3],
			(float)floats[4] );
	fputs(buffer, outfile);
}

//===================================================================================
//===================================================================================
FPATargetSize::FPATargetSize(	PFloat inFloat00, PFloat inFloat01, PFloat inFloat02,
								PFloat inFloat03, PFloat inFloat04, PFloat inFloat05 )
	: FlowParticleAction()
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_TargetSize ;
	
	actionName = "Target Size";
	actionType = "TargetSize";
	floatName[0]	= "size x";
	floatName[1]	= "size y";
	floatName[2]	= "size z";
	floatName[3]	= "scale x";
	floatName[4]	= "scale y";
	floatName[5]	= "scale z";
	
	floats[0] = inFloat00;
	floats[1] = inFloat01;
	floats[2] = inFloat02;
	floats[3] = inFloat03;
	floats[4] = inFloat04;
	floats[5] = inFloat05;
	
	numFloats_v		= 6;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPATargetSize::~FPATargetSize()
{
}
FPATargetSize::FPATargetSize ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_TargetSize ;
}
//--------------------------------------------------------------------
void	FPATargetSize::Execute(int frame)
{ 
	frame = frame;
	if (enabled)
		pTargetSize(floats[0], floats[1], floats[2], floats[3], floats[4], floats[5]);
}
//--------------------------------------------------------------------
void	FPATargetSize::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpTargetSize( %f, %f, %f, %f, %f, %f );\n\n", 
			(float)floats[0], (float)floats[1], (float)floats[2], (float)floats[3], (float)floats[4], (float)floats[5]);
	fputs(buffer, outfile);
}

//===================================================================================
//===================================================================================
FPAVertex::FPAVertex(	PFloat inFloat00, PFloat inFloat01, PFloat inFloat02 )
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Vertex ;
	
	actionName = "Vertex";
	actionType = "Vertex";
	floatName[0] = "x";
	floatName[1] = "y";
	floatName[2] = "z";
	
	floats[0] = inFloat00;
	floats[1] = inFloat01;
	floats[2] = inFloat02;
	
	numFloats_v		= 3;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPAVertex::~FPAVertex()
{}

FPAVertex::FPAVertex ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Vertex ;
}

//--------------------------------------------------------------------
void FPAVertex::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pVertex( floats[0], floats[1], floats[2] );
}
//--------------------------------------------------------------------
void	FPAVertex::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpVertex( %f, %f, %f );\n\n", 
			(float)floats[0], 
			(float)floats[1],
			(float)floats[2] );
	fputs(buffer, outfile);
}

//===================================================================================
//===================================================================================
FPAVortex::FPAVortex(	PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, 
					PFloat inFloat03, PFloat inFloat04, PFloat inFloat05, 
					PFloat inFloat06, PFloat inFloat07, PFloat inFloat08 )
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Vortex ;
	
	actionName = "Vortex";
	actionType = "Vortex";
	floatName[0] = "center x";
	floatName[1] = "center y";
	floatName[2] = "center z";
	floatName[3] = "axis x";
	floatName[4] = "axis y";
	floatName[5] = "axis z";
	floatName[6] = "magnitude";
	floatName[7] = "tightness";
	floatName[8] = "maxRadius";
	
	floats[0] = inFloat00;
	floats[1] = inFloat01;
	floats[2] = inFloat02;
	floats[3] = inFloat03;
	floats[4] = inFloat04;
	floats[5] = inFloat05;
	floats[6] = inFloat06;
	floats[7] = inFloat07;
	floats[8] = inFloat08;
	
	numFloats_v		= 9;
	numDomains_v	= 0;
	numBools_v		= 0;
}
FPAVortex::~FPAVortex()
{}

FPAVortex::FPAVortex ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Vortex ;
}

//--------------------------------------------------------------------
void FPAVortex::drawGL(int)
{
	
	glBegin(GL_LINES);
		glVertex3f(floats[0], floats[1], floats[2]);
		glVertex3f(	(float)floats[0]+(float)floats[3],
					(float)floats[1]+(float)floats[4],
					(float)floats[2]+(float)floats[5]);
	glEnd();
	if(g_glThick) glPointSize(7.0);
	glBegin(GL_POINTS);
		glVertex3f(floats[0], floats[1], floats[2]);
	glEnd();
}

//--------------------------------------------------------------------
void FPAVortex::Execute(int frame)
{
	frame = frame;
	
	if (enabled)
		pVortex(	floats[0], floats[1], floats[2],
				floats[3], floats[4], floats[5], 
				floats[6], floats[7], floats[8] );
}
//--------------------------------------------------------------------
void	FPAVortex::WriteCode(FILE* outfile)
{
	char	buffer[256];
	sprintf (buffer, "\t//-- %s\n", (const char*)actionName);
	fputs(buffer, outfile);
	sprintf (buffer, "\tpVortex( %f, %f, %f, %f, %f, %f, %f, %f, %f );\n\n", 
			(float)floats[0], 
			(float)floats[1],
			(float)floats[2],
			(float)floats[3],
			(float)floats[4],
			(float)floats[5],
			(float)floats[6],
			(float)floats[7],
			(float)floats[8] );
	fputs(buffer, outfile);
}

//===================================================================================
//===================================================================================
FPATurbulence::FPATurbulence(	)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Turbulence ;
	
	actionName = "Turbulence";
	actionType = "Turbulence";
	floatName[0] = "frequency";
	floatName[1] = "octaves";
	floatName[2] = "magnitude";
	floatName[3] = "delta";
	floatName[4] = "DrawArea";
	floatName[5] = "DrawDetail";
	floatName[6] = "PointSize";
	domainName[0] = "Movement";
	boolName[0] = "DrawNoise";
	boolName[1] = "DrawCube";
	boolName[2] = "DrawPositive";
	boolName[3] = "DrawNegative";
	
	floats[0] = 1.0;
	floats[1] = 2.0;
	floats[2] = 1.0;
	floats[3] = 0.01;
	floats[4] = 1.0;
	floats[5] = 25.0;
	floats[6] = 5.0;
	bools[0] = false;
	bools[1] = false;
	bools[2] = true;
	bools[3] = true;
	
	numFloats_v		= 7;
	numDomains_v	= 1;
	numBools_v		= 4;
	
	nval = 0;
}
FPATurbulence::~FPATurbulence()
{
//FIXME -  delete nval here FIXME - 
}

FPATurbulence::FPATurbulence ( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = P_Action;
	type	= FlowParticleAction::PA_Turbulence ;
	
	nval = 0;
}

//--------------------------------------------------------------------
void FPATurbulence::drawGL(int)
{
	float	vec[3];
	int		i, j, k;
	int		detail = (int)floats[5];
	int		kb;
	int		ke;
	float	color[4];
	
	//-- draw, if necessary
	if(bools[0])
	{
		if (attributesChanged || nval == 0)
		{
			if(nval != 0)
			{
				for(i = 0; i < oldDetail; i++)
				{
					for(j = 0; j < oldDetail; j++)
						delete [] nval[i][j];
				}
				for(i = 0; i < oldDetail; i++)
					delete [] nval[i];
				delete [] nval;
			}

			nval = new float**[detail];
			for(i = 0; i < detail; i++)
			{
				nval[i] = new float*[detail];
				for(j = 0; j < detail; j++)
					nval[i][j] = new float[detail];
			}
			oldDetail = detail;

			for (i = 0; i < detail; i++)
			{
				for (j = 0; j < detail; j++)
				{
					for (k = 0; k <  detail; k++)
					{
						vec[0] =	(((float)i/(float)detail)-0.5)*2.0*(float)floats[4] + ((float)g_frame*domains[0].A0());
						vec[1] =	(((float)j/(float)detail)-0.5)*2.0*(float)floats[4] + ((float)g_frame*domains[0].A1());
						vec[2] =	(((float)k/(float)detail)-0.5)*2.0*(float)floats[4] + ((float)g_frame*domains[0].A2());
						nval[i][j][k] = fractalsum3(vec, floats[0], floats[1]);
					}
				}
			}
		}
	
		if (GL__lighting)
		{	GL__lighting = false; glDisable(GL_LIGHTING);	}
		if(g_glThick) glPointSize(floats[6]);
		for (i = 0; i < detail; i++)
		{
			for (j = 0; j < detail; j++)
			{
				if(bools[1])
				{
					kb = 0;
					ke = detail;
				}
				else
				{
					kb = detail/2;
					ke = detail/2+1;
				}
				glDepthMask(GL_FALSE);
				for (k = kb; k < ke; k++)
				{
					vec[0] = (((float)i/(float)detail)-0.5)*2.0*(float)floats[4];
					vec[1] = (((float)j/(float)detail)-0.5)*2.0*(float)floats[4];
					vec[2] = (((float)k/(float)detail)-0.5)*2.0*(float)floats[4];
					
					color[0] = color[1] = color[2] = color[3] = 0.0;
					if((bool)bools[2] && (bool)bools[3])
					{
						if(nval[i][j][k] > 0.0)
						{
							color[0] = nval[i][j][k];
							color[3] = nval[i][j][k];
						}
						else
						{
							color[2] = fabs(nval[i][j][k]);
							color[3] = fabs(nval[i][j][k]);
						}
					}
					else if (bools[2])
					{
						if(nval[i][j][k] > 0.0)
						{
							color[0] = color[1] = color[2] = color[3] = nval[i][j][k];
						}
					}
					else if (bools[3])
					{
						if(nval[i][j][k] < 0.0)
						{
							color[0] = color[1] = color[2] = color[3] = fabs(nval[i][j][k]);
						}
					}
					glColor4fv(color);
					glBegin(GL_POINTS);
					glVertex3fv(vec);
					glEnd();
				}
				glDepthMask(GL_TRUE);
			}
		}
		glEnable(GL_LIGHTING);
	}
}

//--------------------------------------------------------------------
void FPATurbulence::Execute(int)
{
	temp += 0.01;
//	temp += 0.002;
//	temp -= 0.002;
//	temp = 0.0;
//	if(frame < 2)
//		temp = 0;
		
	if(enabled)
	{
		ParticleGroup *pg = _ps.pgrp;
		if(pg == NULL)
			return; // ERROR

		float pV[3];
		float vX[3];
		float vY[3];
		float vZ[3];
		for(int i = 0; i < pg->p_count; i++)
		{
			Particle &m = pg->list[i];
			pV[0] = m.pos.x + ((float)g_frame*domains[0].A0());
			pV[1] = m.pos.y + ((float)g_frame*domains[0].A1());
			pV[2] = m.pos.z + ((float)g_frame*domains[0].A2());

			vX[0] = pV[0]+(float)floats[3];
			vX[1] = pV[1];
			vX[2] = pV[2];

			vY[0] = pV[0];
			vY[1] = pV[1]+(float)floats[3];
			vY[2] = pV[2];

			vZ[0] = pV[0];
			vZ[1] = pV[1];
			vZ[2] = pV[2]+(float)floats[3];


			float d =  	fractalsum3(pV, floats[0], floats[1]);
			float dx = 	(fractalsum3(vX, floats[0], floats[1]) - d)*(float)floats[2];
			float dy = 	(fractalsum3(vY, floats[0], floats[1]) - d)*(float)floats[2];
			float dz = 	(fractalsum3(vZ, floats[0], floats[1]) - d)*(float)floats[2];

			//fprintf(stdout, "%f %f %f\n", dx, dy, dz);
			float velMagOrig = sqrt( m.vel.x*m.vel.x + m.vel.y*m.vel.y + m.vel.z*m.vel.z );

			m.vel.x += dx;
			m.vel.y += dy;
			m.vel.z += dz;

			float	velMagNow = sqrt( m.vel.x*m.vel.x + m.vel.y*m.vel.y + m.vel.z*m.vel.z );
			float	valMagScale = velMagOrig/velMagNow;
			m.vel.x *= valMagScale;
			m.vel.y *= valMagScale;
			m.vel.z *= valMagScale;
		}
	}
}

//--------------------------------------------------------------------
void	FPATurbulence::WriteCode(FILE*)
{
}


//--------------------------------------------------------------------
void	FlowParticleAction::DrawPlaneRIB(float depth)
{
	if ( domains[0].Type() == (PDomainEnum)PDPlane )
	{
//	warning ("DrawPlaneRIB(%f)", depth);
		// Get corners A, B, C and D
		float	A0[3] = {	domains[0].A0(), 
							domains[0].A1(),
							domains[0].A2() };
		float	B0[3] = {	A0[0] + domains[0].A6(),
							A0[1] + domains[0].A7(),
							A0[2] + domains[0].A8() };
		float	C0[3] = {	B0[0] + domains[0].A3(),
							B0[1] + domains[0].A4(),
							B0[2] + domains[0].A5() };
		float	D0[3] = {	A0[0] + domains[0].A3(), 
							A0[1] + domains[0].A4(), 
							A0[2] + domains[0].A5() };


		//- - - - - - - - - - - - - - -
		RtPoint	p0[2][2];
		p0[0][0][0] = C0[0];    p0[0][0][1] = C0[1];    p0[0][0][2] = -C0[2];
		p0[0][1][0] = B0[0];    p0[0][1][1] = B0[1];    p0[0][1][2] = -B0[2];
		p0[1][1][0] = A0[0];    p0[1][1][1] = A0[1];    p0[1][1][2] = -A0[2];
		p0[1][0][0] = D0[0];    p0[1][0][1] = D0[1];    p0[1][0][2] = -D0[2];

		RiPatch(RI_BILINEAR, RI_P,  (RtPointer)p0, RI_NULL);

		MVertex	v1(D0[0], D0[1], D0[2]);
		MVertex v2(A0[0], A0[1], A0[2]);
		MVertex v3(B0[0], B0[1], B0[2]);
		MVertex normal;
		normal.CreateNormal(v1, v2, v3);
//		normal.CreateNormal(v3, v2, v1);
		normal.setX(normal.X()*depth);
		normal.setY(normal.Y()*depth);
		normal.setZ(normal.Z()*depth);

		//-- if plane has depth, draw it...
		if (depth != 0.0f)
		{
			float	A1[3];
			float	B1[3];
			float	C1[3];
			float	D1[3];
			A1[0] = A0[0]-normal.X();
			A1[1] = A0[1]-normal.Y();
			A1[2] = A0[2]-normal.Z();
			B1[0] = B0[0]-normal.X();
			B1[1] = B0[1]-normal.Y();
			B1[2] = B0[2]-normal.Z();
			C1[0] = C0[0]-normal.X();
			C1[1] = C0[1]-normal.Y();
			C1[2] = C0[2]-normal.Z();
			D1[0] = D0[0]-normal.X();
			D1[1] = D0[1]-normal.Y();
			D1[2] = D0[2]-normal.Z();
			
			//-- p0 now becomes the bottom...
			p0[0][0][0] = C1[0];	p0[0][0][1] = C1[1];	p0[0][0][2] = -C1[2];
			p0[0][1][0] = D1[0];	p0[0][1][1] = D1[1];	p0[0][1][2] = -D1[2];
			p0[1][1][0] = A1[0];	p0[1][1][1] = A1[1];	p0[1][1][2] = -A1[2];
			p0[1][0][0] = B1[0];	p0[1][0][1] = B1[1];	p0[1][0][2] = -B1[2];
			RtPoint	p1[2][2];
			RtPoint	p2[2][2];
			RtPoint	p3[2][2];
			RtPoint	p4[2][2];
			RtPoint	p5[2][2];
			p1[0][0][0] = B0[0];	p1[0][0][1] = B0[1];	p1[0][0][2] = -B0[2];
			p1[0][1][0] = B1[0];	p1[0][1][1] = B1[1];	p1[0][1][2] = -B1[2];
			p1[1][1][0] = A1[0];	p1[1][1][1] = A1[1];	p1[1][1][2] = -A1[2];
			p1[1][0][0] = A0[0];	p1[1][0][1] = A0[1];	p1[1][0][2] = -A0[2];
			p2[0][0][0] = C0[0];	p2[0][0][1] = C0[1];	p2[0][0][2] = -C0[2];
			p2[0][1][0] = C1[0];	p2[0][1][1] = C1[1];	p2[0][1][2] = -C1[2];
			p2[1][1][0] = B1[0];	p2[1][1][1] = B1[1];	p2[1][1][2] = -B1[2];
			p2[1][0][0] = B0[0];	p2[1][0][1] = B0[1];	p2[1][0][2] = -B0[2];
			p3[0][0][0] = D0[0];	p3[0][0][1] = D0[1];	p3[0][0][2] = -D0[2];
			p3[0][1][0] = D1[0];	p3[0][1][1] = D1[1];	p3[0][1][2] = -D1[2];
			p3[1][1][0] = C1[0];	p3[1][1][1] = C1[1];	p3[1][1][2] = -C1[2];
			p3[1][0][0] = C0[0];	p3[1][0][1] = C0[1];	p3[1][0][2] = -C0[2];
			p4[0][0][0] = A0[0];	p4[0][0][1] = A0[1];	p4[0][0][2] = -A0[2];
			p4[0][1][0] = A1[0];	p4[0][1][1] = A1[1];	p4[0][1][2] = -A1[2];
			p4[1][1][0] = D1[0];	p4[1][1][1] = D1[1];	p4[1][1][2] = -D1[2];
			p4[1][0][0] = D0[0];	p4[1][0][1] = D0[1];	p4[1][0][2] = -D0[2];
			
			p5[0][0][0] = C1[0];	p5[0][0][1] = C1[1];	p5[0][0][2] = -C1[2];
			p5[0][1][0] = B1[0];	p5[0][1][1] = B1[1];	p5[0][1][2] = -B1[2];
			p5[1][1][0] = A1[0];	p5[1][1][1] = A1[1];	p5[1][1][2] = -A1[2];
			p5[1][0][0] = D1[0];	p5[1][0][1] = D1[1];	p5[1][0][2] = -D1[2];

			RiPatch(RI_BILINEAR, RI_P,  (RtPointer)p0, RI_NULL);
			RiPatch(RI_BILINEAR, RI_P,  (RtPointer)p1, RI_NULL);
			RiPatch(RI_BILINEAR, RI_P,  (RtPointer)p2, RI_NULL);
			RiPatch(RI_BILINEAR, RI_P,  (RtPointer)p3, RI_NULL);
			RiPatch(RI_BILINEAR, RI_P,  (RtPointer)p4, RI_NULL);
		}
	}
}

//--------------------------------------------------------------------
void	FlowParticleAction::DrawSphereRIB()
{
	RiTransformBegin();
	RiTranslate(domains[0].A0(), domains[0].A1(), -domains[0].A2() );
	RiSphere(domains[0].A3(), -domains[0].A3(), domains[0].A3(), 360.0, RI_NULL );
	RiTransformEnd();
}

//--------------------------------------------------------------------
void	FlowParticleAction::DrawDiskRIB(float depth)
{
	MMatrix	rot;
	rot.pointAt(MVertex(domains[0].A3(), domains[0].A4(),-domains[0].A5()));
	rot.setTranslate(domains[0].A0(), domains[0].A1(),-domains[0].A2());
	
	RiTransformBegin();
	rot.RIBConcatTransform();
	RiDisk(0.0,  domains[0].A6(), 360.0, RI_NULL );
	if(depth)
	{
		RiCylinder(domains[0].A6(), -depth, 0.0, 360.0, RI_NULL);
	}
	RiTransformEnd();
}

