#ifndef PARTICLEACTION_H
#define PARTICLEACTION_H

#include <qobject.h>
#include <stdio.h>

#include "pDomain.h"
#include "pFloat.h"
#include "pBool.h"

#include "glDomain.h"

#include "mbaGLSurface.h"
#include "mbaRMShader.h"

#define	PA_NUM_FLOATS	10
#define PA_NUM_DOMAINS	1
#define PA_NUM_BOOLS	4

/**
 * I should rework this base class. There's too much in here that's
 * not used by all that inherit from it.
 */
//====================================================================
class FlowParticleAction 
{
public:
	FlowParticleAction(	);
	virtual ~FlowParticleAction();
	FlowParticleAction(	FlowParticleAction& );
	
	enum PAType
	{
		Undefined=-1, PS_Color, PS_ColorDomain, PS_Size, PS_SizeDomain, PS_StartingAge, PS_TimeStep, PS_Velocity, PS_VelocityDomain, 
		PS_Vertex2ndPosition, PS_Vertex2ndPositionDomain, PS_Vertex2ndPositionTracks,
		PA_Bounce, PA_Damping, PA_Explosion, PA_Follow, PA_Gravitate, PA_Gravity, PA_Jet, 
		PA_KillOld, PA_KillSlow, PA_OrbitLine, PA_OrbitPoint, PA_RandomAccel, PA_RandomDisplace, 
		PA_RandomVelocity, PA_Restore, PA_Sink, PA_SinkVelocity, PA_Source, PA_TargetColor, PA_TargetSize,
		PA_Vertex, PA_Vortex, PA_Turbulence, 
		PPA_PolygonSource, PPA_PolygonBounce, PPA_PolygonNoOp, PPA_PolygonEmitter,
		PG_Sphere, PG_Cone, PG_Disk, PG_Cylinder, PG_Hyperboloid, PG_Paraboloid, PG_Torus
	};
	
	//--
	enum PBaseType {	P_Undefined = -1, P_State, P_Action, P_PolyAction, P_Geometry	};
	
	void Print();
	virtual void Execute(int frame);
	virtual void drawRIB(int frame);
	virtual void drawGL(int frame);
	virtual void drawGLFill(int frame);
	virtual void drawGLThin(int frame);
	
	static	float dtAdjust;	/**< used to adjust dt for framerate */
	
	virtual void WriteCode(FILE* outfile);
	
	virtual	void SaveToFile(FILE *outfile, int tabs);
	
	inline	const QCString&	Name()				{	return actionName;		}
	QCString	DisabledName();
	inline	const char*		ActionType()		{	return (const char*)actionType;		}
	inline	void			SetActionType(const char* type)	{	actionType = type;		}
	inline	PAType			Type()				{	return type;	}
	inline	void			SetType(PAType in)	{	type = in;		}
	inline	PBaseType		BaseType()			{	return baseType;}
	
	inline	const QCString&	FloatName(uint i)	
	{	
		if (i < numFloats_v) return floatName[i];
		else return nullString;
	}
	inline	const QCString&	DomainName(uint i)	
	{	
		if (i < numDomains_v) return domainName[i];
		else return nullString;
	}
	inline	const QCString&	BoolName(uint i)	
	{	
		if (i < numBools_v) return boolName[i];
		else return nullString;
	}
	
	/** @return enable status */
	inline bool	Enabled()			{ return enabled;	}
	/** set enable status */
	inline bool	setEnabled(bool in)	{ enabled = in; return enabled; }	
	
	/** @return number of floats in action */
	inline uint	numFloats()	{	return numFloats_v;	}
	/** @return number of domains in action */
	inline uint	NumDomains(){	return numDomains_v;	}
	/** @return number of booleans in action */
	inline uint	NumBools()	{	return numBools_v;	}
	
	/** set number of floats in action */
	inline void	SetNumFloats(int in)	{	numFloats_v		= in;	}
	/** set number of domains in action */
	inline void	SetNumDomains(int in)	{	numDomains_v	= in;	}
	/** set number of booleans in action */
	inline void	SetNumBools(int in)		{	numBools_v		= in;	}
	
	/** @return pointer to float object at index i */
	inline PFloat	*Float(int i)	{	return &floats[i];	}
	/** @return pointer to domain object at index i */
	inline PDomain	*Domain(int i)	{	return &domains[i];	}
	/** @return pointer to boolean object at index i */
	inline PBool	*Bool(int i)	{	return &bools[i];	}
	
	inline MBA_GLSurface	*GLSurface()			{	return glSurf;	}
	inline void SetGLSurface(MBA_GLSurface* in)	{	if(glSurf) delete glSurf;	glSurf = in;	}
	
	inline MBA_RMShader		**pointerToRMSurface()	{	return &rmSurf;	}
	inline MBA_RMShader		*RMSurface()			{	return rmSurf;	}
	inline void SetRMSurface(MBA_RMShader* in)		{	if(rmSurf) delete rmSurf;	rmSurf = in;	}
	
	inline MBA_RMShader		**pointerToRMDisplace()	{	return &rmDisp;	}
	inline MBA_RMShader		*RMDisplace()			{	return rmDisp;	}
	inline void SetRMDisplacement(MBA_RMShader* in)	{	if(rmDisp) delete rmDisp;	rmDisp = in;	}
	
	bool	SetFloat( int index, float val );

//	void	DrawPointRIB();
//	void	DrawLineRIB();
//	virtual void	DrawTriangleRIB(float depth = 0.0);	// Virtual to add 
	virtual void	DrawPlaneRIB(float depth = 0.0);	// Virtual to add 
//	void	DrawBoxRIB();
	virtual	void	DrawSphereRIB();
//	void	DrawCylinderRIB();
//	void	DrawConeRIB();
//	void	DrawBlobRIB();
	virtual	void	DrawDiskRIB(float depth = 0.0);

	void 	setDomainA0(int index, float val);
	void 	setDomainA1(int index, float val);
	void 	setDomainA2(int index, float val);
	void 	setDomainA3(int index, float val);
	void 	setDomainA4(int index, float val);
	void 	setDomainA5(int index, float val);
	void 	setDomainA6(int index, float val);
	void 	setDomainA7(int index, float val);
	void 	setDomainA8(int index, float val);

	void	DrawPointGL();
	void	DrawLineGL();
	void	DrawTriangleGL(float detail = 0, float depth = 0.0);
	void	DrawPlaneGL(float, float depth = 0.0);
	void	DrawBoxGL();
	void	DrawSphereGL(float);
	void	DrawCylinderGL();
	void	DrawConeGL();
	void	DrawBlobGL();

//signals:
//	void	Updated();
	
//public slots:
public:
	void	SetName		(const char* in, bool updateRequired = true);
	void	SetFloatName	(int i, const char* in, bool updateRequired = true);
	void	SetDomainName	(int i, const char* in, bool updateRequired = true);
	void	SetBoolName		(int i, const char* in, bool updateRequired = true);

public:
	bool	attributesChanged;
	
protected:
	static	int	icount;
	QCString	actionName;
	QCString	actionType;
	PAType		type;
	
//FIXME temporary
	bool	glListOnce;
	
	PBaseType	baseType;
	
	QCString nullString;
	QCString floatName[PA_NUM_FLOATS];
	QCString domainName[PA_NUM_DOMAINS];
	QCString boolName[PA_NUM_BOOLS];

	PFloat	floats[PA_NUM_FLOATS];
	PDomain	domains[PA_NUM_DOMAINS];
	PBool	bools[PA_NUM_BOOLS];

	uint	numFloats_v;
	uint	numBools_v;
	uint	numDomains_v;
	bool	enabled;

	GLDomain			*glDomain;
	MBA_GLSurface		*glSurf;
	MBA_RMShader		*rmSurf;
	MBA_RMShader		*rmDisp;
	
};

//====================================================================
// STATE FUNCTIONS
//====================================================================
/**
 *
 */
//====================================================================
class FPSColor	: public FlowParticleAction
{
public:	
	FPSColor (PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, PFloat inFloat03 = 1.0f);
	~FPSColor();
	FPSColor(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPSColorDomain	: public FlowParticleAction
{
public:	
	FPSColorDomain (PFloat inFloat00, PDomain inDomain00);
	~FPSColorDomain();
	FPSColorDomain(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPSSize	: public FlowParticleAction
{
public:	
	FPSSize (PFloat inFloat00, PFloat inFloat01 = 1.0, PFloat inFloat02 = 1.0);
	~FPSSize();
	FPSSize(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPSSizeDomain	: public FlowParticleAction
{
public:	
	FPSSizeDomain (PDomain inDomain00);
	~FPSSizeDomain();
	FPSSizeDomain(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPSStartingAge	: public FlowParticleAction
{
public:	
	FPSStartingAge(PFloat inFloat00);
	~FPSStartingAge();
	FPSStartingAge(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPSTimeStep	: public FlowParticleAction
{
public:	
	FPSTimeStep (PFloat inFloat00);
	~FPSTimeStep();
	FPSTimeStep(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPSVelocity	: public FlowParticleAction
{
public:	
	FPSVelocity (PFloat inFloat00, PFloat inFloat01, PFloat inFloat02);
	~FPSVelocity();
	FPSVelocity(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPSVelocityDomain	: public FlowParticleAction
{
public:
	FPSVelocityDomain(PDomain& in);
	~FPSVelocityDomain();
	FPSVelocityDomain(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPSVertex2ndPosition	: public FlowParticleAction
{
public:	
	FPSVertex2ndPosition (PFloat inFloat00, PFloat inFloat01, PFloat inFloat02);
	~FPSVertex2ndPosition();
	FPSVertex2ndPosition(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPSVertex2ndPositionDomain	: public FlowParticleAction
{
public:
	FPSVertex2ndPositionDomain(PDomain& in);
	~FPSVertex2ndPositionDomain();
	FPSVertex2ndPositionDomain(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPSVertex2ndPositionTracks	: public FlowParticleAction
{
public:	
	FPSVertex2ndPositionTracks (PBool inBool00);
	~FPSVertex2ndPositionTracks();
	FPSVertex2ndPositionTracks(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

//====================================================================
// ACTION FUNCTIONS
//====================================================================
/**
 *
 */
//====================================================================
class FPABounce	: public FlowParticleAction
{
public:
	FPABounce(PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, PDomain& inDomain00, PFloat inFloat03 = 8.0, PFloat inFloat04 = 0.0, PBool inBool00 = true);
	~FPABounce();
	FPABounce(FlowParticleAction&);

	void Execute(int frame);
	void drawGL(int frame);
	void drawRIB(int frame);
	void WriteCode(FILE* outfile);
	
};

/**
 *
 */
//====================================================================
class FPADamping	: public FlowParticleAction
{
public:
	FPADamping(PFloat dX, PFloat dY, PFloat dZ, PFloat vLow = 0.0f, PFloat vHigh = P_MAXFLOAT);
	~FPADamping();
	FPADamping(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPAExplosion	: public FlowParticleAction
{
public:
	FPAExplosion(PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, 
				PFloat inFloat03, PFloat inFloat04, PFloat inFloat05, 
				PFloat inFloat06 = P_EPS, PFloat beginAt = 0.0, PFloat recycle = 150.0 );
	~FPAExplosion();
	FPAExplosion(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
	void drawGL(int frame);
	
	static int	explosionCount;
	int			explosionNumber;
	float		icount;
};

/**
 *
 */
//====================================================================
class FPAFollow	: public FlowParticleAction
{
public:
	FPAFollow(PFloat inFloat00 = 1.0, PFloat inFloat = P_EPS);
	~FPAFollow();
	FPAFollow(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPAGravitate	: public FlowParticleAction
{
public:
	FPAGravitate(PFloat inFloat00, PFloat inFloat = P_EPS);
	~FPAGravitate();
	FPAGravitate(FlowParticleAction&);
	
	void Execute(int frame);
//	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPAGravity	: public FlowParticleAction
{
public:
	FPAGravity(PFloat inFloat00, PFloat inFloat01, PFloat inFloat02);
	~FPAGravity();
	FPAGravity(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPAJet	: public FlowParticleAction
{
public:
	FPAJet(PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, 
			PFloat inFloat03 = 1.0f, PFloat inFloat04 = P_EPS, PFloat inFloat05 = P_MAXFLOAT );
	~FPAJet();
	FPAJet(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPAKillOld	: public FlowParticleAction
{
public:
	FPAKillOld(PFloat inFloat00, PBool inBool00 = false);
	~FPAKillOld();
	FPAKillOld(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

//====================================================================
//class FPAKillSlow	: public FlowParticleAction
//{
//public:
//	FPAKillSlow(PFloat inFloat00, PBool inBool00 = true);
//	~FPAKillSlow();
//	FPAKillSlow(FlowParticleAction&);
//	
//	void Execute(int frame);
//	void WriteCode(FILE* outfile);
//};

/**
 *
 */
//====================================================================
class FPAOrbitLine	: public FlowParticleAction
{
public:
	FPAOrbitLine(	PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, 
					PFloat inFloat03, PFloat inFloat04, PFloat inFloat05, 
				PFloat inFloat06 = 1.0f, PFloat inFloat07 = P_EPS, PFloat inFloat08 = P_MAXFLOAT);
	~FPAOrbitLine();
	FPAOrbitLine(FlowParticleAction&);
	
	void drawGL(int frame);
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPAOrbitPoint	: public FlowParticleAction
{
public:
	FPAOrbitPoint(	PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, 
				PFloat inFloat03 = 1.0f, PFloat inFloat04 = P_EPS, PFloat inFloat05 = P_MAXFLOAT);
	~FPAOrbitPoint();
	FPAOrbitPoint(FlowParticleAction&);
	
	void drawGL(int frame);
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPARandomAccel	: public FlowParticleAction
{
public:
	FPARandomAccel(PDomain& in);
	~FPARandomAccel();
	FPARandomAccel(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPARandomDisplace	: public FlowParticleAction
{
public:
	FPARandomDisplace(PDomain& in);
	~FPARandomDisplace();
	FPARandomDisplace(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPARandomVelocity	: public FlowParticleAction
{
public:
	FPARandomVelocity(PDomain& in);
	~FPARandomVelocity();
	FPARandomVelocity(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPARestore	: public FlowParticleAction
{
public:
	FPARestore(PFloat inFloat00 );
	~FPARestore();
	FPARestore(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPASink	: public FlowParticleAction
{
public:
	FPASink(PBool inBool00, PDomain& in);
	~FPASink();
	FPASink(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPASinkVelocity	: public FlowParticleAction
{
public:
	FPASinkVelocity(PBool inBool00, PDomain& in);
	~FPASinkVelocity();
	FPASinkVelocity(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPASource	: public FlowParticleAction
{
public:
	FPASource(PFloat inFloat00, PDomain& in);
	~FPASource();
	FPASource(FlowParticleAction&);

	void drawGL(int frame);
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPATargetColor	: public FlowParticleAction
{
public:
	FPATargetColor(PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, 
								PFloat inFloat03, PFloat inFloat04);
	~FPATargetColor();
	FPATargetColor(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPATargetSize	: public FlowParticleAction
{
public:
	FPATargetSize(	PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, 
					PFloat inFloat03, PFloat inFloat04, PFloat inFloat05);
	~FPATargetSize();
	FPATargetSize(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPAVertex	: public FlowParticleAction
{
public:
	FPAVertex(	PFloat inFloat00, PFloat inFloat01, PFloat inFloat02 );
	~FPAVertex();
	FPAVertex(FlowParticleAction&);
	
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPAVortex	: public FlowParticleAction
{
public:
	FPAVortex(	PFloat inFloat00, PFloat inFloat01, PFloat inFloat02, 
				PFloat inFloat03, PFloat inFloat04, PFloat inFloat05, 
				PFloat inFloat06, PFloat inFloat07, PFloat inFloat08 = P_MAXFLOAT);
	~FPAVortex();
	FPAVortex(FlowParticleAction&);
	
	void drawGL(int frame);
	void Execute(int frame);
	void WriteCode(FILE* outfile);
};

/**
 *
 */
//====================================================================
class FPATurbulence	: public FlowParticleAction
{
public:
	FPATurbulence();
	~FPATurbulence();
	FPATurbulence(FlowParticleAction&);
	
	void drawGL(int frame);
	void Execute(int frame);
	void WriteCode(FILE* outfile);
	
protected:
	float	***nval;
	int		oldDetail;
};



#endif
