#include "pDomainWidget.h"
#include "extern.h"
#include <particle/papi.h>

PDomainWidget::PDomainWidget(PDomain *d, const char* lbl, QWidget* parent, const char* name)
	: QWidget (parent, name)
{
	domain = d;
		
	label = new QLabel(this);
	label->setFixedSize(120, 20);
	label->setAlignment(AlignHCenter | AlignBottom);
	label->setText(lbl);
//	label->setFont( *g_font_MB );
	
	mode = DomainNormal;
	typeCombo = new QComboBox(this);
	typeCombo->setFixedHeight(g_comboHeight);
	typeCombo->setFixedWidth(g_numInputWidth);
//	typeCombo->insertItem("Point");
//	typeCombo->insertItem("Line");
//	typeCombo->insertItem("Triangle");
//	typeCombo->insertItem("Plane");
//	typeCombo->insertItem("Box");
//	typeCombo->insertItem("Sphere");
//	typeCombo->insertItem("Cylinder          ");
//	typeCombo->insertItem("Cone");
//	typeCombo->insertItem("Blob");
//	typeCombo->insertItem("Transform");
	SwitchMode(mode);
	if (domain)
		typeCombo->setCurrentItem((int)domain->Type());
	
	int i;
	for (i = 0; i < MAX_DOMAIN_FLOATS; i++)
	{
		dInput[i] = new NumInput(NumInput::Double, this);
		dInput[i]->SetLabelWidth(g_numLabelWidth);
		dInput[i]->SetTotalWidth(g_numInputWidth);
	}
	
	//-- Connect ------------------------
	//QObject::connect(typeCombo, SIGNAL(activated(int)), SLOT(SwitchType(int)) );
	QObject::connect(typeCombo, SIGNAL(activated(int)), SLOT(TranslateType(int)) );
	
	if (domain)
	{
		TranslateType(typeCombo->currentItem());
		SetFloatValues();
	}
	QObject::connect(dInput[0], SIGNAL(ValueChanged(float)), SLOT(SetA0(float)) );
	QObject::connect(dInput[1], SIGNAL(ValueChanged(float)), SLOT(SetA1(float)) );
	QObject::connect(dInput[2], SIGNAL(ValueChanged(float)), SLOT(SetA2(float)) );
	QObject::connect(dInput[3], SIGNAL(ValueChanged(float)), SLOT(SetA3(float)) );
	QObject::connect(dInput[4], SIGNAL(ValueChanged(float)), SLOT(SetA4(float)) );
	QObject::connect(dInput[5], SIGNAL(ValueChanged(float)), SLOT(SetA5(float)) );
	QObject::connect(dInput[6], SIGNAL(ValueChanged(float)), SLOT(SetA6(float)) );
	QObject::connect(dInput[7], SIGNAL(ValueChanged(float)), SLOT(SetA7(float)) );
	QObject::connect(dInput[8], SIGNAL(ValueChanged(float)), SLOT(SetA8(float)) );
	
	QObject::connect(dInput[0], SIGNAL(Updated()), SIGNAL(Updated()) );
	QObject::connect(dInput[1], SIGNAL(Updated()), SIGNAL(Updated()) );
	QObject::connect(dInput[2], SIGNAL(Updated()), SIGNAL(Updated()) );
	QObject::connect(dInput[3], SIGNAL(Updated()), SIGNAL(Updated()) );
	QObject::connect(dInput[4], SIGNAL(Updated()), SIGNAL(Updated()) );
	QObject::connect(dInput[5], SIGNAL(Updated()), SIGNAL(Updated()) );
	QObject::connect(dInput[6], SIGNAL(Updated()), SIGNAL(Updated()) );
	QObject::connect(dInput[7], SIGNAL(Updated()), SIGNAL(Updated()) );
	QObject::connect(dInput[8], SIGNAL(Updated()), SIGNAL(Updated()) );
	
	//-- Layout ------------------------
	topLayout = new QVBoxLayout(this, 0, 0);
	
	topLayout->addWidget(label);
	topLayout->addWidget(typeCombo);
	for (i = 0; i < MAX_DOMAIN_FLOATS; i++)
		topLayout->addWidget(dInput[i]);

	topLayout->activate();
	
}

PDomainWidget::~PDomainWidget()
{
}

//--------------------------------------------------------------------
void	PDomainWidget::Update()	
{
	if(domain)
		SetFloatValues();
}

//--------------------------------------------------------------------
void	PDomainWidget::SetDomainPointer(PDomain* inP, const char* lbl)	
{
	int i;
	int newT = 0;
	
	label->setText(lbl);
	if (inP)
	{
		domain = inP;
		typeCombo->setEnabled(true);
		newT = 0;
		switch(mode)
		{
			case DomainNormal:
				newT = (int)domain->Type();
				break;
			case DomainBounce:
				if((int)domain->Type() == PDPlane)
					newT = 0;
				else if ((int)domain->Type() == PDSphere)
					newT = 1;
				else if ((int)domain->Type() == PDDisc)
					newT = 2;
				break;
			case DomainTransform:
				newT = 0;
				break;
		}
		typeCombo->setCurrentItem(newT);
		//warning("-->> mode = %d, type = %d, switch to %d", mode, (int)domain->Type(), newT);
		TranslateType(typeCombo->currentItem());
		SetFloatValues();
	}
	else
	{
		typeCombo->setEnabled(false);
		for (i = 0; i < MAX_DOMAIN_FLOATS; i++)
			dInput[i]->setEnabled(false);
		domain = inP;
	}
}

//--------------------------------------------------------------------
void  PDomainWidget::SwitchMode(ComboMode m) 
{
	mode = m;
	
	typeCombo->clear();
	switch(mode)
	{
		case DomainNormal:
			typeCombo->insertItem("  Point  ");
			typeCombo->insertItem("  Line  ");
			typeCombo->insertItem("  Triangle  ");
			typeCombo->insertItem("  Plane  ");
			typeCombo->insertItem("  Box  ");
			typeCombo->insertItem("  Sphere  ");
			typeCombo->insertItem("  Cylinder  ");
			typeCombo->insertItem("  Cone  ");
			typeCombo->insertItem("  Blob  ");
			typeCombo->insertItem("  Disk  ");
			break;
		case DomainBounce:
			typeCombo->insertItem("  Plane  ");
			typeCombo->insertItem("  Sphere  ");
			typeCombo->insertItem("  Disk  ");
			break;
		case DomainTransform:
			typeCombo->insertItem("  Transform  ");
			break;
	}
}

//--------------------------------------------------------------------
void	PDomainWidget::TranslateType(int t)
{
	int newT = 0;
	switch(mode)
	{
		case DomainNormal:
			newT = t;
			break;
		case DomainBounce:
			switch(t)
			{
				case 0:		newT = 3; break;
				case 1:		newT = 5; break;
				case 2:		newT = 9; break;
				default:	newT = 5; break;
			}
			break;
		case DomainTransform:
			newT = -1;
			break;
	}
	SwitchType( newT );
	emit Updated();
}

//--------------------------------------------------------------------
void	PDomainWidget::SwitchType(int t)
{
	int i;
	
	domain->SetType((PDomainEnum)t);
	for (i = 0; i < MAX_DOMAIN_FLOATS; i++)
	{
		if (i < domain->numFloats())
		{
			dInput[i]->setEnabled(true);
			dInput[i]->setEnabled(domain->FloatName(i));
		}
		else
			dInput[i]->setEnabled(false);
	}
	SetFloatValues();
}

//--------------------------------------------------------------------
void	PDomainWidget::SetFloatValues()
{
	dInput[0]->SetValueSolid(domain->A0());
	dInput[1]->SetValueSolid(domain->A1());
	dInput[2]->SetValueSolid(domain->A2());
	dInput[3]->SetValueSolid(domain->A3());
	dInput[4]->SetValueSolid(domain->A4());
	dInput[5]->SetValueSolid(domain->A5());
	dInput[6]->SetValueSolid(domain->A6());
	dInput[7]->SetValueSolid(domain->A7());
	dInput[8]->SetValueSolid(domain->A8());
	
	for(int i = 0; i < domain->numFloats(); i++)
		dInput[i]->SetLabel(domain->FloatName(i));
}

//--------------------------------------------------------------------
void	PDomainWidget::setEnabled(bool status)
{
	int i;
	if (status && domain)
	{
		typeCombo->setEnabled(true);
		TranslateType(typeCombo->currentItem());
		//SwitchType(typeCombo->currentItem());
	}
	else
	{
		typeCombo->setEnabled(false);
		for (i = 0; i < MAX_DOMAIN_FLOATS; i++)
			dInput[i]->setEnabled(false);
	}
}	

//====================================================================
void	PDomainWidget::SetDomain()
{
	domain->SetA0(dInput[0]->dblVal());
	domain->SetA1(dInput[1]->dblVal());
	domain->SetA2(dInput[2]->dblVal());
	domain->SetA3(dInput[3]->dblVal());
	domain->SetA4(dInput[4]->dblVal());
	domain->SetA5(dInput[5]->dblVal());
	domain->SetA6(dInput[6]->dblVal());
	domain->SetA7(dInput[7]->dblVal());
	domain->SetA8(dInput[8]->dblVal());
}

void  PDomainWidget::SetType(PDomainEnum in) 
{
	domain->SetType(in);   
}
void	PDomainWidget::SetA0(float in)
{   
	domain->SetA0(dInput[0]->SetValue(in));	 
}
void	PDomainWidget::SetA1(float in) 
{   
	domain->SetA1(dInput[1]->SetValue(in));	 
}
void	PDomainWidget::SetA2(float in) 
{   
	domain->SetA2(dInput[2]->SetValue(in));	 
}
void	PDomainWidget::SetA3(float in) 
{   
	domain->SetA3(dInput[3]->SetValue(in));	 
}
void	PDomainWidget::SetA4(float in) 
{	
	domain->SetA4(dInput[4]->SetValue(in));	 
}
void	PDomainWidget::SetA5(float in) 
{   
	domain->SetA5(dInput[5]->SetValue(in));	 
}
void	PDomainWidget::SetA6(float in)
{   
	domain->SetA6(dInput[6]->SetValue(in));	
}
void	PDomainWidget::SetA7(float in) 
{   
	domain->SetA7(dInput[7]->SetValue(in));	 
}
void	PDomainWidget::SetA8(float in) 
{   
	domain->SetA8(dInput[8]->SetValue(in));	 
}






