#include "pGeometry.h"

#include <GL/gl.h>
#include <math.h>

#include "extern.h"
#include "mmath.h"

//==============================================================================
PGSphere::PGSphere(float r, float zn, float zx, float thm, float d, bool drw, bool cls)
	: FlowParticleAction()
{
	baseType = FlowParticleAction::P_Geometry;
	type = FlowParticleAction::PG_Sphere;
	
	actionName = "Sphere";
	actionType = "Sphere";
	
	floatName[0] = "radius";
	floatName[1] = "z min";
	floatName[2] = "z max";
	floatName[3] = "theta max";
	floatName[4] = "Detail";
	domainName[0] = "Transform";
	boolName[0] = "Draw Surface";
	boolName[1] = "Close Theta";
	boolName[2] = "Flat Caps";

	glDomain = new GLDomain();
	glListOnce = true;
	
	glSurf = new MBA_GLSurface();
	rmSurf = 0;
	
	floats[0] = r;
	floats[1] = zn;
	floats[2] = zx;
	floats[3] = thm;
	floats[4] = d;
	bools[0] = drw;
	bools[1] = cls;
	domains[0].SetA6(1.0);
	domains[0].SetA7(1.0);
	domains[0].SetA8(1.0);
	
	numFloats_v		= 5;
	numDomains_v	= 1;
	numBools_v		= 3;
}

//--------------------------------------------------------------------
PGSphere::~PGSphere()
{
}

//--------------------------------------------------------------------
PGSphere::PGSphere( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = FlowParticleAction::P_Geometry;
	type = FlowParticleAction::PG_Sphere;
}

//--------------------------------------------------------------------
void PGSphere::drawRIB(int /*frame*/)
{
	MMatrix	trans;
	if (enabled)
	{
		char	idString[128];
		sprintf(idString, "--------------------%s----------> \"%s\"", (const char*)actionType, (const char*)actionName);
		RiArchiveRecord ("comment", "", RI_NULL );		
		RiArchiveRecord ("comment", idString, RI_NULL );		
		if (rmSurf)
		{
			rmSurf->DeclareVariables();
			rmSurf->Activate();
		}
		if (rmDisp)
		{
			rmDisp->DeclareVariables();
			rmDisp->Activate();
		}
		//--
		if (bools[0])
		{
			trans.setIdentity();
			trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
			trans.rotateX(-domains[0].A3());
			trans.rotateZ( domains[0].A5());
			trans.rotateY(-domains[0].A4());
			trans.translate(domains[0].A0(), domains[0].A1(), -domains[0].A2());
			RiTransformBegin();
			trans.RIBConcatTransform();
			if (bools[1])
			{
			}
			if (bools[2])
			{
				float cap;
				if ( fabs((float)floats[1]) < fabs((float)floats[0]) )
				{
					cap = sqrt(floats[0]*floats[0] - floats[1]*floats[1]);
					RiDisk(-(float)floats[1], cap, -(float)floats[3], RI_NULL );
				}
				if ( fabs((float)floats[2]) < fabs((float)floats[0]) )
				{
					cap = sqrt(floats[0]*floats[0] - floats[2]*floats[2]);
					RiDisk(-(float)floats[2], cap, -(float)floats[3], RI_NULL );
				}
			}
			RiSphere(floats[0], -(float)floats[1], -(float)floats[2],  -(float)floats[3], RI_NULL);
			RiTransformEnd();
		}
	}
}

//--------------------------------------------------------------------
void PGSphere::drawGL(int /*frame*/)
{
	static	MMatrix	trans;
	if (enabled)
	{
		if (bools[0])
		{
			if (!glSurf)
			{
				glSurf = new MBA_GLSurface();
				warning ("No GLSurface for \"%s\". Creating default surface", (const char*)actionName);
			}
			glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,		glSurf->Diffuse()  );
			glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,	glSurf->Specular() );
			glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,		glSurf->Ambient()  );
			glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,	glSurf->Emission() );
			glMaterialf  (GL_FRONT_AND_BACK, GL_SHININESS,	glSurf->Shininess());

			if (!GL__lighting)
			{	GL__lighting = true; glEnable(GL_LIGHTING);	}
			//if ( glListOnce || attributesChanged )
			{
				//warning("glListOnce(%d), attributesChanged(%d) -- \"%s\"", glListOnce, attributesChanged, (const char*)actionName);
				
				trans.setIdentity();
				trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
				trans.rotateX(domains[0].A3());
				trans.rotateZ(domains[0].A5());
				trans.rotateY(domains[0].A4());
				trans.translate(domains[0].A0(), domains[0].A1(), domains[0].A2());
				glDomain->CreateListRMSphere(trans, floats[0], floats[1], floats[2], floats[3], floats[4], bools[1], bools[2]);
				glListOnce = false;
			}
			glDomain->CallList();
		}
	}
}

//--------------------------------------------------------------------
void PGSphere::WriteCode(FILE*)
{
}

////--------------------------------------------------------------------
//void PGSphere::SaveToFile(FILE *outfile, int tabs)
//{
//	tabs = tabs;
//}


//==============================================================================
PGCone::PGCone(float h, float r, float thm, float d, bool drw, bool cls)
	: FlowParticleAction()
{
	baseType = FlowParticleAction::P_Geometry;
	type = FlowParticleAction::PG_Cone;
	
	actionName = "Cone";
	actionType = "Cone";
	
	floatName[0] = "height";
	floatName[1] = "radius";
	floatName[2] = "theta max";
	floatName[3] = "Detail";
	domainName[0] = "Transform";
	boolName[0] = "Draw Surface";
	boolName[1] = "Close Theta";
	boolName[2] = "Flat Cap";
	boolName[3] = "Round Cap";
	
	glDomain = new GLDomain();
	glListOnce = true;
	
	glSurf = new MBA_GLSurface();
	rmSurf = 0;
	
	floats[0] = h;
	floats[1] = r;
	floats[2] = thm;
	floats[3] = d;
	bools[0] = drw;
	bools[1] = cls;
	domains[0].SetA6(1.0);
	domains[0].SetA7(1.0);
	domains[0].SetA8(1.0);
	
	numFloats_v		= 4;
	numDomains_v	= 1;
	numBools_v		= 4;
}

//--------------------------------------------------------------------
PGCone::~PGCone()
{
}

//--------------------------------------------------------------------
PGCone::PGCone( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = FlowParticleAction::P_Geometry;
	type = FlowParticleAction::PG_Cone;
}

//--------------------------------------------------------------------
void PGCone::drawRIB(int /*frame*/)
{
	MMatrix	trans;
	if (enabled)
	{
		char	idString[128];
		sprintf(idString, "--------------------%s----------> \"%s\"", (const char*)actionType, (const char*)actionName);
		RiArchiveRecord ("comment", "", RI_NULL );		
		RiArchiveRecord ("comment", idString, RI_NULL );		
		if (rmSurf)
		{
			rmSurf->DeclareVariables();
			rmSurf->Activate();
		}
		if (rmDisp)
		{
			rmDisp->DeclareVariables();
			rmDisp->Activate();
		}
		//--
		if (bools[0])
		{
			trans.setIdentity();
			trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
			trans.rotateX(-domains[0].A3());
			trans.rotateZ( domains[0].A5());
			trans.rotateY(-domains[0].A4());
			trans.translate(domains[0].A0(), domains[0].A1(), -domains[0].A2());
			RiTransformBegin();
			trans.RIBConcatTransform();
			if (bools[2])
			{
				RiDisk(0.0, floats[1], -(float)floats[2], RI_NULL);
			}
			if (bools[3])
			{
				RiSphere(floats[1], floats[1], 0.0f, -(float)floats[2], RI_NULL);
			}
			if (bools[1])
			{
				if(bools[3])
				{
				}
			}
			RiCone( -(float)floats[0], floats[1], -(float)floats[2], RI_NULL);
			RiTransformEnd();
		}
	}
}

//--------------------------------------------------------------------
void PGCone::drawGL(int /*frame*/)
{
	static	MMatrix	trans;
	if (enabled)
	{
		if (bools[0])
		{
			if (!glSurf)
			{
				glSurf = new MBA_GLSurface();
				warning ("No GLSurface for \"%s\". Creating default surface", (const char*)actionName);
			}
			glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,		glSurf->Diffuse()  );
			glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,	glSurf->Specular() );
			glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,		glSurf->Ambient()  );
			glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,	glSurf->Emission() );
			glMaterialf  (GL_FRONT_AND_BACK, GL_SHININESS,	glSurf->Shininess());

			if (!GL__lighting)
			{	GL__lighting = true; glEnable(GL_LIGHTING);	}
			if ( glListOnce || attributesChanged )
			{
				trans.setIdentity();
				trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
				trans.rotateX(domains[0].A3());
				trans.rotateZ(domains[0].A5());
				trans.rotateY(domains[0].A4());
				trans.translate(domains[0].A0(), domains[0].A1(), domains[0].A2());
				glDomain->CreateListRMCone(trans, floats[0], floats[1], floats[2], floats[3], bools[1], bools[2], bools[3]);
				glListOnce = false;
			}
			glDomain->CallList();
		}
	}
}

//--------------------------------------------------------------------
void PGCone::WriteCode(FILE*)
{
}

////--------------------------------------------------------------------
//void PGCone::SaveToFile(FILE *outfile, int tabs)
//{
//	tabs = tabs;
//}
	
//==============================================================================
PGDisk::PGDisk(float h, float r, float thm, float d, bool drw, bool cls)
	: FlowParticleAction()
{
	baseType = FlowParticleAction::P_Geometry;
	type = FlowParticleAction::PG_Disk;
	
	actionName = "Disk";
	actionType = "Disk";
	
	floatName[0] = "height";
	floatName[1] = "radius";
	floatName[2] = "theta max";
	floatName[3] = "Detail";
	domainName[0] = "Transform";
	boolName[0] = "Draw Surface";
	boolName[1] = "Close Surface";
	
	glDomain = new GLDomain();
	glListOnce = true;
	
	glSurf = new MBA_GLSurface();
	rmSurf = 0;
	
	floats[0] = h;
	floats[1] = r;
	floats[2] = thm;
	floats[3] = d;
	bools[0] = drw;
	bools[1] = cls;
	domains[0].SetA6(1.0);
	domains[0].SetA7(1.0);
	domains[0].SetA8(1.0);
	
	numFloats_v		= 4;
	numDomains_v	= 1;
	numBools_v		= 2;
}

//--------------------------------------------------------------------
PGDisk::~PGDisk()
{
}

//--------------------------------------------------------------------
PGDisk::PGDisk( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = FlowParticleAction::P_Geometry;
	type = FlowParticleAction::PG_Disk;
}

//--------------------------------------------------------------------
void PGDisk::drawRIB(int /*frame*/)
{
	MMatrix	trans;
	if (enabled)
	{
		char	idString[128];
		sprintf(idString, "--------------------%s----------> \"%s\"", (const char*)actionType, (const char*)actionName);
		RiArchiveRecord ("comment", "", RI_NULL );		
		RiArchiveRecord ("comment", idString, RI_NULL );		
		if (rmSurf)
		{
			rmSurf->DeclareVariables();
			rmSurf->Activate();
		}
		if (rmDisp)
		{
			rmDisp->DeclareVariables();
			rmDisp->Activate();
		}
		//--
		if (bools[0])
		{
			trans.setIdentity();
			trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
			trans.rotateX(-domains[0].A3());
			trans.rotateZ( domains[0].A5());
			trans.rotateY(-domains[0].A4());
			trans.translate(domains[0].A0(), domains[0].A1(), -domains[0].A2());
			RiTransformBegin();
			trans.RIBConcatTransform();
			RiDisk( -(float)floats[0], floats[1], -(float)floats[2], RI_NULL);
			RiTransformEnd();
		}
	}
}

//--------------------------------------------------------------------
void PGDisk::drawGL(int /*frame*/)
{
	static	MMatrix	trans;
	if (enabled)
	{
		if (bools[0])
		{
			if (!glSurf)
			{
				glSurf = new MBA_GLSurface();
				warning ("No GLSurface for \"%s\". Creating default surface", (const char*)actionName);
			}
			glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,		glSurf->Diffuse()  );
			glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,	glSurf->Specular() );
			glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,		glSurf->Ambient()  );
			glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,	glSurf->Emission() );
			glMaterialf  (GL_FRONT_AND_BACK, GL_SHININESS,	glSurf->Shininess());

			if (!GL__lighting)
			{	GL__lighting = true; glEnable(GL_LIGHTING);	}
			if ( glListOnce || attributesChanged )
			{
				trans.setIdentity();
				trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
				trans.rotateX(domains[0].A3());
				trans.rotateZ(domains[0].A5());
				trans.rotateY(domains[0].A4());
				trans.translate(domains[0].A0(), domains[0].A1(), domains[0].A2());
				glDomain->CreateListRMDisk(trans, floats[0], floats[1], floats[2], floats[3]);
				glListOnce = false;
			}
			glDomain->CallList();
		}
	}
}

//--------------------------------------------------------------------
void PGDisk::WriteCode(FILE*)
{
}

////--------------------------------------------------------------------
//void PGDisk::SaveToFile(FILE *outfile, int tabs)
//{
//	tabs = tabs;
//}
	
//==============================================================================
PGCylinder::PGCylinder(float r, float zn, float zx, float thm, float d, bool drw, bool cls)
	: FlowParticleAction()
{
	baseType = FlowParticleAction::P_Geometry;
	type = FlowParticleAction::PG_Cylinder;
	
	actionName = "Cylinder";
	actionType = "Cylinder";
	
	floatName[0] = "radius";
	floatName[1] = "z min";
	floatName[2] = "z max";
	floatName[3] = "theta max";
	floatName[4] = "Detail";
	domainName[0] = "Transform";
	boolName[0] = "Draw Surface";
	boolName[1] = "Close Theta";
	boolName[2] = "Flat Caps";
	boolName[3] = "Round Caps";
	
	glDomain = new GLDomain();
	glListOnce = true;
	
	glSurf = new MBA_GLSurface();
	rmSurf = 0;
	
	floats[0] = r;
	floats[1] = zn;
	floats[2] = zx;
	floats[3] = thm;
	floats[4] = d;
	bools[0] = drw;
	bools[1] = cls;
	bools[2] = false;
	bools[3] = false;
	domains[0].SetA6(1.0);
	domains[0].SetA7(1.0);
	domains[0].SetA8(1.0);
	
	numFloats_v		= 5;
	numDomains_v	= 1;
	numBools_v		= 4;
}

//--------------------------------------------------------------------
PGCylinder::~PGCylinder()
{
}

//--------------------------------------------------------------------
PGCylinder::PGCylinder( FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
	baseType = FlowParticleAction::P_Geometry;
	type = FlowParticleAction::PG_Cylinder;
}

//--------------------------------------------------------------------
void PGCylinder::drawRIB(int /*frame*/)
{
	MMatrix	trans;
	if (enabled)
	{
		char	idString[128];
		sprintf(idString, "--------------------%s----------> \"%s\"", (const char*)actionType, (const char*)actionName);
		RiArchiveRecord ("comment", "", RI_NULL );		
		RiArchiveRecord ("comment", idString, RI_NULL );		
		if (rmSurf)
		{
			rmSurf->DeclareVariables();
			rmSurf->Activate();
		}
		if (rmDisp)
		{
			rmDisp->DeclareVariables();
			rmDisp->Activate();
		}
		//--
		if (bools[0])
		{
			trans.setIdentity();
			trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
			trans.rotateX(-domains[0].A3());
			trans.rotateZ( domains[0].A5());
			trans.rotateY(-domains[0].A4());
			trans.translate(domains[0].A0(), domains[0].A1(), -domains[0].A2());
			RiTransformBegin();
			trans.RIBConcatTransform();
			if (bools[2])	// draw flat caps
			{
				RiDisk(-(float)floats[1], floats[0], -(float)floats[3], RI_NULL);
				RiDisk(-(float)floats[2], floats[0], -(float)floats[3], RI_NULL);
			}
			if (bools[3])	// draw round caps
			{
				RiTranslate(0.0f, 0.0f, -(float)floats[1]);
				RiSphere(floats[0],  floats[0], 0.0f, -(float)floats[3], RI_NULL);
				RiTranslate(0.0f, 0.0f, floats[1]-floats[2]);
				RiSphere(floats[0], 0.0f, -(float)floats[0], -(float)floats[3], RI_NULL);
				RiTranslate(0.0f, 0.0f, floats[2]);
			}
			if (bools[1])	// seal 
			{
				if(bools[3])	// seal round caps
				{
				}
			}
			RiCylinder(floats[0], -(float)floats[1], -(float)floats[2], -(float)floats[3], RI_NULL);
			RiTransformEnd();
		}
	}
}

//--------------------------------------------------------------------
void PGCylinder::drawGL(int /*frame*/)
{
	static	MMatrix	trans;
	if (enabled)
	{
		if (bools[0])
		{
			if (!glSurf)
			{
				glSurf = new MBA_GLSurface();
				warning ("No GLSurface for \"%s\". Creating default surface", (const char*)actionName);
			}
			glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,		glSurf->Diffuse()  );
			glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,	glSurf->Specular() );
			glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,		glSurf->Ambient()  );
			glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,	glSurf->Emission() );
			glMaterialf  (GL_FRONT_AND_BACK, GL_SHININESS,	glSurf->Shininess());

			if (!GL__lighting)
			{	GL__lighting = true; glEnable(GL_LIGHTING);	}
			if ( glListOnce || attributesChanged )
			{
				//warning("glListOnce(%d), attributesChanged(%d) -- \"%s\"", glListOnce, attributesChanged, (const char*)actionName);
				
				trans.setIdentity();
				trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
				trans.rotateX(domains[0].A3());
				trans.rotateZ(domains[0].A5());
				trans.rotateY(domains[0].A4());
				trans.translate(domains[0].A0(), domains[0].A1(), domains[0].A2());
				glDomain->CreateListRMCylinder(	trans, floats[0], floats[1], floats[2], 
												floats[3], floats[4], bools[1], bools[2], bools[3]);
				glListOnce = false;
			}
			glDomain->CallList();
		}
	}
}

//--------------------------------------------------------------------
void PGCylinder::WriteCode(FILE*)
{
}

////--------------------------------------------------------------------
//void PGCylinder::SaveToFile(FILE *outfile, int tabs)
//{
//	tabs = tabs;
//}




