#include "pPolyAction.h"

#include <GL/gl.h>

#include "extern.h"
#include "pPolygonObject.h"

//===================================================================================
//===================================================================================
PPAPolygonAction::PPAPolygonAction(	)
	: FlowParticleAction()
{
	actionList.setAutoDelete(true);
}

PPAPolygonAction::~PPAPolygonAction()
{
//	warning("PPAPolygonAction::~PPAPolygonAction()");
	delete pPolyObj;
}

PPAPolygonAction::PPAPolygonAction(FlowParticleAction& copy)
	: FlowParticleAction(copy)
{
}

//--------------------------------------------------------------------
void	PPAPolygonAction::drawGL(int /*frame*/)
{
	if ( enabled && pPolyObj )
	{	
		if ( (bool)bools[0] || (bool)bools[1] )
		{
			if( domains[0].updated || attributesChanged || bools[0].updated || bools[1].updated || pPolyObj->updated )
			{
				// don't set domains[0].updated to false here or particle actions won't be updated properly
				pPolyObj->updated = false;
				bools[0].updated = false;
				bools[1].updated = false;
				trans.setIdentity();
				trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
				trans.rotateX(domains[0].A3());
				trans.rotateZ(domains[0].A5());
				trans.rotateY(domains[0].A4());
				trans.translate(domains[0].A0(), domains[0].A1(), domains[0].A2());
				RecreateDisplayList();
			}
			glCallList(glList);
		}
	}
}

//--------------------------------------------------------------------
void	PPAPolygonAction::RecreateDisplayList()
{
	attributesChanged = false;
	if ( glIsList(glList) )
	{
		glDeleteLists(glList, 1);
	}
	glList = glGenLists(1);
	glNewList(glList, GL_COMPILE);
	if (glSurf)
	{
		glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,		glSurf->Diffuse()  );
		glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,	glSurf->Specular() );
		glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,		glSurf->Ambient()  );
		glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,	glSurf->Emission() );
		glMaterialf  (GL_FRONT_AND_BACK, GL_SHININESS,	glSurf->Shininess());
	}

	glPushMatrix();
	glMultMatrixf( trans.mat() );
	glNormal3f(0.0, 1.0, 0.0);
	if (bools[1])
		pPolyObj->DrawGLBounds();
	if (bools[0])
		pPolyObj->drawGL();
	glPopMatrix();
	glEndList();
}

//==============================================================================
PPAPolygonBounce::PPAPolygonBounce(	 PPolygonObject &polys, float fric, float res, float cutoff, bool draw, bool bound )
	: PPAPolygonAction()
{
	baseType = P_PolyAction;
	type	= FlowParticleAction::PPA_PolygonBounce ;
		
	actionName		= "PolygonBounce";
	actionType		= "PolygonMetaBounce";
	floatName[0]	= "friction";
	floatName[1]	= "resilience";
	floatName[2]	= "cutoff";
	domainName[0]	= "Transform";
	boolName[0]		= "DrawSurface";
	boolName[1]		= "DrawBounds";
	boolName[2]		= "EnableAction";
	
	numFloats_v		= 3;
	numDomains_v	= 1;
	numBools_v		= 3;
	
	glSurf = new MBA_GLSurface();
	rmSurf = 0;
	rmDisp = 0;
	
	floats[0] = fric;
	floats[1] = res;
	floats[2] = cutoff;
	bools[0] = draw;
	bools[1] = bound;
	bools[2] = true;
	domains[0].SetA6(1.0);
	domains[0].SetA7(1.0);
	domains[0].SetA8(1.0);
	pPolyObj = new PPolygonObject(polys);
	attributesChanged = true;
	RecreateActionList();
	
}

PPAPolygonBounce::~PPAPolygonBounce()
{
//	warning("PPAPolygonBounce::~PPAPolygonBounce()");
}

PPAPolygonBounce::PPAPolygonBounce ( FlowParticleAction& copy, PPolygonObject &polys )
	: PPAPolygonAction(copy)
{
//	warning("PPAPolygonBounce copy");
	baseType	= P_PolyAction;
	type		= FlowParticleAction::PPA_PolygonBounce;
	
	actionList.setAutoDelete(true);
	
	floatName[0]	= "friction";
	floatName[1]	= "resilience";
	floatName[2]	= "cutoff";
	domainName[0]	= "Transform";
	boolName[0]		= "DrawSurface";
	boolName[1]		= "DrawBounds";
	boolName[2]		= "EnableAction";
	
	numFloats_v		= 3;
	numDomains_v	= 1;
	numBools_v		= 3;
	
	pPolyObj = new PPolygonObject(polys);
	attributesChanged = true;
	RecreateActionList();
}

//--------------------------------------------------------------------
void PPAPolygonBounce::Execute(int frame)
{
	int i;
	
	if ( enabled && pPolyObj )
	{
		if (floats[0].updated || floats[1].updated || floats[2].updated || 
			domains[0].updated )
		{
			//warning("PPAPolygonBounce::Execute -- updated");
			trans.setIdentity();
			trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
			trans.rotateX(domains[0].A3());
			trans.rotateZ(domains[0].A5());
			trans.rotateY(domains[0].A4());
			trans.translate(domains[0].A0(), domains[0].A1(), domains[0].A2());
			RecreateActionList();
			floats[0].updated	= false;
			floats[1].updated	= false;
			floats[2].updated	= false;
			domains[0].updated	= false;
			attributesChanged = true;
		}
	}
	if(bools[2])
	{
		for (i = 0; i < (int)actionList.count(); i++)
		{
			((FPABounce*)(actionList.at(i)))->Execute(frame);
		}
	}
}

//--------------------------------------------------------------------
void	PPAPolygonBounce::WriteCode(FILE* outfile)
{
	outfile = outfile;
}

//--------------------------------------------------------------------
void PPAPolygonBounce::drawRIB(int frame)
{
	frame = frame;
	MMatrix	trans;
	
	if (pPolyObj && (bool)bools[0])
	{
		char	idString[128];
		sprintf(idString, "--------------------%s----------> \"%s\"", (const char*)actionType, (const char*)actionName);		
		RiArchiveRecord ("comment", "", RI_NULL );		
		RiArchiveRecord ("comment", idString, RI_NULL );		
		if (rmSurf)
		{
			rmSurf->DeclareVariables();
			rmSurf->Activate();
		}
		if (rmDisp)
		{
			rmDisp->DeclareVariables();
			rmDisp->Activate();
		}

		trans.setIdentity();
		trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
		trans.rotateX(-domains[0].A3());
		trans.rotateZ( domains[0].A5());
		trans.rotateY(-domains[0].A4());
		trans.translate(domains[0].A0(), domains[0].A1(), -domains[0].A2());
		RiTransformBegin();
		trans.RIBConcatTransform();
		pPolyObj->drawRIB();
		RiTransformEnd();
	}
}

//--------------------------------------------------------------------
void	PPAPolygonBounce::RecreateActionList()
{
//	warning("RecreateActionList");
	int i;
	PPolygon	*poly;
	PDomain		tmpPDomain;
	MVertex		tVert0;
	MVertex		tVert1;
	MVertex		tVert2;
	
	actionList.clear();
	if (pPolyObj)
	{
		for (i = 0; i < pPolyObj->NumPolygons(); i++)
		{
			poly = pPolyObj->PolygonAt(i);
			tVert0 = trans*poly->v0;
			tVert1 = trans*poly->v1;
			tVert2 = trans*poly->v2;
			
			tmpPDomain.SetType(PDTriangle);
			tmpPDomain.SetA0( tVert0.X() );
			tmpPDomain.SetA1( tVert0.Y() );
			tmpPDomain.SetA2( tVert0.Z() );
			tmpPDomain.SetA3( tVert1.X() );
			tmpPDomain.SetA4( tVert1.Y() );
			tmpPDomain.SetA5( tVert1.Z() );
			tmpPDomain.SetA6( tVert2.X() );
			tmpPDomain.SetA7( tVert2.Y() );
			tmpPDomain.SetA8( tVert2.Z() );

			actionList.append(new FPABounce(floats[0], floats[1], floats[2], tmpPDomain, 1.0, 0.0, false));
		}
	}
}

//==============================================================================
PPAPolygonNoOp::PPAPolygonNoOp(	 PPolygonObject &polys, bool draw, bool bound )
	: PPAPolygonAction()
{
	warning("PPAPolygonNoOp constructor");
	baseType = P_PolyAction;
	type	= FlowParticleAction::PPA_PolygonNoOp ;
	
	actionList.setAutoDelete(true);
	
	actionName		= "PolygonNoOp";
	actionType		= "PolygonMetaNoOp";
	domainName[0]	= "Transform";
	boolName[0]		= "DrawSurface";
	boolName[1]		= "DrawBounds";
	
	numFloats_v		= 0;
	numDomains_v	= 1;
	numBools_v		= 2;
	
	glSurf = new MBA_GLSurface();
	rmSurf = 0;
	rmDisp = 0;
	
	bools[0] = draw;
	bools[1] = bound;
	domains[0].SetA6(1.0);
	domains[0].SetA7(1.0);
	domains[0].SetA8(1.0);
	pPolyObj = new PPolygonObject(polys);
	attributesChanged = true;
	RecreateActionList();
}

PPAPolygonNoOp::~PPAPolygonNoOp()
{
//	warning("PPAPolygonNoOp::~PPAPolygonNoOp()");
}

PPAPolygonNoOp::PPAPolygonNoOp ( FlowParticleAction& copy, PPolygonObject &polys )
	: PPAPolygonAction(copy)
{
	//warning("PPAPolygonNoOp copy");
	baseType	= P_PolyAction;
	type		= FlowParticleAction::PPA_PolygonNoOp;
	
	actionList.setAutoDelete(true);
	
	domainName[0]	= "Transform";
	boolName[0]		= "DrawSurface";
	boolName[1]		= "DrawBounds";
	
	numFloats_v		= 0;
	numDomains_v	= 1;
	numBools_v		= 2;
	
	pPolyObj = new PPolygonObject(polys);
	attributesChanged = true;
	RecreateActionList();
}

//--------------------------------------------------------------------
void PPAPolygonNoOp::Execute(int)
{
	if ( enabled && pPolyObj )
	{
		if (domains[0].updated )
		{
			//warning("PPAPolygonNoOp::Execute -- updated");
			trans.setIdentity();
			trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
			trans.rotateX(domains[0].A3());
			trans.rotateZ(domains[0].A5());
			trans.rotateY(domains[0].A4());
			trans.translate(domains[0].A0(), domains[0].A1(), domains[0].A2());
			domains[0].updated	= false;
			attributesChanged = true;
		}
	}
}
////--------------------------------------------------------------------
//void PPAPolygonNoOp::drawGL(int)
//{
//	if ( enabled && pPolyObj )
//	{	
//		if ( (bool)bools[0] || (bool)bools[1] )
//		{
//			if(attributesChanged || bools[0].updated || bools[1].updated )
//			{
//				bools[0].updated = false;
//				bools[1].updated = false;
//				trans.setIdentity();
//				trans.rotateX(floats[6]);
//				trans.rotateZ(floats[8]);
//				trans.rotateY(floats[7]);
//				trans.translate((float)floats[3], (float)floats[4], (float)floats[5]);
//				RecreateDisplayList();
//			}
//			glCallList(glList);
//		}
//	}
//}

//--------------------------------------------------------------------
void	PPAPolygonNoOp::WriteCode(FILE* outfile)
{
	outfile = outfile;
}

//--------------------------------------------------------------------
void PPAPolygonNoOp::drawRIB(int frame)
{
	frame = frame;
	MMatrix	trans;
	
	if (pPolyObj && (bool)bools[0])
	{
		char	idString[128];
		sprintf(idString, "--------------------%s----------> \"%s\"", (const char*)actionType, (const char*)actionName);		
		RiArchiveRecord ("comment", "", RI_NULL );		
		RiArchiveRecord ("comment", idString, RI_NULL );		
		if (rmSurf)
		{
			rmSurf->DeclareVariables();
			rmSurf->Activate();
		}
		if (rmDisp)
		{
			rmDisp->DeclareVariables();
			rmDisp->Activate();
		}

		trans.setIdentity();
		trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
		trans.rotateX(-domains[0].A3());
		trans.rotateZ( domains[0].A5());
		trans.rotateY(-domains[0].A4());
		trans.translate(domains[0].A0(), domains[0].A1(), -domains[0].A2());
		RiTransformBegin();
		trans.RIBConcatTransform();
		pPolyObj->drawRIB();
		RiTransformEnd();
	}
}


//--------------------------------------------------------------------
void	PPAPolygonNoOp::RecreateActionList()
{
}

//===================================================================================
PPAPolygonSource::PPAPolygonSource(	 PPolygonObject &polys, float rt, bool draw, bool bound   )
	: PPAPolygonAction()
{
	baseType = P_PolyAction;
	type	= FlowParticleAction::PPA_PolygonSource ;
	
	actionList.setAutoDelete(true);
	velocityList.setAutoDelete(true);
	
	actionName		= "PolygonSource";
	actionType		= "PolygonMetaSource";
	floatName[0]	= "rate";
	floatName[1]	= "vel 1";
	floatName[2]	= "vel 2";
	domainName[0]	= "Transform";
	boolName[0]		= "DrawSurface";
	boolName[1]		= "DrawBounds";
	boolName[2]		= "EnableAction";
	
	numFloats_v		= 3;
	numDomains_v	= 1;
	numBools_v		= 3;
	
	glSurf = new MBA_GLSurface();
	rmSurf = 0;
	rmDisp = 0;
	
	floats[0] = rt;
	floats[1] = 0.01;
	floats[2] = -1.0;
	bools[0] = draw;
	bools[1] = bound;
	bools[2] = true;
	domains[0].SetA6(1.0);
	domains[0].SetA7(1.0);
	domains[0].SetA8(1.0);
	pPolyObj = new PPolygonObject(polys);
	attributesChanged = true;
	RecreateActionList();
}

PPAPolygonSource::~PPAPolygonSource()
{
//	warning("PPAPolygonSource::~PPAPolygonSource()");
}

PPAPolygonSource::PPAPolygonSource ( FlowParticleAction& copy, PPolygonObject &polys )
	: PPAPolygonAction(copy)
{
//	warning("PPAPolygonSource copy");
	baseType	= P_PolyAction;
	type		= FlowParticleAction::PPA_PolygonSource;
	
	actionList.setAutoDelete(true);
	velocityList.setAutoDelete(true);
	
	floatName[0]	= "rate";
	floatName[1]	= "vel 1";
	floatName[2]	= "vel 2";
	domainName[0]	= "Transform";
	boolName[0]		= "DrawSurface";
	boolName[1]		= "DrawBounds";
	boolName[2]		= "EnableAction";
	
	numFloats_v		= 3;
	numDomains_v	= 1;
	numBools_v		= 3;
	
	pPolyObj = new PPolygonObject(polys);
	attributesChanged = true;
	RecreateActionList();
}

//--------------------------------------------------------------------
void PPAPolygonSource::Execute(int frame)
{
	int i;
	
	if ( enabled && pPolyObj )
	{
		if (floats[0].updated || floats[1].updated || floats[2].updated || 
			domains[0].updated )
		{
			//warning("PPAPolygonSource::Execute -- updated");
			rot.setIdentity();
			rot.rotateX(domains[0].A3());
			rot.rotateZ(domains[0].A5());
			rot.rotateY(domains[0].A4());
			trans.setIdentity();
			trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
			trans.rotateX(domains[0].A3());
			trans.rotateZ(domains[0].A5());
			trans.rotateY(domains[0].A4());
			trans.translate(domains[0].A0(), domains[0].A1(), domains[0].A2());
			RecreateActionList();
			floats[0].updated	= false;
			floats[1].updated	= false;
			floats[2].updated	= false;
			domains[0].updated	= false;
			attributesChanged = true;
		}
		
		if(bools[2])
		{
			for (i = 0; i < (int)actionList.count(); i++)
			{
				PPolygon *curPoly = pPolyObj->PolygonAt(i);
				if (curPoly)
				{
					MVertex	N = rot * curPoly->nP;
					if ( (float)floats[2] >= 0 )
						pVelocityD(PDLine,	N.X()*(float)floats[1], N.Y()*(float)floats[1], N.Z()*(float)floats[1], 
											N.X()*(float)floats[2], N.Y()*(float)floats[2], N.Z()*(float)floats[2]); 
					else
						pVelocity(N.X()*(float)floats[1], N.Y()*(float)floats[1], N.Z()*(float)floats[1]);
				}
				//velocityList.at(i)->Execute(frame);
				((FPASource*)(actionList.at(i)))->Execute(frame);
			}
		}
	}
}

//--------------------------------------------------------------------
void	PPAPolygonSource::WriteCode(FILE* outfile)
{
	outfile = outfile;
}

//--------------------------------------------------------------------
void PPAPolygonSource::drawRIB(int frame)
{
	frame = frame;
	MMatrix	trans;
	
	if (pPolyObj && (bool)bools[0])
	{
		char	idString[128];
		sprintf(idString, "--------------------%s----------> \"%s\"", (const char*)actionType, (const char*)actionName);		
		RiArchiveRecord ("comment", "", RI_NULL );		
		RiArchiveRecord ("comment", idString, RI_NULL );		
		if (rmSurf)
		{
			rmSurf->DeclareVariables();
			rmSurf->Activate();
		}
		if (rmDisp)
		{
			rmDisp->DeclareVariables();
			rmDisp->Activate();
		}

	//--
		trans.setIdentity();
		trans.setScale( domains[0].A6(), domains[0].A7(), domains[0].A8() );
		trans.rotateX(-domains[0].A3());
		trans.rotateZ( domains[0].A5());
		trans.rotateY(-domains[0].A4());
		trans.translate(domains[0].A0(), domains[0].A1(), -domains[0].A2());
		RiTransformBegin();
		trans.RIBConcatTransform();
		pPolyObj->drawRIB();
		RiTransformEnd();
	}
}

//--------------------------------------------------------------------
void	PPAPolygonSource::RecreateActionList()
{
//	warning("RecreateActionList");
	int i;
	PPolygon	*poly;
	PDomain		tmpPDomain;
	MVertex		tVert0;
	MVertex		tVert1;
	MVertex		tVert2;
	MVertex		tVertC;
	MVertex		tNorm;
	float		rateArea;
	
	actionList.clear();
	velocityList.clear();
	if (pPolyObj)
	{
		for (i = 0; i < pPolyObj->NumPolygons(); i++)
		{
			poly = pPolyObj->PolygonAt(i);
			tVert0 = trans*poly->v0;
			tVert1 = trans*poly->v1;
			tVert2 = trans*poly->v2;
			tVertC = trans*poly->centroid;
			tNorm = trans*poly->nP;
			tNorm.normalize();
			tNorm = tNorm * (float)floats[1];
			
			tmpPDomain.SetType(PDTriangle);
			tmpPDomain.SetA0( tVert0.X() );
			tmpPDomain.SetA1( tVert0.Y() );
			tmpPDomain.SetA2( tVert0.Z() );
			tmpPDomain.SetA3( tVert1.X() );
			tmpPDomain.SetA4( tVert1.Y() );
			tmpPDomain.SetA5( tVert1.Z() );
			tmpPDomain.SetA6( tVert2.X() );
			tmpPDomain.SetA7( tVert2.Y() );
			tmpPDomain.SetA8( tVert2.Z() );
//			tmpPDomain.SetType(PDPoint);
//			tmpPDomain.SetA0( tVertC.X() );
//			tmpPDomain.SetA1( tVertC.Y() );
//			tmpPDomain.SetA2( tVertC.Z() );

			rateArea = (poly->area)/pPolyObj->MaxPArea();
			actionList.append(new FPASource(rateArea*(float)floats[0], tmpPDomain));
//			actionList.append(new FPASource(1.0, tmpPDomain));
			velocityList.append(new FPSVelocity(tNorm.X(), tNorm.Y(), tNorm.Z()) );
//			velocityList.append(new FPSVelocity(0.0, (float)floats[1], 0.0) );
		}
	}
	
}


//===================================================================================
PPAPolygonEmitter::PPAPolygonEmitter(	 PPolygonObject &polys, float rt, bool draw, bool bound   )
	: PPAPolygonSource(polys, rt, draw, bound)
{
	baseType = P_PolyAction;
	type	= FlowParticleAction::PPA_PolygonSource ;
	actionName		= "PolygonEmitter";
	actionType		= "PolygonMetaEmitter";
	RecreateActionList();
}

PPAPolygonEmitter::~PPAPolygonEmitter()
{
//	warning("PPAPolygonEmitter::~PPAPolygonEmitter()");
}

PPAPolygonEmitter::PPAPolygonEmitter ( FlowParticleAction& copy, PPolygonObject &polys )
	: PPAPolygonSource(copy, polys)
{
	baseType	= P_PolyAction;
	type		= FlowParticleAction::PPA_PolygonEmitter;
	actionType	= "PolygonMetaEmitter";
	RecreateActionList();
}

//--------------------------------------------------------------------
void	PPAPolygonEmitter::RecreateActionList()
{
//	warning("RecreateActionList");
	int i;
	PPolygon	*poly;
	PDomain		tmpPDomain;
	MVertex		tVert0;
	MVertex		tVert1;
	MVertex		tVert2;
	MVertex		tVertC;
	MVertex		tNorm;
	
	actionList.clear();
	velocityList.clear();
	if (pPolyObj)
	{
		for (i = 0; i < pPolyObj->NumPolygons(); i++)
		{
			poly = pPolyObj->PolygonAt(i);
			tVert0 = trans*poly->v0;
			tVert1 = trans*poly->v1;
			tVert2 = trans*poly->v2;
			tVertC = trans*poly->centroid;
			tNorm = trans*poly->nP;
			tNorm.normalize();
			tNorm = tNorm * (float)floats[1];

			tmpPDomain.SetType(PDPoint);
			tmpPDomain.SetA0( tVertC.X() );
			tmpPDomain.SetA1( tVertC.Y() );
			tmpPDomain.SetA2( tVertC.Z() );

			actionList.append(new FPASource(1.0, tmpPDomain));
			velocityList.append(new FPSVelocity(tNorm.X(), tNorm.Y(), tNorm.Z()) );
		}
	}
}

//--------------------------------------------------------------------
void PPAPolygonAction::SaveToFile(FILE* outfile, int tabs)
{
	int		i;
	int		j;
	bool	tmpBool;
	char	buffer[128];
	PDomain	*domain;

	
	char	*walker = buffer;
	for (i = 0; i < tabs; i++)
	{	*walker = '\t';	*(walker+1) = '\0';	walker++;	}
	
	switch(baseType)
	{
		case P_Undefined:
			return;
		case P_State:
			sprintf(walker, "\tPState %s\n", ActionType() );
			break;
		case P_Action:
			sprintf(walker, "\tPAction %s\n", ActionType() );
			break;
		case P_PolyAction:
			sprintf(walker, "\tPAction %s\n", ActionType() );
			break;
		case P_Geometry:
			sprintf(walker, "\tPGeometry %s\n", ActionType() );
			break;
	}
	fputs(buffer, outfile);
		
	sprintf(walker, "\t{\n" );
	fputs(buffer, outfile);
	sprintf(walker, "\t\tNAME	%s\n", (const char*)Name() );
	fputs(buffer, outfile);
	for (j = 0; j < (int)numFloats(); j++)
	{
		sprintf(walker, "\t\tFLOAT	%s\n", (const char*)FloatName(j) );
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", (float)*(Float(j)), Float(j)->Animation() );
		fputs(buffer, outfile);
	}
	for (j = 0; j < (int)NumDomains(); j++)
	{
		domain = Domain(j);
		sprintf(walker, "\t\tDOMAIN	%s\n", (const char*)DomainName(j));
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%d\n", (int)domain->Type());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A0(), domain->PFloatA0()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A1(), domain->PFloatA1()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A2(), domain->PFloatA2()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A3(), domain->PFloatA3()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A4(), domain->PFloatA4()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A5(), domain->PFloatA5()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A6(), domain->PFloatA6()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A7(), domain->PFloatA7()->Animation());
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%f\t%d\n", domain->A8(), domain->PFloatA8()->Animation());
		fputs(buffer, outfile);
	}
	for (j = 0; j < (int)NumBools(); j++)
	{
		tmpBool = (*(Bool(j))).Val();
		sprintf(walker, "\t\tBOOL	%s\n", (const char*)BoolName(j) );
		fputs(buffer, outfile);
		sprintf(walker, "\t\t\t%d\n", tmpBool );
		fputs(buffer, outfile);
	}

	if (GLSurface())
		GLSurface()->SaveToFile(outfile, tabs+2);

	if (RMSurface())
		RMSurface()->SaveToFile(outfile, tabs+2);

	if (RMDisplace())
		RMDisplace()->SaveToFile(outfile, tabs+2);

	if (pPolyObj)
		pPolyObj->SaveToFile(outfile, tabs+2);
		
	sprintf(walker, "\t}\n" );
	fputs(buffer, outfile);
}
	





