#ifndef pMetaAction_h
#define pMetaAction_h

#include <qlist.h>
#include "pAction.h"
#include "mmath.h"

class PPolygonObject;

/**
 * Poly-Actions - collects multiple simple actions for higer level
 *   objects
 */
//====================================================================
class PPAPolygonAction	: public FlowParticleAction
{
public:
	PPAPolygonAction();
	~PPAPolygonAction();
	PPAPolygonAction(FlowParticleAction& copy);
	
	virtual void	RecreateActionList() = 0;
	virtual void	drawGL(int frame);
	virtual void	RecreateDisplayList();
	virtual	void	SaveToFile(FILE *outfile, int tabs);
	
	
	inline	PPolygonObject	*PolygonObject()	{	return pPolyObj;	}

protected:
	float			largestArea;
	PPolygonObject	*pPolyObj;
	QList<FlowParticleAction>	actionList;
	
	MMatrix		trans;
	GLuint		glList;
};
	
/**
 *
 */
//====================================================================
class PPAPolygonBounce	: public PPAPolygonAction
{
public:
	PPAPolygonBounce( PPolygonObject &, float fric = 0.0, float res = 0.0, float cutoff = 0.0, bool draw = true, bool bound = true );
	~PPAPolygonBounce();
	PPAPolygonBounce(FlowParticleAction&, PPolygonObject &);
	
//	void	drawGL(int frame);
	void	Execute(int frame);
	void	drawRIB(int frame);
	void	WriteCode(FILE* outfile);
	
	void	RecreateActionList();
};

/**
 *
 */
//====================================================================
class PPAPolygonNoOp	: public PPAPolygonAction
{
public:
	PPAPolygonNoOp( PPolygonObject &, bool draw = true, bool bound = true );
	~PPAPolygonNoOp();
	PPAPolygonNoOp(FlowParticleAction&, PPolygonObject &);
	
//	void	drawGL(int frame);
	void	Execute(int frame);
	void	drawRIB(int frame);
	void	WriteCode(FILE* outfile);
	
	void	RecreateActionList();
};

/**
 *
 */
//====================================================================
class PPAPolygonSource	: public PPAPolygonAction
{
public:
	PPAPolygonSource( PPolygonObject &, float r = 0.01, bool draw = true, bool bound = true );
	~PPAPolygonSource();
	PPAPolygonSource(FlowParticleAction&, PPolygonObject &);
	
//	void	drawGL(int frame);
	void	Execute(int frame);
	void	drawRIB(int frame);
	void	WriteCode(FILE* outfile);
	
	virtual void	RecreateActionList();
	QList<FPSVelocity>	velocityList;

	MMatrix		rot;
};

/**
 *
 */
//====================================================================
class PPAPolygonEmitter	: public PPAPolygonSource
{
public:
	PPAPolygonEmitter( PPolygonObject &, float r = 0.01, bool draw = true, bool bound = true );
	~PPAPolygonEmitter();
	PPAPolygonEmitter(FlowParticleAction&, PPolygonObject &);

	void	RecreateActionList();
};


#endif
