#ifndef pPolygonObject_h
#define pPolygonObject_h

#include <stdio.h>

#include "mmath.h"
#include <qlist.h>
#include <qstring.h>

/**
 *
 */
//====================================================================
class PPolygon
{
public:
	PPolygon();
	PPolygon(const char*);
	PPolygon( MVertex* v, MVertex* n );
	PPolygon(PPolygon &copy);
	~PPolygon();

	bool	ReadFromString(const char*);
	
	void	drawGL();
	void	drawGL(float addX, float addY, float addZ);
	void	DrawGLNormals(float scale = 1.0);
	void	drawRIB();
	void	drawRIB(float addX, float addY, float addZ);
	
	void	InvertNormal(bool x = true, bool y = true, bool z = true);
	void	InvertPolygon();
	
protected:
    void    calculateArea( void );

public:
	//-----
	MVertex 	v0;
	MVertex 	v1;
	MVertex 	v2;
	MVertex 	n0;
	MVertex 	n1;
	MVertex		n2;
	MVertex		nP;
	MVertex		centroid;
	float		area;

	static	int icount;
};

/**
 *
 */
//====================================================================
class PPolygonObject
{
public:
	PPolygonObject(FILE*);
	PPolygonObject(PPolygonObject&);
	~PPolygonObject();

	void	drawGL();
	void	DrawGLBounds();
	void	drawRIB();
	
	void	readPointsPolygons(FILE *infile, QString &tmpString, int &byteSize);
	
	void	SaveToFile(FILE *infile, int tabs);
	
	inline int	NumPolygons()		{	return (int)polygonList.count();	}
	PPolygon	*PolygonAt(int i);
	
	inline	void	SetHighlightPoly(int in)	{	updated = true; hilightPoly = in;	}
	inline	void	SetShowAllNorms(bool in)	{	updated = true; allNorms = in;		}
	inline	void	SetNormalScale(float in)	{	updated = true; normalScale = in;	}
	inline	float	NormalScale()				{	return normalScale;	}
	inline	float	MaxPArea()					{	return maxArea;	}
	
	void	InvertNormal(int, bool x = true, bool y = true, bool z = true);
	void	InvertPolygon(int);
	
	MVertex&	PolyNormal(int);

	bool	updated;

protected:
	void    updateBoundingBox( PPolygon* newPoly );
	
protected:
	int		hilightPoly;
	bool	allNorms;
	float	normalScale;
	QList<PPolygon>	polygonList;
	static	MVertex	nullVertex;
	
	float	maxX;
	float	minX;
	float	maxY;
	float	minY;
	float	maxZ;
	float	minZ;
	float	maxArea;
};

#endif //pPolygonObject_h


