#include "pPolygonTweak.h"

#include "extern.h"
#include <qpixmap.h>
//#include "Pixmaps/flowIcon.xpm"
#include "pPolygonObject.h"

//==============================================================================
PolygonTweak::PolygonTweak(QWidget *parent, const char* name)
	: QWidget (parent, name)
{
	setCaption("flow::Polygon Tweaker");
	setIcon(QPixmap((const char**)flowIcon_xpm));
	
//	polyObject = _pPolygonObject;
	polyObject = 0;
	
	polyScroll = new QScrollBar( QScrollBar::Horizontal, this);
	polyScroll->setFixedHeight(25);
	polyScroll->setSteps(1, 1);
	polyScroll->setTracking(true);
	polyScroll->setRange(0, 0);
	
	polyNumLCD	= new QLCDNumber(6, this);
	polyNumLCD->setFixedSize(g_numInputWidth/2, 25);
	polyNumLCD->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	polyNumLCD->setSegmentStyle( QLCDNumber::Flat );
//	polyNumLCD->setSegmentStyle( QLCDNumber::Filled );
	
	showAllNorms	= new QPushButton(this);
	showAllNorms->setText("show all norms");
	showAllNorms->setFixedSize(g_numInputWidth, 25);
	showAllNorms->setToggleButton(true);
	
	normalScaleInput = new NumInput(NumInput::Double, this);
	normalScaleInput->SetLabelWidth(g_numLabelWidth);
	normalScaleInput->SetTotalWidth(g_numInputWidth);
	normalScaleInput->SetLabel("norm scale");
	
	invertNormalBut	= new QPushButton(this);
	invertNormalBut->setText("invert norm");
	invertNormalBut->setFixedSize(g_numInputWidth, 25);
	
	invertPolyBut	= new QPushButton(this);
	invertPolyBut->setText("invert poly");
	invertPolyBut->setFixedSize(g_numInputWidth, 25);

	//--
	modeSelectCombo = new QComboBox(this);
	modeSelectCombo->setFixedSize(g_numInputWidth, 25);
	modeSelectCombo->insertItem("invert normal");
	modeSelectCombo->insertItem("invert polygon");

	startAtInput = new NumInput(NumInput::Int, this);
	startAtInput->SetLabelWidth(g_numLabelWidth);
	startAtInput->SetTotalWidth(g_numInputWidth);
	startAtInput->SetLabel("start at");
	startAtInput->SetValueSolid(0.0);
	
	finishAtInput = new NumInput(NumInput::Int, this);
	finishAtInput->SetLabelWidth(g_numLabelWidth);
	finishAtInput->SetTotalWidth(g_numInputWidth);
	finishAtInput->SetLabel("finish at");
	finishAtInput->SetValueSolid(0.0);
	
	skipInput = new NumInput(NumInput::Int, this);
	skipInput->SetLabelWidth(g_numLabelWidth);
	skipInput->SetTotalWidth(g_numInputWidth);
	skipInput->SetLabel("skip");
	skipInput->SetValueSolid(0.0);
		
	invertBut	= new QPushButton(this);
	invertBut->setText("invert go");
	invertBut->setFixedSize(g_numInputWidth, 25);

	int	normButHeight = 20;
	xNormBut	= new MToolButton(this);
	xNormBut->setText("x");
	xNormBut->setFixedHeight(normButHeight);
	xNormBut->setToggleButton(true);

	yNormBut	= new MToolButton(this);
	yNormBut->setText("y");
	yNormBut->setFixedHeight(normButHeight);
	yNormBut->setToggleButton(true);

	zNormBut	= new MToolButton(this);
	zNormBut->setText("z");
	zNormBut->setFixedHeight(normButHeight);
	zNormBut->setToggleButton(true);

	xNormBut->setOn(true);
	yNormBut->setOn(true);
	zNormBut->setOn(true);
	
	//-- Connect -----------------------
	QObject::connect (polyScroll, SIGNAL(valueChanged(int)), SLOT(SetHighlightPoly(int)) );
	QObject::connect (polyScroll, SIGNAL(valueChanged(int)), polyNumLCD, SLOT(display(int)) );
	QObject::connect (normalScaleInput, SIGNAL(ValueChanged(float)), SLOT(SetNormalScale(float)) );
	QObject::connect (showAllNorms, SIGNAL(toggled(bool)), SLOT(SetShowAllNorms(bool)) );
	QObject::connect (invertNormalBut, SIGNAL(clicked()), SLOT(InvertNormal()) );
	QObject::connect (invertPolyBut, SIGNAL(clicked()), SLOT(InvertPolygon()) );
	QObject::connect (invertBut, SIGNAL(clicked()), SLOT(DoBatch()) );

	
	//-- Layout ------------------------
	topVBoxLayout	= new QVBoxLayout(this, 5, 5);
	h1Layout		= new QHBoxLayout(5);
	h2Layout		= new QHBoxLayout(5);
	v1Layout		= new QVBoxLayout(5);
	v2Layout		= new QVBoxLayout(5);
	v3Layout		= new QVBoxLayout(5);
	normButLayout	= new QHBoxLayout(1);
	
	topVBoxLayout->addLayout(h1Layout, 1);
		h1Layout->addLayout(v1Layout, 0);
			v1Layout->addWidget(normalScaleInput);
			v1Layout->addWidget(showAllNorms);
			v1Layout->addStretch(5);
		h1Layout->addLayout(v2Layout, 0);
			v2Layout->addWidget(invertPolyBut);
			v2Layout->addWidget(invertNormalBut);
			v2Layout->addLayout(normButLayout);
				normButLayout->addWidget(xNormBut);
				normButLayout->addWidget(yNormBut);
				normButLayout->addWidget(zNormBut);
			v2Layout->addStretch(5);
		h1Layout->addStretch(5);
		h1Layout->addLayout(v3Layout, 0);
			v3Layout->addWidget( modeSelectCombo );
			v3Layout->addWidget( startAtInput );
			v3Layout->addWidget( finishAtInput );
			v3Layout->addWidget( skipInput );
			v3Layout->addWidget( invertBut );
			v3Layout->addStretch(5);
	topVBoxLayout->addLayout(h2Layout);
		h2Layout->addWidget(polyScroll, 1);
		h2Layout->addWidget(polyNumLCD, 0);
	
	topVBoxLayout->activate();
	
	SetPolyObject(polyObject);
	resize(400, 200);
}

PolygonTweak::~PolygonTweak()
{
}

//--------------------------------------------------------------------
void	PolygonTweak::SetHighlightPoly(int in)
{
	if (polyObject)
	{
		polyObject->SetHighlightPoly(in);
	}
}

//--------------------------------------------------------------------
void	PolygonTweak::SetShowAllNorms(bool in)
{
	if (polyObject)
	{
		polyObject->SetShowAllNorms(in);
	}
}

//--------------------------------------------------------------------
void	PolygonTweak::SetNormalScale(float in)
{
	if (polyObject)
	{
		polyObject->SetNormalScale(in);
	}
}

//--------------------------------------------------------------------
void	PolygonTweak::InvertNormal()
{
	if (polyObject)
		polyObject->InvertNormal(polyScroll->value(), 
								 xNormBut->isOn(), yNormBut->isOn(), zNormBut->isOn() );
}

//--------------------------------------------------------------------
void	PolygonTweak::InvertPolygon()
{
	if (polyObject)
		polyObject->InvertPolygon(polyScroll->value());
}

//--------------------------------------------------------------------
void	PolygonTweak::DoBatch()
{
	int i;
	if (polyObject)
	{
		switch(modeSelectCombo->currentItem())
		{
			case 0: // normal
				for( i = startAtInput->intVal(); i <= finishAtInput->intVal(); i += 1 + skipInput->intVal() )
				{
					polyObject->InvertNormal(i, xNormBut->isOn(), yNormBut->isOn(), zNormBut->isOn() );
				}
				break;
			case 1: // polygon
				for( i = startAtInput->intVal(); i <= finishAtInput->intVal(); i += 1 + skipInput->intVal() )
				{
					polyObject->InvertPolygon(i);
				}
				break;
		}
	}
}

//--------------------------------------------------------------------
void	PolygonTweak::SetPolyObject(PPolygonObject* in)
{
	PPolygonObject	*oldPolyObj = polyObject;
	
	polyObject = in;
	if (polyObject)
	{
		polyScroll->setEnabled(true);
		normalScaleInput->setEnabled(true);
		skipInput->setEnabled(true);
		startAtInput->setEnabled(true);
		finishAtInput->setEnabled(true);
		invertBut->setEnabled(true);
		invertNormalBut->setEnabled(true);
		invertPolyBut->setEnabled(true);
		modeSelectCombo->setEnabled(true);
		polyScroll->setRange(0, polyObject->NumPolygons()-1);
		normalScaleInput->SetValueSolid(polyObject->NormalScale());
		skipInput->SetValueSolid( 0.0 );
		startAtInput->SetValueSolid( 0.0 );
		finishAtInput->SetValueSolid( polyObject->NumPolygons()-1 );
	}
	else
	{	
		if (oldPolyObj)
			oldPolyObj->SetHighlightPoly(-1);
		polyScroll->setEnabled(false);
		normalScaleInput->setEnabled(false);
		skipInput->setEnabled(false);
		startAtInput->setEnabled(false);
		finishAtInput->setEnabled(false);
		polyScroll->setRange(0, 0);
		invertBut->setEnabled(false);
		invertNormalBut->setEnabled(false);
		invertPolyBut->setEnabled(false);
		modeSelectCombo->setEnabled(false);
	}
}

