#include "partDrawOGL.h"

#include <GL/glu.h>

#ifdef SORT_TIME
	#include <sys/time.h>
	#include <unistd.h>
#endif

#include "glPrimitives.h"
#include "pPolygonObject.h"
#include "particleControl.h"
#include "camera.h"
#include "extern.h"

#include "GLExt.h"

#include "tex_constant.h"
#include "tex_glow.h"

extern	int				g_frame;
extern	CameraView		*g_globalCam;
extern _ParticleState	_ps;


GLuint	 PartDrawOGL::tex[2];
int		 PartDrawOGL::curTex = 0;

bool	 PartDrawOGL::sort			=  true;
int		 PartDrawOGL::poolSize		=  0;
int		 PartDrawOGL::p_countLast	= -1;
int		*PartDrawOGL::randIndex		=  0;
float	 PartDrawOGL::scaleColorOut[4]	=  { 1.0, 1.0, 1.0, 1.0 };
PNode	*PartDrawOGL::partPool		=  0;
MVertex	 PartDrawOGL::eyePos;
MMatrix	 PartDrawOGL::camRot;
MMatrix	 PartDrawOGL::velRot;

int		 PartDrawOGL::skip		= 1;
int		 PartDrawOGL::drawCount	= 0;

int		 PartDrawOGL::tmpCount = 0;

//==============================================================================
PartDrawOGL::PartDrawOGL()
{
}

PartDrawOGL::~PartDrawOGL()
{
}

//--------------------------------------------------------------------
void PartDrawOGL::initTextures()
{
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glGenTextures(1, &tex[0]);
	glBindTexture(GL_TEXTURE_2D, tex[0]);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	glTexImage2D(GL_TEXTURE_2D, 0,   tex_0Comp, tex_0w, tex_0h, 0, (GLenum)tex_0Format, GL_UNSIGNED_BYTE, tex_0); 
	gluBuild2DMipmaps(GL_TEXTURE_2D, tex_0Comp, tex_0w, tex_0h,    (GLenum)tex_0Format, GL_UNSIGNED_BYTE, tex_0);
	
	
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glGenTextures(1, &tex[1]);
	glBindTexture(GL_TEXTURE_2D, tex[1]);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	glTexImage2D(GL_TEXTURE_2D, 0,   tex_1Comp, tex_1w, tex_1h, 0, (GLenum)tex_1Format, GL_UNSIGNED_BYTE, tex_1); 
	gluBuild2DMipmaps(GL_TEXTURE_2D, tex_1Comp, tex_1w, tex_1h,    (GLenum)tex_1Format, GL_UNSIGNED_BYTE, tex_1);
	
	curTex = 1;

	glAlphaFunc(GL_GEQUAL, 0.05);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glBindTexture(GL_TEXTURE_2D, tex[curTex]);
}	

//--------------------------------------------------------------------
void PartDrawOGL::setSorting(bool status)
{
	sort = status;
}	

//--------------------------------------------------------------------
void PartDrawOGL::setEyePoint(float x, float y, float z)
{
	eyePos.setX(x);
	eyePos.setY(y);
	eyePos.setZ(z);
}
void PartDrawOGL::setEyePoint(float *pos)
{
	eyePos.setX(pos[0]);
	eyePos.setY(pos[1]);
	eyePos.setZ(pos[2]);
}

////--------------------------------------------------------------------
//GLfloat* PartDrawOGL::scaleColor(float *in)
//{
//	static GLfloat	out[4];
//	out[0] = (float)in[0];
//	out[1] = (float)in[1];
//	out[2] = (float)in[2];
//	out[3] = (float)in[3];
//	float max = 1.0;
//	if (out[0] > max)
//		max = out[0];
//	if (out[1] > max)
//		max = out[1];
//	if (out[2] > max)
//		max = out[2];
//	if (max > 1.0)
//	{
//		float tmp = 1.0/max;
//		out[0] *= tmp;
//		out[1] *= tmp;
//		out[2] *= tmp;
//	}
//	return out;
//}

/**
 * Draw current particle group. 
 * @param primitive primitive type
 */
//--------------------------------------------------------------------
void PartDrawOGL::pDrawGroupp(ParticleControl::PPrimitiveType primitive, int skip)
{	
	skip += 1;
	// Get a pointer to the particles in gp memory
	ParticleGroup *pg = _ps.pgrp;
	if(pg == NULL)
		return; // ERROR
		
	if(pg->p_count < 1)
		return;
	
//	if(GLExt::hasPointsParameters())
//	{
//		GLfloat quadratic[3] = { 0.25, 0.0, 1/60.0 };
//		glPointParameterfvEXT(GL_DISTANCE_ATTENUATION_EXT, quadratic);
//	}
	
	//-- Set material to a known state...
	float	emis[4] = { 0.00, 0.00, 0.00, 1.00 };
	float	ambt[4] = { 0.00, 0.00, 0.00, 1.00 };
	float	diff[4] = { 1.00, 1.00, 1.00, 1.00 };
	float	spec[4] = { 0.50, 0.50, 0.50, 1.00 };
	glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION,	emis);
	glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,	ambt);
	glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,	diff);
	glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR,	spec);
	glMaterialf  (GL_FRONT_AND_BACK, GL_SHININESS,	100);
	
	glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
	
	if(!g_glFastParticles)
	{
		if(g_glAlphaTest)
			glEnable(GL_ALPHA_TEST);
		if(!tex[curTex])
			initTextures();
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex[curTex]);
	}
	
	int i;
	if(sort)
	{
		sortParticles(pg, skip);
		switch(primitive)
		{ 
			case ParticleControl::Point:
			case ParticleControl::Disk:
				#ifndef RIVA_GLX 
				if(g_glFastParticles)
					glBegin(GL_POINTS);
				#endif
				break;
			case ParticleControl::Line:
				if(g_glFastParticles)
					glBegin(GL_LINES);
				break;
			default:
				break;
		}
		walkTree(&partPool[0], primitive);
//	fprintf(stdout, "\n\n");
		
		#ifdef RIVA_GLX
		if(primitive == ParticleControl::Line)
		#endif
		if(g_glFastParticles)
			glEnd();
	}
	else
	{
		switch(primitive)
		{
			case ParticleControl::Point:
				#ifndef RIVA_GLX
				if(g_glFastParticles)	
					glBegin(GL_POINTS);
				#endif
				for(i = 0; i < pg->p_count; i+=skip)
					drawSphere(pg->list[i], g_glFastParticles);
				#ifndef RIVA_GLX
				if(g_glFastParticles)	
					glEnd();
				#endif
				break;
			case ParticleControl::Disk:
				#ifndef RIVA_GLX
				if(g_glFastParticles)	
					glBegin(GL_POINTS);
				#endif
				for(i = 0; i < pg->p_count; i+=skip)
					drawDisk(pg->list[i], g_glFastParticles);
				#ifndef RIVA_GLX
				if(g_glFastParticles)	
					glEnd();
				#endif
				break;
			case ParticleControl::Line:
				if(g_glFastParticles)	
					glBegin(GL_LINES);
				for(i = 0; i < pg->p_count; i+=skip)
					drawLine(pg->list[i], g_glFastParticles);
				if(g_glFastParticles)	
					glEnd();
				break;
			case ParticleControl::Polygon:
				break;
			case ParticleControl::Connect:
				break;
		}
	}

	glDisable(GL_COLOR_MATERIAL);
	if(!g_glFastParticles)
	{
		glDisable(GL_TEXTURE_2D);
		if(g_glAlphaTest)
			glDisable(GL_ALPHA_TEST);
	}

}

/**
 * Sort particle group contents into an unbalanced binary tree.
 * A pool of particle nodes are allocated, and is dynamically resized 
 * to 1.5 * num particles. This memory is NOT autmatically freed for the 
 * sake of speed. Call setPoolSize() to reclaim memory.
 * 
 */ 
//--------------------------------------------------------------------
void	PartDrawOGL::sortParticles(ParticleGroup *pg, int skip)
{
	static int lastCount = 0;
	
	int numParts = pg->p_count/skip;
	//-- resize the pool, if necessary
	//PNode	*newPool = 0;
	if(poolSize <= numParts)
	{
		poolSize = numParts+numParts/2;
		setPoolSize(poolSize);
	}
	
#ifdef SORT_TIME
	static double timeaccum = 0;
	struct timeval	begtime;
	struct timeval	endtime;
	struct timeval	difftime;
	gettimeofday(&begtime, 0);
#endif
	
//	if(lastCount != pg->p_count)
	{
		float	rank;
		float	diff[3];
		int		poolIndex = 0;
		for(int i = 0; i < pg->p_count; i+=skip)
		{
			Particle &p =  pg->list[ i ];
//			Particle &p =  pg->list[ randIndex[i] ];
			diff[0] = p.pos.x - eyePos.x();
			diff[1] = p.pos.y - eyePos.y();
			diff[2] = p.pos.z - eyePos.z();
			//-- don't bother taking square root
			rank = diff[0]*diff[0] + diff[1]*diff[1] + diff[2]*diff[2];
			partPool[poolIndex].rank	 = rank;
			partPool[poolIndex].particle = &pg->list[i];
			partPool[poolIndex].less	 = 0;
			partPool[poolIndex].more	 = 0;
			if(poolIndex)
				insertParticle(&partPool[poolIndex], &partPool[0]);
			poolIndex++;
		}
	}

	lastCount = pg->p_count;

#ifdef SORT_TIME
	gettimeofday(&endtime, 0);
	difftime.tv_sec  = endtime.tv_sec  - begtime.tv_sec ;
	difftime.tv_usec = endtime.tv_usec - begtime.tv_usec;
	timeaccum = (double)(1000000*difftime.tv_sec + difftime.tv_usec);
	if(g_frame%100 == 0)
	{
		warning("100 frame average for depth sorting: %f usecs", timeaccum/100.0);
		timeaccum = 0;
	}
#endif

//	warning("sortParticles -- poolSize = %d, numParts = %d, pg->p_count = %d", poolSize, numParts, pg->p_count);
}
		
//--------------------------------------------------------------------
void	PartDrawOGL::insertParticle(PNode* node, PNode *treeNode)
{
	if(node->rank > treeNode->rank)
	{
		if(treeNode->more)
			insertParticle(node, treeNode->more);
		else
			treeNode->more = node;
	}
	else if(node->rank < treeNode->rank)
	{
		if(treeNode->less)
			insertParticle(node, treeNode->less);
		else
			treeNode->less = node;
	}		
}

		
//--------------------------------------------------------------------
void	PartDrawOGL::walkTree(PNode *node, ParticleControl::PPrimitiveType primitive)
{
	if(node->more)
		walkTree(node->more, primitive);
	
	Particle	*p = node->particle;
	switch(primitive)
	{
		case ParticleControl::Point:
			drawSphere(*p, g_glFastParticles);
			break;
		case ParticleControl::Disk:
			drawDisk(*p, g_glFastParticles);
			break;
		case ParticleControl::Line:
			drawLine(*p, g_glFastParticles);
			break;
		case ParticleControl::Polygon:
			break;
		case ParticleControl::Connect:
			break;
	}
	if(node->less)
		walkTree(node->less, primitive);
}
				
//--------------------------------------------------------------------
void	PartDrawOGL::walkTreePrint(PNode *node, ParticleControl::PPrimitiveType primitive)
{
	if(node->more)
		walkTreePrint(node->more, primitive);
		
	warning("%04d  %f", tmpCount, node->rank);
	tmpCount++;
	
	if(node->less)
		walkTreePrint(node->less, primitive);
}

/**
 * Set size of particle sort pool. If num particles > pool size, 
 * it is automatically resized to num particles * 1.5.
 * This member is mostly used for reclaiming memory if a huge
 * particle system has been sorted, but the current system
 * is smaller.
 */
//--------------------------------------------------------------------
void	PartDrawOGL::setPoolSize(int size)
{
	int i;
	delete [] partPool;
	poolSize = size;
	partPool	= new PNode[poolSize];
	for(i = 0; i < poolSize; i++)
	{
		partPool[i].index = i;
	}
	
	// Only resize if smaller...
	if(p_countLast < size)
	{
		delete [] randIndex;
		randIndex = new int[poolSize];
		for(i = 0; i < poolSize; i++)
		{
			randIndex[i] = -1;
		}
		int done = 0;
		int current = 0;
		while(done < poolSize-1)
		{
			if(randIndex[current] < 0)
			{
				randIndex[current] = done;
				done++;
				current++;
			}
			current += rand()/(RAND_MAX/7);
			current = current%poolSize;
		}
	}
}

//--------------------------------------
void	PartDrawOGL::drawSphere	(Particle &m, bool fast)
{
	drawCount++;
	glColor4fv(scaleColor((float*)&m.color));
	if(fast )
	{
		#ifdef RIVA_GLX	
		glBegin(GL_POINTS);
		#endif			
		glVertex3fv((GLfloat *)&m.pos);
		#ifdef RIVA_GLX	
		glEnd();
		#endif			
	}
	else
	{
		if(!(drawCount%skip))
		{
			MVertex	vel( 0.0, 1.0, 0.0 ); // point billboard straight up to avoid flickering
			camRot.pointAt(	MVertex(eyePos.x() - m.pos.x, eyePos.y() - m.pos.y, eyePos.z() - m.pos.z), vel);
		}
		drawTexQuad(m, false);
	}
}

//--------------------------------------
void	PartDrawOGL::drawDisk	(Particle &m, bool fast)
{
	drawCount++;
	glColor4fv(scaleColor((float*)&m.color));
	if(fast)
	{
		#ifdef RIVA_GLX	
		glBegin(GL_POINTS);
		#endif			
		glVertex3fv((GLfloat *)&m.pos);
		#ifdef RIVA_GLX	
		glEnd();
		#endif			
	}
	else
	{
		if(!(drawCount%skip))
		{
			MVertex	vel(-m.vel.x, -m.vel.y, -m.vel.z);
			camRot.pointAt(	MVertex(eyePos.x()- m.pos.x, eyePos.y() - m.pos.y, eyePos.z() - m.pos.z), vel);
		}
		drawTexQuad(m, true);
	}
}

//--------------------------------------
void	PartDrawOGL::drawLine	(Particle &m, bool fast)
{
	drawCount++;
	glColor4fv(scaleColor((float*)&m.color));
	if(fast)
	{
		glVertex3fv((GLfloat *)&m.pos);
		// For lines, make a tail with the velocity vector's direction and
		// a length
		pVector tail(-m.vel.x, -m.vel.y, -m.vel.z);
		tail *= m.size.y;
		tail += m.pos;
		glVertex3fv((GLfloat *)&tail);
	}
	else
	{	
		if(!(drawCount%skip))
		{
			MVertex	vel(-m.vel.x, -m.vel.y, -m.vel.z);
			camRot.pointAt(	MVertex(eyePos.x()- m.pos.x, eyePos.y() - m.pos.y, eyePos.z() - m.pos.z), vel);
		}
		drawTexQuad(m, false, true);
	}
}

//--------------------------------------
void	PartDrawOGL::drawPolygon(Particle &m, bool fast)
{
	if(true || fast)
	{
		glColor4fv(scaleColor((float*)&m.color));
	}
	else
	{
	}
}

//--------------------------------------
void	PartDrawOGL::drawConnect(Particle &m, bool fast)
{
	if(true || fast)
	{
		glColor4fv(scaleColor((float*)&m.color));
	}
	else
	{
	}
}

//--------------------------------------
void	PartDrawOGL::drawTexQuad(Particle &m, bool flat, bool stretch)
{
		/* stuff for tail */
		GLfloat	r = 1.0;
		GLfloat	l = 0.0;
		if(stretch)
		{
			float	camDist;

			MVertex	velVec(m.vel.x, m.vel.y, m.vel.z);
			MVertex camVec(m.pos.x-eyePos.x(), m.pos.y-eyePos.y(), m.pos.z-eyePos.z());
			camDist = camVec.length();
			velVec.normalize();
			camVec.normalize();
			float	angle = fabs(velVec.dot(camVec));
			float	angleShrink = 1.0 - angle*angle;

			l = m.size.y * m.vel.length();
			
			if(camDist < l)		/* get rid of funky artifacts when particles get too close to camera */
				angleShrink *= camDist/l;
				
			l *= angleShrink;	/* faked perspective correction for length */
		} 
		r = m.size.x;

		GLfloat x = m.pos.x;
		GLfloat y = m.pos.y;
		GLfloat z = m.pos.z;
		
		glPushMatrix();	//-- Push/Pop on the GeForce seems to be a bottleneck...
		glTranslatef(x, y, z);
		glMultMatrixf(camRot.matx);

		float nx, ny, nz;
		if(flat)
		{	nx = 0.0;	ny = 0.0;	nz = 1.0;	}
		else
		{	nx = 0.58;	ny = 0.58;	nz = 0.58;	}
		
		glBegin(GL_TRIANGLE_STRIP);
			glNormal3f( nx,  ny,  nz);
			glTexCoord2f(0.0, 0.0); glVertex3f( r,  r+l, 0.0f);
			glNormal3f(-nx,  ny,  nz);
			glTexCoord2f(1.0, 0.0); glVertex3f(-r,  r+l, 0.0f);
			glNormal3f( nx, -ny,  nz);
			glTexCoord2f(0.0, 1.0); glVertex3f( r,   -r, 0.0f);
			glNormal3f(-nx, -ny,  nz);
			glTexCoord2f(1.0, 1.0); glVertex3f(-r,   -r, 0.0f);
		glEnd();
		
		glPopMatrix();
}
	

//==============================================================================
PNode::PNode()
{
	rank		= -1.0;
	particle	=  0;
	less		=  0;
	more		=  0;
}

PNode::~PNode()
{
}
