#ifndef partDrawOGL_h
#define partDrawOGL_h

#include "compatibility.h"
#include <GL/gl.h>
#include <particle/general.h>

#include "particleControl.h"
#include "mmath.h"

/**
 * PNode
 */
//==============================================================================
class PNode
{
public:
	PNode();
	~PNode();

	int			index;
	float		rank;
	Particle	*particle;
	PNode		*less;
	PNode		*more;
	
};

/**
 * PartDrawOGL
 */
//==============================================================================
class PartDrawOGL
{
public:
	PartDrawOGL();
	~PartDrawOGL();

	static	void	pDrawGroupp(ParticleControl::PPrimitiveType primitive, int skip);
	
	static	void	initTextures();
	
	static	void	sortParticles(ParticleGroup *pg, int skip = 0);
	static	void	setPoolSize(int size = 100);
	
	static	void	setSorting(bool);
	static	void	setEyePoint(float x, float y, float z);
	static	void	setEyePoint(float *pos);
	
	static	void	drawSphere	(Particle &m, bool fast = true);
	static	void	drawDisk	(Particle &m, bool fast = true);
	static	void	drawLine	(Particle &m, bool fast = true);
	static	void	drawPolygon	(Particle &m, bool fast = true);
	static	void	drawConnect	(Particle &m, bool fast = true);
	
	static	void	drawTexQuad	(Particle &m, bool flat = true, bool stretch = false);

protected:

private:
	static	GLfloat	scaleColorOut[4];
	static	GLfloat* scaleColor(float *in)
	{
		scaleColorOut[0] = (float)in[0];
		scaleColorOut[1] = (float)in[1];
		scaleColorOut[2] = (float)in[2];
		scaleColorOut[3] = (float)in[3];
		float max = 1.0;
		if (scaleColorOut[0] > max)
			max = scaleColorOut[0];
		if (scaleColorOut[1] > max)
			max = scaleColorOut[1];
		if (scaleColorOut[2] > max)
			max = scaleColorOut[2];
		if (max > 1.0)
		{
			float tmp = 1.0/max;
			scaleColorOut[0] *= tmp;
			scaleColorOut[1] *= tmp;
			scaleColorOut[2] *= tmp;
		}
		return scaleColorOut;
	}	
	static	void	insertParticle(PNode* node, PNode *treeNode);
	static	void	walkTree(PNode* node, ParticleControl::PPrimitiveType primitive);
	static	void	walkTreePrint(PNode* node, ParticleControl::PPrimitiveType primitive);
//	static	void	walkSort(ParticleControl::PPrimitiveType primitive);
	
//	static	void	sortIndexes();
	
	static	bool		sort;
//	static	float		eyePos[3];
	static	MVertex		eyePos;
	
	static	int			poolSize;
	static	PNode		*partPool;
//	static	int			index;
	static	int			*randIndex;
	static	int			p_countLast;
	static	int			drawCount;
	static	int			skip;
	
	static	int	tmpCount;
	
	static	MMatrix	camRot;	//-- this shouldn't be static. Don't keep this class as singleton
	static	MMatrix	velRot;	//-- this shouldn't be static. Don't keep this class as singleton

	static	int		curTex;
	static	GLuint	tex[2];
};

#endif // partDrawOGL_h




