#include "particleControl.h"
#include "extern.h"

#include "mbaScene.h"
#include "lightGroup.h"
#include "pPolyAction.h"
#include "hitList.h"

#include "partDrawOGL.h"

//*************************//
//My versions of pDrawGroup//
void m_pDrawGroupp(int primitive, int skip = 0, PPolygonObject *polygons = 0);
void m_pDrawGroupl(int dlist, bool const_size=false, bool const_color=false, bool const_rotation=false);

void m_pDrawGrouppSort(int primitive, int skip = 0, PPolygonObject *polygons = 0);
//*************************//

/**
 * Default constructor.
 */
//==============================================================================
ParticleControl::ParticleControl()
{
	hasLighting		= false;
	enabled			= true;
	name			= "default";
	maxParticles	= 1000;
//	maxParticles	= 0;
	primitiveType	= Point;
	rmSurface		= new MBA_RMShader(g_defaultSurfaceShader);
	rmDisplace		= 0;
//	rmDisplace		= new MBA_RMShader(g_defaultDisplaceShader);;
	pointSize	= 1.0;
	lineWidth	= 1.0;
	
	//-- Init particle stuff
	particle_handle = pGenParticleGroups(1, 0);
	pCurrentGroup(particle_handle);
	
	//action_handle = pGenActionLists(1);
	
	PDomain	tmpDomain0(PDLine, 
						 1.0,
						 1.0,
						 1.0, 
						 1.0, 
						 1.0,
						 1.0	);
	PDomain	tmpDomain1(PDCylinder, 
						0.0,
						0.065,
						0.0, 
						0.0, 
						0.0645,
						0.0,
						0.005,
						0.0045);
	PDomain	tmpDomain2(PDPoint, 
						0.0,
						0.1,
						0.0 );	
	PDomain	tmpDomain3(PDPlane, 
						-5.0,
						 0.0,
						-5.0, 
						 10.0, 
						 0.0,
						 0.0,
						 0.0,
						 0.0,
						 10.0	);
	PDomain	tmpDomain4(PDLine, 
						 0.1,
						 2.0,
						 0.0, 
						 0.05, 
						 2.0,
						 0.0	);
	FlowParticleAction	*newAction;
	
	actionList.append(new FPSTimeStep(1.0));
	actionList.append(new FPSColorDomain(1.0, tmpDomain0));
	actionList.append(new FPSSizeDomain(tmpDomain4));
	actionList.append(new FPSVelocityDomain(tmpDomain1));
	actionList.append(new FPASource( 3, tmpDomain2));
	actionList.append(new FPAGravity(0.0, -0.002, 0.0) );
	actionList.append(new FPAKillOld(400.0, false) );
	newAction = new FPABounce( 0.0, 0.5, 0.0, tmpDomain3, 10.0, 0.0, true);
	newAction->SetName("Bounce - floor plane");
//	newAction->setEnabled(false);
	actionList.append(newAction);
	
}

/**
 * Copy constructor.
 * @param blurCopy omit particle actions that create or destroy particles.
 */
//==============================================================================
ParticleControl::ParticleControl(ParticleControl &copy, bool blurCopy)
{
	enabled			= copy.enabled;
	name			= copy.name;
	maxParticles	= copy.maxParticles;
	primitiveType	= copy.primitiveType;
	if(copy.rmSurface)
		rmSurface	= new MBA_RMShader( *(copy.rmSurface) );
	else
		rmSurface	= 0;
	if(copy.rmDisplace)
		rmDisplace	= new MBA_RMShader( *(copy.rmDisplace) );
	else
		rmDisplace	= 0;

	pointSize	= copy.pointSize;
	lineWidth	= copy.lineWidth;
	
	//-- Init particle stuff
	particle_handle = pGenParticleGroups(1, 0);
	pCurrentGroup(particle_handle);
	pSetMaxParticles(maxParticles);
	
	pCopyGroup(copy.particle_handle);
	
	//action_handle = pGenActionLists(1);
	
	for(int i = 0; i < copy.numActions(); i++)
	{
		FlowParticleAction	*tmpAction = copy.actionList.at(i);
		if(blurCopy)
		{
			if(	tmpAction->BaseType() == FlowParticleAction::P_State ||
				tmpAction->Type() == FlowParticleAction::PA_KillOld ||
				tmpAction->Type() == FlowParticleAction::PA_KillSlow ||
				tmpAction->Type() == FlowParticleAction::PA_Sink ||
				tmpAction->Type() == FlowParticleAction::PA_SinkVelocity ||
				tmpAction->Type() == FlowParticleAction::PA_Source ||
				tmpAction->Type() == FlowParticleAction::PA_Vertex ||
				tmpAction->Type() == FlowParticleAction::PPA_PolygonSource				
			)
				continue;
		}
		actionList.append( new FlowParticleAction( *(copy.actionList.at(i)) ) );
	}
}

/**
 * destroy
 */
ParticleControl::~ParticleControl()
{
	pDeleteParticleGroups(particle_handle, 1);
//	warning("ParticleControl::~ParticleControl() -- %d in actionList", actionList.count());
	actionList.setAutoDelete(true);
	actionList.clear();
//	warning("                                       %d in actionList", actionList.count());
}

/**
 * look up index from FlowParticleAction pointer
 */
//----------------------------------------------------------
int		ParticleControl::findAction(FlowParticleAction* cur)
{
	return actionList.find(cur);	
}

/**
 * Execute all actions in actionList.
 */
//----------------------------------------------------------
void	ParticleControl::Execute(int frame)
{
	if (enabled)
	{
		int i;
		frame = frame;
		pCurrentGroup(particle_handle);
		for (i = 0; i < (int)actionList.count(); i++)
		{
			(actionList.at(i))->Execute(frame);	
		}
		pMove();
	}
}

/**
 * Render particles and actions
 */
//----------------------------------------------------------
void	ParticleControl::drawGL(int frame)
{
	if (enabled)
	{
		int i;
		pCurrentGroup(particle_handle);
		//-- draw solids before particles which are likely to be transparent...
		for (i = 0; i < (int)actionList.count(); i++)
		{
			GL__lighting = true; glEnable(GL_LIGHTING);
			(actionList.at(i))->drawGL(frame);	
		}
		
		if(!hasLighting)
		{	GL__lighting = false; glDisable(GL_LIGHTING);	}
		
		PartDrawOGL::setEyePoint(g_globalCam->pos());
		PartDrawOGL::setSorting(false);
		if(g_glBlend)
		{
			glDepthMask(GL_FALSE);
			if(g_glSortParticles)
				PartDrawOGL::setSorting(true);
		}
		 
		
//		PPolygonObject *polyObj = 0;
		switch (primitiveType)
		{
			case Point:
				if(g_glThick) glPointSize(pointSize); else glPointSize(1.0);
				//m_pDrawGroupp(Point, g_partSkip);
				PartDrawOGL::pDrawGroupp(Point, g_partSkip);
				break;
			case Line:
				if(g_glThick) glLineWidth(lineWidth);
				//m_pDrawGroupp(Line, g_partSkip);
				PartDrawOGL::pDrawGroupp(Line, g_partSkip);
				break;
			case Disk:
				if(g_glThick) glPointSize(pointSize); else glPointSize(1.0);
				//m_pDrawGroupp(Disk, g_partSkip);
				PartDrawOGL::pDrawGroupp(Disk, g_partSkip);
				break;
			case Polygon:
//				for (i = 0; i < (int)actionList.count(); i++)
//					if ( (actionList.at(i))->Type() == FlowParticleAction::PPA_PolygonEmitter )
//						polyObj = ((PPAPolygonAction*)actionList.at(i))->PolygonObject();
//				m_pDrawGroupp(Polygon, g_partSkip, polyObj);
				break;
			case Connect:
//				if(g_glThick) glLineWidth(lineWidth);
//				m_pDrawGroupp(Connect);
				break;
		}
		glPointSize(1.0);
		glLineWidth(1.0);
	}
	glDepthMask(GL_TRUE);
}

/**
 * Clear all particles. Start from scratch.
 */
//----------------------------------------------------------
void	ParticleControl::ResetSystem( )
{
	pCurrentGroup(particle_handle);
	pSetMaxParticles(0);
	pSetMaxParticles(maxParticles);
}

/**
 * Set max particle count for group
 */
//----------------------------------------------------------
void	ParticleControl::SetMaxParticles( int in )
{
	pCurrentGroup(particle_handle);
	pSetMaxParticles(maxParticles = in);
}

/**
 * Rename action
 */
//----------------------------------------------------------
void	ParticleControl::SetActionName( uint index, const char* newName )
{
	if (index >= actionList.count() )
		warning ("ERROR: ParticleControl::SetActionName(%d) - max is %d", index, actionList.count() );
	else
		(actionList.at(index))->SetName(newName);
}

/**
 * Bogus hack for mouse-picking selection
 */
//----------------------------------------------------------
void	ParticleControl::generateHitList(HitList &hL)
{
	int 	i;
	hL.reset();
	FlowParticleAction::PAType	paType;
	hL.addCameraPrimitive(g_globalCam);
	for(i = 0; i < numActions(); i++)
	{
		if(actionList.at(i)->Enabled())
		{
			paType = actionList.at(i)->Type();
			if (	paType == FlowParticleAction::PA_Bounce ||
					paType == FlowParticleAction::PA_Sink ||
					paType == FlowParticleAction::PA_Source)
			{
				hL.addActionPrimitive(actionList.at(i));
			}
		}
	}
	hL.checkForOldActiveTarget();
}
