#ifndef particleControl_h
#define particleControl_h

#include "compatibility.h"
#include <GL/gl.h>
#include <stdio.h>
#include "particle/papi.h"

#include <qlist.h> 
#include <qstring.h> 

#include "pAction.h"
#include "mbaRMShader.h"

class HitList;
//#define PRIMITIVE_POINTS 0
//#define PRIMITIVE_LINES	1
//

/**
 * Class that interfaces with the particle API
 */
//===========================================================
class ParticleControl
{
public:
	ParticleControl();
	ParticleControl(ParticleControl &copy, bool blurCopy = false);
	~ParticleControl();

	enum PPrimitiveType	{	
		Point,		/**< change this to 'Sphere' */
		Line, 		/**< line, from position to position-velocity */
		Disk,		/**< camera-facing disk */
		Polygon, 	/**< UNUSED */
		Connect 	/**< UNUSED */
	};
	
	void	drawGL(int frame);
	void	Execute(int frame);
	void	WriteCode(FILE *outfile);
	
	void	RenderParticles();
	void	Render_RIBShader();

	void	ResetSystem();
	bool	SaveParticleSystem(FILE* outfile, int tabs = 0);
	bool	LoadParticleSystem(FILE* infile, const char *inName = 0);
	void	SetMaxParticles (int in);
	void	SetActionName (uint index, const char* newName);
	
	void	generateHitList(HitList &hL);
	int		findAction(FlowParticleAction*);
	
	/** @return name of particle system */
	inline	const char*	Name()				{	return (const char*)name;	}		
	/** set name of particle system  */
	inline	void 	SetName(const char* n)	{	name = n;					}		
	/** count of particle actions in system*/
	inline	int		numActions()			{	return (int)actionList.count();	}	
	/** enable/disable particle system */
	inline	void	setEnabled(bool status)	{	enabled = status;			}		
	/** @return enabled/disabled state */
	inline	bool	isEnabled()				{	return enabled;				}		
	/** @return max particles allowed in this system */
	inline	int		getMaxParticles()		{	return maxParticles;		}		
	/** @return PAPI particle handle for this system */
	inline	int		getParticleHandle()		{	return particle_handle;		}		
	/** set primitive type */
	inline	void	SetPrimitiveType(int in)	{	primitiveType = (PPrimitiveType)in;	}	
	/** @return primitiveType */
	inline	PPrimitiveType	PrimitiveType()		{	return primitiveType;	}				
	/** @return point size */
	inline	float	PointSize()				{	return pointSize;	}
	/** @return line width */		
	inline	float	LineWidth()				{	return lineWidth;	}
	/** set size of points for GL rendering */		
	inline	void	SetPointSize(float in)	{	pointSize = in;		}
	/** set width of lines for GL rendering */		
	inline	void	SetLineWidth(float in)	{	lineWidth = in;		}
	
	inline void setLighting(bool status)	{ 	hasLighting = status;	}
	inline bool lighting()					{ 	return hasLighting;	}

	/** ugly */    
	inline MBA_RMShader **pointerToRMSurface()  {   return &rmSurface;  }
	/** ugly */    
	inline MBA_RMShader *RMSurface() 		   {   return rmSurface;   }
	/** ugly */    
	inline MBA_RMShader **pointerToRMDisplace() {   return &rmDisplace; }
	/** ugly */    
	inline MBA_RMShader *RMDisplace()		   {   return rmDisplace;  }

public: /*protected*/
	QList<FlowParticleAction>	actionList;
	
protected: 
	FlowParticleAction *ReadActionGeneric(const char* type, FILE *infile);
	FlowParticleAction *ReadPolygon (const char* inType, const char* buffer);

	//----------------------------------
//	int				action_handle;

	

private:
	char	*KillNewline (char *);
	bool	ReadDomain (FILE*, PDomain&);
	bool	ReadFloat (FILE*, PFloat&);
	bool	ReadBool (FILE*, PBool&);
	
	bool			enabled;
	bool			hasLighting;
	QCString		name;
	PPrimitiveType	primitiveType;
	int				maxParticles;
	int				particle_handle;
	float	pointSize;
	float	lineWidth;
	
	MBA_RMShader	*rmSurface;
	MBA_RMShader	*rmDisplace;

};


#endif
