#include "particleControlWidget.h"
#include <qmessagebox.h>
#include <qfiledialog.h>
#include "pPolyAction.h"
#include "pGeometry.h"

#include "mbaSurfaceTabDialog.h"
#include "extern.h"
#include "glPortCam.h"
#include "pPolygonObject.h"

/**
 * Delete action from list.
 */
//----------------------------------------------------------
void	ParticleControlWidget::DeleteAction()
{
	int i = actionListBox->currentItem();
	FlowParticleAction		*action;
	if (partCtrl)
	{
	   	switch( QMessageBox::warning( this, "confirm delete", "delete current action?", "&ok", "&cancel" )) 
		{
			case 0:
				if ( i >= 0 && i < (int)partCtrl->actionList.count() )
				{
					//warning("actionListBox->count( %d )", actionListBox->count() );
					actionListBox->removeItem(i);
					action = partCtrl->actionList.at(i);
					if(deferedActiveAction == action)
						deferedActiveAction = 0;
					partCtrl->actionList.remove(i);

					//FIXME!!! We will crash if there's no actions in the system...
					
					currentActionIndex = -1;
					if(i < (int)partCtrl->actionList.count())
						SwitchAction(i);
					else
						SwitchAction(0);
					delete action;
				}
				break;
		}
	}
	emit Updated();
}

//----------------------------------------------------------
void	ParticleControlWidget::CopyAction()
{
	QCString	currentType;
	FlowParticleAction *oldAction = 0;
	FlowParticleAction *newAction = 0;
	
	if (partCtrl)
	{	
		int index = actionListBox->currentItem();
		if (index >= 0)
		{
			oldAction = partCtrl->actionList.at(index);
			currentType = (const char*)oldAction->ActionType();
		}
		else
			return;

		//---- STATE		
		if ( currentType == "Color" )
			partCtrl->actionList.append( newAction = new FPSColor( *oldAction ) );
		else if ( currentType == "ColorDomain" )
			partCtrl->actionList.append( newAction = new FPSColorDomain( *oldAction ) );
		else if ( currentType == "Size" )
			partCtrl->actionList.append( newAction = new FPSSize( *oldAction ) );
		else if ( currentType == "SizeDomain" )
			partCtrl->actionList.append( newAction = new FPSSizeDomain( *oldAction ) );
		else if ( currentType == "StartingAge" )
			partCtrl->actionList.append( newAction = new FPSStartingAge( *oldAction ) );
		else if ( currentType == "TimeStep" )
			partCtrl->actionList.append( newAction = new FPSTimeStep( *oldAction ) );
		else if ( currentType == "Velocity" )
			partCtrl->actionList.append( newAction = new FPSVelocity( *oldAction ) );
		else if ( currentType == "VelocityDomain" )
			partCtrl->actionList.append( newAction = new FPSVelocityDomain( *oldAction ) );
		else if ( currentType == "Vertex2ndPosition" )
			partCtrl->actionList.append( newAction = new FPSVertex2ndPosition( *oldAction ) );
		else if ( currentType == "Vertex2ndPositionDomain" )
			partCtrl->actionList.append( newAction = new FPSVertex2ndPositionDomain( *oldAction ) );
		else if ( currentType == "Vertex2ndPositionTracks" )
			partCtrl->actionList.append( newAction = new FPSVertex2ndPositionTracks( *oldAction ) );
		//---- ACTIONS		
		else if ( currentType == "Bounce" )
			partCtrl->actionList.append( newAction = new FPABounce( *oldAction ) );
		else if ( currentType == "Damping" )
			partCtrl->actionList.append( newAction = new FPADamping( *oldAction ) );
		else if ( currentType == "Explosion" )
			partCtrl->actionList.append( newAction = new FPAExplosion( *oldAction ) );
		else if ( currentType == "Follow" )
			partCtrl->actionList.append( newAction = new FPAFollow( *oldAction ) );
		else if ( currentType == "Gravitate" )
			partCtrl->actionList.append( newAction = new FPAGravitate( *oldAction ) );
		else if ( currentType == "Gravity" )
			partCtrl->actionList.append( newAction = new FPAGravity( *oldAction ) );
		else if ( currentType == "Jet" )
			partCtrl->actionList.append( newAction = new FPAJet( *oldAction ) );
		else if ( currentType == "KillOld" )
			partCtrl->actionList.append( newAction = new FPAKillOld( *oldAction ) );
//		else if ( currentType == "KillSlow" )
//			partCtrl->actionList.append( newAction = new FPAKillSlow( *oldAction ) );
		else if ( currentType == "OrbitLine" )
			partCtrl->actionList.append( newAction = new FPAOrbitLine( *oldAction ) );
		else if ( currentType == "OrbitPoint" )
			partCtrl->actionList.append( newAction = new FPAOrbitPoint( *oldAction ) );
		else if ( currentType == "RandomAccel" )
			partCtrl->actionList.append( newAction = new FPARandomAccel( *oldAction ) );
		else if ( currentType == "RandomDisplace" )
			partCtrl->actionList.append( newAction = new FPARandomDisplace( *oldAction ) );
		else if ( currentType == "RandomVelocity" )
			partCtrl->actionList.append( newAction = new FPARandomVelocity( *oldAction ) );
		else if ( currentType == "Restore" )
			partCtrl->actionList.append( newAction = new FPARestore( *oldAction ) );
		else if ( currentType == "Sink" )
			partCtrl->actionList.append( newAction = new FPASink( *oldAction ) );
		else if ( currentType == "Sink Velocity" )
			partCtrl->actionList.append( newAction = new FPASinkVelocity( *oldAction ) );
		else if ( currentType == "Source" )
			partCtrl->actionList.append( newAction = new FPASource( *oldAction ) );
		else if ( currentType == "TargetColor" )
			partCtrl->actionList.append( newAction = new FPATargetColor( *oldAction ) );
		else if ( currentType == "TargetSize" )
			partCtrl->actionList.append( newAction = new FPATargetSize( *oldAction ) );
		else if ( currentType == "Vertex" )
			partCtrl->actionList.append( newAction = new FPAVertex( *oldAction ) );
		else if ( currentType == "Vortex" )
			partCtrl->actionList.append( newAction = new FPAVortex( *oldAction ) );
		else if ( currentType == "Turbulence" )
			partCtrl->actionList.append( newAction = new FPATurbulence( *oldAction ) );
			
		else if ( currentType == "Sphere" )
			partCtrl->actionList.append( newAction = new PGSphere( *oldAction ));
		else if ( currentType == "Cone" )
			partCtrl->actionList.append( newAction = new PGCone( *oldAction ));
		else if ( currentType == "Disk" )
			partCtrl->actionList.append( newAction = new PGDisk( *oldAction ));
		else if ( currentType == "Cylinder" )
			partCtrl->actionList.append( newAction = new PGCylinder( *oldAction ));

		int	i;
		if (newAction)
		{
			actionListBox->insertItem((const char*)(newAction->Name()));
			i = actionListBox->count()-1;
			actionListBox->setCurrentItem( i );
		}
	}
	emit Updated();
}

//----------------------------------------------------------
void	ParticleControlWidget::AddAction()
{
	int		actionIndex;
	uint	listIndex;
	
	if (partCtrl)
	{
		if ( currentActionIndex >= 0 && currentActionIndex < (int)partCtrl->actionList.count() )
			listIndex = (uint)currentActionIndex;
		else
			listIndex = partCtrl->actionList.count();

		FlowParticleAction *newAction = 0;

		addParticleAction->show();
		if (addParticleAction->result() == QDialog::Accepted) 
			actionIndex = addParticleAction->selected();
		else
			actionIndex = -1;

		PDomain tmpDomain;
		PDomain bounceDomain(PDPlane,  -1.0,  0.0, -1.0,
										2.0,  0.0,  0.0, 
										0.0,  0.0,  2.0 );

		switch( addParticleAction->WhichType() )
		{
			case AddParticleActionWidget::PFuncUndefined:
				return /*FIXME - false*/;
			case AddParticleActionWidget::PFuncState: 
				switch(actionIndex)
				{
					case -1:
						break;
					case 0://Color
						partCtrl->actionList.insert( listIndex, newAction = new FPSColor( PFloat(1.0), PFloat(1.0), PFloat(1.0) ) );
						break;
					case 1://Color Domain
						warning ("list index = %d", listIndex );
						partCtrl->actionList.insert( listIndex, newAction = new FPSColorDomain( 1.0, tmpDomain ) );
						break;
					case 2://Particle Size
						partCtrl->actionList.insert( listIndex, newAction = new FPSSize( 1.0 ) );
						break;
					case 3://Particle SizeDomain
						partCtrl->actionList.insert( listIndex, newAction = new FPSSizeDomain( tmpDomain ) );
						break;
					case 4://Starting Age
						partCtrl->actionList.insert( listIndex, newAction = new FPSStartingAge( 0.0 ) );
						break;
					case 5://Time Step
						partCtrl->actionList.insert( listIndex, newAction = new FPSTimeStep( 1.0 ) );
						break;
					case 6://Velocity
						partCtrl->actionList.insert( listIndex, newAction = new FPSVelocity( 0.0, 0.0, 0.0 ) );
						break;
					case 7://Velocity Domain
						partCtrl->actionList.insert( listIndex, newAction = new FPSVelocityDomain( tmpDomain ) );
						break;
					case 8://Secondary position
						partCtrl->actionList.insert( listIndex, newAction = new FPSVertex2ndPosition( 0.0, 0.0, 0.0 ) );
						break;
					case 9://Secondary position domain
						partCtrl->actionList.insert( listIndex, newAction = new FPSVertex2ndPositionDomain( tmpDomain ) );
						break;
					case 10://Track secondary position
						partCtrl->actionList.insert( listIndex, newAction = new FPSVertex2ndPositionTracks( true ) );
						break;

				}
				break;
			case AddParticleActionWidget::PFuncAction: 
				switch(actionIndex)
				{
					case -1:
						break;
					case 0://Bounce
						partCtrl->actionList.insert( listIndex, newAction = new FPABounce(0.0, 0.0, 0.0, bounceDomain) );
						break;
					case 1://Damping
						partCtrl->actionList.insert( listIndex, newAction = new FPADamping( 1.0, 1.0, 1.0) );
						break;
					case 2://Explosion
						partCtrl->actionList.insert( listIndex, newAction = new FPAExplosion( 0.0, 0.0, 0.0, 
																		0.1, 0.01, 3.0, P_EPS) );
						break;
					case 3://Follow
						partCtrl->actionList.insert( listIndex, newAction = new FPAFollow() );
						break;
					case 4://Gravitate
						partCtrl->actionList.insert( listIndex, newAction = new FPAGravitate(1.0) );
						break;
					case 5://Gravity
						partCtrl->actionList.insert( listIndex, newAction = new FPAGravity( 0.0, 0.0, 0.0) );
						break;
					case 6://Jet
						partCtrl->actionList.insert( listIndex, newAction = new FPAJet( 0.0, 0.0, 0.0) );
						break;
					case 7://Kill Old
						partCtrl->actionList.insert( listIndex, newAction = new FPAKillOld( 0.0 ) );
						break;
//					case 8://Kill Slow
//						partCtrl->actionList.insert( listIndex, newAction = new FPAKillSlow( 0.0 ) );
//						break;
					case 9://Orbit Line
						partCtrl->actionList.insert( listIndex, newAction = new FPAOrbitLine(	0.0, 0.0, 0.0,
																		0.0, 1.0, 0.0, 0.01) );
						break;
					case 10://Orbit Point
						partCtrl->actionList.insert( listIndex, newAction = new FPAOrbitPoint(0.0, 0.0, 0.0, 0.01) );
						break;
					case 11://Random Acceleration
						partCtrl->actionList.insert( listIndex, newAction = new FPARandomAccel( tmpDomain ) );
						break;
					case 12://Random Displace
						partCtrl->actionList.insert( listIndex, newAction = new FPARandomDisplace( tmpDomain ) );
						break;
					case 13://Random Velocity
						partCtrl->actionList.insert( listIndex, newAction = new FPARandomVelocity( tmpDomain ) );
						break;
					case 14://Restore
						partCtrl->actionList.insert( listIndex, newAction = new FPARestore( 0.0 ) );
						break;
					case 15://Sink
						partCtrl->actionList.insert( listIndex, newAction = new FPASink( false, tmpDomain ) );
						break;
					case 16://Sink Velocity
						partCtrl->actionList.insert( listIndex, newAction = new FPASinkVelocity( true, tmpDomain ) );
						break;
					case 17://Source
						partCtrl->actionList.insert( listIndex, newAction = new FPASource( 0.0, tmpDomain ) );
						break;
					case 18://Target Color
						partCtrl->actionList.insert( listIndex, newAction = new FPATargetColor( 1.0, 1.0, 1.0, 1.0, 0.005 ) );
						break;
					case 19://Target Size
						partCtrl->actionList.insert( listIndex, newAction = new FPATargetSize( 0.1,  0.1,  0.1, 0.005, 0.005 , 0.005) );
						break;
					case 20://Vertex
						partCtrl->actionList.insert( listIndex, newAction = new FPAVertex( 0.0, 0.0, 0.0 ) );
						break;
					case 21://Vortex
						partCtrl->actionList.insert( listIndex, newAction = new FPAVortex(	0.0, 0.0, 0.0, 
																		0.0, 0.0, 1.0,
																		1.0, 1.0 ) );
						break;
					case 22://Turbulence
						partCtrl->actionList.insert( listIndex, newAction = new FPATurbulence() );
						break;
				}
				break;
			case AddParticleActionWidget::PFuncPoly: 
				if (actionIndex >= 0)
				{
					PPolygonObject *pPolygonObject = 0;
					FILE	*tmpFile;
					const char *filename = GetPolyFileName();
					if (filename)
					{
						tmpFile = fopen(filename, "r");
						if (tmpFile)
						{
							pPolygonObject = new PPolygonObject(tmpFile);
							fclose(tmpFile);
							switch(actionIndex)
							{
								case -1:
									break;
								case 0:	// polygonal bounce
									partCtrl->actionList.insert( listIndex, newAction = new PPAPolygonBounce(*pPolygonObject));
									break;
								case 1:	// polygonal source
									partCtrl->actionList.insert( listIndex, newAction = new PPAPolygonSource(*pPolygonObject));
									break;
								case 2:	// polygonal NoOp
									partCtrl->actionList.insert( listIndex, newAction = new PPAPolygonNoOp(*pPolygonObject));
									break;
							}
							delete pPolygonObject;
						}
					}
				}
				break;
			case AddParticleActionWidget::PFuncGeom:
				switch(actionIndex)
				{
					case 0:// Sphere
						partCtrl->actionList.insert( listIndex, newAction = new PGSphere());
						break;
					case 1:// Cone
						partCtrl->actionList.insert( listIndex, newAction = new PGCone());
						break;
					case 2:// Disk
						partCtrl->actionList.insert( listIndex, newAction = new PGDisk());
						break;
					case 3:// Cylinder
						partCtrl->actionList.insert( listIndex, newAction = new PGCylinder());
						break;
				}
				break;
		}

		if (newAction)
		{
			if(newAction->Enabled())
				actionListBox->insertItem((const char*)(newAction->Name()), listIndex);
			else
				actionListBox->insertItem((const char*)(newAction->DisabledName()), listIndex);
			currentActionIndex = -1;
			actionListBox->setCurrentItem( listIndex );
		}
	}
	emit Updated();
}

//----------------------------------------------------------
const char*	ParticleControlWidget::GetPolyFileName()
{
	int		result;
	static	QCString	remDir(".");
	static	QCString	filename;
	
	QFileDialog	fileDialog(remDir, "*.rib", this, 0, true);
	fileDialog.resize(fileDialog.width(), (2*g_dskTopH)/3);
	
	if ( (result = fileDialog.exec()) == QDialog::Accepted)
	{
		filename = fileDialog.selectedFile();
		return (const char*)filename;
	}
	else
		return 0;
}	
