#include "particleControl.h"
#include "extern.h"
#include "particleControlWidget.h"
#include "pPolygonObject.h"
#include "pPolyAction.h"
#include "pGeometry.h"
#include "camera.h"

/**
 * Save particle system to file.
 * @param outfile open file handle to write to.
 * @param tabs number of tabs to indent.
 */
//----------------------------------------------------------
bool	ParticleControl::SaveParticleSystem(FILE* outfile, int tabs)
{
	int		i;
	FlowParticleAction	*curAction;
	char	buffer[128];
	char	tabBuf[32] = { 0 };
	
	char	*walker = 0;
	walker = tabBuf;
	for (i = 0; i < tabs; i++)
	{	*walker = '\t';	*(walker+1) = '\0';	walker++;	}
	walker = buffer;
	for (i = 0; i < tabs; i++)
	{	*walker = '\t';	*(walker+1) = '\0';	walker++;	}
	
	fprintf(outfile, "#----------------------------------------------------------------------\n");
	fprintf(outfile, "%sParticleSystem %s\n", tabBuf, (const char*)name );
	fprintf(outfile, "%s{\n", tabBuf);
	
	//--Write out max particles
	fprintf(outfile, "%s\tMaxParticles\n", tabBuf );
	fprintf(outfile, "%s\t\t%d\n", tabBuf, maxParticles);
	
	fprintf(outfile, "%s\tDrawAs\n", tabBuf );
	fprintf(outfile, "%s\t\t%d\n", tabBuf, (int)primitiveType);
	
	fprintf(outfile, "%s\tEnabled\n", tabBuf );
	fprintf(outfile, "%s\t\t%d\n", tabBuf, (int)enabled);
	
	fprintf(outfile, "%s\tLighting\n", tabBuf );
	fprintf(outfile, "%s\t\t%d\n", tabBuf, (int)hasLighting);
	
	if (rmSurface)
	{
		rmSurface->SaveToFile(outfile, tabs+1);
	}
	
	for (i = 0; i < (int)actionList.count(); i++)
	{
		curAction = actionList.at(i);
		curAction->SaveToFile(outfile, tabs);
	}

	sprintf(walker, "}\n");
	fputs(buffer, outfile);
	
//	WriteCode();
		
	return true;
}

/**
 * Load particle system from file.
 * @param infile open file handle
 * @param name of system
 */
//--------------------------------------------------------------------
bool	ParticleControl::LoadParticleSystem(FILE* infile, const char* inName)
{
	static	int	genericName = 0;
	char	buffer[128];
	QCString	qBuffer;
	FlowParticleAction	*newAction;
	
	g_frame = 0;
	
	if (inName)
		name = inName;
	else
		name.sprintf("generic particle %d", genericName++);
		
	
//	FlowParticleAction		*action;
	//Delete current system...
//	int i;

	actionList.setAutoDelete(true);
	actionList.clear();
	actionList.setAutoDelete(false);
	
//	for ( i = (int)actionList.count()-1; i >= 0; i--)
//	{
//		action = actionList.at(i);
//		actionList.remove(i);
////		delete action;
//	}

	fprintf(stderr, "---- ");
	while ( fgets(buffer, 128, infile) )
	{
		fprintf(stderr, "-");
		newAction = 0;
		qBuffer = buffer;
		if(buffer[0] == '#')
			continue;
		qBuffer = qBuffer.simplifyWhiteSpace();
		if ( qBuffer.length() < 1 )
			continue;
				
		if (qBuffer == "MaxParticles")
		{
			fgets(buffer, 128, infile);
			maxParticles =  atoi(buffer);
			SetMaxParticles(maxParticles);
		}
		else if (qBuffer == "DrawAs")
		{
			fgets(buffer, 128, infile);
			primitiveType =  (PPrimitiveType)atoi(buffer);
		}
		else if (qBuffer == "Enabled")
		{
			fgets(buffer, 128, infile);
			enabled =  (bool)atoi(buffer);
		}
		else if (qBuffer == "Lighting")
		{
			fgets(buffer, 128, infile);
			hasLighting =  (bool)atoi(buffer);
		}
		else if (qBuffer == "RM_SURFACE")
		{
			if (rmSurface)
				delete rmSurface;
			rmSurface = new MBA_RMShader(infile);
		}
		if (qBuffer == "}")
		{
			break;
		}
		else if ( qBuffer.find("PState") == 0)
		{
			qBuffer.remove(0, 7);
			newAction = ReadActionGeneric((const char*)qBuffer, infile);
		}
		else if ( qBuffer.find("PAction") == 0)
		{
			qBuffer.remove(0, 8);
			newAction = ReadActionGeneric((const char*)qBuffer, infile);
		}
		else if ( qBuffer.find("PGeometry") == 0)
		{
			qBuffer.remove(0, 10);
			newAction = ReadActionGeneric((const char*)qBuffer, infile);
		}
		if (newAction)
			actionList.append(newAction );
	}
	fprintf(stderr, "\n");
//	for (i = 0; i < (int)actionList.count(); i++)
//	{
//		warning("Action at %d  = %s", i, (const char*)actionList.at(i)->Name());
//	}
	return true;	
}

/**
 * Read action generically, then cast to specific type.
 */
//--------------------------------------------------------------------
FlowParticleAction *ParticleControl::ReadActionGeneric (const char* inType, FILE *infile)
{
	// read in generic action, then create specific action object
	FlowParticleAction	action;
	char	buffer[128];
	int		i;
	int		numFloats_v		= 0;
	int		numDomains_v	= 0;
	int		numBools_v		= 0;
	QCString	tmp;
	QCString	tmpName;
	QCString	type;
	PPolygonObject	*pPolygonObj = 0;

	//-- Get type from passed buffer 'inType' & get rid of 'PAction' prefix
	type = inType;
	//type = type.simplifyWhiteSpace();
//	warning ("ReadActionGeneric type = \'%s\'", (const char*)type);
	
	while ( fgets(buffer, 128, infile) > 0 )
	{
		tmp = KillNewline(buffer);
		//warning ("%s", (const char*)tmp);
		tmp = tmp.simplifyWhiteSpace();
		if (tmp.find('{') >= 0)			//ignore open brace
			continue;
		if (tmp.find('}') >= 0)			//get out
			break;

		tmpName = (const char*)tmp;		
		i = tmp.find(' ');
		//tmp.truncate((uint)i);
		tmpName.remove(0, (uint)i+1);
		
//		warning ("%s", (const char*) tmp);
		
		if (tmp.find("Enabled") >= 0)
			action.setEnabled(tmpName.toInt());
		else if (tmp.find("NAME") >= 0)
			action.SetName((const char*)tmpName);
		else if (tmp.find("FLOAT") >= 0)
		{
			action.SetNumFloats(numFloats_v+1);
			action.SetFloatName(numFloats_v, (const char*)tmpName, false);
			if (!ReadFloat(infile, *(action.Float(numFloats_v)) ))
			{
				warning ("Error reading Float%02d(%s) in %s", numFloats_v, (const char*)tmpName, (const char*)tmp);	
				return 0;
			}
			numFloats_v++;
		}	
		else if (tmp.find("DOMAIN") >= 0)
		{
			action.SetNumDomains(numDomains_v+1);
			action.SetDomainName(numDomains_v, (const char*)tmpName, false);
			if (!ReadDomain(infile, *(action.Domain(numDomains_v)) ))
			{
				warning ("Error reading Domain%02d(%s) in %s", numDomains_v, (const char*)tmpName, (const char*)tmp);
				return 0;
			}
			numDomains_v++;
		}
		else if (tmp.find("BOOL") >= 0)
		{
			action.SetNumBools(numBools_v+1);
			action.SetBoolName(numBools_v, (const char*)tmpName, false);
			if (!ReadBool(infile, *(action.Bool(numBools_v)) ))
			{
				warning ("Error reading Bool%02d(%s) in %s", numBools_v, (const char*)tmpName, (const char*)tmp);
				return 0;
			}
			numBools_v++;
		}
		else if (tmp.find("GL_SURFACE") >= 0)
		{
			if (action.GLSurface())
				delete action.GLSurface();
			MBA_GLSurface* tmpGLSurf = new MBA_GLSurface();
			tmpGLSurf->LoadFromFile(infile);
			action.SetGLSurface(tmpGLSurf);
		}
		else if (tmp.find("RM_SURFACE") >= 0)
		{
			if (action.RMSurface())
				delete action.RMSurface();
			MBA_RMShader* tmpRMSurf = new MBA_RMShader(infile);
			action.SetRMSurface(tmpRMSurf);
		}
		
		else if (tmp.find("RM_DISPLACEMENT") >= 0)
		{
			if (action.RMDisplace())
				delete action.RMDisplace();
			MBA_RMShader* tmpRMDisp = new MBA_RMShader(infile);
			action.SetRMDisplacement(tmpRMDisp);
		}
		else if (tmp.find("POLYGONS") >= 0)
		{
			pPolygonObj = new PPolygonObject(infile);
		}
		
	}		
	
	action.SetActionType((const char*)type);
	
//	if (type == "Bounce")
//	{
////		if (action.RMSurface())
////			delete action.RMSurface();
////		MBA_RMShader* tmpRMSurf = new MBA_RMShader();
////		action.SetRMSurface(tmpRMSurf);
////		warning ("Added default shader (%s) to Bounce action", tmpRMSurf->Name());
//
//		if (!action.RMSurface())
//		{
//		  if (action.RMSurface())
//			  delete action.RMSurface();
//		  MBA_RMShader* tmpRMSurf = new MBA_RMShader();
//		  action.SetRMSurface(tmpRMSurf);
//		  warning ("Added default shader (%s) to Bounce action", tmpRMSurf->Name());
//		}
//	}
	
//	warning("read in \"%s\"", (const char*)type);
	FlowParticleAction	*newAction = 0;
	if (type == "Color")
		newAction = new FPSColor(action);
	else if (type == "ColorDomain" )
		newAction = new FPSColorDomain (action);
	else if (type == "Size" )
		newAction = new FPSSize (action);
	else if (type == "SizeDomain" )
		newAction = new FPSSizeDomain (action);
	else if (type == "StartingAge" )
		newAction = new FPSStartingAge (action);
	else if (type == "TimeStep" )
		newAction = new FPSTimeStep (action);
	else if (type == "Velocity" )
		newAction = new FPSVelocity(action);
	else if (type == "VelocityDomain" )
		newAction = new FPSVelocityDomain(action);
	//-------------------------
	// ACTION
	else if (type == "Bounce" )
		newAction = new FPABounce(action);
	else if (type == "Damping" )
		newAction = new FPADamping(action);
	else if (type == "Explosion" )
		newAction = new FPAExplosion (action);
	else if (type == "Follow" )
		newAction = new FPAFollow (action);
	else if (type == "Gravitate" )
		newAction = new FPAGravitate (action);
	else if (type == "Gravity" )
		newAction = new FPAGravity(action);
	else if (type == "Jet" )
		newAction = new FPAJet(action);
	else if (type == "KillOld" )
		newAction = new FPAKillOld(action);
//	else if (type == "KillSlow" )
//		newAction = new FPAKillSlow(action);
	else if (type == "OrbitLine" )
		newAction = new FPAOrbitLine (action);
	else if (type == "OrbitPoint" )
		newAction = new FPAOrbitPoint (action);
	else if (type == "RandomAccel" )
		newAction = new FPARandomAccel (action);
	else if (type == "RandomDisplace" )
		newAction = new FPARandomDisplace (action);
	else if (type == "RandomVelocity" )
		newAction = new FPARandomVelocity (action);
	else if (type == "Restore" )
		newAction = new FPARestore (action);
	else if (type == "Sink" )
		newAction = new FPASink (action);
	else if (type == "SinkVelocity" )
		newAction = new FPASinkVelocity (action);
	else if (type == "Source" )
	{
		newAction = new FPASource(action);
//		action.Print();
//		newAction->Print();
	}
	else if (type == "TargetColor" )
		newAction = new FPATargetColor (action);
	else if (type == "TargetSize" )
		newAction = new FPATargetSize (action);
	else if (type == "Vertex" )
		newAction = new FPAVertex (action);
	else if (type == "Vortex" )
		newAction = new FPAVortex (action);
	else if (type == "Turbulence" )
		newAction = new FPATurbulence (action);
	//-------------------------
	// META-ACTION
	else if (type == "PolygonMetaBounce" )
		if (pPolygonObj)
			newAction = new PPAPolygonBounce(action, *pPolygonObj);
		else 
			warning("ERROR READING PolygonMetaBounce...");
	else if (type == "PolygonMetaSource")
		if (pPolygonObj)
			newAction = new PPAPolygonSource(action, *pPolygonObj);
		else 
			warning("ERROR READING PolygonMetaSource...");
	else if (type == "PolygonMetaNoOp")
		if (pPolygonObj)
			newAction = new PPAPolygonNoOp(action, *pPolygonObj);
		else 
			warning("ERROR READING PolygonMetaNoOp...");
	else if (type == "PolygonMetaEmitter")
		if (pPolygonObj)
			newAction = new PPAPolygonEmitter(action, *pPolygonObj);
		else 
			warning("ERROR READING PolygonMetaEmitter...");
	//-------------------------
	// GEOMETRY
	else if (type == "Sphere" )
		newAction = new PGSphere(action);
	else if (type == "Cone" )
		newAction = new PGCone(action);
	else if (type == "Disk" )
		newAction = new PGDisk(action);
	else if (type == "Cylinder" )
		newAction = new PGCylinder(action);
	//-------------------------
	// HUH?
	else
		warning("!! -- %s not recognized.", (const char*)type);
	
	delete pPolygonObj;
	return newAction;
}

/**
 * read single polygon and create triangular domain action, if necessary
 */
//--------------------------------------------------------------------
FlowParticleAction *ParticleControl::ReadPolygon (const char* inType, const char* buff)
{
	FlowParticleAction	action;
	char	buffer[256];
	QCString	type;
	float	a0;
	float	a1;
	float	a2;
	float	b0;
	float	b1;
	float	b2;
	float	c0;
	float	c1;
	float	c2;

	type = inType;
	action.SetActionType(inType);
	
	strcpy(buffer, buff);
	sscanf(buffer, "Polygon \"P\" [%f %f %f %f %f %f %f %f %f ]", &a0, &a1, &a2, &b0, &b1, &b2, &c0, &c1, &c2 );
	action.SetName("polygon");
	action.Domain(0)->set(PDTriangle, a0, a1, a2, b0, b1, b2, c0, c1, c2);
	
	FlowParticleAction	*newAction = 0;
	if (type == "Bounce" )
	{
		action.Float(0)->SetVal(0.0);
		action.Float(1)->SetVal(0.0);
		action.Float(2)->SetVal(0.0);
		action.Float(3)->SetVal(1.0);
		action.Float(4)->SetVal(0.0);
		action.Bool(0)->SetVal(true);
		newAction = new FPABounce(action);
	}
	else if (type == "Source" )
	{
		action.Float(0)->SetVal(0.005);
		newAction = new FPASource(action);
	}
	
	
	return newAction;
}

/**
 * read float from file
 */
//--------------------------------------------------------------------
bool	ParticleControl::ReadFloat(FILE* infile, PFloat& pFloat)
{
	char	buffer[128];
//	bool	ok;
	QCString	qBuffer;
	
	fgets(buffer, 128, infile);
	qBuffer = KillNewline(buffer);
	qBuffer = qBuffer.simplifyWhiteSpace();
	float	val;
	int		anim;
	sscanf((const char*)qBuffer, "%f %d", &val, &anim);
	pFloat.SetVal(val);
	if (anim < 0)
		pFloat.SetAnimation(0);
	else
		pFloat.SetAnimation(anim);
		
	return true;
}

/**
 * read bool from file
 */
//----------------------------------------------------------------------------------------
bool	ParticleControl::ReadBool(FILE* infile, PBool& pBool)
{
	char	buffer[128];
	bool	ok;
	QCString	qBuffer;
	
	fgets(buffer, 128, infile);
	qBuffer = KillNewline(buffer);
	pBool.SetVal((bool)qBuffer.toInt(&ok));
	if (!ok)
	{
		warning ("error with ReadBool");
		return false;
	}
//	warning ("Bool read with value ( %d )", (int)((bool)pBool) );
	return true;
}

/**
 * read domain from file
 */
//----------------------------------------------------------------------------------------
bool	ParticleControl::ReadDomain(FILE* infile, PDomain& domain)
{
	char	buffer[128];
	bool	ok;
	QCString	qBuffer;
	PFloat	tmpPFloat;

	// Read type
	fgets(buffer, 128, infile);
	qBuffer = KillNewline(buffer);
	domain.SetType((PDomainEnum)qBuffer.toInt(&ok));
//	warning("----------\ndomain type %d", (PDomainEnum)qBuffer.toInt(&ok));
	if (!ok)
		return false;
	ReadFloat(infile, tmpPFloat);		domain.SetA0(tmpPFloat);
	ReadFloat(infile, tmpPFloat);		domain.SetA1(tmpPFloat);
	ReadFloat(infile, tmpPFloat);		domain.SetA2(tmpPFloat);
	ReadFloat(infile, tmpPFloat);		domain.SetA3(tmpPFloat);
	ReadFloat(infile, tmpPFloat);		domain.SetA4(tmpPFloat);
	ReadFloat(infile, tmpPFloat);		domain.SetA5(tmpPFloat);
	ReadFloat(infile, tmpPFloat);		domain.SetA6(tmpPFloat);
	ReadFloat(infile, tmpPFloat);		domain.SetA7(tmpPFloat);
	ReadFloat(infile, tmpPFloat);		domain.SetA8(tmpPFloat);
//	warning("----------domain END");
		
	return true;
}

/**
 * KillNewline
 */
//--------------------------------------------------------------------
char*	ParticleControl::KillNewline (char *string)
{
	char	*whiteSpace;
		
	char	*newline;
	newline = strrchr(string, '\n');
	if (newline)
		*newline = '\0';
	whiteSpace = string;
	
	while ( (*whiteSpace == ' ' || *whiteSpace == '\t') && *whiteSpace != '\0' )
		whiteSpace++;
		
	return whiteSpace;
}

/**
 * Write out GLUT sourcecode for the system. This has not been maintained,
 * and will have lots of holes. 
 */
//--------------------------------------------------------------------
void	ParticleControl::WriteCode(FILE *outfile)
{
	int		i;
	FlowParticleAction	*curAction;
	
	CameraVals	curVals (g_globalCam->GetVals());
	
	if(outfile)
	{
		for (i = 0; i < (int)actionList.count(); i++)
		{
			curAction = actionList.at(i);
			curAction->WriteCode(outfile);
		}
	}
}

