#include "particleControlWidget.h"

#include "extern.h"
#include "define.h"
#include "particleControl.h"
#include "mbaGLSurfaceWidget.h"
#include "pPolyAction.h"
#include "pPolygonTweak.h"
#include "glPortCam.h"
#include "mbaSurfaceTabDialog.h"
#include "mbaScene.h"

#include <qtooltip.h>
#include <qkeycode.h>
#include <qiconset.h>
#include <qtimer.h>

#define PARTICLE_CONTROL_WIDTH 225

//==========================================================
ParticleControlWidget::ParticleControlWidget(ParticleControlFrame *parent, const char* name, QToolBar *toolbar)
	:	QWidget (parent, name)
{
	pcFrame = parent;
	deferedActiveAction = 0;
	
// 	setFont( *g_font_MB );
	
	currentActionIndex = -1;
	editPanel = new QWidget(toolbar);
	toolbar->setStretchableWidget(editPanel);

	partCtrl = 0;
	
	//---------------
	addParticleAction = new AddParticleActionWidget(this, "addParticleAction");
	addParticleAction->hide();
	
	//-- Widgets ---------------------------	
	actionListBox = new QListBox(this);
	actionListBox->setMinimumSize(80, 80);
 	actionListBox->setFont( *g_font_M );
	actionListBox->setMultiSelection( false );
	
	glLightingCheck = new QCheckBox(this);
	glLightingCheck->setText("particle lighting");
	glLightingCheck->setFont(*g_font_S);
	
	partCtrlSelCombo = new QComboBox(true, this, "partCtrlSelCombo");
	partCtrlSelCombo->setFont( *g_font_LB );
	partCtrlSelCombo->insertItem("NULL SYSTEM");
	partCtrlSelCombo->setInsertionPolicy(QComboBox::NoInsertion);
	int	sz = partCtrlSelCombo->sizeHint().height();
	partCtrlSelCombo->setMinimumWidth(sz*2);
	
	pShaderBut = new MToolButton(this, "pShaderBut");
	pShaderBut->setFixedSize(sz, sz);
	pShaderBut->setUsesBigPixmap(true);
	pShaderBut->setIconSet(*icoSet_shade_s);
	pShaderBut->setAutoRaise(true);
	
	pControlEnable = new MToolButton(this, "pControlEnable");
	pControlEnable->setFixedSize(sz, sz);
	pControlEnable->setFont(*g_font_LB);
	pControlEnable->setToggleButton(true);
	pControlEnable->setUsesBigPixmap(true);
	pControlEnable->setIconSet(*icoSet_dis_s);
	pControlEnable->setAutoRaise(true);
	//pControlEnable->setPixmap(QPixmap((const char**)but_lt_disable_xpm ));
	
	maxParticlesInput = new NumInput(NumInput::Int, this, "maxParticlesInput");
	maxParticlesInput->SetLabel("# particles");
	maxParticlesInput->SetLabelWidth(g_numLabelWidth);
	SetMaxParticles(0);
	
	primitiveTypeCombo = new QComboBox(this, "primitiveTypeCombo");
	primitiveTypeCombo->setFixedHeight(sz);
	primitiveTypeCombo->setMinimumWidth(sz*2);
	primitiveTypeCombo->insertItem(" spheres  ");
	primitiveTypeCombo->insertItem(" lines  ");
	primitiveTypeCombo->insertItem(" disks  ");
//	primitiveTypeCombo->insertItem(" polygons  ");
//	primitiveTypeCombo->insertItem(" connect  ");

	primitiveSizeCombo = new QComboBox(this, "primitiveSizeCombo");
	primitiveSizeCombo->setFixedHeight(sz);
	primitiveSizeCombo->setMinimumWidth(sz);
	primitiveSizeCombo->setMaximumWidth(sz*2);
	primitiveSizeCombo->insertItem(" 1 ");
	primitiveSizeCombo->insertItem(" 2 ");
	primitiveSizeCombo->insertItem(" 4 ");
	primitiveSizeCombo->insertItem(" 8 ");
	primitiveSizeCombo->insertItem(" 16 ");
	primitiveSizeCombo->insertItem(" 32 ");
	primitiveSizeCombo->insertItem(" 64 ");
	
	//---------------
	actionParamListFloat	= new QListBox(editPanel, "actionParamListFloat");
	actionParamListDomain	= new QListBox(editPanel, "actionParamListDomain");
	actionParamListBool		= new QListBox(editPanel, "actionParamListBool");
	actionParamListFloat->setFont ( *g_font_M );
	actionParamListDomain->setFont( *g_font_M );
	actionParamListBool->setFont  ( *g_font_M );
	actionParamListFloat->setMinimumSize (90, 20);
	actionParamListDomain->setMinimumSize(90, 20);
	actionParamListBool->setMinimumSize  (90, 20);
	actionParamListFloat->setEnabled (false);
	actionParamListDomain->setEnabled(false);
	actionParamListBool->setEnabled  (false);
	
	//---------------
	actionNameEdit  = new QLineEdit(editPanel, "actionNameEdit");
	pFloatWidget	= new PFloatWidget	(0, 0, editPanel, 0, false);
	pDomainWidget	= new PDomainWidget	(0, 0, editPanel);
	pBoolWidget		= new PBoolWidget	(0, 0, editPanel);
	actionNameEdit->setEnabled(false);
	pDomainWidget->setEnabled(false);
	pFloatWidget->setEnabled(false);
	pBoolWidget->setEnabled(false);
	
	//-- Connect -----------------------
	QObject::connect(maxParticlesInput, SIGNAL(ValueChanged(int)), SLOT(SetMaxParticles(int)) );
	QObject::connect(primitiveTypeCombo, SIGNAL(activated(int)), SLOT(SetPrimitiveType(int)) );
	QObject::connect(primitiveSizeCombo, SIGNAL(activated(int)), SLOT(SetPrimitiveSize(int)) );

	QObject::connect(actionListBox, SIGNAL(highlighted(int)), SLOT(SwitchAction(int)) );
	QObject::connect(actionListBox, SIGNAL(selected(int)),    SLOT(SetActionFocus(int)) );
	QObject::connect(actionParamListFloat, SIGNAL(highlighted(int)), SLOT(SwitchFloat(int)) );
	QObject::connect(actionParamListFloat, SIGNAL(selected(int)),    SLOT(SetFloatFocus(int)) );
	QObject::connect(actionParamListDomain, SIGNAL(highlighted(int)), SLOT(SwitchDomain(int)) );
	QObject::connect(actionParamListDomain, SIGNAL(selected(int)),    SLOT(SetDomainFocus(int)) );
	QObject::connect(actionParamListBool, SIGNAL(highlighted(int)), SLOT(SwitchBool(int)) );
	QObject::connect(actionParamListBool, SIGNAL(selected(int)),    SLOT(SetBoolFocus(int)) );
	QObject::connect(actionNameEdit, SIGNAL(returnPressed()), SLOT(SetActionName()) );
	
	QObject::connect(pShaderBut, SIGNAL(clicked()), SLOT(ShowShaderDialog()) );
	QObject::connect(partCtrlSelCombo, SIGNAL(activated(int)), SLOT(selectNewSystem(int)) );
	QObject::connect(partCtrlSelCombo, SIGNAL(activated(const QString&)), SLOT(changeSystemName(const QString&)) );
	QObject::connect(pControlEnable, SIGNAL(toggled(bool)), SLOT(DisableSystem(bool)) );
	
	QObject::connect(glLightingCheck, SIGNAL(toggled(bool)), SLOT(setLighting(bool)) );
	
	if(g_camGLPort)
	{
		QObject::connect(pDomainWidget, SIGNAL(Updated()), SIGNAL(Updated()) );
		QObject::connect(pFloatWidget, SIGNAL(Updated()), SIGNAL(Updated()) );
		QObject::connect(pBoolWidget, SIGNAL(Updated()), SIGNAL(Updated()) );
	}
	
	//-- Edit Panel Layout ------------------------
	topEditLayout = new QVBoxLayout(editPanel, 5, 5, "topEditLayout");
	numInputLayout		= new QVBoxLayout(0, "numInputLayout");
	actionHLayout		= new QHBoxLayout(5, "actionHLayout");
	paramHLayout		= new QHBoxLayout(5, "paramHLayout");
	maxNflowLayout		= new QVBoxLayout(5, "maxNflowLayout");
	actionParamListLayout= new QVBoxLayout(5, "actionParamListLayout");
	
	
	topEditLayout->addWidget(actionNameEdit);
	topEditLayout->addLayout(paramHLayout, 0);
		paramHLayout->addLayout(actionParamListLayout, 1);
			actionParamListLayout->addWidget(actionParamListFloat, 8);
			actionParamListLayout->addWidget(actionParamListDomain, 1);
			actionParamListLayout->addWidget(actionParamListBool, 3);
		paramHLayout->addLayout(numInputLayout, 0);
			numInputLayout->addWidget(pFloatWidget);
			numInputLayout->addStretch(10);
			numInputLayout->addWidget(pDomainWidget);
			numInputLayout->addStretch(3);
			numInputLayout->addWidget(pBoolWidget);
			numInputLayout->addStretch(3);

	topEditLayout->activate();
	
	//-- Layout ------------------------
	topLayout			= new QVBoxLayout(this, 5, 0, "topLayout");
	listHLayout 		= new QHBoxLayout(5, "listHLayout");
	listVLayout 		= new QVBoxLayout(2, "listVLayout");
	drawLayout			= new QHBoxLayout(5, "listButtonLayout");
	pOptionsLayout		= new QHBoxLayout(5, "pOptionsLayout");
	partCtrlSelComboLayout= new QHBoxLayout(5, "pControlSelectLayout");
	
	topLayout->addLayout(actionHLayout, 10);
		actionHLayout->addLayout(listHLayout, 1);
			listHLayout->addLayout(listVLayout, 1);
				listVLayout->addLayout(partCtrlSelComboLayout, 0);
					partCtrlSelComboLayout->addWidget(partCtrlSelCombo, 1);
					partCtrlSelComboLayout->addWidget(pControlEnable, 0);
				listVLayout->addLayout(pOptionsLayout, 0);
					pOptionsLayout->addWidget(primitiveTypeCombo, 1);
					pOptionsLayout->addWidget(primitiveSizeCombo, 1);
					pOptionsLayout->addWidget(pShaderBut, 0);
				listVLayout->addWidget(maxParticlesInput);
				listVLayout->addWidget(actionListBox, 2);
				listVLayout->addWidget(glLightingCheck, 0);

	topLayout->activate();
	
	if(g_toolTipsEnabled)
	{
		QToolTip::add(partCtrlSelCombo,	"select particle system to edit...");
		QToolTip::add(pShaderBut, 		"particle surface shader.");
		QToolTip::add(pControlEnable, 	"enable/disable this particle system.");
		QToolTip::add(primitiveTypeCombo,	"render particles as...");
		QToolTip::add(primitiveSizeCombo,	"particle point size in preview window.");
//		QToolTip::add(pFloatWidget,		"float edit widget");
//		QToolTip::add(pDomainWidget,	"domain edit widget");
//		QToolTip::add(pBoolWidget,		"bool edit widget");
	}
	
//	//-- TEMPORARY --------------------
	SetParticleControl((ParticleControl*)0);
}

ParticleControlWidget::~ParticleControlWidget()
{
}
	
//----------------------------------------------------------
void ParticleControlWidget::setLighting(bool status)
{
	if (partCtrl)
	{
		partCtrl->setLighting(status);
	}
}

//----------------------------------------------------------
void ParticleControlWidget::SceneChange()
{
	int	i;
	partCtrlSelCombo->clear();
	nameIndex = -1;
	partCtrlSelCombo->blockSignals(true);
	for (i = 0; i < g_scene->numParticleSystems(); i++ )
	{
		partCtrlSelCombo->insertItem(g_scene->ParticleSystemAt(i)->Name());
	}
	partCtrlSelCombo->blockSignals(false);
	selectNewSystem(0);
}

//----------------------------------------------------------
void	ParticleControlWidget::deferActiveAction(FlowParticleAction *cur)
{
	deferedActiveAction = cur;
//	QTimer::singleShot(0, this, SLOT(activateDeferredAction()));
}

//----------------------------------------------------------
void	ParticleControlWidget::activateDeferredAction()
{
	if(deferedActiveAction)
		setActiveAction(deferedActiveAction);
	deferedActiveAction = 0;
}

//----------------------------------------------------------
void	ParticleControlWidget::setActiveAction(FlowParticleAction *cur)
{
	int tmp = partCtrl->findAction(cur);
	if(tmp >= 0)
	{
		actionListBox->setCurrentItem(tmp);
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::selectNewSystem(int index)
{
//	warning("ParticleControlWidget::selectNewSystem(%d)", index);
	if(index != nameIndex) 
	{
//		warning("--SetParticleControl");
		SetParticleControl(g_scene->ParticleSystemAt(index));
		g_scene->setActiveParticleSystem(index);
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::changeSystemName(const QString& newText)
{
//	warning("ParticleControlWidget::changeSystemName(\"%s\")", (const char*)newText);
	int index = partCtrlSelCombo->currentItem();
	if ( nameIndex == index )
	{
//		warning("--Name changed");
		partCtrl->SetName(newText);
		partCtrlSelCombo->blockSignals(true);
		partCtrlSelCombo->changeItem(newText, index);
		partCtrlSelCombo->blockSignals(false);
	}
	nameIndex = index;
}

//----------------------------------------------------------
void	ParticleControlWidget::SetParticleControl(int index)
{
	partCtrlSelCombo->setCurrentItem(index);
	selectNewSystem(index);
}

//----------------------------------------------------------
void	ParticleControlWidget::SetParticleControl(ParticleControl *in)
{
	partCtrl = in;
	int	tmp;
	tmp = actionListBox->currentItem();
	actionListBox->clear();

	if (partCtrl)
	{
		pShaderBut->setEnabled(true);
		pControlEnable->setEnabled(true);
		if (partCtrl->isEnabled())
			pControlEnable->setOn(false);
		else
			pControlEnable->setOn(true);
		partCtrlSelCombo->setEnabled(true);
		
		glLightingCheck->setChecked(partCtrl->lighting());
		
		primitiveTypeCombo->setEnabled(true);
		primitiveTypeCombo->setCurrentItem( (int)partCtrl->PrimitiveType() );
		switch(partCtrl->PrimitiveType())
		{
			case ParticleControl::Point:
				primitiveSizeCombo->setCurrentItem((int)(partCtrl->PointSize()-1.0));
				break;
			case ParticleControl::Line:
				primitiveSizeCombo->setCurrentItem((int)(partCtrl->LineWidth()-1.0));
				break;
			case ParticleControl::Disk:
				primitiveSizeCombo->setCurrentItem((int)(partCtrl->PointSize()-1.0));
				break;
			case ParticleControl::Polygon:
				break;
			case ParticleControl::Connect:
				primitiveSizeCombo->setCurrentItem((int)(partCtrl->LineWidth()-1.0));
				break;
		}
		maxParticlesInput->setEnabled(true);
		SetMaxParticles( partCtrl->getMaxParticles() );
		
		int 	i;
		actionListBox->setEnabled(true);
		for (i = 0; i < (int)partCtrl->actionList.count(); i++)
		{
			if((partCtrl->actionList.at(i))->Enabled())
				actionListBox->insertItem((const char*)(partCtrl->actionList.at(i))->Name());
			else
				actionListBox->insertItem((const char*)(partCtrl->actionList.at(i))->DisabledName());
		}
		if (tmp > 0 && tmp < i )
		{
			currentActionIndex = -1;
			actionListBox->setCurrentItem(tmp);
		}
		else
		{
			currentActionIndex = -1;
			actionListBox->setCurrentItem(0);
		}
	}
	else
	{
		pShaderBut->setEnabled(false);
		pControlEnable->setEnabled(false);
		partCtrlSelCombo->setEnabled(false);
		
		pDomainWidget->SetDomainPointer(0);
		pFloatWidget->SetFloatPointer(0);
		pBoolWidget->SetBoolPointer(0);

		primitiveTypeCombo->setEnabled(false);
		maxParticlesInput->setEnabled(false);
		
		actionListBox->clear();
		actionListBox->setEnabled(false);
		
		actionNameEdit->setText(" ");
		actionNameEdit->setEnabled(false);

		actionParamListFloat->clear();
		actionParamListDomain->clear();
		actionParamListBool->clear();
		//shouldn't need this, but the empty list boxes aren't updating by themselves
		actionParamListFloat->update();
		actionParamListDomain->update();
		actionParamListBool->update();
		actionParamListFloat->setEnabled(false);
		actionParamListDomain->setEnabled(false);
		actionParamListBool->setEnabled(false);
		
		if (g_surfaceEditWidget)
		{
			g_surfaceEditWidget->SetGLSurfacePointer(0);
			g_surfaceEditWidget->SetRMSurfacePointerPointer(0);
		}
		
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SetActionFocus(int)
{
	warning("ParticleControlWidget::SetActionFocus( )" );
	if (partCtrl)
	{
		if ( partCtrl->actionList.at(currentActionIndex)->numFloats() > 0)
			pFloatWidget->SetFocus();
		else if ( partCtrl->actionList.at(currentActionIndex)->NumDomains() > 0)
			pDomainWidget->SetFocus();
		else if ( partCtrl->actionList.at(currentActionIndex)->NumBools() > 0)
			pBoolWidget->SetFocus();
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SetFloatFocus(int)
{
	pFloatWidget->SetFocus();
}

//----------------------------------------------------------
void	ParticleControlWidget::SetDomainFocus(int)
{
	//warning("ParticleControlWidget::SetDomainFocus(int)");
	pDomainWidget->SetFocus();
}
//----------------------------------------------------------
void	ParticleControlWidget::SetBoolFocus(int)
{
	pBoolWidget->SetFocus();
}

//----------------------------------------------------------
void	ParticleControlWidget::DisableSystem(bool status)
{
	if(partCtrl)
	{
		partCtrl->setEnabled(!status);
		emit Updated();
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::DisableAction(bool status)
{
	FlowParticleAction	*currentAction = 0;
	int		index;
	
	if (partCtrl)
	{
		index = actionListBox->currentItem();

		if ( index >= 0 )
		{
			currentAction = partCtrl->actionList.at(index);
			currentAction->setEnabled(!status);
			actionListBox->blockSignals(true);
			if ( status )
				actionListBox->changeItem((const char*)(currentAction->DisabledName()), index);
			else
				actionListBox->changeItem((const char*)(currentAction->Name()), index);
			actionListBox->blockSignals(false);
			
		}
	}
	emit Updated();
}
//----------------------------------------------------------
void	ParticleControlWidget::MoveActionUp()
{
	int	origIndex;
	FlowParticleAction	*curAction;
	QString	tmpString;
	
	if (partCtrl)
	{
		origIndex = actionListBox->currentItem();
		if ( origIndex > 0 )
		{
			curAction = partCtrl->actionList.at(origIndex);
			partCtrl->actionList.insert(origIndex-1, curAction);
			partCtrl->actionList.remove(origIndex+1);

			tmpString = actionListBox->text(origIndex-1);
			actionListBox->changeItem(actionListBox->text(origIndex), origIndex-1);
			actionListBox->changeItem(tmpString, origIndex);
			actionListBox->setCurrentItem(origIndex-1);

		}
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::MoveActionDown()
{
	int	origIndex;
	FlowParticleAction	*curAction;
	QString	tmpString;
	
	if (partCtrl)
	{
		origIndex = actionListBox->currentItem();
		if (origIndex < (int)partCtrl->actionList.count()-1)
		{
			curAction = partCtrl->actionList.at(origIndex);
			partCtrl->actionList.remove(origIndex);
			partCtrl->actionList.insert(origIndex+1, curAction);

			tmpString = actionListBox->text(origIndex+1);
			actionListBox->changeItem(actionListBox->text(origIndex), origIndex+1);
			actionListBox->changeItem(tmpString, origIndex);
			actionListBox->setCurrentItem(origIndex+1);
		}
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SetMaxParticles(int in)
{
	if (partCtrl)
	{
		if ( in != maxParticlesInput->GetIntValue() )
			maxParticlesInput->SetValueSolid(in);
		partCtrl->SetMaxParticles(in);
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SetPrimitiveType(int in)
{
	if (partCtrl)
	{
		partCtrl->SetPrimitiveType(in);
		switch((ParticleControl::PPrimitiveType)in)
		{
			case ParticleControl::Point:
				primitiveSizeCombo->setCurrentItem((int)(partCtrl->PointSize()-1.0));
				break;
			case ParticleControl::Line:
				primitiveSizeCombo->setCurrentItem((int)(partCtrl->LineWidth()-1.0));
				break;
			case ParticleControl::Disk:
				primitiveSizeCombo->setCurrentItem((int)(partCtrl->PointSize()-1.0));
				break;
			case ParticleControl::Polygon:
				break;
			case ParticleControl::Connect:
				primitiveSizeCombo->setCurrentItem((int)(partCtrl->LineWidth()-1.0));
				break;
		}
		emit Updated();
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SetPrimitiveSize(int in)
{
	in = (int)pow(2, in);
	if (partCtrl)
	{
		switch(partCtrl->PrimitiveType())
		{
			case ParticleControl::Point:
				partCtrl->SetPointSize((float)(in+1));
				break;
			case ParticleControl::Line:
				partCtrl->SetLineWidth((float)(in+1));
				break;
			case ParticleControl::Disk:
				partCtrl->SetPointSize((float)(in+1));
				break;
			case ParticleControl::Polygon:
				break;
			case ParticleControl::Connect:
				partCtrl->SetLineWidth((float)(in+1));
				break;
		}
		emit Updated();
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SetActionName()
{
	if (partCtrl)
	{
		QString	currentText = actionNameEdit->text();
		int i = actionListBox->currentItem();
		if(i >= 0)
		{
			actionListBox->changeItem(currentText, i);
			partCtrl->SetActionName((uint)i, currentText);
		}
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SetParticleSystemName()
{
	if (partCtrl)
	{
		partCtrl->SetName(partCtrlSelCombo->text(partCtrlSelCombo->currentItem()));
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SwitchFloat(int index)
{
	if (partCtrl)
	{
		pFloatWidget->SetFloatPointer((partCtrl->actionList.at(currentActionIndex))->Float(index), (partCtrl->actionList.at(currentActionIndex))->FloatName(index));
	//	warning("ParticleControlWidget::SwitchFloat - pFloatWidget->SetFloatPointer(%p)", (actionList.at(currentActionIndex))->Float(index) );
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SwitchBool(int index)
{
	if (partCtrl)
	{
		pBoolWidget->SetBoolPointer((partCtrl->actionList.at(currentActionIndex))->Bool(index), (partCtrl->actionList.at(currentActionIndex))->BoolName(index));
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SwitchDomain(int index)
{
	//warning("ParticleControlWidget::SwitchDomain(%d)", index);
	if (partCtrl)
	{
		FlowParticleAction	*currentAction;
		currentAction = partCtrl->actionList.at(currentActionIndex);
		
		if(currentAction->Type() == FlowParticleAction::PA_Bounce)
			pDomainWidget->SwitchMode(PDomainWidget::DomainBounce);
		else if (currentAction->BaseType() == FlowParticleAction::P_Geometry ||
				 currentAction->BaseType() == FlowParticleAction::P_PolyAction )
			pDomainWidget->SwitchMode(PDomainWidget::DomainTransform);
		else
			pDomainWidget->SwitchMode(PDomainWidget::DomainNormal);
			
		pDomainWidget->SetDomainPointer( currentAction->Domain(index), currentAction->DomainName(index));

	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SwitchAction(int index)
{
	uint	i;
	int		curFloat;
	int		curDomain;
	int		curBool;
	
//	warning("ParticleControlWidget::SwitchAction( %d )", index);
	if(partCtrl /*&& partCtrl->numActions() > 0*/)
	{
		if(currentActionIndex == index)	//XXX goddam Qt2 is emitting signals when it shouldn't...
			return;
			
		//FIXME -  Temporary hack for display list updation
		if (currentActionIndex >= 0)
			partCtrl->actionList.at(currentActionIndex)->attributesChanged = false;
		partCtrl->actionList.at(index)->attributesChanged = true;

		currentActionIndex = index;

		FlowParticleAction	*currentAction;
		currentAction = partCtrl->actionList.at(index);

		static	FlowParticleAction* lastAction = 0;
		lastAction = partCtrl->actionList.at(index);
		actionNameEdit->setEnabled(true);
		actionNameEdit->setText(lastAction->Name());

		curFloat	= actionParamListFloat->currentItem();
		curDomain	= actionParamListDomain->currentItem();
		curBool		= actionParamListBool->currentItem();

		actionParamListFloat->setUpdatesEnabled(false);
		actionParamListDomain->setUpdatesEnabled(false);
		actionParamListBool->setUpdatesEnabled(false);
		actionParamListFloat->clear();
		actionParamListBool->clear();
		actionParamListDomain->clear();
	

		for (i = 0; i < currentAction->numFloats(); i++)
			if ( ! currentAction->FloatName(i).isNull() )
				actionParamListFloat->insertItem((const char*)(currentAction->FloatName(i)));

		for (i = 0; i < currentAction->NumDomains(); i++)
			if ( ! currentAction->DomainName(i).isNull() )
				actionParamListDomain->insertItem( (const char*)(currentAction->DomainName(i)) );

		for (i = 0; i < currentAction->NumBools(); i++)
			if ( ! currentAction->BoolName(i).isNull() )
				actionParamListBool->insertItem( (const char*)(currentAction->BoolName(i)) );

		if ( currentAction->NumBools() == 0 )
		{
			actionParamListBool->setEnabled(false);
			pBoolWidget->SetBoolPointer(0, 0);
		}
		else
		{
			actionParamListBool->setEnabled(true);
			pBoolWidget->setEnabled(true);
			//pBoolWidget->SetBoolPointer(currentAction->Bool(0), currentAction->BoolName(0));
			if (curBool >= 0 && curBool < (int)actionParamListBool->count() )
				actionParamListBool->setCurrentItem(curBool);
			else
				actionParamListBool->setCurrentItem(0);
		}
		actionParamListBool->setUpdatesEnabled(true);
		actionParamListBool->repaint();

		if ( currentAction->NumDomains() == 0 )
		{
			actionParamListDomain->setEnabled(false);
			pDomainWidget->setEnabled(false);
			pDomainWidget->SetDomainPointer(0, 0);
		}
		else
		{
			actionParamListDomain->setEnabled(true);
			pDomainWidget->setEnabled(true);
			if (curDomain >= 0 && curDomain < (int)actionParamListDomain->count() )
				actionParamListDomain->setCurrentItem(curDomain);
			else
				actionParamListDomain->setCurrentItem(0);
		}
		actionParamListDomain->setUpdatesEnabled(true);
		actionParamListDomain->repaint();

		if ( currentAction->numFloats() == 0 )
		{
			actionParamListFloat->setEnabled(false);
			pFloatWidget->setEnabled(false);
			pFloatWidget->SetFloatPointer(0, 0);
		}
		else
		{
			actionParamListFloat->setEnabled(true);
			pFloatWidget->setEnabled(true);
			//pFloatWidget->SetFloatPointer(currentAction->Float(0), currentAction->FloatName(0));
			if (curFloat >= 0 && curFloat < (int)actionParamListFloat->count() )
				actionParamListFloat->setCurrentItem(curFloat);
			else
				actionParamListFloat->setCurrentItem(0);
		}
		actionParamListFloat->setUpdatesEnabled(true);
		actionParamListFloat->repaint();

		pcFrame->SetDisableOn( ! currentAction->Enabled() );

		if (	currentAction->Type() == FlowParticleAction::PA_Bounce ||
				currentAction->BaseType() == FlowParticleAction::P_PolyAction ||
				currentAction->BaseType() == FlowParticleAction::P_Geometry	)
		{
			g_surfaceEditWidget->SetGLSurfacePointer(currentAction->GLSurface());
			g_surfaceEditWidget->SetRMDisplacePointerPointer(currentAction->pointerToRMDisplace(), true);
			g_surfaceEditWidget->SetRMSurfacePointerPointer(currentAction->pointerToRMSurface(), true);
		}
		else
		{
			g_surfaceEditWidget->SetGLSurfacePointer(0);
			g_surfaceEditWidget->SetRMDisplacePointerPointer(0);
			g_surfaceEditWidget->SetRMSurfacePointerPointer(0);
		}
		
		if ( currentAction->BaseType() == FlowParticleAction::P_PolyAction	)
		{
			g_polyTweakWidget->SetPolyObject( ((PPAPolygonAction*)currentAction)->PolygonObject() );
		}
		else
		{
			g_polyTweakWidget->SetPolyObject( 0 );
		}
	}
	else
	{
		actionNameEdit->setEnabled(false);
		pFloatWidget->setEnabled(false);
		pDomainWidget->setEnabled(false);
		pBoolWidget->setEnabled(false);
		actionParamListFloat->repaint();
		actionParamListDomain->repaint();
		actionParamListBool->repaint();
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SetDomainDone()
{
	pDomainWidget->Update();
}

//----------------------------------------------------------
void	ParticleControlWidget::SetDomain123(float val00, float val01, float val02)
{
	FlowParticleAction	*currentAction;
	PDomain			*pDomain;
	
	if (partCtrl)
	{
		currentAction = partCtrl->actionList.at(currentActionIndex);
		if (currentAction)
		{
			if (currentAction->NumDomains())
			{
				pDomain = currentAction->Domain(0);
				pDomain->MoveXYZ(val00, val01, val02);
				//pDomainWidget->SetA0(pDomain->A0()+val00);
				//pDomainWidget->SetA1(pDomain->A1()+val01);
				//pDomainWidget->SetA2(pDomain->A2()+val02);
				//warning("Setit");
			}
//			else if ( currentAction->numFloats() > 2 )
//			{
//				currentAction->SetFloat(0, (float)*(currentAction->Float(0))+val00);
//				currentAction->SetFloat(1, (float)*(currentAction->Float(1))+val01);
//				currentAction->SetFloat(2, (float)*(currentAction->Float(2))+val02);
//			}
		}
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SetDomain456(float val03, float val04, float val05 )
{
	FlowParticleAction	*currentAction;
	PDomain			*pDomain;
	
	if (partCtrl)
	{
		currentAction = partCtrl->actionList.at(currentActionIndex);
		if (currentAction)
		{
			if (currentAction->NumDomains())
			{
				pDomain = currentAction->Domain(0);
				pDomain->RotateXYZ(val03, val04, val05);
//				pDomainWidget->SetA3(pDomain->A3()+val03);
//				pDomainWidget->SetA4(pDomain->A4()+val04);
//				pDomainWidget->SetA5(pDomain->A5()+val05);
				//warning("Setit");
			}
		}
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::SetDomain789(float val06, float val07, float val08)
{
	FlowParticleAction	*currentAction;
	PDomain			*pDomain;
	
	if (partCtrl)
	{
		currentAction = partCtrl->actionList.at(currentActionIndex);
		if (currentAction)
		{
			if (currentAction->NumDomains())
			{
				pDomain = currentAction->Domain(0);
				pDomainWidget->SetA6(pDomain->A6()+val06);
				pDomainWidget->SetA7(pDomain->A7()+val07);
				pDomainWidget->SetA8(pDomain->A8()+val08);
			}
		}
	}
}

//----------------------------------------------------------
void	ParticleControlWidget::ShowShaderDialog()
{
	if (partCtrl)
	{
		g_surfaceEditWidget->SetGLSurfacePointer( 0 );
		g_surfaceEditWidget->SetRMSurfacePointerPointer( partCtrl->pointerToRMSurface(), false );
		g_surfaceEditWidget->SetRMDisplacePointerPointer( partCtrl->pointerToRMDisplace(), false );
		g_surfaceEditWidget->show();
	}
}

//==================================================================================================
//==================================================================================================
ParticleControlFrame::ParticleControlFrame(QWidget *nullParent, const char* name, ToolBarDock buttonBar, ToolBarDock editBar)
	:	MbaMainWindow (nullParent, name)
{
	setCaption("flow::Particle Control");
	setIcon(QPixmap((const char**)flowIcon_xpm));
	
	manipToolBar = new QToolBar(0, this, buttonBar, true);
	dockList.append(new QMainWindow::ToolBarDock(buttonBar));
	
	paramEditToolBar = new QToolBar(0, this, editBar, true);
	dockList.append(new QMainWindow::ToolBarDock(editBar));
	
	if(g_winStyleToolBars)
	{
		manipToolBar->setStyle(winStyle);
		paramEditToolBar->setStyle(winStyle);
	}
	particleControlWidget = new ParticleControlWidget(this, 0, paramEditToolBar);
	setCentralWidget(particleControlWidget);
	setRightJustification(true);
	
	closeBut = new MToolButton(manipToolBar);
	closeBut->setAccel(ALT+'Q');
	closeBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	closeBut->setUsesBigPixmap(true);
	//closeBut->setPixmap(QPixmap((const char**)but_close_xpm ));
	closeBut->setIconSet(*icoSet_close);
	
	MLabel *stretch;
	
	stretch = new MLabel(manipToolBar);
	stretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	
	disableBut = new MToolButton(manipToolBar);
	disableBut->setAccel(ALT+'E');
	disableBut->setToggleButton(true);
	disableBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	disableBut->setUsesBigPixmap(true);
	//disableBut->setText("disable");
	//disableBut->setPixmap(QPixmap((const char**)but_disable_xpm ));
	disableBut->setIconSet(*icoSet_dis);
	disableBut->setIconSet(*icoSet_dis_on, true);
	
	stretch = new MLabel(manipToolBar);
	stretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	
	copyBut = new MToolButton(manipToolBar);
	copyBut->setAccel(CTRL+'C');
	copyBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	copyBut->setUsesBigPixmap(true);
	//copyBut->setPixmap(QPixmap((const char**)but_copy_xpm ));
	copyBut->setIconSet(*icoSet_clone);
	
	addBut = new MToolButton(manipToolBar);
	addBut->setAccel(ALT+'A');
	addBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	addBut->setUsesBigPixmap(true);
	//addBut->setPixmap(QPixmap((const char**)but_add_xpm ));
	addBut->setIconSet(*icoSet_add);
	
	delBut = new MToolButton(manipToolBar);
	delBut->setAccel(ALT+'D');
	delBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	delBut->setUsesBigPixmap(true);
	//delBut->setPixmap(QPixmap((const char**)but_del_xpm ));
	delBut->setIconSet(*icoSet_del);

	stretch = new MLabel(manipToolBar);
	stretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	
	upBut = new MToolButton(manipToolBar);
	upBut->setAccel(CTRL+Key_Up);
	upBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	upBut->setUsesBigPixmap(true);
	//upBut->setPixmap(QPixmap((const char**)but_up_xpm ));
	upBut->setIconSet(*icoSet_up);
	
	downBut = new MToolButton(manipToolBar);
	downBut->setAccel(CTRL+Key_Down);
	downBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	downBut->setUsesBigPixmap(true);
	//downBut->setPixmap(QPixmap((const char**)but_dwn_xpm ));
	downBut->setIconSet(*icoSet_dwn);
	
	manipToolBar->setStretchableWidget(stretch);

	if(g_toolTipsEnabled)
	{
		QToolTip::add(closeBut,		"close (alt-Q)");
		QToolTip::add(disableBut,	"enable/disable current action (alt-E)");
		QToolTip::add(copyBut,		"clone action (ctrl-C)");
		QToolTip::add(addBut,		"add action (alt-A)");
		QToolTip::add(delBut,		"delete action (alt-D)");
		QToolTip::add(upBut,		"move action up (ctrl-(up))");
		QToolTip::add(downBut, 		"move action down (ctrl-(down))");
	}
	
	//----------------------------------
	QObject::connect(closeBut,	SIGNAL(clicked()), SLOT(hide()) );
	QObject::connect(copyBut,	SIGNAL(clicked()), particleControlWidget, SLOT(CopyAction()) );
	QObject::connect(addBut,	SIGNAL(clicked()), particleControlWidget, SLOT(AddAction()) );
	QObject::connect(delBut,	SIGNAL(clicked()), particleControlWidget, SLOT(DeleteAction()) );
	QObject::connect(upBut,		SIGNAL(clicked()), particleControlWidget, SLOT(MoveActionUp()) );
	QObject::connect(downBut,	SIGNAL(clicked()), particleControlWidget, SLOT(MoveActionDown()) );
	QObject::connect(disableBut, SIGNAL(toggled(bool)), particleControlWidget, SLOT(DisableAction(bool)) );
	
	//----------------------------------
//	setDockEnabled( QMainWindow::TornOff, false);
	setDockEnabled( QMainWindow::Top	, true);
	setDockEnabled( QMainWindow::Bottom	, true); 
	setDockEnabled( QMainWindow::Right	, true);
	setDockEnabled( QMainWindow::Left	, true);
}

ParticleControlFrame::~ParticleControlFrame()
{
}


//----------------------------------------------------------
void ParticleControlFrame::show()
{
	//Work around bug in Qt...don't want the toolbar, 
	//but it shows up if you do a hide(), then a show()
	QMainWindow::show();
//	menuBar()->hide();
}

//--------------------------------------------------------------------
QToolBar*	ParticleControlFrame::GetToolBar(int toolBarNum)
{
	switch(toolBarNum)
	{
		case 0:
			return manipToolBar;
		case 1:
			return paramEditToolBar;
		default:
			return 0;
	}
}

//----------------------------------------------------------
bool ParticleControlFrame::setToolBar(int num, QMainWindow::ToolBarDock dock, bool newline)
{
	switch(num)
	{
		case 0:
			removeToolBar(manipToolBar);
			addToolBar(manipToolBar, 0, dock, newline);
			*dockList.at(0) = dock;
			return true;
		case 1:
			removeToolBar(paramEditToolBar);
			addToolBar(paramEditToolBar, 0, dock, newline);
			*dockList.at(1) = dock;
			return true;
		default:
			return false;
	}
}


