#ifndef particleControlWidget_h
#define particleControlWidget_h

#include <qtoolbar.h> 
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qlist.h> 

#include "compatibility.h"
#include <GL/gl.h>
#include <stdio.h>
#include <particle/papi.h>

#include "mwidgets.h"
#include "mbaMainWindow.h"
#include "particleControl.h"
#include "pDomainWidget.h"
#include "pFloatWidget.h"
#include "pBoolWidget.h"
#include "pAction.h"
#include "addParticleAction.h"
#include "renderPanelRIB.h"

#include "numInput.h"

class ParticleControlFrame;

/**
 *
 */
//===========================================================
class ParticleControlWidget : public QWidget
{
	Q_OBJECT
	
public:
	ParticleControlWidget(ParticleControlFrame *parent = 0, const char* name = 0, QToolBar *toolbar = 0);
	~ParticleControlWidget();

	void	SetParticleControl(ParticleControl *);
	void	SetParticleControl(int);
	
	
	inline bool SaveParticleSystem(FILE* outfile)	{	if(partCtrl) return partCtrl->SaveParticleSystem(outfile);	else return false;	}/** Pass along to ParticleControl */
	inline void	RenderParticles()	{	if(partCtrl) partCtrl->RenderParticles();	}	/** Pass along to ParticleControl */
	inline void	ResetSystem()		{	if(partCtrl) partCtrl->ResetSystem();	}		/** Pass along to ParticleControl */

	inline ParticleControl	*GetParticleControl()	{	return partCtrl;	} /** ugly */
	
	void	SceneChange();
	
	void	setActiveAction(FlowParticleAction *cur);
	void	deferActiveAction(FlowParticleAction *cur);

signals:
	void	Updated();
	
protected:
	const char *GetPolyFileName();
	
public slots:
	void	SetDomain123( float val00, float val01, float val02);
	void	SetDomain456( float val03, float val04, float val05);
	void	SetDomain789( float val06, float val07, float val08);
	void	SetDomainDone( );
	void	activateDeferredAction();

protected slots:
	void	selectNewSystem(int);
	void	changeSystemName(const QString&);
	void	SetPrimitiveType (int in);
	void	SetPrimitiveSize(int);
	void	SetMaxParticles (int in);
	void	SwitchFloat (int in);
	void	SwitchBool (int in);
	void	SwitchDomain (int in);
	void	SwitchAction (int in);
	
	void	SetActionFocus	(int in);
	void	SetFloatFocus	(int in);
	void	SetDomainFocus	(int in);
	void	SetBoolFocus	(int in);
	
	void	ShowShaderDialog();
	
	void	SetActionName ();
	void	SetParticleSystemName ();
	
	void	MoveActionUp();
	void	MoveActionDown();
	void	DeleteAction();
	void	AddAction();
	void	CopyAction();
	
	void	setLighting(bool);
	
	void	DisableAction(bool);
	void	DisableSystem(bool);
	
protected:
	ParticleControlFrame	*pcFrame;
	
	QWidget		*editPanel;
	
	QComboBox	*partCtrlSelCombo;
	int			nameIndex;
	MToolButton	*pControlEnable;
	MToolButton	*pShaderBut;
	QLineEdit	*actionNameEdit;
	
	QListBox	*actionListBox;
	QListBox	*actionParamListFloat;
	QListBox	*actionParamListDomain;
	QListBox	*actionParamListBool;
	
	QCheckBox	*glLightingCheck;
	
	PFloatWidget	*pFloatWidget;
	PDomainWidget	*pDomainWidget;
	PBoolWidget		*pBoolWidget;
	
	QComboBox	*primitiveTypeCombo;
	QPushButton	*drawFullBut;
	QComboBox	*primitiveSizeCombo;

	ParticleControl	*partCtrl;

	FlowParticleAction	*deferedActiveAction;	/**< storage used to allow deferred activation of an action */

	int			currentActionIndex;
	NumInput	*maxParticlesInput;
	
	AddParticleActionWidget	*addParticleAction;

private:
	QVBoxLayout	*topLayout;
	QVBoxLayout	*maxNflowLayout;
	QHBoxLayout	*actionHLayout;
	QHBoxLayout	*paramHLayout;
	QVBoxLayout	*numInputLayout;
	QHBoxLayout	*listHLayout;
	QVBoxLayout	*listVLayout;
	QVBoxLayout	*listButtonLayout;
	QVBoxLayout	*actionParamListLayout;
	QVBoxLayout	*topEditLayout;
	QHBoxLayout	*drawLayout;
	QHBoxLayout	*pOptionsLayout;
	QHBoxLayout	*partCtrlSelComboLayout;
};

/**
 * 
 */
//===========================================================
class ParticleControlFrame : public MbaMainWindow
{
	Q_OBJECT
	
public:
	ParticleControlFrame(QWidget *nullParent = 0, const char* name = 0, ToolBarDock buttonBar = Left, ToolBarDock editBar = Right);
	~ParticleControlFrame();
	
	inline ParticleControlWidget	*GetParticleControlWidget() {	return particleControlWidget;	}
	inline void	SetDisableOn(bool status)	{	disableBut->setOn(status);	}
	
	QToolBar	*GetToolBar(int toolBarNum);
	bool	setToolBar(int num, QMainWindow::ToolBarDock dock, bool newline = true);
	
public slots:
	void show();

protected:
	ParticleControlWidget	*particleControlWidget;
	QToolBar		*manipToolBar;
	QToolBar		*paramEditToolBar;
	
	MToolButton		*closeBut;
	MToolButton		*copyBut;
	MToolButton		*disableBut;
	MToolButton		*addBut;
	MToolButton		*delBut;
	MToolButton		*upBut;
	MToolButton		*downBut;
	
	
	QLabel	*stretchM2;
};


#endif
