#include "prefDialogCode.h"

#include <qtimer.h>
#include <qtooltip.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qtextstream.h>

#include "extern.h"
#include "mbaScene.h"

#include "define.h"
#include "extern.h"

//==============================================================================
PrefDialogCode::PrefDialogCode(QWidget *parent, const char *name)
	: QFrame(parent, name)
{
//	setFont(*g_font_MB);
		
//	configLabel = new QLabel(this);
//	configLabel->setText("configuration file :");
//	configLabel->setAlignment( AlignLeft | AlignBottom );
//	configLabel->setFixedHeight(configLabel->sizeHint().height()*2);
	
	configEdit = new QLineEdit(this);
	configEdit->setFont(*g_font_S);
	configEdit->setReadOnly(true);
	
	int	fileButSize = configEdit->sizeHint().height();
	configBrowseBut = new MToolButton(this);
	configBrowseBut->setFixedSize(fileButSize, fileButSize);
	configBrowseBut->setPixmap(*pix_file);
	
	configSaveBut = new MToolButton(this);
	configSaveBut->setFixedSize(fileButSize, fileButSize);
	configSaveBut->setPixmap(*pix_save);
	
//	fileLabel = new QLabel(this);
//	fileLabel->setText("files to scan & copy :");
//	fileLabel->setAlignment( AlignLeft | AlignBottom );
//	fileLabel->setFixedHeight(fileLabel->sizeHint().height()*2);
	
	addFileBut    = new	MToolButton(this);
	addFileBut->setText("&add file");
	addFileBut->setFont(*g_font_S);
	addFileBut->setAutoRaise(true);
	
	editFileBut    = new	MToolButton(this);
	editFileBut->setText("&edit file");
	editFileBut->setFont(*g_font_S);
	editFileBut->setAutoRaise(true);
	
	removeFileBut    = new MToolButton(this);
	removeFileBut->setText("&remove file");
	removeFileBut->setFont(*g_font_S);
	removeFileBut->setAutoRaise(true);
	
	fileListBox = new QListBox(this);
	fileListBox->setFont(*g_font_S);
	
//	outputLabel = new QLabel(this);
//	outputLabel->setText("output directory :");
//	outputLabel->setAlignment( AlignLeft | AlignBottom );
//	outputLabel->setFixedHeight(configLabel->sizeHint().height()*2);
	
	outputEdit = new QLineEdit(this);
	outputEdit->setFont(*g_font_S);
	outputEdit->setReadOnly(true);
	
	outputBrowseBut = new MToolButton(this);
	outputBrowseBut->setPixmap(*pix_file);
	
	if(g_toolTipsEnabled)
	{
		//QToolTip::add(fastParticlesCheck, "draw particles as points/lines, or full primitives (sphere, disk, etc.)");
	}
	
	//-- Connect -----------------------
	QObject::connect(addFileBut, SIGNAL(clicked()), SLOT(addFile()) );
	QObject::connect(editFileBut, SIGNAL(clicked()), SLOT(editFile()) );
	QObject::connect(removeFileBut, SIGNAL(clicked()), SLOT(removeFile()) );
	QObject::connect(configBrowseBut, SIGNAL(clicked()), SLOT(browseForConfig()) );
	QObject::connect(configSaveBut,   SIGNAL(clicked()), SLOT(saveConfig()) );
	QObject::connect(outputBrowseBut, SIGNAL(clicked()), SLOT(browseForOutputDir()) );
	
	//-- Layout ------------------------
	topGridLayout	= new QGridLayout(this, 4, 2, 2);
	butLayout		= new QHBoxLayout(5);
	outputLayout	= new QHBoxLayout(2);
	configLayout	= new QHBoxLayout(2);
	
//	topVBoxLayout->addWidget(configLabel, 0);
	topGridLayout->addLayout(configLayout, 0, 1);
		configLayout->addWidget(configEdit, 1);
		configLayout->addWidget(configSaveBut, 0);
		configLayout->addWidget(configBrowseBut, 0);
//	topVBoxLayout->addWidget(fileLabel, 0);
	topGridLayout->addWidget(fileListBox,  1, 1);
	topGridLayout->addLayout(butLayout,    2, 1);
		butLayout->addWidget(addFileBut);
		butLayout->addWidget(editFileBut);
		butLayout->addWidget(removeFileBut);
//	topVBoxLayout->addWidget(outputLabel, 0);
	topGridLayout->addLayout(outputLayout, 3, 1);
		outputLayout->addWidget(outputEdit, 1);
		outputLayout->addWidget(outputBrowseBut, 0);

}

PrefDialogCode::~PrefDialogCode()
{
}

//--------------------------------------------------------------------
void	PrefDialogCode::show()
{	
	fileListBox->clear();
	
	fileListBox->insertStringList(g_scene->getCodeFiles());
	lastOutputDir  = g_scene->getCodeOutDir();
	lastConfigFile = g_scene->getCodeConfigFile();
	
	outputEdit->setText(lastOutputDir);
	configEdit->setText(lastConfigFile);
	fileListBox->clear();
	fileListBox->insertStringList(g_scene->getCodeFiles());
 
 	if(lastFile.isEmpty())
	{
		lastFile = fileListBox->text(0);
	}
	
	QWidget::show();
}

//--------------------------------------------------------------------
void	PrefDialogCode::addFile()
{
	QFileDialog	fileDialog(lastFile, "*", this, "PrefDialogCode::addTemplate", true);
	fileDialog.resize(fileDialog.width(), (2*g_dskTopH)/3);
	fileDialog.setFont(*g_font_M);
	if(fileDialog.exec() == QDialog::Accepted)
	{
		QString addFile = fileDialog.selectedFile();
		if(!addFile.isNull())
		{
			lastFile = addFile;
			fileListBox->insertItem(addFile);
			g_scene->getCodeFiles().append(addFile);
			saveConfig();
		}
	}
}

//--------------------------------------------------------------------
void	PrefDialogCode::removeFile()
{
	int  remove;
	int	 but;
	char buffer[512];
	
	if( (remove = fileListBox->currentItem()) >= 0)
	{
		sprintf(buffer, "remove file\n\"%s\"\nfrom config?", (const char*)fileListBox->text(remove) );
		but = QMessageBox::warning(this, "flow::Remove File?", buffer, 1, 2);
		QStringList::Iterator it;
		if(but == QMessageBox::Ok)
		{
			fileListBox->removeItem(remove);
			it = g_scene->getCodeFiles().at(remove);
			g_scene->getCodeFiles().remove(it);
			saveConfig();
		}
	}
}

//--------------------------------------------------------------------
void	PrefDialogCode::browseForOutputDir()
{
	QString dirName = QFileDialog::getExistingDirectory(lastOutputDir, this);
	if(!dirName.isNull())
	{
		lastOutputDir = dirName;
		g_scene->getCodeOutDir() = lastOutputDir;
	}
}

//--------------------------------------------------------------------
void	PrefDialogCode::browseForConfig()
{
	QFileDialog	fileDialog(lastConfigFile, "*", this, "PrefDialogCode::Locate Code Config", true);
	fileDialog.resize(fileDialog.width(), (2*g_dskTopH)/3);
	fileDialog.setFont(*g_font_M);
	if(fileDialog.exec() == QDialog::Accepted)
	{
		QString cfgFile = fileDialog.selectedFile();
		if(!cfgFile.isNull())
		{
			if(QFile::exists(cfgFile))
			{
				lastConfigFile = cfgFile;
				loadConfig();
			}
			else
			{
				if(QMessageBox::information(this, "flow::Create file?", "File does not exist. Create?", 1, 2) == 1)
				{
					lastConfigFile = cfgFile;
					saveConfig();
					loadConfig();
				}
			}
		}
	}
}

//--------------------------------------------------------------------
void	PrefDialogCode::saveConfig()
{
	g_scene->saveCodeConfig(lastConfigFile);
}

//--------------------------------------------------------------------
void	PrefDialogCode::loadConfig()
{
	QFile		infile;
	QString		lineString;
	char		buffer[512];
	
	if(g_scene->loadCodeConfig(lastConfigFile))
	{
		fileListBox->insertStringList(g_scene->getCodeFiles());
		lastOutputDir  = g_scene->getCodeOutDir();
		lastConfigFile = g_scene->getCodeConfigFile();
	
		outputEdit->setText(lastOutputDir);
		configEdit->setText(lastConfigFile);
		fileListBox->clear();
		fileListBox->insertStringList(g_scene->getCodeFiles());
	}
	else
	{
		sprintf(buffer, "code configuration file : \n%s\ndoes not exist. Create?", (const char*)lastConfigFile);
		if(QMessageBox::warning(this, "flow::File Not Found", buffer, "Create",  "Browse") == 0)
			saveConfig();
		else
			browseForConfig();
	}
}


//--------------------------------------------------------------------
void	PrefDialogCode::editFile()
{
	
}

