#include "prefDialogGeneral.h"

#include <qmessagebox.h>
#include <qtooltip.h>

#include "extern.h"
#include "define.h"
#include "mainFlow.h"
#include "lightGroupWidget.h"
#include "particleControlWidget.h"
#include "mbaSurfaceTabDialog.h"

#include "extern.h"

//==============================================================================
PrefDialogGeneral::PrefDialogGeneral(QWidget *parent, const char *name)
	: QFrame(parent, name)
{
//	setFont(*g_font_MB);
		
	//-- NumInputs
	previewSizeInput = new NumInput(NumInput::Int, this);
	previewSizeInput->SetLabelWidth(g_numLabelWidth*2);
	previewSizeInput->SetLabel("preview size");
	previewSizeInput->SetValueSolid(g_previewRenderSize);
	
	previewInterlaceInput = new NumInput(NumInput::Int, this);
	previewInterlaceInput->SetLabelWidth(g_numLabelWidth*2);
	previewInterlaceInput->SetLabel("preview interlace");
	previewInterlaceInput->SetValueSolid(g_rendInterlace);
	
	randomSeedInput = new NumInput(NumInput::Int, this);
	randomSeedInput->SetLabelWidth(g_numLabelWidth*2);
	randomSeedInput->SetLabel("random seed");
	randomSeedInput->SetValueSolid(g_randSeed);
	
	//-- Text edit
	rendNameEdit = new MLineEdit(this);
	rendNameEdit->setText(g_rendName);
	rendNameEdit->setFont(*g_font_M);
	
	rendererEdit = new MLineEdit(this);
	rendererEdit->setText(g_rendermanExec);
	rendererEdit->setFont(*g_font_M);
	
	renderOptsEdit = new MLineEdit(this);
	renderOptsEdit->setText(g_rendermanOpts);
	renderOptsEdit->setFont(*g_font_M);
//	renderOptsEdit->setEnabled(false);
	
	shaderInfoEdit = new MLineEdit(this);
	shaderInfoEdit->setText(g_shaderInfoExec);
	shaderInfoEdit->setFont(*g_font_M);
	
	shaderEnvEdit = new MLineEdit(this);
	shaderEnvEdit->setText(g_envShaderPath);
	shaderEnvEdit->setFont(*g_font_M);
	
	defaultSurfaceEdit = new MLineEdit(this);
	defaultSurfaceEdit->setText(g_defaultSurfaceShader);
	defaultSurfaceEdit->setFont(*g_font_M);
	
	defaultDisplaceEdit = new MLineEdit(this);
	defaultDisplaceEdit->setText(g_defaultDisplaceShader);
	defaultDisplaceEdit->setFont(*g_font_M);
	
	shaderExtEdit = new MLineEdit(this);
	shaderExtEdit->setText(g_compiledShaderExt);
	shaderExtEdit->setFont(*g_font_M);
	
	tiffToJpgEdit = new MLineEdit(this);
	tiffToJpgEdit->setText(g_tiffToJpegExec);
	tiffToJpgEdit->setFont(*g_font_M);

	//-- Text edit labels
	rendNameLabel = new QLabel(this);
	rendNameLabel->setAlignment( AlignVCenter | AlignRight );
	rendNameLabel->setText("render name");
	rendNameLabel->setMinimumWidth(g_numLabelWidth*2);
	rendNameLabel->setFont(*g_font_SB);
	
	rendererLabel = new QLabel(this);
	rendererLabel->setAlignment( AlignVCenter | AlignRight );
	rendererLabel->setText("RM renderer");
	rendererLabel->setFont(*g_font_SB);	
	
	renderOptsLabel = new QLabel(this);
	renderOptsLabel->setAlignment( AlignVCenter | AlignRight );
	renderOptsLabel->setText("render options");
	renderOptsLabel->setFont(*g_font_SB);	
	
	shaderEnvLabel = new QLabel(this);
	shaderEnvLabel->setAlignment( AlignVCenter | AlignRight );
	shaderEnvLabel->setText("shader environment");
	shaderEnvLabel->setFont(*g_font_SB);	
	
	defaultSurfaceLabel = new QLabel(this);
	defaultSurfaceLabel->setAlignment( AlignVCenter | AlignRight );
	defaultSurfaceLabel->setText("default surface");
	defaultSurfaceLabel->setFont(*g_font_SB);	
	
	defaultDisplaceLabel = new QLabel(this);
	defaultDisplaceLabel->setAlignment( AlignVCenter | AlignRight );
	defaultDisplaceLabel->setText("default displacement");
	defaultDisplaceLabel->setFont(*g_font_SB);	
	
	shaderInfoLabel = new QLabel(this);
	shaderInfoLabel->setAlignment( AlignVCenter | AlignRight );
	shaderInfoLabel->setText("shader info");
	shaderInfoLabel->setFont(*g_font_SB);	
	
	shaderExtLabel = new QLabel(this);
	shaderExtLabel->setAlignment( AlignVCenter | AlignRight );
	shaderExtLabel->setText("shader extension");
	shaderExtLabel->setFont(*g_font_SB);	
	
	tiffToJpgLabel = new QLabel(this);
	tiffToJpgLabel->setAlignment( AlignVCenter | AlignRight );
	tiffToJpgLabel->setText("tiff to jpeg");
	tiffToJpgLabel->setFont(*g_font_SB);	
	
	//-- Check boxes
	saveBaseSceneCheck = new QCheckBox(this);
	saveBaseSceneCheck->setMinimumWidth(g_numInputWidth);
	saveBaseSceneCheck->setText("save \'base.scene\' on exit");
	saveBaseSceneCheck->setChecked(g_saveBaseSceneOnExit);
	
	killChildrenCheck = new QCheckBox(this);
	killChildrenCheck->setMinimumWidth(g_numInputWidth);
	killChildrenCheck->setText("kill child processes on exit");
	killChildrenCheck->setChecked(g_killChildrenOnExit);
	
	//-- Buttons
	rescanShadersBut = new QPushButton(this);
	rescanShadersBut->setText("r e s c a n   s h a d e r s");
	rescanShadersBut->setEnabled(true);
	
	
	if(g_toolTipsEnabled)
	{
		QToolTip::add(rendNameLabel,		"base name for rendered files.\ni.e. 'flow' -> 'flow0000.tif', 'flow0001.tif', etc.");
		QToolTip::add(rendererLabel,		"full path to RenderMan renderer");
		QToolTip::add(renderOptsLabel,		"command-line options for renderer");
		QToolTip::add(shaderEnvLabel,		"name of environment variable that contains the shader path");
		QToolTip::add(defaultSurfaceLabel,	"default surface shader");
		QToolTip::add(defaultDisplaceLabel,	"default displacement shader");
		QToolTip::add(shaderInfoLabel,		"full path to shader info utility");
		QToolTip::add(tiffToJpgLabel,		"full path to TIFF->JPEG conversion utility");
		QToolTip::add(shaderExtLabel,		"extension used for compiled shaders");
		QToolTip::add(rescanShadersBut,		"rescan shader path and update the\nlist of available shaders.");
		QToolTip::add(saveBaseSceneCheck, 	"on exit, save the current scene as 'base.scene'.\n'base.scene' is loaded every time flow starts.");
		QToolTip::add(killChildrenCheck, 	"on exit, send kill signal to all child processes.");
	}
	
	//-- Connect -----------------------
	QObject::connect(previewSizeInput, SIGNAL(ValueChanged(int)), SLOT(SetPreviewSize(int)) );
	QObject::connect(previewInterlaceInput, SIGNAL(ValueChanged(int)), SLOT(SetPreviewInterlace(int)) );
	QObject::connect(randomSeedInput, SIGNAL(ValueChanged(int)), SLOT(SetRandSeed(int)) );
	QObject::connect(killChildrenCheck, SIGNAL(toggled(bool)), SLOT(SetKillChildren(bool)) );
	QObject::connect(saveBaseSceneCheck, SIGNAL(toggled(bool)), SLOT(SetSaveBaseScene(bool)) );

	QObject::connect( rendNameEdit 		 , SIGNAL(returnPressed()), SLOT( SetRendName()				) );
	QObject::connect( rendererEdit		 , SIGNAL(returnPressed()), SLOT( SetRMRenderer()			) );
	QObject::connect( renderOptsEdit	 , SIGNAL(returnPressed()), SLOT( SetRMRenderOpts()			) );
	QObject::connect( shaderEnvEdit 	 , SIGNAL(returnPressed()), SLOT( SetShaderEnvironment()	) );
	QObject::connect( defaultSurfaceEdit , SIGNAL(returnPressed()), SLOT( SetDefaultSurface()		) );
	QObject::connect( defaultDisplaceEdit, SIGNAL(returnPressed()), SLOT( SetDefaultDisplacement()	) );
	QObject::connect( shaderInfoEdit	 , SIGNAL(returnPressed()), SLOT( SetShaderInfo()			) );
	QObject::connect( shaderExtEdit 	 , SIGNAL(returnPressed()), SLOT( SetShaderExtension()		) );
	QObject::connect( tiffToJpgEdit 	 , SIGNAL(returnPressed()), SLOT( SetTiffToJpeg()			) );
	
	QObject::connect( rescanShadersBut 	 , SIGNAL(clicked()), SLOT( RescanShaders()) );
	//-- Layout ------------------------
	topVBoxLayout	= new QVBoxLayout(this, 10, 5);
//	combo0Layout	= new QVBoxLayout(0);
	numInputHLayout	= new QHBoxLayout(10);
	numInput0Layout	= new QVBoxLayout(2);
	butVLayout		= new QVBoxLayout(5);
	textEditGLayout = new QGridLayout(9, 2, 1);
	textEditGLayout->setColStretch(0, 0);
	textEditGLayout->setColStretch(1, 1);
//	//-- Combobox
//	topVBoxLayout->addLayout(combo0Layout, 0);	
	//-- Num Input
	topVBoxLayout->addLayout(numInput0Layout);
		numInput0Layout->addWidget(previewSizeInput);
		numInput0Layout->addWidget(previewInterlaceInput);
		numInput0Layout->addWidget(randomSeedInput);
	//-- Text edit
	topVBoxLayout->addStretch(1);
	topVBoxLayout->addLayout(textEditGLayout, 0);
		textEditGLayout->addWidget(rendNameLabel,        0, 0);
		textEditGLayout->addWidget(rendererLabel,        1, 0);
		textEditGLayout->addWidget(renderOptsLabel,      2, 0);
		textEditGLayout->addWidget(shaderInfoLabel,      3, 0);
		textEditGLayout->addWidget(shaderEnvLabel,       4, 0);
		textEditGLayout->addWidget(defaultSurfaceLabel,  5, 0);
		textEditGLayout->addWidget(defaultDisplaceLabel, 6, 0);
		textEditGLayout->addWidget(shaderExtLabel,       7, 0);
		textEditGLayout->addWidget(tiffToJpgLabel,       8, 0);

		textEditGLayout->addWidget(rendNameEdit,		 0, 1);
		textEditGLayout->addWidget(rendererEdit,		 1, 1);
		textEditGLayout->addWidget(renderOptsEdit,		 2, 1);
		textEditGLayout->addWidget(shaderInfoEdit,  	 3, 1);
		textEditGLayout->addWidget(shaderEnvEdit, 		 4, 1);
		textEditGLayout->addWidget(defaultSurfaceEdit,   5, 1);
		textEditGLayout->addWidget(defaultDisplaceEdit,  6, 1);
		textEditGLayout->addWidget(shaderExtEdit, 		 7, 1);
		textEditGLayout->addWidget(tiffToJpgEdit, 		 8, 1);

	//-- Checkbox
	topVBoxLayout->addStretch(2);
	topVBoxLayout->addWidget(saveBaseSceneCheck);
	topVBoxLayout->addWidget(killChildrenCheck);
	topVBoxLayout->addLayout(butVLayout, 0);
		butVLayout->addWidget(rescanShadersBut);
		
	topVBoxLayout->activate();
}

PrefDialogGeneral::~PrefDialogGeneral()
{
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::show()
{
	previewSizeInput->SetValueSolid(g_previewRenderSize);
	previewInterlaceInput->SetValueSolid(g_rendInterlace);
	randomSeedInput->SetValueSolid(g_randSeed);
	rendNameEdit->setText(g_rendName);
	tiffToJpgEdit->setText(g_tiffToJpegExec);
	shaderExtEdit->setText(g_compiledShaderExt);
	defaultSurfaceEdit->setText(g_defaultSurfaceShader);
	shaderEnvEdit->setText(g_envShaderPath);
	renderOptsEdit->setText(g_rendermanOpts);
	shaderInfoEdit->setText(g_shaderInfoExec);
	rendererEdit->setText(g_rendermanExec);
	killChildrenCheck->setChecked(g_killChildrenOnExit);
	saveBaseSceneCheck->setChecked(g_saveBaseSceneOnExit);
	QFrame::show();
}

//--------------------------------------------------------------------
bool	PrefDialogGeneral::SavePrefs()
{
	return true;
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetRendName()
{
	delete [] g_rendName;
	g_rendName = new char[strlen(rendNameEdit->text())+1];
	strcpy(g_rendName, rendNameEdit->text());
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetPreviewSize(int in)
{
	g_previewRenderSize = in;
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetPreviewInterlace(int in)
{
	g_rendInterlace = in;
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetRandSeed(int in)
{
	g_randSeed = in;
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetKillChildren(bool status)
{
	g_killChildrenOnExit = status;
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetSaveBaseScene(bool status)
{
	g_saveBaseSceneOnExit = status;
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetRMRenderer()
{
	char	message[256];
	if(g_rendermanExec)
		delete [] g_rendermanExec;
	g_rendermanExec = new char[strlen(rendererEdit->text())+1];
	strcpy(g_rendermanExec, rendererEdit->text());
	
	warning("---- Check for renderer ( \"%s\" )", g_rendermanExec);
	QFileInfo	rendExecCheck(g_rendermanExec);
	if (rendExecCheck.isExecutable())
		g_hasRendermanExec = true;
	else
	{
		sprintf (message, "ERROR: render exec\n\"%s\"\ndoes not exist or is not executable.\nSet the correct value in File->Preferences.\n(absolute pathname required)", g_rendermanExec);
		QMessageBox::critical( this, FLOW_ERROR, message, "Understood");
		g_hasRendermanExec = false;
	}
	
	this->setFocus();
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetRMRenderOpts()
{
	if(g_rendermanOpts)
		delete [] g_rendermanOpts;
	QCString	tmpString(renderOptsEdit->text());
	tmpString = tmpString.simplifyWhiteSpace();
	g_rendermanOpts = new char[strlen((const char*)tmpString)+1];
	strcpy(g_rendermanOpts, (const char*)tmpString);
	renderOptsEdit->setText(g_rendermanOpts);
	this->setFocus();
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetShaderEnvironment()
{
	if(g_envShaderPath)
		delete [] g_envShaderPath;
	g_envShaderPath = new char[strlen(shaderEnvEdit->text())+1];
	strcpy(g_envShaderPath, shaderEnvEdit->text());
	this->setFocus();
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetDefaultSurface()
{
	if(g_defaultSurfaceShader)
		delete [] g_defaultSurfaceShader;
	g_defaultSurfaceShader = new char[strlen(defaultSurfaceEdit->text())+1];
	strcpy(g_defaultSurfaceShader, defaultSurfaceEdit->text());
	this->setFocus();
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetDefaultDisplacement()
{
	if(g_defaultDisplaceShader)
		delete [] g_defaultDisplaceShader;
	g_defaultDisplaceShader = new char[strlen(defaultDisplaceEdit->text())+1];
	strcpy(g_defaultDisplaceShader, defaultDisplaceEdit->text());
	this->setFocus();
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetShaderInfo()
{
	char	message[256];
	if(g_shaderInfoExec)
		delete [] g_shaderInfoExec;
	g_shaderInfoExec = new char[strlen(shaderInfoEdit->text())+1];
	strcpy(g_shaderInfoExec, shaderInfoEdit->text());
	
	QFileInfo	shadeInfoExecCheck(g_shaderInfoExec);
	if (shadeInfoExecCheck.isExecutable())
		g_hasShaderInfoExec = true;
	else
	{
		sprintf (message, "ERROR: shader info exec\n\"%s\"\ndoes not exist or is not executable.\nSet the correct value in File->Preferences.\n(absolute pathname required)", g_shaderInfoExec);
		QMessageBox::critical( this, FLOW_ERROR, message, "Understood");
		g_hasShaderInfoExec = false;
	}
	
	this->setFocus();
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetShaderExtension()
{
	if(g_compiledShaderExt)
		delete [] g_compiledShaderExt;
	g_compiledShaderExt = new char[strlen(shaderExtEdit->text())+1];
	strcpy(g_compiledShaderExt, shaderExtEdit->text());
	this->setFocus();
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::SetTiffToJpeg()
{
	char	message[256];
	if(g_tiffToJpegExec)
		delete [] g_tiffToJpegExec;
	g_tiffToJpegExec = new char[strlen(tiffToJpgEdit->text())+1];
	strcpy(g_tiffToJpegExec, tiffToJpgEdit->text());
	
	QFileInfo	convertExecCheck(g_tiffToJpegExec);
	if (convertExecCheck.isExecutable())
		g_hasTiffToJpegExec = true;
	else
	{
		sprintf (message, "ERROR: tiff to jpeg convert exec\n\"%s\"\ndoes not exist or is not executable.\nSet the correct value in File->Preferences.\n(absolute pathname required)", g_tiffToJpegExec);
		QMessageBox::critical( this, FLOW_ERROR, message, "Understood");
		g_hasTiffToJpegExec = false;
	}
	this->setFocus();
}

//--------------------------------------------------------------------
void	PrefDialogGeneral::RescanShaders()
{
	::CheckShaders();
	g_surfaceEditWidget->reloadShaders();
}

