#include "prefDialogUI.h"

#include <qtimer.h>
#include <qtooltip.h>
#include <qfontdialog.h>
#include <qcolordialog.h>

#include "extern.h"
#include "mainFlow.h"
#include "lightGroupWidget.h"
#include "particleControlWidget.h"
#include "mbaSurfaceTabDialog.h"
#include "renderPanelTabDialog.h"
#include "configFlowIO.h"

#include "extern.h"

//==============================================================================
PrefDialogUI::PrefDialogUI(QWidget *parent, const char *name)
	: QFrame(parent, name)
{
	setFont(*g_font_MB);

	tbIndex	= -1;
	window	= 0;
	
	//-- startup options label
	startupOptionsLabel = new QLabel(this);
	startupOptionsLabel->setText("s t a r t u p   o p t i o n s");
	startupOptionsLabel->setAlignment( AlignCenter );
	startupOptionsLabel->setFrameStyle( QFrame::Sunken | QFrame::Panel );
	startupOptionsLabel->setFont(*g_font_M);
	
	//-- Check boxes
	toolTipsCheck = new QCheckBox(this);
	toolTipsCheck->setMinimumWidth(g_numInputWidth);
	toolTipsCheck->setText("tooltips");
	
	persistentCheck = new QCheckBox(this);
	persistentCheck->setMinimumWidth(g_numInputWidth);
	persistentCheck->setText("persistent geometry");
	
	persistentCheck->setChecked(g_persistentGeom);
	toolTipsCheck->setChecked(g_toolTipsEnabled);	
	
	//-- ToolBar label
	widgetToolLabel = new QLabel(this);
	widgetToolLabel->setText("t o o l b a r   p o s i t i o n s");
	widgetToolLabel->setAlignment( AlignCenter );
	widgetToolLabel->setFrameStyle( QFrame::Sunken | QFrame::Panel );
	widgetToolLabel->setFont(*g_font_M);

	//-- ToolBar stuff
	widgetToolList = new QListBox(this);
	widgetToolList->setMinimumSize(60, 25);
	widgetToolList->setFont(*g_font_M);
	widgetToolList->insertItem("main 0 - toolBar");			// 0
	widgetToolList->insertItem("main 1 - infoBar"); 		// 1
	widgetToolList->insertItem("particle 0 - toolBar"); 	// 2
	widgetToolList->insertItem("particle 1 - valueEdit");	// 3
	widgetToolList->insertItem("lights 0 - toolBar");		// 4
	widgetToolList->insertItem("lights 1 - colorEdit"); 	// 5
	widgetToolList->insertItem("lights 2 - valueEdit"); 	// 6
	widgetToolList->insertItem("surface 0 - toolbar");		// 7
	widgetToolList->insertItem("render 0 - toolbar");		// 8
	
	toolTopRadio	= new QRadioButton(this);
	toolTopRadio->setFixedSize(g_numInputWidth/2, 20);
	toolTopRadio->setText("top");
	toolBotRadio	= new QRadioButton(this);
	toolBotRadio->setFixedSize(g_numInputWidth/2, 20);
	toolBotRadio->setText("bottom");
	toolLeftRadio	= new QRadioButton(this);
	toolLeftRadio->setFixedSize(g_numInputWidth/2, 20);
	toolLeftRadio->setText("left");
	toolRightRadio	= new QRadioButton(this);
	toolRightRadio->setFixedSize(g_numInputWidth/2, 20);
	toolRightRadio->setText("right");
	toolPosGroup = new QButtonGroup();
	toolPosGroup->setExclusive(true);
	toolPosGroup->insert(toolTopRadio);
	toolPosGroup->insert(toolBotRadio);
	toolPosGroup->insert(toolLeftRadio);
	toolPosGroup->insert(toolRightRadio);
	newLineCheck = new QCheckBox(this);
	newLineCheck->setText("newline");
	newLineCheck->setChecked(true);
	
	char	tmpString[128];
	//-- Push Buttons
	fontBut = new QPushButton(this);
	fontBut->setText("f o n t");
	fontBut->setFixedWidth(g_numInputWidth);
	
	fontLabel = new QLabel(this);
	fontLabel->setFrameStyle( QFrame::Sunken | QFrame::Panel );
	sprintf(tmpString, "%s, %d", g_fontName, g_fontSize);
	fontLabel->setText(tmpString);
	
	fgColorBut = new QPushButton(this);
	fgColorBut->setText("fg");
	fgColorBut->setFixedWidth(g_numInputWidth);
	
	fgColorLabel = new QLabel(this);
	fgColorLabel->setMaximumHeight(fgColorBut->height());
	fgColorLabel->setText("b u t t o n  c o l o u r");
	fgColorLabel->setAlignment( AlignVCenter | AlignRight );
	
	bgColorBut = new QPushButton(this);
	bgColorBut->setText("bg");
	bgColorBut->setFixedWidth(g_numInputWidth);
	
	bgColorLabel = new QLabel(this);
	bgColorLabel->setMaximumHeight(bgColorBut->height());
	bgColorLabel->setText("b a s e  c o l o u r");
	bgColorLabel->setAlignment( AlignVCenter | AlignRight );
		
	saveGeomBut = new QPushButton(this);
	saveGeomBut->setText("s a v e   c u r r e n t   g e o m e t r y");
	
	//-- Num inputs
	xWinManAdd = new NumInput(NumInput::Int, this);
	xWinManAdd->SetLabelWidth(g_numLabelWidth);
	xWinManAdd->SetTotalWidth(g_numInputWidth);
	xWinManAdd->SetLabel("x add");
	xWinManAdd->SetValueSolid(g_winManXAdd);
	
	yWinManAdd = new NumInput(NumInput::Int, this);
	yWinManAdd->SetLabelWidth(g_numLabelWidth);
	yWinManAdd->SetTotalWidth(g_numInputWidth);
	yWinManAdd->SetLabel("y add");
	yWinManAdd->SetValueSolid(g_winManYAdd);
	
	winManAddLabel = new QLabel(this);
	winManAddLabel->setText("some window managers \nincorrectly report x,y position.\nuse this to compensate:");
	winManAddLabel->setAlignment (AlignTop | AlignRight);
	
	if(g_toolTipsEnabled)
	{
		QToolTip::add(saveGeomBut,		"save window positions and layout");
		QToolTip::add(persistentCheck,	"save window positions and layout\nevery time flow exits.");
		QToolTip::add(toolTipsCheck,	"enable tooltip hints\n(takes effect next time flow is run)");
	}
	
	//-- Connect -----------------------
	QObject::connect(widgetToolList, SIGNAL(highlighted(int)), SLOT(SelectToolBar(int)) );
	QObject::connect(toolTopRadio, SIGNAL(clicked()), SLOT(MoveToolTo_Top()) );
	QObject::connect(toolBotRadio, SIGNAL(clicked()), SLOT(MoveToolTo_Bot()) );
	QObject::connect(toolLeftRadio, SIGNAL(clicked()), SLOT(MoveToolTo_Left()) );
	QObject::connect(toolRightRadio, SIGNAL(clicked()), SLOT(MoveToolTo_Right()) );

	QObject::connect(persistentCheck, SIGNAL(toggled(bool)), SLOT(SetPersistent(bool)) );
	QObject::connect(toolTipsCheck, SIGNAL(toggled(bool)), SLOT(SetToolTips(bool)) );

	QObject::connect(fontBut, SIGNAL(clicked()), SLOT(setBaseFont()) );

	QObject::connect(fgColorBut, SIGNAL(clicked()), SLOT(setFgColor()) );
	QObject::connect(bgColorBut, SIGNAL(clicked()), SLOT(setBgColor()) );

	QObject::connect(saveGeomBut, SIGNAL(clicked()), SLOT(SaveCurrentGeometry()) );

	QObject::connect(xWinManAdd, SIGNAL(ValueChanged(int)), SLOT(SetWinManAddX(int)) );
	QObject::connect(yWinManAdd, SIGNAL(ValueChanged(int)), SLOT(SetWinManAddY(int)) );
	
	//-- Layout ------------------------
	topVBoxLayout	= new QVBoxLayout(this, 10, 0);
	h1Layout		= new QHBoxLayout(5);
	toolPosVLayout	= new QVBoxLayout(1);
	toolPosH1Layout	= new QHBoxLayout(1);
	toolPosH2Layout	= new QHBoxLayout(1);
	toolPosH3Layout	= new QHBoxLayout(1);
	check0Layout	= new QVBoxLayout(1);
	saveAndXYHLayout= new QHBoxLayout(5);
	winManAddLayout	= new QVBoxLayout(1);
	fontHLayout		= new QHBoxLayout(1);
	colorGLayout	= new QGridLayout(2, 2, 1);
	
	topVBoxLayout->addWidget(startupOptionsLabel, 0);
	topVBoxLayout->addLayout(check0Layout, 0);
		check0Layout->addWidget(toolTipsCheck);
		check0Layout->addWidget(persistentCheck);
		check0Layout->addStretch(1);
	//topVBoxLayout->addSpacing(5);
	topVBoxLayout->addLayout(saveAndXYHLayout, 0);
		saveAndXYHLayout->addWidget(winManAddLabel, 1);
		saveAndXYHLayout->addLayout(winManAddLayout, 0);
			winManAddLayout->addWidget(xWinManAdd, 0);
			winManAddLayout->addWidget(yWinManAdd, 0);
	topVBoxLayout->addSpacing(10);
	topVBoxLayout->addWidget(widgetToolLabel, 0);
	topVBoxLayout->addSpacing(3);
	topVBoxLayout->addLayout(h1Layout, 1);
		h1Layout->addWidget(widgetToolList, 1);
		h1Layout->addLayout(toolPosVLayout, 0);
			toolPosVLayout->addWidget(newLineCheck);
			toolPosVLayout->addStretch(1);
			toolPosVLayout->addLayout(toolPosH1Layout, 0);
				toolPosH1Layout->addStretch(1);
				toolPosH1Layout->addWidget(toolTopRadio);
				toolPosH1Layout->addStretch(1);
			toolPosVLayout->addLayout(toolPosH2Layout, 0);
				toolPosH2Layout->addWidget(toolLeftRadio);
				toolPosH2Layout->addStretch(1);
				toolPosH2Layout->addWidget(toolRightRadio, 0);
			toolPosVLayout->addLayout(toolPosH3Layout);
				toolPosH3Layout->addStretch(1);
				toolPosH3Layout->addWidget(toolBotRadio, 0);
				toolPosH3Layout->addStretch(1);
			//toolPosVLayout->addSpacing(10);
			toolPosVLayout->addStretch(2);
	topVBoxLayout->addSpacing(10);
	topVBoxLayout->addLayout(fontHLayout, 1);
		fontHLayout->addWidget(fontBut, 0);
		fontHLayout->addWidget(fontLabel, 1);
	topVBoxLayout->addSpacing(10);
	topVBoxLayout->addLayout(colorGLayout, 1);
		colorGLayout->addWidget(fgColorLabel, 0, 0);
		colorGLayout->addWidget(bgColorLabel, 1, 0);
		colorGLayout->addWidget(fgColorBut,   0, 1);
		colorGLayout->addWidget(bgColorBut,   1, 1);
	topVBoxLayout->addSpacing(5);
	topVBoxLayout->addWidget(saveGeomBut, 1);

	topVBoxLayout->activate();
	
}

PrefDialogUI::~PrefDialogUI()
{
}

//--------------------------------------------------------------------
void	PrefDialogUI::show()
{	
	xWinManAdd->SetValueSolid(g_winManXAdd);
	yWinManAdd->SetValueSolid(g_winManYAdd);
	QWidget::show();
}

//--------------------------------------------------------------------
bool	PrefDialogUI::SavePrefs()
{
	return true;
}

//--------------------------------------------------------------------
void	PrefDialogUI::MoveToolTo_Top()
{
	if(window && tbIndex > -1)
	{
		window->setToolBar(tbIndex, QMainWindow::Top, newLineCheck->isChecked());
	}
}

//--------------------------------------------------------------------
void	PrefDialogUI::MoveToolTo_Bot()
{
	if(window && tbIndex > -1)
	{
		window->setToolBar(tbIndex, QMainWindow::Bottom, newLineCheck->isChecked());
	}
}

//--------------------------------------------------------------------
void	PrefDialogUI::MoveToolTo_Left()
{
	if(window && tbIndex > -1)
	{
		window->setToolBar(tbIndex, QMainWindow::Left, newLineCheck->isChecked());
	}
}

//--------------------------------------------------------------------
void	PrefDialogUI::MoveToolTo_Right()
{
	if(window && tbIndex > -1)
	{
		window->setToolBar(tbIndex, QMainWindow::Right, newLineCheck->isChecked());
	}
}

//--------------------------------------------------------------------
void	PrefDialogUI::SelectToolBar(int index)
{
	//warning("PrefDialogUI::SelectToolBar(%d)", index);
	switch(index)
	{
		case 0:								// main - toolbar 0";
			window = g_mainFlow;
			tbIndex = 0;
			toolPosGroup->setButton(TranslateDock(g_mainFlow->toolBarDock(0)) );
			break;
		case 1:								// main - toolbar 1);
			window = g_mainFlow;
			tbIndex = 1;
			toolPosGroup->setButton(TranslateDock(g_mainFlow->toolBarDock(1)) );
			break;
		case 2:								// particle - toolbar 0
			window = g_particleControlFrame;
			tbIndex = 0;
			toolPosGroup->setButton(TranslateDock(g_particleControlFrame->toolBarDock(0)) );
			break;
		case 3:								// particle - toolbar 1
			window = g_particleControlFrame;
			tbIndex = 1;
			toolPosGroup->setButton(TranslateDock(g_particleControlFrame->toolBarDock(1)) );
			break;
		case 4:								// lights - toolbar 0
			window = g_lightGroupWidgetFrame;
			tbIndex = 0;
			toolPosGroup->setButton(TranslateDock(g_lightGroupWidgetFrame->toolBarDock(0)) );
			break;
		case 5:								// lights - toolbar 1
			window = g_lightGroupWidgetFrame;
			toolPosGroup->setButton(TranslateDock(g_lightGroupWidgetFrame->toolBarDock(1)) );
			tbIndex = 1;
			break;
		case 6:								// lights - toolbar 2
			window = g_lightGroupWidgetFrame;
			tbIndex = 2;
			toolPosGroup->setButton(TranslateDock(g_lightGroupWidgetFrame->toolBarDock(2)) );
			break;
		case 7:								// surface - toolbar 0
			window = g_surfaceEditWidget;
			tbIndex = 0;
			//toolPosGroup->setButton(TranslateDock(_tb0_surfaceTab));
			toolPosGroup->setButton(TranslateDock(g_surfaceEditWidget->toolBarDock(0)) );
			break;
		case 8:								// render - toolbar 0
			window = g_renderOptions;
			tbIndex = 0;
			//toolPosGroup->setButton(TranslateDock(_tb0_renderTab));
			toolPosGroup->setButton(TranslateDock(g_renderOptions->toolBarDock(0)) );
			break;
		default:
			window = 0;
			tbIndex = 0;
	}
}

//--------------------------------------------------------------------
int	PrefDialogUI::TranslateDock(int dockEnum)
{
	switch((QMainWindow::ToolBarDock)dockEnum)
	{
		case QMainWindow::Top:
			return 0;
		case QMainWindow::Bottom:
			return 1;
		case QMainWindow::Left:
			return 2;
		case QMainWindow::Right:
			return 3;
		case QMainWindow::Unmanaged:
		case QMainWindow::TornOff:
		default:
			return 0;
	}
}

//--------------------------------------------------------------------
void	PrefDialogUI::SetPersistent(bool status)
{
	g_persistentGeom = status;
}

//--------------------------------------------------------------------
void	PrefDialogUI::SetToolTips(bool status)
{
	g_toolTipsEnabled = status;
}

//--------------------------------------------------------------------
void	PrefDialogUI::setBaseFont()
{
	bool	ok;
//	QFontDialog::getFont(&ok, *g_font_M, this);
	QFont tempFont = QFontDialog::getFont(&ok, QFont(g_fontName, g_fontSize), this);
	if(ok)
	{
		delete [] g_fontName;
		g_fontName = new char[strlen(tempFont.family())+1];
		strcpy(g_fontName, tempFont.family());
		g_fontSize = tempFont.pointSize();
		char	tmpString[128];
		sprintf(tmpString, "%s, %d", g_fontName, g_fontSize);
		fontLabel->setText(tmpString);
	}
}

//--------------------------------------------------------------------
void	PrefDialogUI::setFgColor()
{
	QPalette	curPalette;
	QColor 		butColor;
	QColor		backColor;
	
	curPalette	= QApplication::palette();
	butColor	= curPalette.color( QPalette::Normal, QColorGroup::Button );
	butColor	= QColorDialog::getColor(butColor, this);
	if(butColor.isValid())
	{
		backColor	= curPalette.color( QPalette::Normal, QColorGroup::Background );
		curPalette = QPalette(butColor, backColor);
		curPalette.setColor(QColorGroup::Highlight, curPalette.active().dark());
		curPalette.setColor(QPalette::Disabled, QColorGroup::Highlight, curPalette.active().mid());

		QApplication::setPalette(curPalette, true);
	}
}

//--------------------------------------------------------------------
void	PrefDialogUI::setBgColor()
{
	QPalette	curPalette;
	QColor 		butColor;
	QColor		backColor;
	
	curPalette	= QApplication::palette();
	butColor	= curPalette.color( QPalette::Normal, QColorGroup::Button );
	backColor	= curPalette.color( QPalette::Normal, QColorGroup::Background );
	backColor	= QColorDialog::getColor(backColor, this);
	if(backColor.isValid())
	{
		curPalette = QPalette(butColor, backColor);
		curPalette.setColor(QColorGroup::Highlight, curPalette.active().dark());
		curPalette.setColor(QPalette::Disabled, QColorGroup::Highlight, curPalette.active().mid());

		QApplication::setPalette(curPalette, true);
	}
}

//--------------------------------------------------------------------
void	PrefDialogUI::SaveCurrentGeometry()
{
	ConfigFlowIO::WriteGeomFile();
}

//--------------------------------------------------------------------
void	PrefDialogUI::SetWinManAddX(int val)
{
	g_winManXAdd = val;
}

//--------------------------------------------------------------------
void	PrefDialogUI::SetWinManAddY(int val)
{
	g_winManYAdd = val;
}
