#include "renderPanelCode.h"

#ifndef WIN32	
	#include <unistd.h>
	#include <sys/wait.h>
#endif

#include <ri.h>
#include <qapp.h>
#include <qvalidator.h>
#include <qfiledialog.h>
#include <qmessagebox.h>

#include "compatibility.h"
#include <GL/gl.h>

#include "extern.h"
#include "define.h"
#include "numInput.h"
#include "mbaScene.h"
#include "glPortCam.h"
#include "colorButton.h"

//static int checkHeight = 20;
//static int buttonHeight = 25;

//==============================================================================
RenderPanelCode::RenderPanelCode(QWidget *parent, const char* name)
	: QFrame (parent, name)
{
	codeDir  = g_flowHome;
	codeDir += "/Code/";
	
	codeFileName		= codeDir+"pout.cpp";	
	templateFileName	= codeDir+"Templates/flow_template.cpp";
	
//	setFont(*g_font_MB);
	setFrameStyle( QFrame::Sunken | QFrame::Panel );
	setLineWidth( 2 );
		
	//-- buttons
	modeSelButGroup = new QButtonGroup();
	modeSelButGroup->setExclusive(true);
	
	configBut = new MToolButton(this);
	configBut->setText("config");
	configBut->setToggleButton(true);
	configBut->setAutoRaise(true);
	
	optionsBut = new MToolButton(this);
	optionsBut->setText("options");
	optionsBut->setAutoRaise(true);
	optionsBut->setToggleButton(true);
	optionsBut->setEnabled(false);
	
	buildBut = new MToolButton(this);
	buildBut->setText("build");
	buildBut->setAutoRaise(true);
	buildBut->setToggleButton(true);
	buildBut->setEnabled(false);
	
	modeSelButGroup->insert(configBut,  RenderPanelCode::Config);
	modeSelButGroup->insert(optionsBut, RenderPanelCode::Options);
	modeSelButGroup->insert(buildBut,   RenderPanelCode::Build);
	
	writeCodeBut = new QPushButton(this);
	writeCodeBut->setText("&write code");
	
	//-- Widget Stack
	configTab = new RenderPanelCodeConfig();
	optionsTab = new RenderPanelCodeOptions();
	
	widgetStack = new QWidgetStack(this);
	widgetStack->addWidget(configTab, 0);
	widgetStack->addWidget(optionsTab, 0);
	
	widgetStack->raiseWidget(configTab);
	
	//----------------------------------
	QObject::connect(writeCodeBut,    SIGNAL(clicked()), SLOT(writeCode()) );
	QObject::connect(modeSelButGroup, SIGNAL(pressed(int)), SLOT(selectMode(int)) );
	QObject::connect(configBut, SIGNAL(stateChanged(int)), SLOT(setConfigPalette(int)) );
	QObject::connect(optionsBut, SIGNAL(stateChanged(int)), SLOT(setOptionsPalette(int)) );
	QObject::connect(buildBut, SIGNAL(stateChanged(int)), SLOT(setBuildPalette(int)) );

	//----------------------------------
	topVLayout		= new QVBoxLayout(this, 10, 10);
	topHLayout		= new QHBoxLayout(10);
	v1Layout		= new QVBoxLayout(1);
	
	//----------------------------------
	topVLayout->addLayout(topHLayout, 1);
		topHLayout->addLayout(v1Layout, 0);
			v1Layout->addWidget(configBut);
			v1Layout->addWidget(optionsBut);
			v1Layout->addWidget(buildBut);
			v1Layout->addStretch(1);
			//v1Layout->addWidget(modeSelButGroup);
			v1Layout->addWidget(writeCodeBut);
		topHLayout->addWidget(widgetStack, 1);
			
//	configBut->setOn(true);
	modeSelButGroup->setButton(0);
}

RenderPanelCode::~RenderPanelCode()
{
}

//--------------------------------------------------------------------
void RenderPanelCode::show()
{
	QFrame::show();
}

//--------------------------------------------------------------------
void	RenderPanelCode::setConfigPalette(int state)
{
	if(state == 2)
		configBut->setPalette(QPalette(QColor(128, 128, 255)));
	else
		configBut->setPalette(QApplication::palette());
	
}

//--------------------------------------------------------------------
void	RenderPanelCode::setOptionsPalette(int state)
{
	if(state == 2)
		optionsBut->setPalette(QPalette(QColor(128, 255, 128)));
	else
		optionsBut->setPalette(QApplication::palette());
	
}
	
//--------------------------------------------------------------------
void	RenderPanelCode::setBuildPalette(int state)
{
	if(state == 2)
		buildBut->setPalette(QPalette(QColor(255, 128, 128)));
	else
		buildBut->setPalette(QApplication::palette());
	
}
	
//--------------------------------------------------------------------
void RenderPanelCode::writeCode()
{
	if(!g_scene->writeCode())
	{
		char buffer[256];
		sprintf(buffer, "Error writing code.");
		QMessageBox::warning(this, FLOW_ERROR, buffer);
	}
}

//--------------------------------------------------------------------
void RenderPanelCode::selectMode(int mode)
{
	switch(mode)
	{
		case Config:
			widgetStack->raiseWidget(configTab);
			break;
		case Build:
			warning("RenderPanelCode::selectMode -- Build not here yet.");
			break;
		case Options:
			widgetStack->raiseWidget(optionsTab);
			//warning("RenderPanelCode::selectMode -- Options not here yet.");
			break;
		default:
			warning("RenderPanelCode::selectMode -- unknown mode");
			break;
	}
}















