#include "renderPanelGL.h"

#ifndef WIN32	
	#include <unistd.h>
	#include <sys/wait.h>
#endif

#include <ri.h>
#include <qvalidator.h>
#include <qapp.h>

#include "compatibility.h"
#include <GL/gl.h>

#include "extern.h"
#include "numInput.h"
#include "mbaScene.h"
#include "glPortCam.h"
#include "camera.h"
#include "colorButton.h"

//==============================================================================
RenderPanelGL::RenderPanelGL(QWidget *parent, const char* name)
	: QFrame (parent, name)
{
	setFrameStyle( QFrame::Sunken | QFrame::Panel );
	setLineWidth( 2 );
	setFont(*g_font_SB);		
	
	glFillCheck = new QCheckBox("Wireframe", this);
	glFillCheck->setChecked(false);
	
	glCullCheck = new QCheckBox("Backface Cull", this);
	glCullCheck->setChecked(g_glCullFace);
	
	glBlendCheck = new QCheckBox("Blend", this);
	glBlendCheck->setChecked(g_glBlend);
	
	glAlphaTestCheck = new QCheckBox("Alpha Test", this);
	glAlphaTestCheck->setChecked(g_glAlphaTest);
	
	glTwoSideLightCheck = new QCheckBox("2 Side Lighting", this);
	glTwoSideLightCheck->setChecked(g_glTwoSideLight);
	
	glPointSmoothCheck = new QCheckBox("Point Smooth", this);
	glPointSmoothCheck->setChecked(false);
	
	glLineSmoothCheck = new QCheckBox("Line Smooth", this);
	glLineSmoothCheck->setChecked(false);
	
	gridYZCheck = new QCheckBox("grid YZ", this);
	gridYZCheck->setFixedWidth(g_numInputWidth);
	
	gridXZCheck = new QCheckBox("grid XZ", this);
	gridXZCheck->setChecked(true);
	
	gridXYCheck = new QCheckBox("grid XY", this);
	
	unitSizeInput = new NumInput(NumInput::Double, this);
	unitSizeInput->SetLabelWidth(g_numLabelWidth);
	unitSizeInput->SetTotalWidth(g_numInputWidth);
	unitSizeInput->SetLabel("unit size");
	unitSizeInput->SetValueSolid(g_globalCam->gridUnitSize());

	numUnitsInput = new NumInput(NumInput::Int, this);
	numUnitsInput->SetLabelWidth(g_numLabelWidth);
	numUnitsInput->SetTotalWidth(g_numInputWidth);
	numUnitsInput->SetLabel("num units");
	numUnitsInput->SetValueSolid(g_globalCam->gridNumUnits());
	
	//-- Combo box
	particleSkipCombo = new QComboBox(this);
	particleSkipCombo->setFixedHeight(g_comboHeight);
	particleSkipCombo->insertItem(" 0 ");
	particleSkipCombo->insertItem(" 1 ");
	particleSkipCombo->insertItem(" 2 ");
	particleSkipCombo->insertItem(" 5 ");
	particleSkipCombo->insertItem(" 10 ");
	particleSkipCombo->insertItem(" 25 ");
	particleSkipCombo->insertItem(" 50 ");
	//-- Label
	particleSkipLabel = new QLabel(this);
	//particleSkipLabel->setAlignment(AlignVCenter | AlignRight);
	particleSkipLabel->setText("Particle Skip");
	
	fastParticlesCheck = new QCheckBox(this);
	fastParticlesCheck->setMinimumWidth(g_numInputWidth);
	fastParticlesCheck->setText("simple particles");
	fastParticlesCheck->setChecked(g_glFastParticles);
	
	sortParticlesCheck = new QCheckBox(this);
	sortParticlesCheck->setMinimumWidth(g_numInputWidth);
	sortParticlesCheck->setText("sort particles");
	sortParticlesCheck->setChecked(g_glFastParticles);
	
	//----------------------------------
	QObject::connect(glFillCheck,		SIGNAL(toggled(bool)), SLOT(setGLFillMode(bool)) );
	QObject::connect(glCullCheck,		SIGNAL(toggled(bool)), SLOT(setGLCullFace(bool)) );
	QObject::connect(glBlendCheck,		SIGNAL(toggled(bool)), SLOT(setGLBlend(bool)) );
	QObject::connect(glAlphaTestCheck,		SIGNAL(toggled(bool)), SLOT(setGLAlphaTest(bool)) );
	QObject::connect(glTwoSideLightCheck,SIGNAL(toggled(bool)), SLOT(setTwoSideLighting(bool)) );
	QObject::connect(glPointSmoothCheck,SIGNAL(toggled(bool)), SLOT(setPointSmooth(bool)) );
	QObject::connect(glLineSmoothCheck,	SIGNAL(toggled(bool)), SLOT(setLineSmooth(bool)) );

	QObject::connect(gridYZCheck,		SIGNAL(toggled(bool)), SLOT(setYZCheck(bool)) );
	QObject::connect(gridXZCheck,		SIGNAL(toggled(bool)), SLOT(setXZCheck(bool)) );
	QObject::connect(gridXYCheck,		SIGNAL(toggled(bool)), SLOT(setXYCheck(bool)) );
	QObject::connect(unitSizeInput,		SIGNAL(ValueChanged(float)), SLOT(setUnitSize(float)) );
	QObject::connect(numUnitsInput,		SIGNAL(ValueChanged(int)), SLOT(setNumUnits(int)) );

	QObject::connect(particleSkipCombo, SIGNAL(activated(int)), SLOT(setParticleSkip(int)) );

	QObject::connect(fastParticlesCheck, SIGNAL(toggled(bool)), SLOT(setFastParticles(bool)) );
	QObject::connect(sortParticlesCheck, SIGNAL(toggled(bool)), SLOT(setSortParticles(bool)) );
	
	//----------------------------------
	topVLayout		= new QVBoxLayout(this, 10, 10);
	topHLayout		= new QHBoxLayout(10);
	v1Layout		= new QVBoxLayout(0);
	v2Layout		= new QVBoxLayout(0);
	v3Layout		= new QVBoxLayout(0);
	particleSkipLayout	= new QHBoxLayout(5);
		
	//----------------------------------
	topVLayout->addLayout(topHLayout, 1);
		topHLayout->addLayout(v1Layout, 0);
			v1Layout->addWidget(glFillCheck);
			v1Layout->addWidget(glCullCheck);
			v1Layout->addWidget(glBlendCheck);
			v1Layout->addWidget(glAlphaTestCheck);
			v1Layout->addWidget(glTwoSideLightCheck);
			v1Layout->addWidget(glPointSmoothCheck);
			v1Layout->addWidget(glLineSmoothCheck);
			v1Layout->addStretch(1);
			v1Layout->addLayout(particleSkipLayout, 0);
				particleSkipLayout->addWidget(particleSkipCombo, 5);
				particleSkipLayout->addWidget(particleSkipLabel, 1);
			v1Layout->addStretch(1);
		topHLayout->addLayout(v2Layout, 0);
			v2Layout->addWidget(fastParticlesCheck, 0);
			v2Layout->addWidget(sortParticlesCheck, 0);
			v2Layout->addStretch(1);
		topHLayout->addLayout(v3Layout, 0);
			v3Layout->addWidget(gridYZCheck);
			v3Layout->addWidget(gridXZCheck);
			v3Layout->addWidget(gridXYCheck);
			v3Layout->addWidget(unitSizeInput);
			v3Layout->addWidget(numUnitsInput);
			v3Layout->addStretch(1);
		topHLayout->addStretch(1);
}

RenderPanelGL::~RenderPanelGL()
{
}

//--------------------------------------------------------------------
void RenderPanelGL::show()
{
	glBlendCheck->setChecked(g_glBlend);
	glCullCheck->setChecked(g_glCullFace);
	glTwoSideLightCheck->setChecked(g_glTwoSideLight);
	
	warning("g_glBlend = %d", g_glBlend);
	QFrame::show();
}

//--------------------------------------------------------------------
void RenderPanelGL::setGLFillMode(bool status)
{
	g_camGLPort->makeCurrent();
	if(status)
	{
		GL__polygonMode = GL_LINE;
	}
	else
	{
		GL__polygonMode = GL_FILL;
	}
	glPolygonMode(GL_FRONT_AND_BACK, (GLenum)GL__polygonMode ); 
	emit Updated();
}

//--------------------------------------------------------------------
void RenderPanelGL::setGLCullFace(bool status)
{
	g_camGLPort->makeCurrent();
	if(status)
	{
		glEnable(GL_CULL_FACE);
	}
	else
	{
		glDisable(GL_CULL_FACE);
	}
	emit Updated();
}

//--------------------------------------------------------------------
void RenderPanelGL::setGLBlend(bool status)
{
	g_camGLPort->makeCurrent();
	if(status)
	{
		g_glBlend = true;
		glEnable(GL_BLEND);
	}
	else
	{
		g_glBlend = false;
		glDisable(GL_BLEND);
	}
	emit Updated();
}

//--------------------------------------------------------------------
void RenderPanelGL::setGLAlphaTest(bool status)
{
	g_camGLPort->makeCurrent();
	if(status)
	{
		g_glAlphaTest = true;
	}
	else
	{
		g_glAlphaTest = false;
	}
	emit Updated();
}


//--------------------------------------------------------------------
void RenderPanelGL::setTwoSideLighting(bool status)
{
	g_camGLPort->makeCurrent();
	g_glTwoSideLight = status;
	glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, g_glTwoSideLight );
	emit Updated();
}

//--------------------------------------------------------------------
void RenderPanelGL::setPointSmooth(bool status)
{
	g_camGLPort->makeCurrent();
	if (status)
		glEnable(GL_POINT_SMOOTH);
	else
		glDisable(GL_POINT_SMOOTH);
	emit Updated();
}

//--------------------------------------------------------------------
void RenderPanelGL::setLineSmooth(bool status)
{
	g_camGLPort->makeCurrent();
	if (status)
		glEnable(GL_LINE_SMOOTH);
	else
		glDisable(GL_LINE_SMOOTH);
	emit Updated();
}

void	RenderPanelGL::setYZCheck(bool in)
{	g_camGLPort->SetYZGrid(in);	emit Updated();	}
void	RenderPanelGL::setXZCheck(bool in)
{	g_camGLPort->SetXZGrid(in);	emit Updated();	}
void	RenderPanelGL::setXYCheck(bool in)
{	g_camGLPort->SetXYGrid(in);	emit Updated();	}

//--------------------------------------------------------------------
void RenderPanelGL::setUnitSize(float in)
{
	g_gridUnitSize = in;
	g_globalCam->SetGridUnitSize(g_gridUnitSize);
	g_camGLPort->RecreateGridDisplayList();
	//FIXME -- update ortho views
	emit Updated();
}

//--------------------------------------------------------------------
void RenderPanelGL::setNumUnits(int in)
{
	g_gridNumUnits = in;
	g_globalCam->SetGridNumUnits(g_gridNumUnits);
	g_camGLPort->RecreateGridDisplayList();
	//FIXME -- update ortho views
	emit Updated();
}

//--------------------------------------------------------------------
void	RenderPanelGL::setParticleSkip(int in)
{
	switch(in)
	{
		case 0:
			g_partSkip = 0;
			break;
		case 1:
			g_partSkip = 1;
			break;
		case 2:
			g_partSkip = 2;
			break;
		case 3:
			g_partSkip = 5;
			break;
		case 4:
			g_partSkip = 10;
			break;
		case 5:
			g_partSkip = 25;
			break;
		case 6:
			g_partSkip = 50;
			break;
	}
	emit Updated();
}


//--------------------------------------------------------------------
void	RenderPanelGL::setFastParticles(bool status)
{
	g_glFastParticles = status;
	emit Updated();

}

//--------------------------------------------------------------------
void	RenderPanelGL::setSortParticles(bool status)
{
	g_glSortParticles = status;
	emit Updated();
}


