#include "renderPanelGeneral.h"

#ifndef WIN32	
	#include <unistd.h>
	#include <sys/wait.h>
#endif

#include <ri.h>
#include <qvalidator.h>
#include <qapp.h>

#include "compatibility.h"
#include <GL/gl.h>

#include "extern.h"
#include "numInput.h"
#include "mbaScene.h"
#include "glPortCam.h"
#include "colorButton.h"

//static int checkHeight = 20;
//static int buttonHeight = 25;

//==============================================================================
RenderPanelGeneral::RenderPanelGeneral(QWidget *parent, const char* name)
	: QFrame (parent, name)
{
	setFrameStyle( QFrame::Sunken | QFrame::Panel );
	setLineWidth( 2 );
		
	//-- background color
	bkGndColorLabel = new QLabel(this);
	bkGndColorLabel->setText("background");
	bkGndColorLabel->setFont(*g_font_SB);
	bkGndColorLabel->setAlignment( AlignCenter );

	bkGndColorButton = new ColorButton (this);
	bkGndColorButton->setFixedWidth(g_numInputWidth);
	bkGndColorButton->setEnabled(true);
	bkGndColorButton->setColor(0.0, 0.0, 0.0);

	//-- camera
	fovInput = new NumInput(NumInput::Double, this);
	fovInput->SetLabelWidth(g_numLabelWidth);
	fovInput->SetTotalWidth(g_numInputWidth);
	fovInput->SetLabel("fov");
	fovInput->SetValueSolid(g_globalCam->fov());
	
	//----------------------------------
	topVLayout		= new QVBoxLayout(this, 10, 10);
	topHLayout		= new QHBoxLayout(10);
	v1Layout		= new QVBoxLayout(0);
	v2Layout		= new QVBoxLayout(0);
	v3Layout		= new QVBoxLayout(0);
		
	//----------------------------------
	QObject::connect(bkGndColorButton,	SIGNAL(RGBChanged(float, float, float)), SLOT(SetBkGndColor(float, float, float)) );
	QObject::connect(fovInput, SIGNAL(ValueChanged(float)), SLOT(SetCamFOV(float)) );

	//----------------------------------
	topVLayout->addLayout(topHLayout, 1);
		topHLayout->addLayout(v1Layout, 0);
			v1Layout->addWidget(bkGndColorLabel);
			v1Layout->addWidget(bkGndColorButton);
			v1Layout->addStretch(1);
			v1Layout->addWidget(fovInput);
		topHLayout->addLayout(v2Layout, 0);
			v2Layout->addStretch(1);
		topHLayout->addLayout(v3Layout, 0);
			v3Layout->addStretch(1);
		topHLayout->addStretch(1);
			
	topVLayout->activate();
}

RenderPanelGeneral::~RenderPanelGeneral()
{
}

//--------------------------------------------------------------------
void RenderPanelGeneral::show()
{
	fovInput->SetValueSolid(g_globalCam->fov());
	bkGndColorButton->setColor( *(g_scene->BackgroundColor()) );
	QFrame::show();
}

//--------------------------------------------------------------------
void RenderPanelGeneral::SetBkGndColor(float inR, float inG, float inB)
{
	if ( g_scene->BackgroundColor() )
	{
		g_scene->BackgroundColor()->SetRGB(inR, inG, inB);
	}
	else
	{
		GColor	*tmpColor;
		tmpColor = new GColor(inR, inG, inB);
		g_scene->SetBkGndColorPointer(tmpColor);
	}
	emit Updated();
}

//--------------------------------------------------------------------
void	RenderPanelGeneral::SetCamFOV(float in)
{
	g_camGLPort->makeCurrent();
	g_globalCam->SetFov(in);
	g_globalCam->ProjectGL();
	emit Updated();
}

