#include "renderPanelRIB.h"

#ifndef WIN32
	#include <unistd.h>
	#include <sys/wait.h>
	#include <sys/types.h>
	#include <signal.h>
#endif

#include <ri.h>
#include <qvalidator.h>
#include <qbuttongroup.h>
#include <qmessagebox.h>
#include <qtooltip.h>
#include <qapp.h>

#include "extern.h"
#include "particleControl.h"
#include "lightGroup.h"
#include "glPortCam.h"
#include "camera.h"
#include "numInput.h"
#include "renderman.h"
#include "mbaScene.h"
#include "renderRIB.h"

#include "mainFlow.h"
#include "lightGroupWidget.h"
#include "particleControlWidget.h"
#include "mbaSurfaceTabDialog.h"
#include "renderPanelTabDialog.h"
#include "mbaSceneWidget.h"

#include "pPolygonObject.h"

/**
 * default constructor
 */
//==============================================================================
RenderPanelRIB::RenderPanelRIB(QWidget *parent, const char* name)
	: QFrame (parent, name)
{
	setFrameStyle( QFrame::Sunken | QFrame::Panel );
	setLineWidth( 2 );
	
//	setFont(*g_font_MB);
	
	renderMode = RI_RGBA;
	
	startFrameInput = new NumInput(NumInput::Int, this);
	startFrameInput->SetLabelWidth(g_numLabelWidth);
	startFrameInput->SetTotalWidth(g_numInputWidth);
	startFrameInput->SetLabel("start frame");
	endFrameInput = new NumInput(NumInput::Int, this);
	endFrameInput->SetLabelWidth(g_numLabelWidth);
	endFrameInput->SetTotalWidth(g_numInputWidth);
	endFrameInput->SetLabel("end frame");
	
	sizeXInput = new NumInput(NumInput::Int, this);
	sizeXInput->SetLabelWidth(g_numLabelWidth);
	sizeXInput->SetTotalWidth(g_numInputWidth);
	sizeXInput->SetLabel("width");
	sizeYInput = new NumInput(NumInput::Int, this);
	sizeYInput->SetLabelWidth(g_numLabelWidth);
	sizeYInput->SetTotalWidth(g_numInputWidth);
	sizeYInput->SetLabel("height");
	sizeAspectInput = new NumInput(NumInput::Double, this);
	sizeAspectInput->SetLabelWidth(g_numLabelWidth);
	sizeAspectInput->SetTotalWidth(g_numInputWidth);
	sizeAspectInput->SetLabel("aspect");
	
	samplesXInput = new NumInput(NumInput::Int, this);
	samplesXInput->SetLabelWidth(g_numLabelWidth);
	samplesXInput->SetTotalWidth(g_numInputWidth);
	samplesXInput->SetLabel("x samples");
	samplesYInput = new NumInput(NumInput::Int, this);
	samplesYInput->SetLabelWidth(g_numLabelWidth);
	samplesYInput->SetTotalWidth(g_numInputWidth);
	samplesYInput->SetLabel("y samples");
	
	sidesInput = new NumInput(NumInput::Int, this);
	sidesInput->SetLabelWidth(g_numLabelWidth);
	sidesInput->SetTotalWidth(g_numInputWidth);
	sidesInput->SetLabel("sides");
	
	maxRaysInput = new NumInput(NumInput::Int, this);
	maxRaysInput->SetLabelWidth(g_numLabelWidth);
	maxRaysInput->SetTotalWidth(g_numInputWidth);
	maxRaysInput->SetLabel("max rays");
	
	shadeRateInput = new NumInput(NumInput::Double, this);
	shadeRateInput->SetLabelWidth(g_numLabelWidth);
	shadeRateInput->SetTotalWidth(g_numInputWidth);
	shadeRateInput->SetLabel("shade rate");
	
	rendermanOptions = new MLineEdit(this);
	rendermanOptions->setFixedHeight(20);
	rendermanOptions->setText(g_rendermanOpts);
	
	renderModeCombo = new QComboBox(this);
	renderModeCombo->setFixedHeight(g_comboHeight);
	renderModeCombo->setFixedWidth(g_numInputWidth);
	renderModeCombo->insertItem("  RGB  ");
	renderModeCombo->insertItem("  RGBA  ");
	renderModeCombo->insertItem("  RGBZ  ");
	renderModeCombo->insertItem("  RGBAZ  ");
	renderModeCombo->insertItem("  A  ");
	renderModeCombo->insertItem("  Z  ");
	renderModeCombo->insertItem("  AZ  ");
	renderModeCombo->setFont(*g_font_MB);
	
	allowShadowsCheck = new QCheckBox("allow shadows", this);
	allowDisplaceCheck = new QCheckBox("true displace", this);
	motionBlurCheck = new QCheckBox("motion blur", this);
	
	blurStepsCombo = new QComboBox(this);
	blurStepsCombo->setFixedHeight(g_comboHeight);
	blurStepsCombo->insertItem("1 step");
	blurStepsCombo->insertItem("2 steps");
	blurStepsCombo->insertItem("3 steps");
	blurStepsCombo->insertItem("4 steps");
	blurStepsCombo->insertItem("5 steps");
	blurStepsCombo->insertItem("6 steps");
	blurStepsCombo->insertItem("7 steps");
	blurStepsCombo->insertItem("8 steps");
	blurStepsCombo->insertItem("9 steps");
	
	if(g_toolTipsEnabled)
	{
		QToolTip::add(allowShadowsCheck, "global toggle for raytraced shadows");
		QToolTip::add(allowDisplaceCheck, "global toggle for BMRT's \'true displacement\'");
		QToolTip::add(motionBlurCheck, "toggle motion blur\n(currently works for particles only)");
	}
		
	//----------------------------------
	QObject::connect(startFrameInput,	SIGNAL(ValueChanged(int)), SLOT(setStartFrame(int)) );
	QObject::connect(endFrameInput,		SIGNAL(ValueChanged(int)), SLOT(setEndFrame(int)) );
	QObject::connect(sizeXInput,		SIGNAL(ValueChanged(int)), SLOT(setSizeX(int)) );
	QObject::connect(sizeYInput,		SIGNAL(ValueChanged(int)), SLOT(setSizeY(int)) );
	QObject::connect(sizeAspectInput,	SIGNAL(ValueChanged(float)), SLOT(setSizeAspect(float)) );
	QObject::connect(renderModeCombo,	SIGNAL(activated(int)), SLOT(setRenderMode(int)) );
	QObject::connect(samplesXInput,		SIGNAL(ValueChanged(float)), SLOT(setSamplesX(float)) );
	QObject::connect(samplesYInput,		SIGNAL(ValueChanged(float)), SLOT(setSamplesY(float)) );
	QObject::connect(sidesInput,		SIGNAL(ValueChanged(int)), SLOT(setSides(int)) );
	QObject::connect(maxRaysInput,		SIGNAL(ValueChanged(int)), SLOT(setMaxRays(int)) );
	QObject::connect(shadeRateInput,	SIGNAL(ValueChanged(float)), SLOT(setShadeRate(float)) );
	QObject::connect(allowShadowsCheck,	SIGNAL(toggled(bool)), SLOT(setAllowShadows(bool)) );
	QObject::connect(allowDisplaceCheck,SIGNAL(toggled(bool)), SLOT(setTrueDisplace(bool)) );
	QObject::connect(motionBlurCheck,	SIGNAL(toggled(bool)), SLOT(setMotionBlur(bool)) );
	QObject::connect(blurStepsCombo,	SIGNAL(activated(int)), SLOT(setNumBlurSteps(int)) );

	//----------------------------------
	topVLayout		= new QVBoxLayout(this, 10, 0);
	topHLayout		= new QHBoxLayout(10);
	v0Layout		= new QVBoxLayout(0);
	v1Layout		= new QVBoxLayout(0);
	v2Layout		= new QVBoxLayout(0);
//	v3Layout		= new QVBoxLayout(0);
	
//	topVLayout->addWidget(renderLabel, 0);
	topVLayout->addLayout(topHLayout, 1);
		topHLayout->addLayout(v0Layout, 0);
			v0Layout->addWidget(startFrameInput);
			v0Layout->addWidget(endFrameInput);
			v0Layout->addSpacing(5);
			v0Layout->addWidget(sizeXInput);
			v0Layout->addWidget(sizeYInput);
			v0Layout->addWidget(sizeAspectInput);
			v0Layout->addSpacing(5);
			v0Layout->addWidget(renderModeCombo);
			v0Layout->addStretch(1);
		topHLayout->addLayout(v1Layout, 0);
			v1Layout->addWidget(samplesXInput);
			v1Layout->addWidget(samplesYInput);
			v1Layout->addSpacing(5);
			v1Layout->addWidget(sidesInput);
			v1Layout->addWidget(maxRaysInput);
			v1Layout->addWidget(shadeRateInput);
			v1Layout->addStretch(1);
		topHLayout->addLayout(v2Layout, 1);
			v2Layout->addWidget(allowShadowsCheck);
			v2Layout->addWidget(allowDisplaceCheck);
			v2Layout->addStretch(1);
			v2Layout->addWidget(motionBlurCheck);
			v2Layout->addWidget(blurStepsCombo, 0);
//		topHLayout->addLayout(v3Layout, 0);
//			v3Layout->addStretch(1);
	rendermanOptions->hide();
	
	topVLayout->activate();
	
}

RenderPanelRIB::~RenderPanelRIB()
{
}

//--------------------------------------------------------------------
/** set global true displace flag */
void	RenderPanelRIB::setTrueDisplace(bool status)
{	g_allowTrueDisplace = status;	}

/** set global allow shadows flag */
void	RenderPanelRIB::setAllowShadows(bool status)
{	g_allowShadows = status;		}

/** set global enable motion blur flag */
void	RenderPanelRIB::setMotionBlur(bool status)
{	g_motionBlur = status;			}

/** set global number of blur steps flag */
void	RenderPanelRIB::setNumBlurSteps(int val)
{	g_numBlurSteps = val+1;			}

/** forwards info to RenderRIB::setStartFrame(int) */
void	RenderPanelRIB::setStartFrame(int f)
{	g_renderRIB->setStartFrame(f);		}
	
/** forwards info to RenderRIB::setEndFrame(int) */
void	RenderPanelRIB::setEndFrame(int f)
{	g_renderRIB->setEndFrame(f);		}

/**
 * before showing the widget, set all values to reflect current state.
 */
//------------------------------------------------------------------------------
void RenderPanelRIB::show()
{
	startFrameInput->SetValueSolid(g_renderRIB->StartFrame());
	endFrameInput->SetValueSolid(g_renderRIB->EndFrame());
	sidesInput->SetValueSolid(g_renderRIB->Sides());
	maxRaysInput->SetValueSolid(g_renderRIB->MaxRays());
	shadeRateInput->SetValueSolid(g_renderRIB->ShadeRate());
	sizeXInput->SetValueSolid(g_rendW);
	sizeYInput->SetValueSolid(g_rendH);
	sizeAspectInput->SetValueSolid(g_rendA);
	samplesXInput->SetValueSolid(g_rendSamples[0]);
	samplesYInput->SetValueSolid(g_rendSamples[1]);
	allowShadowsCheck->setChecked(g_allowShadows);
	allowDisplaceCheck->setChecked(g_allowTrueDisplace);
	motionBlurCheck->setChecked(g_motionBlur);
	if (!strcmp(g_renderRIB->RenderMode(), RI_RGB ))
		renderModeCombo->setCurrentItem(0);
	else if (!strcmp(g_renderRIB->RenderMode(), RI_RGBA ))
		renderModeCombo->setCurrentItem(1);
	else if (!strcmp(g_renderRIB->RenderMode(), RI_RGBZ ))
		renderModeCombo->setCurrentItem(2);
	else if (!strcmp(g_renderRIB->RenderMode(), RI_RGBAZ ))
		renderModeCombo->setCurrentItem(3);
	else if (!strcmp(g_renderRIB->RenderMode(), RI_A ))
		renderModeCombo->setCurrentItem(4);
	else if (!strcmp(g_renderRIB->RenderMode(), RI_Z ))
		renderModeCombo->setCurrentItem(5);
	else if (!strcmp(g_renderRIB->RenderMode(), RI_AZ ))
		renderModeCombo->setCurrentItem(6);
	QWidget::show();
}

//------------------------------------------------------------------------------
// resetting the numInput is redundant when being set from *this, but makes setting 
// values from other objects easier.
/** sets global RenderMan width and updates num input widget */
void	RenderPanelRIB::setSizeX(int in)
{
	g_rendW = in;
	sizeXInput->SetValueSolid(g_rendW);
}
/** sets global RenderMan height and updates num input widget */
void	RenderPanelRIB::setSizeY(int in)
{
	g_rendH = in;
	sizeYInput->SetValueSolid(g_rendH);
}
/** sets global RenderMan aspect ratio and updates num input widget */
void	RenderPanelRIB::setSizeAspect(float in)
{
	g_rendA = in;
	sizeAspectInput->SetValueSolid(g_rendA);
}
/** sets global RenderMan X samples and updates num input widget */
void	RenderPanelRIB::setSamplesX(float in)
{
	g_rendSamples[0] = in;
	samplesXInput->SetValueSolid(g_rendSamples[0]);
}
/** sets global RenderMan Y samples and updates num input widget */
void	RenderPanelRIB::setSamplesY(float in)
{
	g_rendSamples[1] = in;
	samplesYInput->SetValueSolid(g_rendSamples[1]);
}

/** sets RenderRIB::setSides(int) and updates num input widget */
//------------------------------------------------------------------------------
void	RenderPanelRIB::setSides(int in)
{
	int	val = g_renderRIB->setSides(in);
	if (in != val)
	{
		warning("in = %d...val = %d", in, val);
		sidesInput->SetValueSolid((float)val);
	}
}

/** sets RenderRIB::setMaxRays(int) and updates num input widget */
//------------------------------------------------------------------------------
void	RenderPanelRIB::setMaxRays(int in)
{
	int	val = g_renderRIB->setMaxRays(in);
	if (in != val)
		maxRaysInput->SetValueSolid(val);
}

/** sets RenderRIB::setShadeRate(int) and updates num input widget */
//------------------------------------------------------------------------------
void	RenderPanelRIB::setShadeRate(float in)
{
	float	val = g_renderRIB->setShadeRate(in);
	if (in != val)
		shadeRateInput->SetValueSolid(val);
}

/** sets RenderRIB::setRenderMode(int). (i.e. RI_RGB, RI_RGBA, etc.) */
//------------------------------------------------------------------------------
void	RenderPanelRIB::setRenderMode(int in)
{
	switch(in)
	{
		case 0:
			renderMode = RI_RGB;
			g_renderRIB->setRenderMode( RI_RGB );
			break;
		case 1:
			renderMode = RI_RGBA;
			g_renderRIB->setRenderMode( RI_RGBA );
			break;
		case 2:
			renderMode = RI_RGBZ;
			g_renderRIB->setRenderMode( RI_RGBZ );
			break;
		case 3:
			renderMode = RI_RGBAZ;
			g_renderRIB->setRenderMode( RI_RGBAZ );
			break;
		case 4:
			renderMode = RI_A;
			g_renderRIB->setRenderMode( RI_A );
			break;
		case 5:
			renderMode = RI_Z;
			g_renderRIB->setRenderMode( RI_Z );
			break;
		case 6:
			renderMode = RI_AZ;
			g_renderRIB->setRenderMode( RI_AZ );
			break;
	}
}

//------------------------------------------------------------------------------
void	RenderPanelRIB::RenderCurrentFrame()
{
	g_renderRIB->RenderCurrentFrame();
}

//------------------------------------------------------------------------------
void	RenderPanelRIB::RenderAnimation()
{
	g_camGLPort->SetAnimation(false);
	g_renderRIB->RenderAnimation();
	g_mainFlow->iconify();
//	g_lightGroupWidgetFrame->iconify();
//	g_particleControlFrame->iconify();
//	g_surfaceEditWidget->iconify();
//	g_renderOptions->iconify();
//	g_sceneWidget->iconify();
}









