#include "renderPanelTabDialog.h"
#include "extern.h"
#include "define.h"

#include <qkeycode.h>
#include <qtooltip.h>

#include "Pixmaps/but_rib_s.xpm"
//#include "Pixmaps/flowIcon.xpm"

//==================================================================================================
RenderPanelTabDialog::RenderPanelTabDialog(QWidget *parent, const char* name, ToolBarDock buttonBarDock)
	: MbaMainWindow(parent, name)
{
	setCaption("flow::Render Properties");
	setIcon(QPixmap((const char**)flowIcon_xpm));
//	setFont(*g_font_MB);
		
	renderPanelGeneral	= new RenderPanelGeneral;
	renderPanelGL		= new RenderPanelGL;
	renderPanelRIB		= new RenderPanelRIB;
	renderPanelHosts	= new RenderPanelHosts;
	renderPanelCode		= new RenderPanelCode;
//	renderPanelMPEG		= new RenderPanelMPEG;
	
	tabDialog = new MTabWidget(this);
//	tabDialog->setOkButton(0);
	tabDialog->setFont(*g_font_MB);
	tabDialog->addTab(renderPanelGeneral,	"&general");
	tabDialog->addTab(renderPanelGL,		"&GL");
	tabDialog->addTab(renderPanelRIB,		"&render");
	tabDialog->addTab(renderPanelHosts,		"&hosts");
	tabDialog->addTab(renderPanelCode,		"&code");
//	tabDialog->addTab(renderPanelMPEG,		"&mpeg");

	setCentralWidget(tabDialog);
	setRightJustification(true);
	
	manipToolBar = new QToolBar(0, this, buttonBarDock, true);
	dockList.append(new QMainWindow::ToolBarDock(buttonBarDock));

	if(g_winStyleToolBars)
	{
		manipToolBar->setStyle(winStyle);
	}
	
	closeBut = new MToolButton(manipToolBar);
	closeBut->setAccel(ALT+'Q');
	closeBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	closeBut->setUsesBigPixmap(true);
	closeBut->setIconSet(*icoSet_close);
	
	MLabel *stretch;
	stretch = new MLabel(manipToolBar);
	stretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	
	aniBut = new QPushButton(manipToolBar);
	aniBut->setAccel(CTRL+'V');
//	aniBut->setToggleButton(true);
	aniBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
//	aniBut->setUsesBigPixmap(true);
//	aniBut->setPixmap(QPixmap((const char**)but_ani_xpm ));
	aniBut->setText("ani");
	aniBut->setPixmap(QPixmap((const char**)but_rib_s_xpm ));
	
	stretch = new MLabel(manipToolBar);
	stretch->setMinimumSize(ICON_SPACER, ICON_SPACER);
	
	frameBut = new MToolButton(manipToolBar);
	frameBut->setAccel('R');
	frameBut->setFixedSize(TOOL_SIZE, TOOL_SIZE);
	frameBut->setUsesBigPixmap(true);
	frameBut->setIconSet(*icoSet_rm);
	
	stretch = new MLabel(manipToolBar);
	manipToolBar->setStretchableWidget(stretch);
	
	if(g_toolTipsEnabled)
	{
		QToolTip::add(closeBut, "close (alt-Q)");
		QToolTip::add(aniBut, "render animation (alt-A)");
		QToolTip::add(frameBut, "render single frame (R)");
	}
	
	//----------------------------------
	QObject::connect(closeBut,	SIGNAL(clicked()),	SLOT(hide()) );
	QObject::connect(frameBut,	SIGNAL(clicked()),	SLOT(RenderCurrentFrame()) );
	QObject::connect(aniBut,	SIGNAL(clicked()),	SLOT(RenderAnimation()) );
	
	QObject::connect(renderPanelGL,		 SIGNAL(Updated()),	SIGNAL(Updated()) );
	QObject::connect(renderPanelGeneral, SIGNAL(Updated()),	SIGNAL(Updated()) );
	
	//----------------------------------
	setDockEnabled( QMainWindow::TornOff, true);
	setDockEnabled( QMainWindow::Top	, true);
	setDockEnabled( QMainWindow::Bottom	, true); 
	setDockEnabled( QMainWindow::Right	, true);
	setDockEnabled( QMainWindow::Left	, true);
	
}

RenderPanelTabDialog::~RenderPanelTabDialog()
{
}

//--------------------------------------------------------------------
QToolBar*	RenderPanelTabDialog::GetToolBar(int toolBarNum)
{
	switch(toolBarNum)
	{
		case 0:
			return manipToolBar;
		default:
			return 0;
	}
}

//----------------------------------------------------------
bool RenderPanelTabDialog::setToolBar(int num, QMainWindow::ToolBarDock dock, bool newline)
{
	switch(num)
	{
		case 0:
			removeToolBar(manipToolBar);
			addToolBar(manipToolBar, 0, dock, newline);
			*dockList.at(0) = dock;
			return true;
		default:
			return false;
	}
}

//--------------------------------------------------------------------
void	RenderPanelTabDialog::RenderCurrentFrame()
{
	renderPanelRIB->RenderCurrentFrame();
}

//--------------------------------------------------------------------
void	RenderPanelTabDialog::RenderAnimation()
{
	renderPanelRIB->RenderAnimation();
}




