#include "renderRIB.h"

#ifndef WIN32
	#include <unistd.h>
	#include <sys/wait.h>
	#include <sys/types.h>
	#include <signal.h>
#endif
#include <ri.h>
#include <stdio.h>
#include <stdlib.h>

#include <qapplication.h>
#include <qprogressdialog.h>

#include "extern.h"
#include "mbaScene.h"
#include "camera.h"
#include "lightGroup.h"
#include "glPortCam.h"
#include "forkVector.h"
#include "mainFlow.h"

//==============================================================================
RenderRIB::RenderRIB()
{
	int i;
	startFrame	= 0;
	endFrame	= 1000;
	sides		= 2;
	maxRays		= 5;
	shadeRate	= 1;
	renderMode	= RI_RGBA;
	
	progressDialog		= 0;
	continueRendering	= false;
	
	for (i = 0; i < NUM_RENDER_PROCS; i++)
		localHostList.append(new bool(false));
		
	for (i = 0; i < NUM_RENDER_PROCS; i++)
		pid_wait[i] = 0;
	for (i = 0; i < NUM_PREVIEW_PROCS; i++)
		pid_preview_wait[i] = 0;
	
	QObject::connect(&checkPIDTimer, SIGNAL(timeout()), SLOT(WaitForNextFrame()) );
}

RenderRIB::~RenderRIB()
{
#ifndef WIN32
	// one last check before we go...
	int i;
	int	status;
	for (i = 0; i < NUM_PREVIEW_PROCS; i++)
	{
		if (pid_preview_wait[i])	// check for any g_rendermanExec zombies...
		{
			if (g_killChildrenOnExit)
			{
				kill(pid_preview_wait[i], 15);
				fprintf(stderr, "---- waiting for child render process (pid %d) to die...", pid_preview_wait[i]);
				status = waitpid(pid_preview_wait[i], (int*)0, 0); // block
				fprintf(stderr, "OK.\n");
			}
			else
				status = waitpid(pid_preview_wait[i], (int*)0, WNOHANG);
			if (status > 0)
				pid_preview_wait[i] = 0;
			else if (status < 0)
				warning ("ERROR checking child PID in RenderPanelRIB::~RenderPanelRIB()");
		}
	}
	for (i = 0; i < NUM_RENDER_PROCS; i++)
	{
		if (pid_wait[i])	// check for any g_rendermanExec zombies...
		{
			if (g_killChildrenOnExit)
			{
				kill(pid_wait[i], 15);
				fprintf(stderr, "---- waiting for child render process (pid %d) to die...", pid_wait[i]);
				status = waitpid(pid_wait[i], (int*)0, 0); // block
				fprintf(stderr, "OK.\n");
			}
			else
				status = waitpid(pid_wait[i], (int*)0, WNOHANG);
			if (status > 0)
				pid_wait[i] = 0;
			else if (status < 0)
				warning ("ERROR checking child PID in RenderPanelRIB::~RenderPanelRIB()");
		}
	}
	warning("------------------------------------------------------------");
#endif
}

/**
 * set frame number to begin rendering
 */
//------------------------------------------------------------------------------
int		RenderRIB::setStartFrame(int in)	
{ 
	if (in >= 0) 
		startFrame = in;
	return startFrame;	
}

/**
 * set last frame of animation
 */
//------------------------------------------------------------------------------
int		RenderRIB::setEndFrame(int in)		
{ 
	if (in >= 0) 
		endFrame = in;	
	if (continueRendering && progressDialog)
	{
		fdiff = endFrame-startFrame;
		progressDialog->setTotalSteps(fdiff);
	}
	return endFrame;	
}

/**
 * mark local processor as available for rendering
 */ 
//------------------------------------------------------------------------------
void	RenderRIB::setLocalHostOn(int index, bool state)
{
	if (index < NUM_RENDER_PROCS)
	{
		*(localHostList.at(index)) = state;
		//warning("process %d set %d", index, state);
	}
}

//------------------------------------------------------------------------------
bool	RenderRIB::RenderCurrentFrame()
{
	bool	rendered = false;
#ifndef WIN32
	int		waitstatus;
	int		waitcheck;
	int		i;
	
	for (i = 0; i < NUM_PREVIEW_PROCS; i++)
	{
		if (pid_preview_wait[i])
		{
			waitcheck = waitpid (pid_preview_wait[i], &waitstatus, WNOHANG);
			if (waitcheck == pid_preview_wait[i])
				pid_preview_wait[i] = 0;
		}
		if ( rendered == false && pid_preview_wait[i] == 0 )
		{
			RenderSceneFrame(g_frame, i, true);
			rendered = true;
		}
	}
#endif
	return rendered;
}

//------------------------------------------------------------------------------
bool	RenderRIB::RenderAnimation()
{
	continueRendering = !continueRendering;
	if(continueRendering)
	{
		g_frame = 0;
		g_camGLPort->ResetCamList();
		g_scene->GotoFrame(startFrame);
		fcurrent = 0;
		fdiff = endFrame-startFrame;
//		progressDialog = new QProgressDialog("Rendering frame...", "Cancel", fdiff, 0, "renderProgress", true );
		progressDialog = new QProgressDialog("rendering frame...", "&cancel", fdiff, 0, "renderProgress", false );
		progressDialog->setFont(*g_font_MB);
		progressDialog->setIcon(QPixmap((const char**)flowIcon_xpm));
		progressDialog->setIconText("flow");
		progressDialog->setCaption ("flow");
		progressDialog->setMinimumDuration(0);
		progressDialog->setTotalSteps(fdiff);
		progressDialog->show();
		progressDialog->setProgress(0);
		checkPIDTimer.start(400);
	}
	else
	{
		checkPIDTimer.stop();
		EndProgress();
	}
	
	return continueRendering;
}

//------------------------------------------------------------------------------
int		RenderRIB::UpdateProgress()
{
	char labelText[64];
	char iconText[64];
	sprintf(labelText, "rendering frame...%d", fcurrent+startFrame);
	sprintf(iconText,  "flow %d", fcurrent+startFrame);
	progressDialog->setIconText(iconText);
	progressDialog->setCaption (iconText);
	progressDialog->setLabelText( labelText );
	progressDialog->setProgress(fcurrent++);
	return fcurrent;
}

//------------------------------------------------------------------------------
void	RenderRIB::EndProgress()
{
	if(progressDialog)
	{
		progressDialog->setProgress(fdiff);
		delete progressDialog;
		progressDialog = 0;
		g_mainFlow->show();
		g_mainFlow->raise();
	}
}

//------------------------------------------------------------------------------
void	RenderRIB::WaitForNextFrame()
{
#ifndef WIN32
	int		i;
	int		waitcheck;
	int		waitstatus;
	bool	renderInProgress = false;
	
	if (progressDialog->wasCancelled())
		continueRendering = false;
	
	fprintf(stderr, "RenderRIB::WaitForNextFrame()\n");
	for (i = 0; i < NUM_RENDER_PROCS; i++)
	{
		if (pid_wait[i])						//process has been started & needs to be checked
		{
			waitcheck = waitpid (pid_wait[i], &waitstatus, WNOHANG);
			if (waitcheck == 0)					//still going...
				renderInProgress = true;
			else if	(waitcheck == pid_wait[i])	//process finished
			{
				pid_wait[i] = 0;
				if ( continueRendering && *(localHostList.at(i)))	//if button is down, send another RIB to this host
				{
					renderInProgress = true;
					if ( g_frame >= startFrame )
						RenderSceneFrame(g_frame, i);
					g_camGLPort->updateGL();
					g_frame++;
					UpdateProgress();
					
					if ( g_frame >= endFrame )
					{
						continueRendering = false;
					}
				}
			}
			else if	(waitcheck < 0)				//ERROR - check waitstatus
				warning("ERROR: waitcheck < 0"); 
			else
				warning("ERROR: waitcheck returned %d, pid_wait[%d] = %d", waitcheck, i, pid_wait[i]);
		}
		else if ( continueRendering && *(localHostList.at(i)))	//new process added...
		{
			renderInProgress = true;
			if ( g_frame >= startFrame )
				RenderSceneFrame(g_frame, i);
			g_camGLPort->updateGL();
			g_frame++;
			UpdateProgress();
		}
	}
	
	if (!renderInProgress)
	{
		checkPIDTimer.stop();	
		EndProgress();
	}
#endif
}

//------------------------------------------------------------------------------
int	RenderRIB::RenderSceneFrame(int frame, int process, bool framebuffer)
{
#ifndef WIN32
//	RtToken	renderMode = RI_RGBA;
	char	filename[128];
	char	ribname[128];
	
	sprintf (filename,	"TIF/%s%04d.tif", g_rendName, frame);
	sprintf (ribname,	"RIB/rib%02d.rib", process);
	
	int	available = -1;
	if (framebuffer)
	{
		int i;
		int	status;
		for (i = 0; i < NUM_PREVIEW_PROCS; i++)
		{
			if (pid_preview_wait[i])	// check for any g_rendermanExec zombies...
			{
				status = waitpid(pid_preview_wait[i], (int*)0, WNOHANG);
				if (status > 0)
					pid_preview_wait[i] = 0;
				else if (status < 0)
					warning ("ERROR checking child PID in RenderSceneFrame");
			}
			if (!pid_preview_wait[i])
				available = i;
		}
		
		if (available < 0)
		{
//			QCString	tmpMssg;
//			tmpMssg.sprintf("PID table for previews is full (%d allowed).\nClose some preview windows and\ntry again.", NUM_PREVIEW_PROCS);
//			QMessageBox::warning(0, 0, tmpMssg, "Ok");
			return PreviewFull;
		}
		else
			sprintf(ribname, "RIB/sceneFrame%03d.rib", available);
	}
	
	FILE	*testRibname;
	testRibname = fopen(ribname, "w");
	if (!testRibname)
	{
//		QCString	tmpMssg;
//		tmpMssg.sprintf("ERROR!\n\nCould not open\n\"%s\"\nfor write.\n\nCheck file permissions and disk space.", ribname);
//		QMessageBox::critical(0, "f l o w", tmpMssg, "Ok");
		return WriteError;
	}
	fclose(testRibname);
	warning("\n------------------------------------------------------------");
	warning("---- RenderRIB::RenderSceneFrame(%d, %d, %d);  RiBegin( \"%s\" );", frame, process, framebuffer, ribname);
	RiBegin(ribname);

	if(g_motionBlur)
		RiShutter(0.0, 1.0);

	RiOption("render", "max_raylevel", &maxRays, RI_NULL );
	RiPixelSamples (g_rendSamples[0], g_rendSamples[1]);
	RiFormat (g_rendW, g_rendH, g_rendA);
	if (framebuffer)
		RiDisplay (filename, RI_FRAMEBUFFER, renderMode, RI_NULL);
	else
		RiDisplay (filename, RI_FILE, renderMode, RI_NULL);
	RiSides(sides);
	
	RiCropWindow(g_rendCropLeft, g_rendCropRight, g_rendCropTop, g_rendCropBot);
	
	RiShadingRate(shadeRate);

	if (g_scene->BackgroundColor())
	{
		RiDeclare ("bgcolor", "uniform color" );
		RtColor	bgColor;
		bgColor[0] = g_scene->BackgroundColor()->R();
		bgColor[1] = g_scene->BackgroundColor()->G();
		bgColor[2] = g_scene->BackgroundColor()->B();
		if( (bgColor[0]+bgColor[1]+bgColor[2]) > 0.01)
			RiImager("background", "bgcolor", bgColor, RI_NULL);
	}
		
	g_globalCam->ProjectRIB();
	g_globalCam->PlaceCamRIB();

	RiWorldBegin();
	
	int i;
	if(g_scene->ActiveLightGroup())
		g_scene->ActiveLightGroup()->illuminateRIB(frame);
	//FIXME -  Stepping through the particle systems is the responsibililty of 
	//FIXME -  scene, not this. Move it.
	for (i = 0; i < g_scene->numParticleSystems(); i++ )
		g_scene->ParticleSystemAt(i)->RenderParticles();

	RiWorldEnd();
	RiEnd();

	if(g_hasRendermanExec)
	{
		ForkVector	forkVector;
		qApp->syncX();	//XXX Do I need these? Ask on the Qt interest list...
		qApp->flushX();
		if (framebuffer)
		{
			if( (pid_preview_wait[available] = fork()) == 0 )
			{
				QApplication::exit(0);
				execv( g_rendermanExec,  forkVector.createRenderVector((const char*)ribname, true));
			}
			if( pid_preview_wait[available] < 0)
			{
				warning ("FORK ERROR (pid_preview_wait[%d] = %d)", available, pid_preview_wait[available] );
				return ForkError;
			}
		}
		else
		{
			if( (pid_wait[process] = fork()) == 0 )
			{
				QApplication::exit(0);
				execv( g_rendermanExec,  forkVector.createRenderVector((const char*)ribname, false));
			}
			if( pid_wait[process] < 0)
			{
				warning ("FORK ERROR (pid_wait[%d] = %d)", process, pid_wait[process] );
				return ForkError;
			}
		}
	}
#else
	frame		= frame;
	process		= process;
	framebuffer	= framebuffer;
#endif
	return Ok;
}






