#include "compatibility.h"
#include <particle/general.h>
#include <ri.h>
#include <GL/gl.h>
#include "mmath.h"

#include "pPolygonObject.h"
#include "particleControl.h"
#include "camera.h"

extern _ParticleState _ps;
extern	CameraView		*g_globalCam;

static	char	*radioNone		= "none";
//static	char	*radioReceive	= "zonal_receives";
//static	char	*radioShoots 	= "zonal_shoots";
static	char	*radioFull 		= "fully_zonal";

//--------------------------------------------------------------------
void pRenderGroup(int inPrim, PPolygonObject* polygons)
{
	// Get a pointer to the particles in gp memory
	float	*clr;
	int		primitive;
	
	ParticleGroup *pg = _ps.pgrp;
	
	if(pg == NULL)
		return; // ERROR
		
	if(pg->p_count < 1)
		return;
	
	RiAttribute("radiosity", "zonal", &radioNone, RI_NULL);
	
//	RiSolidBegin(RI_UNION);
	for(int i = 0; i < pg->p_count; i++)
	{
		Particle &m = pg->list[i];
		primitive = inPrim;
		
		clr = (float*)&m.color;
		RtColor rOpacity = { clr[3], clr[3], clr[3] };
		RtColor rColor   = { clr[0], clr[1], clr[2] };
		RiOpacity(rOpacity);
		RiColor(rColor);

		float	tail[3];
		float	len;
		float	pt1[3];
		float	pt2[3];
		float	r;
		float		xzlen, yzlen;
		float		yrot, xrot;
		float		dir[3] = {0.0, 0.0, 0.0};
		float		rot[3] = {0.0, 0.0, 0.0};
		RiTransformBegin();
		
//		RiSolidBegin(RI_PRIMITIVE);
			RiTranslate(m.pos.x, m.pos.y, -m.pos.z);
			if ( primitive == ParticleControl::Line && m.vel.length2() < 0.00001)
				primitive = ParticleControl::Point;
			switch(primitive)
			{
				case ParticleControl::Point:
					RiSphere(m.size.x, -m.size.x, m.size.x, 360.0, RI_NULL);
					break;
				case ParticleControl::Line:
					{
						pVector vN(m.vel);
						vN.normalize();
						vN.z = -vN.z;
						//pVector voN(m.velB);
						//voN.normalize();
						//voN.z = -voN.z;
						pVector voN(0.0, 1.0, 0.0);

						pVector biN;
						if(voN.x == vN.x && voN.y == vN.y && voN.z == fabs(vN.z))
						{
							biN = pVector(0.0, 1.0, 0.0);
						}
						else
							biN = vN ^ voN;
						biN.normalize();

						pVector N;
						if(vN.y > 0.99)
						{
							fprintf(stderr, "(vN.y > 0.99)\n");
							fprintf(stderr, "biN(%f %f %f)\n", biN.x, biN.y, biN.z);
							fprintf(stderr, " vN(%f %f %f)\n", vN.x, vN.y, vN.z);
							fprintf(stderr, "voN(%f %f %f)\n", voN.x, voN.y, voN.z);
							N = pVector(0.0, 1.0, 0.0);
						}
						else
						{
							N = vN ^ biN;
						}

						float M[4][4];
						M[0][0] = vN.x;  M[1][0] = biN.x;  M[2][0] = N.x;  M[3][0] = 0;
						M[0][1] = vN.y;  M[1][1] = biN.y;  M[2][1] = N.y;  M[3][1] = 0;
						M[0][2] = vN.z;  M[1][2] = biN.z;  M[2][2] = N.z;  M[3][2] = 0;
						M[0][3] = 0;	 M[1][3] = 0;	   M[2][3] = 0;    M[3][3] = 1;
						RiConcatTransform(M);
					}
					tail[0] = m.vel.x*m.size.y;
					tail[1] = m.vel.y*m.size.y;
					tail[2] = m.vel.z*m.size.y;
					len = sqrt( tail[0]*tail[0] + tail[1]*tail[1] + tail[2]*tail[2] );
					
					r = m.size.x;
					pt1[0] = 0.0;
					pt1[1] = r;
					pt1[2] = 0.0;
					pt2[0] = 0.0;
					pt2[1] = r;
					pt2[2] = len;

					RiRotate( 90.0, 0.0, 1.0, 0.0 );
					RiSphere( r, -r, 0.0, 360.0, RI_NULL);
					RiHyperboloid( pt1, pt2, 360.0, RI_NULL);
					RiTranslate( 0.0, 0.0, len);
					RiSphere( r, 0.0, r, 360.0, RI_NULL);
					break;
				case ParticleControl::Disk:
					dir[0] =   (g_globalCam->vals->position.X()-m.pos.x);
					dir[1] =  -(g_globalCam->vals->position.Y()-m.pos.y);
					dir[2] =   (g_globalCam->vals->position.Z()-m.pos.z);
//					dir[0] =   m.vel.x;
//					dir[1] =  -m.vel.y;
//					dir[2] =   m.vel.z;
				if( dir[0]+dir[1]+dir[2] )
					{
						xzlen = sqrt(dir[0]*dir[0]+dir[2]*dir[2]);
						if (xzlen == 0)
							yrot = (dir[1]<0.0) ? 180.0 : 0.0;
						else
							yrot = 180.0*acos(dir[2]/xzlen)/M_PI;

						yzlen = sqrt(dir[1]*dir[1]+xzlen*xzlen);
						if (yzlen == 0.0)
							yzlen = 0.0000001;
						xrot = 180.0*acos(xzlen/yzlen)/M_PI;

						if (dir[1] > 0.0)
							rot[0] =  xrot;
						else
							rot[0] = -xrot;
						if (dir[0] > 0.0)
							rot[1] = -yrot;
						else
							rot[1] =  yrot;
					}
					RiRotate(180.0+rot[1], 0.0, 1.0, 0.0);
					RiRotate(rot[0], 1.0, 0.0, 0.0);
					RiDisk(0.0, m.size.x, 360.0, RI_NULL);
					break;
				case ParticleControl::Polygon:
					if (polygons)
					{
						PPolygon	*poly = 0;
						int polyIndex = (i % polygons->NumPolygons());
						poly = polygons->PolygonAt(polyIndex);
						poly->drawRIB(-poly->centroid.X(), -poly->centroid.Y(), -poly->centroid.Z());
					}
					break;
				
			}
//		RiSolidEnd();
		RiTransformEnd();
	}
//	RiSolidEnd();
	RiAttribute("radiosity", "zonal", &radioFull, RI_NULL);
}

//--------------------------------------------------------------------
void pRenderGroupBlur(int inPrim, int cur_handle, ParticleControl &nextTimePC, PPolygonObject* polygons )
{
	// Get a pointer to the particles in gp memory
	float	*clr;
	int		primitive;
	
	pCurrentGroup(cur_handle);
	ParticleGroup *pg = _ps.pgrp;
			
	if(pg == NULL)
		return; // ERROR
		
	if(pg->p_count < 1)
		return;
	
	pCurrentGroup(nextTimePC.getParticleHandle());
	ParticleGroup *pgts = _ps.pgrp;
	if(pgts == NULL)
	{
		pRenderGroup(inPrim, polygons);
		return;
	}
	
	
	RiAttribute("radiosity", "zonal", &radioNone, RI_NULL);
	
	float	r;
	float	len;
	float	yrot, xrot;
	float	xzlen, yzlen;
	float	pt1[3]	= {0.0, 0.0, 0.0};
	float	pt2[3]	= {0.0, 0.0, 0.0};
	float	dir[3]	= {0.0, 0.0, 0.0};
	float	rot[3]	= {0.0, 0.0, 0.0};
	float	tail[3]	= {0.0, 0.0, 0.0};
	float	M[4][4];

	for(int i = 0; i < pg->p_count; i++)
	{
		Particle &m = pg->list[i];
		Particle &mts = pgts->list[i];
		
		clr = (float*)&m.color;
		RtColor rOpacity = { clr[3], clr[3], clr[3] };
		RtColor rColor   = { clr[0], clr[1], clr[2] };
		RiOpacity(rOpacity);
		RiColor(rColor);

		primitive = inPrim;
		if ( primitive == ParticleControl::Line && m.vel.length2() < 0.00001)
			primitive = ParticleControl::Point;
			
		RiTransformBegin();
			RiMotionBegin(2, 0.0, 1.0);
			RiTranslate(m.pos.x, m.pos.y, -m.pos.z);
			RiTranslate(mts.pos.x, mts.pos.y, -mts.pos.z);
			RiMotionEnd();
			
			switch(primitive)
			{
				case ParticleControl::Point:
					RiSphere(m.size.x, -m.size.x, m.size.x, 360.0, RI_NULL);
					break;
				case ParticleControl::Line:
					{
						pVector vN(m.vel);
						vN.normalize();
						vN.z = -vN.z;
//						pVector voN(m.velB);
						pVector voN(0, 1, 0);
						voN.normalize();
						voN.z = -voN.z;
						pVector biN;
						if(voN.x == vN.x && voN.y == vN.y && voN.z == vN.z)
							biN = pVector(0, 1, 0);
						else
							biN = vN ^ voN;
						biN.normalize();
						pVector N(vN ^ biN);
						M[0][0] = vN.x;  M[1][0] = biN.x;  M[2][0] = N.x;  M[3][0] = 0;
						M[0][1] = vN.y;  M[1][1] = biN.y;  M[2][1] = N.y;  M[3][1] = 0;
						M[0][2] = vN.z;  M[1][2] = biN.z;  M[2][2] = N.z;  M[3][2] = 0;
						M[0][3] = 0;	 M[1][3] = 0;	   M[2][3] = 0;    M[3][3] = 1;
						RiConcatTransform(M);
					}
					tail[0] = m.vel.x;	tail[1] = m.vel.y;	tail[2] = m.vel.z;
					len = m.size.y * sqrt( tail[0]*tail[0] + tail[1]*tail[1] + tail[2]*tail[2] );
					
					r = m.size.x;
					pt1[0] = 0.0;
					pt1[1] = r;
					pt1[2] = 0.0;
					pt2[0] = 0.0;
					pt2[1] = r;
					pt2[2] = len;

					RiRotate( 90.0, 0.0, 1.0, 0.0 );
					RiSphere( r, -r, 0.0, 360.0, RI_NULL);
					RiHyperboloid( pt1, pt2, 360.0, RI_NULL);
					RiTranslate( 0.0, 0.0, len);
					RiSphere( r, 0.0, r, 360.0, RI_NULL);
					break;
				case ParticleControl::Disk:
					dir[0] =   (g_globalCam->vals->position.X()-m.pos.x);
					dir[1] =  -(g_globalCam->vals->position.Y()-m.pos.y);
					dir[2] =   (g_globalCam->vals->position.Z()-m.pos.z);
//					dir[0] =   m.vel.x;
//					dir[1] =  -m.vel.y;
//					dir[2] =   m.vel.z;
				if( dir[0]+dir[1]+dir[2] )
					{
						xzlen = sqrt(dir[0]*dir[0]+dir[2]*dir[2]);
						if (xzlen == 0)	yrot = (dir[1]<0.0) ? 180.0 : 0.0;
						else			yrot = 180.0*acos(dir[2]/xzlen)/M_PI;

						yzlen = sqrt(dir[1]*dir[1]+xzlen*xzlen);
						if (yzlen == 0.0)	
							yzlen = 0.0000001;
						xrot = 180.0*acos(xzlen/yzlen)/M_PI;

						if (dir[1] > 0.0)	rot[0] =  xrot;
						else				rot[0] = -xrot;
						
						if (dir[0] > 0.0)	rot[1] = -yrot;
						else				rot[1] =  yrot;
					}
					RiRotate(180.0+rot[1], 0.0, 1.0, 0.0);
					RiRotate(rot[0], 1.0, 0.0, 0.0);
					RiDisk(0.0, m.size.x, 360.0, RI_NULL);
					break;
				case ParticleControl::Polygon:
					if (polygons)
					{
						PPolygon	*poly = 0;
						int polyIndex = (i % polygons->NumPolygons());
						poly = polygons->PolygonAt(polyIndex);
						poly->drawRIB(-poly->centroid.X(), -poly->centroid.Y(), -poly->centroid.Z());
					}
					break;
				
			}
		RiTransformEnd();
	}
	RiAttribute("radiosity", "zonal", &radioFull, RI_NULL);
	
	pCurrentGroup(cur_handle);	// reset current group
}
