#ifndef shaderPreviewList_h
#define shaderPreviewList_h

#include <qtablevw.h>
#include <qpixmap.h>
#include <qlist.h>
#include <qtimer.h>
#include <qstrlist.h>

/**
 * Saved shader variation view widget.
 */
class ShaderPreview : public QTableView
{
    Q_OBJECT
public:
    ShaderPreview(int numCols, QWidget* parent=0, const char* name=0 );
    ~ShaderPreview();
    
	void		changeNumCols(int);
    QPixmap&	cellContent( int row, int col );
    void 		setCellContent( int row, int col, const QPixmap& );
	void		fillTable ( QCString& base, QStrList& previewFiles);
    void 		clear();
	bool		setCurrentItem(int row, int col);
	bool		setCurrentItem(int flat);
	void 		replace( int location, const QPixmap&, const char *cacheKey = 0 );
	void 		setSelected( int );
	void		setEnabled(bool);
	
	inline int	currentItem()	{	return indexOf(curRow, curCol); }

signals:
	void	highlighted(int);
	void	selected(int);

protected:
    void	paintCell( QPainter*, int row, int col );
    void	mousePressEvent( QMouseEvent* );
    void	keyPressEvent( QKeyEvent* );
    void	focusInEvent( QFocusEvent* );
    void	focusOutEvent( QFocusEvent* );
    void	setCellContentNoUpdate( int row, int col, const QPixmap&  );
	void	setNumRowCols(int row, int col);
	
protected slots:
	void	fillTableAction();
	
private:
	bool	enabled;
    int indexOf( int row, int col );
	
	QList<QPixmap> *contents;
	int				curRow;
	int				curCol;
	int				lastRow;
	int				lastCol;
	int				rows;
	int				cols;
	int				maxCols;
	
	QTimer			fillTimer;
	int				fillCount;
	int				fillMax;
	QStrList		prevFiles;
	QCString		prevFilename;
	QCString		baseDir;
};

#endif //shaderPreviewList_h
