#include "startup.h"

#include <qapplication.h>
#include <qpixmap.h>
#include <qpixmapcache.h> 
#include "mbaMainWindow.h"
#include <qmessagebox.h>
#include <qtooltip.h>

#include "extern.h"
#include "Pixmaps/flow_splash.xpm"

#include "glWidget.h"
#include "lightGroupWidget.h"
#include "mainFlow.h"
#include "mbaScene.h"
#include "mbaSceneWidget.h"
#include "mbaSurfaceTabDialog.h"
#include "particleControlWidget.h"
#include "pPolygonTweak.h"
#include "renderPanelTabDialog.h"
#include "renderRIB.h"
#include "configFlowIO.h"
#include "configFileIO.h"

#ifndef WIN32
	#include <unistd.h>
	#include <sys/types.h>
	#include <sys/wait.h>
	#include <sys/time.h>
#endif
	#include <time.h>

bool	ReadConfigFile();

//==============================================================================
MainStartup::MainStartup()
//	: QFrame(0, 0, WStyle_Customize | WStyle_NoBorder )
	: QFrame(0, 0 )
{
	int	mainFrameWidth = 2;
		
	splashPix = new QPixmap((const char**)flow_splash_xpm);
	setBackgroundPixmap(*splashPix);
	setLineWidth(mainFrameWidth);
	setFrameStyle(QFrame::Raised | QFrame::Box);
	
	float	pW = splashPix->width();
	float	pH = splashPix->height();
	QPalette	flowPalette(QColor(125, 125, 125));
	progressLabel = new QLabel(this);
	progressLabel->setFont(QFont("Helvetica", 12, QFont::Bold));
//	progressLabel->setFrameStyle(QFrame::Raised | QFrame::Panel);
//	progressLabel->setFrameStyle(QFrame::Sunken | QFrame::Panel);
//	progressLabel->setFrameStyle(QFrame::Raised | QFrame::Box);
	progressLabel->setFrameStyle(QFrame::Sunken | QFrame::Box);
	progressLabel->setLineWidth(1);
	progressLabel->setAlignment(AlignCenter);
	progressLabel->move(11, (int)(pH-30));
	progressLabel->setFixedSize((int)(pW-15), 25);
	progressLabel->setPalette(flowPalette);
	
	setPalette(flowPalette);
	
	timer = new QTimer(this);
	
	//-- Connect -----------------------
	QObject::connect(timer, SIGNAL(timeout()), SLOT(Startup()) );

	setFixedSize( splashPix->width()+(mainFrameWidth*4), splashPix->height()+(mainFrameWidth*4));
}

MainStartup::~MainStartup()
{
	delete g_renderRIB;
}

//--------------------------------------------------------------------
void	MainStartup::BeginStartup()
{
	warning("------------------------------------------------------------");
	timer->start(5);
}

//--------------------------------------------------------------------
void	MainStartup::Startup()
{
	static int	step = 0;
	QCString	progressString;
	
	
	switch((StartupIndex)step)
	{
		case NoOp:
			break;
		case ReadTheInitFile:
			ReadInitFile();
			break;
		case CheckForErrors:
			ErrorChecks();
			break;
		case CheckForShaders:
			::CheckShaders();
			break;
		case CreateMainFlow:
			g_mainWidget = g_mainFlow = new MainFlow(0, "MainFlow");
			// g_camGLPort has been created.
			break;
		case CreateDefaultScene:
			g_scene = new MBAScene();
			break;
		case CreateSceneWidget:
			g_sceneWidget = new MBASceneWidget((MBAScene*)0, 0, "MBASceneWidget");
			g_sceneWidget->resize(225, 350);
			break;
		case CreateSurfaceTabDialog:
			g_surfaceEditWidget = new MBASurfaceTabDialog(0, "MBASurfaceTabDialog" );
			QObject::connect(g_surfaceEditWidget, SIGNAL(Updated()), g_camGLPort, SLOT(updateGLIfNecessary()) );
			break;
		case CreateLightGroupWidget:
			g_lightGroupWidgetFrame = new LightGroupWidgetFrame(0, "LightGroupWidgetFrame" );
			g_lightGroupWidget = g_lightGroupWidgetFrame->GetLightGroupWidget();
			QObject::connect(g_lightGroupWidget, SIGNAL(Updated()), g_camGLPort, SLOT(updateGLIfNecessary()) );
			break;
		case CreateRenderPanelTabDialog:
			g_renderRIB = new RenderRIB();
			g_renderOptions = new RenderPanelTabDialog(0, "RenderPanelTabDialog" );
			QObject::connect(g_renderOptions, SIGNAL(Updated()), g_camGLPort, SLOT(updateGLIfNecessary()) );
			break;
		case CreateParticleControlWidget:
			g_particleControlFrame	= new ParticleControlFrame(0, "ParticleControlFrame");
			QObject::connect(g_particleControlFrame->GetParticleControlWidget(), SIGNAL(Updated()), g_camGLPort, SLOT(updateGLIfNecessary()) );
			break;
		case CreatePolyTweakWidget:
			g_polyTweakWidget = new PolygonTweak();
			break;
		case NumStartupSteps:
			break;
		default:
			break;
	}
	int	mssg;
	switch((StartupIndex)step+1)
	{
		case NoOp:
			break;
		case ReadTheInitFile:
			progressString.sprintf("Reading init file...");
			break;
		case CheckForErrors:
			progressString.sprintf("Error checks...");
			break;
		case CheckForShaders:
			progressString.sprintf("Finding shaders...");
			break;
		case CreateMainFlow:
			progressString.sprintf("Create MainFlow...");
			break;
		case CreateDefaultScene:
			progressString.sprintf("Create default scene...");
			break;
		case CreateSceneWidget:
			progressString.sprintf("Create SceneWidget...");
			break;
		case CreateSurfaceTabDialog:
			progressString.sprintf("Create SurfaceTabDialog...");
			break;
		case CreateLightGroupWidget:
			progressString.sprintf("Create LightGroupWidget...");
			break;
		case CreateRenderPanelTabDialog:
			progressString.sprintf("Create RenderPanelTabDialog...");
			break;
		case CreateParticleControlWidget:
			progressString.sprintf("Create ParticleControlWidget...");
			break;
		case CreatePolyTweakWidget:
			progressString.sprintf("Create PolyTweakWidget...");
			break;
		case NumStartupSteps:
		default:
			srand((int)time(0));
			mssg = rand()/(RAND_MAX/4);
			switch(mssg)
			{
				case 0:	progressString.sprintf("groovy.");	break;
				case 1:	progressString.sprintf("Ok...");	break;
				case 2:	progressString.sprintf("Ready.");	break;
				default:	progressString.sprintf("Done.");	break;
			}	
		}
	
	if (step == (int)NumStartupSteps)
	{
		timer->stop();
		g_mainFlow->Init();
		MoveAndResize();
		g_mainFlow->show();
		hide();
		//-- hopefully, this will free some memory...
		splashPix->resize(1, 1);
		setBackgroundPixmap(*splashPix);
		setFixedSize(5, 5);
	}
	progressLabel->setText(progressString);
	step++;
}

//--------------------------------------------------------------------
void	MainStartup::ErrorChecks()
{
	char	message[256];
	bool	ok = true;
	
	timer->stop();
	
	//-- I have to reorg the startup stuff...environment is being checked in two places 
	//-- (out of necessity, but it's not very clean, is it?)
	if (! CheckEnvironment() )	ok = false;

//	warning("!!-- skipping CheckExternalExec");
//	if (! CheckExternalExec() )	ok = false;
	CheckExternalExec();
	
	if (!ok)
	{
		sprintf (message, "One or more errors were encountered\nwhile initializing flow.\nFix em and try again.");
		QMessageBox::critical( this, FLOW_ERROR, message, "Ok");
		exit(1);
	}
	
	timer->start(5);
}

//--------------------------------------------------------------------
bool	MainStartup::CheckReptileDirs()
{
	bool	ret = true;
	char	makeDir[256];
	QDir		currentDir;
	
	sprintf (makeDir, "%s/.ReptileLabour", g_homeDir );
	if ( ! currentDir.cd(makeDir) )
		ret = CreateReptileDir(makeDir);
	sprintf (makeDir, "%s/.ReptileLabour/Init", g_homeDir );
	if ( ! currentDir.cd(makeDir) )
		ret = CreateReptileDir(makeDir);
	sprintf (makeDir, "%s/.ReptileLabour/DirHistory", g_homeDir );
	if ( ! currentDir.cd(makeDir) )
		ret = CreateReptileDir(makeDir);
	sprintf (makeDir, "%s/.ReptileLabour/ShaderInfo", g_homeDir );
	if ( ! currentDir.cd(makeDir) )
		ret = CreateReptileDir(makeDir);
	sprintf (makeDir, "%s/.ReptileLabour/ShaderInfo/Surface", g_homeDir );
	if ( ! currentDir.cd(makeDir) )
		ret = CreateReptileDir(makeDir);
	sprintf (makeDir, "%s/.ReptileLabour/ShaderInfo/Surface/Variations", g_homeDir );
	if ( ! currentDir.cd(makeDir) )
		ret = CreateReptileDir(makeDir);
	sprintf (makeDir, "%s/.ReptileLabour/ShaderInfo/Displacement", g_homeDir );
	if ( ! currentDir.cd(makeDir) )
		ret = CreateReptileDir(makeDir);
	sprintf (makeDir, "%s/.ReptileLabour/ShaderInfo/Displacement/Variations", g_homeDir );
	if ( ! currentDir.cd(makeDir) )
		ret = CreateReptileDir(makeDir);
	
	return ret;
}

//--------------------------------------------------------------------
bool	MainStartup::CheckFlowDirs()
{
	bool	ret = true;
	char	makeDir[256];
	QDir		currentDir;
		
	sprintf (makeDir, "%s", g_flowHome);
	if ( ! currentDir.cd(makeDir) )
		ret = CreateReptileDir(makeDir);
	sprintf (makeDir, "%s/Scene", g_flowHome);
	if ( ! currentDir.cd(makeDir) )
		ret = CreateReptileDir(makeDir);
	sprintf (makeDir, "%s/TIF", g_flowHome);
	if ( ! currentDir.cd(makeDir) )
		ret = CreateReptileDir(makeDir);
	sprintf (makeDir, "%s/RIB", g_flowHome);
	if ( ! currentDir.cd(makeDir) )
		ret = CreateReptileDir(makeDir);

#ifndef WIN32
	if(ret)
		chdir(g_flowHome);
#else
#endif
	
	return ret;
}

//--------------------------------------------------------------------
bool	MainStartup::CreateReptileDir(const char* makeDir)
{
	char	message[256];
	QDir	currentDir;
	
	if (! currentDir.mkdir(makeDir) )
	{
		sprintf (message, "ERROR:\n\"%s\"\ncannot be created.\nCheck file permissions.", makeDir);
		QMessageBox::critical( this, FLOW_ERROR, message, "EXIT");
		return false;
	}
	else
		warning ("created directory\"%s\"", makeDir);
	return true;
}

//--------------------------------------------------------------------
bool	MainStartup::CheckEnvironment()
{
	char	message[256];
	bool	ok = true;
	char	*env;
	
//	env = getenv("HOME");
//	if (env)
//	{
//		g_homeDir = env;
//	}
//	else
//	{
//		sprintf (message, "ERROR:\nEnvironment variable \"%s\" is not set.\n\nflow will not work without it.", "HOME");
//		QMessageBox::critical( this, FLOW_ERROR, message, "Understood");
//		ok = false;
//	}
//	env = getenv("FLOWHOME");
//	if (env)
//	{
//		g_flowHome = env;
//	}
//	else
//	{
//		sprintf (message, "ERROR:\nEnvironment variable \"%s\" is not set.\n\nflow will not work without it.", "FLOWHOME");
//		QMessageBox::critical( this, FLOW_ERROR, message, "Understood");
//		ok = false;
//	}
	
	env = getenv(g_envShaderPath);
	if (env)
	{
		g_shadersPath = env;
	}
	else
	{
		sprintf (message, "ERROR:\nEnvironment variable \"%s\" is not set.\n\nflow will not work without it.\n\n( %s is your RenderMan shaders path )", g_envShaderPath, g_envShaderPath);
		QMessageBox::critical( this, FLOW_ERROR, message, "Understood");
		ok = false;
	}
	
	return ok;
}

//--------------------------------------------------------------------
bool	MainStartup::CheckExternalExec()
{
	bool	ok = true;
	#ifndef WIN32
	char	message[256];
	
	warning("---- Check for tiff->jpeg command line converter ( \"%s\" )", g_tiffToJpegExec);
	QFileInfo	convertExecCheck(g_tiffToJpegExec);
	if (convertExecCheck.isExecutable())
		g_hasTiffToJpegExec = true;
	else
	{
		sprintf (message, "ERROR: tiff to jpeg convert exec\n\"%s\"\ndoes not exist or is not executable.\nSet the correct value in File->Preferences.\n(absolute pathname required)", g_tiffToJpegExec);
		QMessageBox::critical( this, FLOW_ERROR, message, "Understood");
		g_hasTiffToJpegExec = false;
		ok = false;
	}

	warning("---- Check for shader info utility ( \"%s\" )", g_shaderInfoExec);
	QFileInfo	shadeInfoExecCheck(g_shaderInfoExec);
	if (shadeInfoExecCheck.isExecutable())
		g_hasShaderInfoExec = true;
	else
	{
		sprintf (message, "ERROR: shader info exec\n\"%s\"\ndoes not exist or is not executable.\nSet the correct value in File->Preferences.\n(absolute pathname required)", g_shaderInfoExec);
		QMessageBox::critical( this, FLOW_ERROR, message, "Understood");
		g_hasShaderInfoExec = false;
		ok = false;
	}

	warning("---- Check for renderer ( \"%s\" )", g_rendermanExec);
	QFileInfo	rendExecCheck(g_rendermanExec);
	if (rendExecCheck.isExecutable())
		g_hasRendermanExec = true;
	else
	{
		sprintf (message, "ERROR: render exec\n\"%s\"\ndoes not exist or is not executable.\nSet the correct value in File->Preferences.\n(absolute pathname required)", g_rendermanExec);
		QMessageBox::critical( this, FLOW_ERROR, message, "Understood");
		g_hasRendermanExec = false;
		ok = false;
	}

	#endif
	return ok;
}

//--------------------------------------------------------------------
bool	MainStartup::ReadInitFile()
{
//	warning ("MainStartup::ReadInitFile()");
	timer->stop();
	
	char	message[256];
	bool	ok = true;
	char	*env;
	
	//-- Check HOME environment
	env = getenv("HOME");
	if (env)
	{
		g_homeDir = env;
	}
	else
	{
		sprintf (message, "ERROR:\nEnvironment variable \"%s\" is not set.\n\nflow will not work without it.", "HOME");
		strcat(message, "\n\n( If HOME isn't set, you're probably in trouble )");
		QMessageBox::critical( this, FLOW_ERROR, message, "Understood");
		ok = false;
		exit(1); // screw it...just exit...
	}
	env = getenv("FLOWHOME");
	if (env)
	{
		g_flowHome = env;
	}
	else
	{
		sprintf (message, "ERROR:\nEnvironment variable \"%s\" is not set.\n\nflow will not work without it.\n\nSuggestion:\n  setenv FLOWHOME \"%s/FlowData\"", "FLOWHOME", g_homeDir );
		QMessageBox::critical( this, FLOW_ERROR, message, "Understood");
		ok = false;
	}
	if (!ok)
	{
		warning("ERROR: MainStartup::ReadInitFile() !ok");
		InitDefaults();
		if(CheckReptileDirs())
			ConfigFlowIO::WriteConfigFile();
		exit(1);
	}
		
	//-- Make sure correct dirs are there...
	ok = CheckReptileDirs();
	if(!ok)
		return false;
	ok = CheckFlowDirs();
	if(!ok)
		return false;
	
	//-- Load default values before reading file
	InitDefaults();
	
	ConfigFlowIO::ReadConfigFile();
	
	setFonts();
	
	//-- sanity checks...
	if (g_numPreviewCols < 1)	
		g_numPreviewCols = 1;
	if (g_pixmapCacheSize < 1024)
		g_pixmapCacheSize = 1024;
	//-- set size of global pixmap cache
	QPixmapCache::setCacheLimit( g_pixmapCacheSize );

//	QApplication::setPalette(flowPalette);
	
	timer->start(5);
	return true;
}


//--------------------------------------------------------------------
void	MainStartup::InitDefaults()
{
	//-- GUI aspects...
	g_toolTipsEnabled	= true;
	g_killChildrenOnExit = true;
	g_fontName = new char[16];
	strcpy(g_fontName, "helvetica");
	g_fontSize = 12;
	
	g_numPreviewCols		= 2;
	g_previewTableCellSize	= 60;
	g_previewRenderSize		= 128;
	g_pixmapCacheSize		= 5120;
	
//--Default
	basePalette = QApplication::palette();
	
	QColor	butColor = QApplication::palette().active().button();
	QColor	bckColor = QApplication::palette().active().mid();
	flowPalette = QPalette(butColor, bckColor);
	flowPalette.setColor(QColorGroup::Highlight, QApplication::palette().active().dark());
	flowPalette.setColor(QPalette::Disabled, QColorGroup::Highlight, QApplication::palette().active().mid());
	
	QApplication::setPalette(flowPalette);

	//-- External executables...
	g_shaderInfoExec = new char[64];
	strcpy(g_shaderInfoExec, "/usr/local/BMRT/bin/slctell");

	g_tiffToJpegExec = new char[64];
	strcpy(g_tiffToJpegExec, "/usr/X11R6/bin/convert");

	g_rendermanExec = new char[64];
	strcpy(g_rendermanExec, "/usr/local/BMRT/bin/rendrib");
	
	g_rendermanOpts = new char[64];
	strcpy(g_rendermanOpts, "");
	
	g_compiledShaderExt = new char[64];
	strcpy(g_compiledShaderExt, ".slc");
	
	g_envShaderPath = new char[64];
	strcpy(g_envShaderPath, "SHADERS");
	
	int homeLen = strlen(g_homeDir);
	g_shaderSurfDir		   = new char[homeLen+64];
	g_shaderDispDir		   = new char[homeLen+64];
	g_defaultSurfaceShader  = new char[10];
	g_defaultDisplaceShader = new char[10];
	sprintf(g_shaderSurfDir,			"%s/.ReptileLabour/ShaderInfo/Surface/", g_homeDir);
	sprintf(g_shaderDispDir,			"%s/.ReptileLabour/ShaderInfo/Displacement/", g_homeDir);
	sprintf(g_defaultSurfaceShader,	"plastic");
	sprintf(g_defaultDisplaceShader,	"dented");

}

//--------------------------------------------------------------------
void	MainStartup::MoveAndResize()
{ 
	int	dock;
	QCString	infileName;
	infileName.sprintf("%s/.ReptileLabour/Init/flow.geom", g_homeDir);
	
	ConfigFileReader configFile((const char*)infileName);
	int		x, y, w, h;
	bool	show;
	if(configFile.IsValid())
	{
		configFile.findInt("WINMANADD_X", g_winManXAdd);
		configFile.findInt("WINMANADD_Y", g_winManYAdd);
		
		configFile.findToolBarDock("TB0_MAIN_FLOW",			dock);
		g_mainFlow->setToolBar(0, (QMainWindow::ToolBarDock)dock, true);
		configFile.findToolBarDock("TB1_MAIN_FLOW",			dock);
		g_mainFlow->setToolBar(1, (QMainWindow::ToolBarDock)dock, true);
		configFile.findToolBarDock("TB0_PARTICLE_CONTROL",	dock);
		g_particleControlFrame->setToolBar(0, (QMainWindow::ToolBarDock)dock, true);
		configFile.findToolBarDock("TB1_PARTICLE_CONTROL",	dock);
		g_particleControlFrame->setToolBar(1, (QMainWindow::ToolBarDock)dock, true);
		configFile.findToolBarDock("TB0_SURFACE_TAB",		dock);
		g_surfaceEditWidget->setToolBar(0, (QMainWindow::ToolBarDock)dock, true);
		configFile.findToolBarDock("TB0_RENDER_TAB",		dock);
		g_renderOptions->setToolBar(0, (QMainWindow::ToolBarDock)dock, true);
		configFile.findToolBarDock("TB0_SCENE_WIDGET",		dock);
		g_sceneWidget->setToolBar(0, (QMainWindow::ToolBarDock)dock, true);
		configFile.findToolBarDock("TB0_LIGHT_GROUP",		dock);
		g_lightGroupWidgetFrame->setToolBar(0, (QMainWindow::ToolBarDock)dock, true);
		configFile.findToolBarDock("TB1_LIGHT_GROUP",		dock);
		g_lightGroupWidgetFrame->setToolBar(1, (QMainWindow::ToolBarDock)dock, true);
		configFile.findToolBarDock("TB2_LIGHT_GROUP",		dock);
		g_lightGroupWidgetFrame->setToolBar(2, (QMainWindow::ToolBarDock)dock, true);
		
		x = y = 0;
		//-- mainFlow
		if(	configFile.findInt("MAINFLOW_X", x) && configFile.findInt("MAINFLOW_Y", y) &&
	 		configFile.findInt("MAINFLOW_W", w) && configFile.findInt("MAINFLOW_H", h) &&
	 		configFile.findBool("MAINFLOW_SHOW", show) )
		{
			g_mainFlow->resize (w, h);
			if(show)
				g_mainFlow->show();
			g_mainFlow->move( x+g_winManXAdd, y+g_winManYAdd);
		}
		x = y = 0;
		//-- particleControlFrame
		if(	configFile.findInt("PARTICLECONTROL_X", x) && configFile.findInt("PARTICLECONTROL_Y", y) &&
	 		configFile.findInt("PARTICLECONTROL_W", w) && configFile.findInt("PARTICLECONTROL_H", h) &&
	 		configFile.findBool("PARTICLECONTROL_SHOW", show) )
		{
			g_particleControlFrame->resize (w, h);
			if(show)
				g_particleControlFrame->show();
			g_particleControlFrame->move( x+g_winManXAdd, y+g_winManYAdd);
		}
		x = y = 0;
		//-- surfaceEditWidget
		if(	configFile.findInt("SURFACE_X", x) && configFile.findInt("SURFACE_Y", y) &&
	 		configFile.findInt("SURFACE_W", w) && configFile.findInt("SURFACE_H", h) &&
	 		configFile.findBool("SURFACE_SHOW", show) )
		{
			g_surfaceEditWidget->resize (w, h);
			if(show)
				g_surfaceEditWidget->show();
			g_surfaceEditWidget->move( x+g_winManXAdd, y+g_winManYAdd);
		}
		x = y = 0;
		//-- renderOptions
		if(	configFile.findInt("RENDER_X", x) && configFile.findInt("RENDER_Y", y) &&
	 		configFile.findInt("RENDER_W", w) && configFile.findInt("RENDER_H", h) &&
	 		configFile.findBool("RENDER_SHOW", show) )
		{
			g_renderOptions->resize (w, h);
			if(show)
				g_renderOptions->show();
			g_renderOptions->move( x+g_winManXAdd, y+g_winManYAdd);
		}
		x = y = 0;
		//-- lightGroupWidgetFrame
		if(	configFile.findInt("LIGHTGROUP_X", x) && configFile.findInt("LIGHTGROUP_Y", y) &&
	 		configFile.findInt("LIGHTGROUP_W", w) && configFile.findInt("LIGHTGROUP_H", h) &&
	 		configFile.findBool("LIGHTGROUP_SHOW", show) )
		{
			g_lightGroupWidgetFrame->resize (w, h); // this resize isn't catching for some reason...
			//warning("g_lightGroupWidgetFrame->resize (%d, %d)", w, h);
			if(show)
				g_lightGroupWidgetFrame->show();
			g_lightGroupWidgetFrame->move( x+g_winManXAdd, y+g_winManYAdd);
			g_lightGroupWidgetFrame->resize (w, h);
		}
		x = y = 0;
		//-- sceneWidget
		if(	configFile.findInt("SCENETREE_X", x) && configFile.findInt("SCENETREE_Y", y) &&
	 		configFile.findInt("SCENETREE_W", w) && configFile.findInt("SCENETREE_H", h) &&
	 		configFile.findBool("SCENETREE_SHOW", show) )
		{
			g_sceneWidget->resize (w, h);
			if(show)
				g_sceneWidget->show();
			g_sceneWidget->move( x+g_winManXAdd, y+g_winManYAdd);
		}
		//-- OrthoViews
	}	
}

//--------------------------------------------------------------------
void MainStartup::setFonts()
{
	g_font_S   = new QFont( g_fontName, g_fontSize-2);
	g_font_M   = new QFont( g_fontName, g_fontSize  );
	g_font_L   = new QFont( g_fontName, g_fontSize+2);
	g_font_SB  = new QFont( g_fontName, g_fontSize-2, QFont::Bold);
	g_font_MB  = new QFont( g_fontName, g_fontSize  , QFont::Bold);
	g_font_LB  = new QFont( g_fontName, g_fontSize+2, QFont::Bold);
	g_font_SBI = new QFont( g_fontName, g_fontSize-2, QFont::Bold, true);
	g_font_MBI = new QFont( g_fontName, g_fontSize  , QFont::Bold, true);
	g_font_LBI = new QFont( g_fontName, g_fontSize+2, QFont::Bold, true);
	QFont::setDefaultFont ( *g_font_MB );
	
	//-- Motif 1.x style combo has hard-coded height (what bullshit). Get a 
	//-- reasonable height here...
	QComboBox	*combo = new QComboBox(false, 0);
	g_comboHeight = combo->sizeHint().height();
	delete combo;
	
	//-- 
	QToolTip::setPalette( QPalette(QColor(255, 255, 192)) );
	QToolTip::setFont( *g_font_S );
	
}

